@@ -58,6 +58,7 @@
 #include "scripting/flash/display/shaderprecision.h"
 #include "scripting/flash/display/swfversion.h"
 #include "scripting/flash/display/triangleculling.h"
+#include "scripting/flash/display/NativeWindow.h"
 #include "scripting/flash/display3d/flashdisplay3d.h"
 #include "scripting/flash/display3d/flashdisplay3dtextures.h"
 #include "scripting/flash/events/flashevents.h"
@@ -75,10 +76,12 @@
 #include "scripting/flash/net/NetStreamInfo.h"
 #include "scripting/flash/net/NetStreamPlayOptions.h"
 #include "scripting/flash/net/NetStreamPlayTransitions.h"
+#include "scripting/flash/net/DatagramSocket.h"
 #include "scripting/flash/printing/flashprinting.h"
 #include "scripting/flash/sampler/flashsampler.h"
 #include "scripting/flash/security/certificatestatus.h"
 #include "scripting/flash/system/flashsystem.h"
+#include "scripting/flash/system/messagechannel.h"
 #include "scripting/flash/system/messagechannelstate.h"
 #include "scripting/flash/system/securitypanel.h"
 #include "scripting/flash/system/systemupdater.h"
@@ -112,6 +115,7 @@
 #include "scripting/flash/globalization/numberparseresult.h"
 #include "scripting/flash/globalization/stringtools.h"
 #include "scripting/flash/external/ExternalInterface.h"
+#include "scripting/flash/external/ExtensionContext.h"
 #include "scripting/flash/media/flashmedia.h"
 #include "scripting/flash/media/audiodecoder.h"
 #include "scripting/flash/media/audiooutputchangereason.h"
@@ -144,6 +148,7 @@
 #include "scripting/avm1/avm1media.h"
 #include "scripting/avm1/avm1net.h"
 #include "scripting/avm1/avm1text.h"
+#include "scripting/avm1/avm1ui.h"
 #include "scripting/avm1/avm1xml.h"
 
 using namespace lightspark;
