@@ -64,6 +64,8 @@ REGISTER_CLASS_NAME(ASCondition,"flash.concurrent")
 //Desktop (AIR)
 REGISTER_CLASS_NAME(NativeApplication,"flash.desktop")
 REGISTER_CLASS_NAME(NativeDragManager,"flash.desktop")
+REGISTER_CLASS_NAME(NativeProcess,"flash.desktop")
+REGISTER_CLASS_NAME(NativeProcessStartupInfo,"flash.desktop")
 REGISTER_CLASS_NAME(ClipboardFormats,"flash.desktop")
 REGISTER_CLASS_NAME(ClipboardTransferMode,"flash.desktop")
 
@@ -127,6 +129,7 @@ REGISTER_CLASS_NAME(StageScaleMode,"flash.display")
 REGISTER_CLASS_NAME(StageQuality,"flash.display")
 REGISTER_CLASS_NAME(SWFVersion,"flash.display")
 REGISTER_CLASS_NAME(TriangleCulling,"flash.display")
+REGISTER_CLASS_NAME(NativeWindow,"flash.display")
 
 //Display3D
 REGISTER_CLASS_NAME(Context3D,"flash.display3D")
@@ -196,9 +199,11 @@ REGISTER_CLASS_NAME(VideoEvent,"flash.events")
 
 //External interface (browser interaction)
 REGISTER_CLASS_NAME(ExternalInterface,"flash.external")
+REGISTER_CLASS_NAME(ExtensionContext,"flash.external")
 
 //filesystem
 REGISTER_CLASS_NAME2(ASFile,"File","flash.filesystem")
+REGISTER_CLASS_NAME(FileMode,"flash.filesystem")
 REGISTER_CLASS_NAME(FileStream,"flash.filesystem")
 
 //Filters
@@ -304,6 +309,8 @@ REGISTER_CLASS_NAME(URLRequestMethod,"flash.net")
 REGISTER_CLASS_NAME(URLStream,"flash.net")
 REGISTER_CLASS_NAME(URLVariables,"flash.net")
 REGISTER_CLASS_NAME(XMLSocket,"flash.net")
+REGISTER_CLASS_NAME(DatagramSocket,"flash.net")
+
 
 REGISTER_CLASS_NAME(DRMManager,"flash.net.drm")
 
@@ -337,6 +344,7 @@ REGISTER_CLASS_NAME(Accelerometer,"flash.sensors")
 REGISTER_CLASS_NAME(ApplicationDomain,"flash.system")
 REGISTER_CLASS_NAME(Capabilities,"flash.system")
 REGISTER_CLASS_NAME(LoaderContext,"flash.system")
+REGISTER_CLASS_NAME(MessageChannel,"flash.system")
 REGISTER_CLASS_NAME(MessageChannelState,"flash.system")
 REGISTER_CLASS_NAME(Security,"flash.system")
 REGISTER_CLASS_NAME(SecurityDomain,"flash.system")
@@ -448,6 +456,7 @@ REGISTER_CLASS_NAME2(AVM1Shape,"Shape","")
 REGISTER_CLASS_NAME2(AVM1SimpleButton,"SimpleButton","")
 REGISTER_CLASS_NAME2(AVM1Stage,"Stage","")
 REGISTER_CLASS_NAME2(AVM1SharedObject,"SharedObject","")
+REGISTER_CLASS_NAME2(AVM1TextFormat,"TextFormat","")
 REGISTER_CLASS_NAME2(AVM1TextField,"TextField","")
 REGISTER_CLASS_NAME2(AVM1XMLDocument,"XML","")
 REGISTER_CLASS_NAME2(AVM1Video,"Video","")
@@ -458,3 +467,6 @@ REGISTER_CLASS_NAME2(AVM1LoadVars,"LoadVars","")
 REGISTER_CLASS_NAME2(AVM1Bitmap,"Bitmap","")
 REGISTER_CLASS_NAME2(AVM1BitmapData,"BitmapData","")
 REGISTER_CLASS_NAME2(AVM1NetConnection,"NetConnection","")
+REGISTER_CLASS_NAME2(AVM1XMLSocket,"XMLSocket","")
+REGISTER_CLASS_NAME2(AVM1ContextMenuItem,"ContextMenuItem","")
+REGISTER_CLASS_NAME2(AVM1NetStream,"NetStream","")
