@@ -24,6 +24,8 @@
 #include "compat.h"
 #include "backends/decoder.h"
 #include <iostream>
+#include <unordered_set>
+#include <SDL2/SDL.h>
 
 namespace lightspark
 {
@@ -38,13 +40,14 @@ private:
 	bool audio_available;
 	int mixeropened;
 	EngineData* engineData;
-	std::list<AudioStream *> streams;
-	typedef std::list<AudioStream *>::iterator stream_iterator;
-	Mutex streamMutex;
 public:
+	Mutex streamMutex;
+	Mutex managerMutex;
+	std::list<AudioStream *> streams;
+	SDL_AudioDeviceID device;
 	AudioManager(EngineData* engine);
 
-	AudioStream *createStream(AudioDecoder *decoder, bool startpaused, IThreadJob *producer, uint32_t playedTime, double volume);
+	AudioStream *createStream(AudioDecoder *decoder, bool startpaused, IThreadJob *producer, int grouptag, uint32_t playedTime, double volume);
 
 	void toggleMuteAll() { muteAllStreams ? unmuteAll() : muteAll(); }
 	bool allMuted() { return muteAllStreams; }
@@ -63,18 +66,26 @@ private:
 	AudioManager* manager;
 	AudioDecoder *decoder;
 	IThreadJob* producer;
+	int grouptag;
 	bool hasStarted;
 	bool isPaused;
 	bool mixingStarted;
+	ACQUIRE_RELEASE_FLAG(isdone);
 	double curvolume;
 	double unmutevolume;
+	float panning[2];
 	uint64_t playedtime;
 	struct timeval starttime;
 	int mixer_channel;
 public:
+	uint8_t* audiobuffer;
 	bool init(double volume);
+	void deinit();
 	void startMixing();
-	AudioStream(AudioManager* _manager,IThreadJob* _producer,uint64_t _playedtime):manager(_manager),decoder(NULL),producer(_producer),hasStarted(false),isPaused(true),mixingStarted(false),playedtime(_playedtime) { }
+	AudioStream(AudioManager* _manager,IThreadJob* _producer, int _grouptag,uint64_t _playedtime):manager(_manager),decoder(nullptr),producer(_producer),grouptag(_grouptag)
+	  ,hasStarted(false),isPaused(true),mixingStarted(false),isdone(false),curvolume(1.0),unmutevolume(1.0),panning{1.0,1.0},playedtime(_playedtime),mixer_channel(-1),audiobuffer(nullptr)
+	{
+	}
 
 	void SetPause(bool pause_on);
 	uint32_t getPlayedTime();
@@ -85,8 +96,13 @@ public:
 	void resume() { SetPause(false); }
 	void setVolume(double volume);
 	void setPlayedTime(uint64_t p) { playedtime = p; }
+	void setPanning(uint16_t left, uint16_t right);
+	void setIsDone();
+	bool getIsDone() const;
 	inline double getVolume() const { return curvolume; }
+	inline float* getPanning() { return panning; }
 	inline AudioDecoder *getDecoder() const { return decoder; }
+	inline int getGroupTag() const { return grouptag; }
 	~AudioStream();
 };
 
