@@ -25,8 +25,8 @@
 
 using namespace lightspark;
 
-BuiltinStreamDecoder::BuiltinStreamDecoder(std::istream& _s, NetStream* _ns):
-	stream(_s),prevSize(0),decodedAudioBytes(0),decodedVideoFrames(0),decodedTime(0),frameRate(0.0),netstream(_ns),headerbuf(NULL),headerLen(0)
+BuiltinStreamDecoder::BuiltinStreamDecoder(std::istream& _s, NetStream* _ns, uint32_t _buffertime):
+	stream(_s),prevSize(0),decodedAudioBytes(0),decodedVideoFrames(0),decodedTime(0),frameRate(0.0),netstream(_ns),headerbuf(nullptr),headerLen(0),buffertime(_buffertime)
 {
 	STREAM_TYPE t=classifyStream(stream);
 	if(t==FLV_STREAM)
@@ -91,14 +91,14 @@ bool BuiltinStreamDecoder::decodeNextFrame()
 				memcpy(headerbuf,tag.packetData,tag.packetLen);
 				headerLen = tag.packetLen;
 			}
-			if(audioDecoder==NULL)
+			if(audioDecoder==nullptr)
 			{
 				audioCodec=tag.SoundFormat;
 				switch(tag.SoundFormat)
 				{
 					case AAC:
 #ifdef ENABLE_LIBAVCODEC
-						audioDecoder=new FFMpegAudioDecoder(netstream->getSystemState()->getEngineData(), tag.SoundFormat,NULL,0);// tag.packetData, tag.packetLen);
+						audioDecoder=new FFMpegAudioDecoder(netstream->getSystemState()->getEngineData(), tag.SoundFormat,nullptr,0,buffertime);// tag.packetData, tag.packetLen);
 #else
 						audioDecoder=new NullAudioDecoder();
 #endif
@@ -106,7 +106,7 @@ bool BuiltinStreamDecoder::decodeNextFrame()
 						break;
 					case MP3:
 #ifdef ENABLE_LIBAVCODEC
-						audioDecoder=new FFMpegAudioDecoder(netstream->getSystemState()->getEngineData(), tag.SoundFormat,NULL,0);
+						audioDecoder=new FFMpegAudioDecoder(netstream->getSystemState()->getEngineData(), tag.SoundFormat,nullptr,0,buffertime);
 #else
 						audioDecoder=new NullAudioDecoder();
 #endif
@@ -148,7 +148,7 @@ bool BuiltinStreamDecoder::decodeNextFrame()
 			//If the framerate is known give the right timing, otherwise use decodedTime from audio
 			uint32_t frameTime=(frameRate!=0.0)?(decodedVideoFrames*1000/frameRate):decodedTime;
 
-			if(videoDecoder==NULL)
+			if(videoDecoder==nullptr)
 			{
 				//If the isHeader flag is on then the decoder becomes the owner of the data
 				if(tag.isHeader())
@@ -165,7 +165,7 @@ bool BuiltinStreamDecoder::decodeNextFrame()
 				{
 					//First packet but no special handling
 #ifdef ENABLE_LIBAVCODEC
-					videoDecoder=new FFMpegVideoDecoder(tag.codec,NULL,0,frameRate);
+					videoDecoder=new FFMpegVideoDecoder(tag.codec,nullptr,0,frameRate);
 #else
 					videoDecoder=new NullVideoDecoder();
 #endif
@@ -211,10 +211,10 @@ bool BuiltinStreamDecoder::decodeNextFrame()
 				while (it != tag.dataobjectlist.end())
 				{
 					ASObject* o = asAtomHandler::getObject((*it));
-					if(o && o->hasPropertyByMultiname(m,true,false))
+					if(o && o->hasPropertyByMultiname(m,true,false,o->getInstanceWorker()))
 					{
 						asAtom v=asAtomHandler::invalidAtom;
-						o->getVariableByMultiname(v,m);
+						o->getVariableByMultiname(v,m,GET_VARIABLE_OPTION::NONE,o->getInstanceWorker());
 						frameRate = asAtomHandler::toNumber(v);
 						break;
 					}
@@ -226,7 +226,7 @@ bool BuiltinStreamDecoder::decodeNextFrame()
 			break;
 		}
 		default:
-			LOG(LOG_ERROR,_("Unexpected tag type ") << (int)TagType << _(" in FLV"));
+			LOG(LOG_ERROR,"Unexpected tag type " << (int)TagType << " in FLV");
 			return false;
 	}
 	return true;
