@@ -77,7 +77,6 @@ struct GeomToken
 		} vec;
 		const FILLSTYLE*  fillStyle; // make sure the pointer is valid until rendering is done
 		const LINESTYLE2* lineStyle; // make sure the pointer is valid until rendering is done
-		const MORPHLINESTYLE2* morphlineStyle; // make sure the pointer is valid until rendering is done
 		number_t value;
 		uint64_t uval;// this is used to have direct access to the value as it is stored in a vector<uint64_t> for performance
 	};
@@ -95,7 +94,6 @@ struct GeomToken
 	GeomToken(const FILLSTYLE& fs):fillStyle(&fs) {}
 	GeomToken(const LINESTYLE2& ls):lineStyle(&ls) {}
 	GeomToken(number_t val):value(val) {}
-	GeomToken(const MORPHLINESTYLE2& mls):morphlineStyle(&mls) {}
 	GeomToken(const Vector2& _vec)
 	{
 		vec.x=_vec.x;
@@ -107,11 +105,16 @@ struct tokensVector
 {
 	std::vector<uint64_t> filltokens;
 	std::vector<uint64_t> stroketokens;
-	tokensVector() {}
+	RECT boundsRect;
+	uint16_t currentLineWidth;
+	bool canRenderToGL;
+	tokensVector():currentLineWidth(0),canRenderToGL(true) {}
 	void clear()
 	{
 		filltokens.clear();
 		stroketokens.clear();
+		canRenderToGL=true;
+		currentLineWidth=0;
 	}
 	uint32_t size() const
 	{
@@ -123,34 +126,38 @@ struct tokensVector
 	}
 };
 
-enum SHAPE_PATH_SEGMENT_TYPE { PATH_START=0, PATH_STRAIGHT, PATH_CURVE_QUADRATIC };
 
 class ShapePathSegment {
 public:
-	SHAPE_PATH_SEGMENT_TYPE type;
-	uint64_t i;
-	ShapePathSegment(SHAPE_PATH_SEGMENT_TYPE _t, uint64_t _i):type(_t),i(_i){}
-	bool operator==(const ShapePathSegment& v)const{return v.i == i;}
+	uint64_t from;
+	uint64_t quadctrl;
+	uint64_t to;
+	int linestyleindex;
+	ShapePathSegment(uint64_t from, uint64_t quadctrl, uint64_t to, int linestyleindex): from(from), quadctrl(quadctrl), to(to),linestyleindex(linestyleindex) {}
+	ShapePathSegment reverse() const {return ShapePathSegment(to, quadctrl, from, linestyleindex);}
 };
 
 class ShapesBuilder
 {
 private:
-	void joinOutlines();
+	std::vector<ShapePathSegment> currentSubpath;
+	std::map< unsigned int, std::vector<ShapePathSegment> > filledShapesMap;
+	std::map< unsigned int, std::vector<ShapePathSegment> > strokeShapesMap;
+
 	static bool isOutlineEmpty(const std::vector<ShapePathSegment>& outline);
-	inline uint64_t makeVertex(const Vector2& v) const { return (uint64_t(v.y)<<32) | (uint64_t(v.x)&0xffffffff); }
+	static uint64_t makeVertex(const Vector2& v) { return (uint64_t(v.y)<<32) | (uint64_t(v.x)&0xffffffff); }
+	std::map<uint16_t,LINESTYLE2>::iterator getStrokeLineStyle(const std::list<MORPHLINESTYLE2>::iterator& stylesIt, uint16_t ratio, std::map<uint16_t, LINESTYLE2>* linestylecache, const RECT& boundsrc);
 public:
-	std::map< unsigned int, std::vector< std::vector<ShapePathSegment> > > filledShapesMap;
-	std::map< unsigned int, std::vector< std::vector<ShapePathSegment> > > strokeShapesMap;
-	std::vector<ShapePathSegment>* extendOutline(std::vector<std::vector<ShapePathSegment> >* outlines, const Vector2& v1, const Vector2& v2, std::vector<ShapePathSegment>* lastOutline);
-	std::vector<ShapePathSegment>* extendOutlineCurve(std::vector<std::vector<ShapePathSegment> >* outlines, const Vector2& start, const Vector2& control, const Vector2& end, std::vector<ShapePathSegment>* lastOutline);
+	void extendOutline(const Vector2& v1, const Vector2& v2, int linestyleindex);
+	void extendOutlineCurve(const Vector2& start, const Vector2& control, const Vector2& end, int linestyleindex);
+	void endSubpathForStyles(unsigned fill0, unsigned fill1, unsigned stroke, bool formorphing);
 	/**
 		Generate a sequence of cachable tokens that defines the geomtries
 		@param styles This list is supposed to survive until as long as the returned tokens array
 		@param tokens A vector that will be filled with tokens
 	*/
 	void outputTokens(const std::list<FILLSTYLE>& styles, const std::list<LINESTYLE2>& linestyles, tokensVector& tokens);
-	void outputMorphTokens(const std::list<MORPHFILLSTYLE>& styles, const std::list<MORPHLINESTYLE2> &linestyles, tokensVector& tokens, uint16_t ratio);
+	void outputMorphTokens(std::list<MORPHFILLSTYLE>& styles, std::list<MORPHLINESTYLE2>& linestyles, tokensVector& tokens, uint16_t ratio, const RECT& boundsrc);
 	void clear();
 };
 
