@@ -35,7 +35,7 @@ InputThread::InputThread(SystemState* s):m_sys(s),engineData(nullptr),terminated
 	curDragged(),currentMouseOver(),lastMouseDownTarget(),
 	lastKeyUp(SDLK_UNKNOWN), dragLimit(nullptr),button1pressed(false)
 {
-	LOG(LOG_INFO,_("Creating input thread"));
+	LOG(LOG_INFO,"Creating input thread");
 }
 
 void InputThread::start(EngineData* e)
@@ -64,6 +64,28 @@ bool InputThread::worker(SDL_Event *event)
 		if (ret)
 			return ret;
 	}
+	if (event->type == LS_USEREVENT_INTERACTIVEOBJECT_REMOVED_FOM_STAGE)
+	{
+		{
+			Locker locker(inputDataSpinlock);
+			if (currentMouseOver && !currentMouseOver->isOnStage())
+				currentMouseOver.reset();
+		}
+		{
+			Locker locker(mutexListeners);
+			if (lastMouseDownTarget && !lastMouseDownTarget->isOnStage())
+				lastMouseDownTarget.reset();
+			if (lastMouseUpTarget && !lastMouseUpTarget->isOnStage())
+				lastMouseUpTarget.reset();
+		}
+		{
+			Locker locker(mutexDragged);
+			if (curDragged && !curDragged->isOnStage())
+				stopDrag(curDragged.getPtr());
+		}
+	}
+	if (!m_sys || m_sys->isShuttingDown())
+		return false;
 	switch(event->type)
 	{
 		case SDL_KEYDOWN:
@@ -84,21 +106,24 @@ bool InputThread::worker(SDL_Event *event)
 		{
 			if(m_sys->currentVm == nullptr)
 				break;
-		
-			Locker locker(mutexListeners);
-		
 			_NR<InteractiveObject> target = m_sys->stage->getFocusTarget();
 			if (target.isNull())
 				break;
-		
-			tiny_string s = event->text.text;
-			target->incRef();
-			m_sys->currentVm->addIdleEvent(NullRef, _MR(new (m_sys->unaccountedMemory) TextInputEvent(target,s)));
+			// SDL_TEXINPUT sometimes seems to send an empty text, we ignore those events
+			tiny_string s = std::string(event->text.text);
+			if (s.numChars()> 0)
+			{
+				target->incRef();
+				m_sys->currentVm->addIdleEvent(NullRef, _MR(new (m_sys->unaccountedMemory) TextInputEvent(target,s)));
+			}
 			break;
 		}
 		case SDL_MOUSEBUTTONDOWN:
 		{
-			button1pressed=false;
+			{
+				Locker locker(mutexListeners);
+				button1pressed=false;
+			}
 			if(event->button.button == SDL_BUTTON_LEFT)
 			{
 				//Grab focus, to receive keypresses
@@ -108,8 +133,7 @@ bool InputThread::worker(SDL_Event *event)
 				m_sys->windowToStageCoordinates(event->button.x,event->button.y,stageX,stageY);
 				if (m_sys->mainClip->usesActionScript3)
 				{
-					if (event->button.clicks == 1)
-						handleMouseDown(stageX,stageY,SDL_GetModState(),event->button.state == SDL_PRESSED);
+					handleMouseDown(stageX,stageY,SDL_GetModState(),event->button.state == SDL_PRESSED);
 					if (event->button.clicks == 2)
 						handleMouseDoubleClick(stageX,stageY,SDL_GetModState(),event->button.state == SDL_PRESSED);
 				}
@@ -129,7 +153,10 @@ bool InputThread::worker(SDL_Event *event)
 			handleMouseUp(stageX,stageY,SDL_GetModState(),event->button.state == SDL_PRESSED,event->button.button);
 			ret=true;
 			if (event->button.button == SDL_BUTTON_LEFT)
+			{
+				Locker locker(mutexListeners);
 				button1pressed=true;
+			}
 			break;
 		}
 		case SDL_MOUSEMOTION:
@@ -143,7 +170,10 @@ bool InputThread::worker(SDL_Event *event)
 			else
 				m_sys->windowToStageCoordinates(event->motion.x,event->motion.y,stageX,stageY);
 			handleMouseMove(stageX,stageY,SDL_GetModState(),event->motion.state == SDL_PRESSED);
-			button1pressed=false;
+			{
+				Locker locker(mutexListeners);
+				button1pressed=false;
+			}
 			ret=true;
 			break;
 		}
@@ -159,10 +189,19 @@ bool InputThread::worker(SDL_Event *event)
 			ret=true;
 			break;
 		}
-		case SDL_WINDOWEVENT_LEAVE:
+		case SDL_WINDOWEVENT:
 		{
-			handleMouseLeave();
-			ret=true;
+			switch (event->window.event)
+			{
+				case SDL_WINDOWEVENT_LEAVE:
+				{
+					handleMouseLeave();
+					ret=true;
+					break;
+				}
+				default:
+					break;
+			}
 			break;
 		}
 		default:
@@ -235,7 +274,7 @@ _NR<InteractiveObject> InputThread::getMouseTarget(uint32_t x, uint32_t y, Displ
 		return selected;
 	try
 	{
-		_NR<DisplayObject> dispobj=m_sys->stage->hitTest(NullRef,x,y, type,true);
+		_NR<DisplayObject> dispobj=m_sys->stage->hitTest(x,y, type,true);
 		if(!dispobj.isNull() && dispobj->is<InteractiveObject>())
 		{
 			dispobj->incRef();
@@ -244,7 +283,7 @@ _NR<InteractiveObject> InputThread::getMouseTarget(uint32_t x, uint32_t y, Displ
 	}
 	catch(LightsparkException& e)
 	{
-		LOG(LOG_ERROR,_("Error in input handling ") << e.cause);
+		LOG(LOG_ERROR,"Error in input handling " << e.cause);
 		m_sys->setError(e.cause);
 		return NullRef;
 	}
@@ -260,12 +299,11 @@ void InputThread::handleMouseDown(uint32_t x, uint32_t y, SDL_Keymod buttonState
 	_NR<InteractiveObject> selected = getMouseTarget(x, y, DisplayObject::MOUSE_CLICK);
 	if (selected.isNull())
 		return;
-	Locker locker(mutexListeners);
 	number_t localX, localY;
 	selected->globalToLocal(x,y,localX,localY);
-	selected->incRef();
 	m_sys->currentVm->addIdleEvent(selected,
-		_MR(Class<MouseEvent>::getInstanceS(m_sys,"mouseDown",localX,localY,true,buttonState,pressed)));
+		_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"mouseDown",localX,localY,true,buttonState,pressed)));
+	Locker locker(mutexListeners);
 	lastMouseDownTarget=selected;
 }
 
@@ -276,12 +314,26 @@ void InputThread::handleMouseDoubleClick(uint32_t x, uint32_t y, SDL_Keymod butt
 	_NR<InteractiveObject> selected = getMouseTarget(x, y, DisplayObject::DOUBLE_CLICK);
 	if (selected.isNull())
 		return;
-	Locker locker(mutexListeners);
+	if (!selected->isHittable(DisplayObject::DOUBLE_CLICK))
+	{
+		// no double click hit found, add additional down-up-click sequence
+		if (lastMouseUpTarget)
+		{
+			// add mousedown event for last mouseUp target
+			number_t localX, localY;
+			lastMouseUpTarget->globalToLocal(x,y,localX,localY);
+			m_sys->currentVm->addIdleEvent(lastMouseUpTarget,
+										   _MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"mouseDown",localX,localY,true,buttonState,pressed)));
+			// reset lastMouseDownTarget to lastMouseUpTarget to ensure a normal "click" event is sended
+			Locker locker(mutexListeners);
+			lastMouseDownTarget = lastMouseUpTarget;
+		}
+		return;
+	}
 	number_t localX, localY;
 	selected->globalToLocal(x,y,localX,localY);
-	selected->incRef();
 	m_sys->currentVm->addIdleEvent(selected,
-		_MR(Class<MouseEvent>::getInstanceS(m_sys,"doubleClick",localX,localY,true,buttonState,pressed)));
+		_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"doubleClick",localX,localY,true,buttonState,pressed)));
 }
 
 void InputThread::handleMouseUp(uint32_t x, uint32_t y, SDL_Keymod buttonState, bool pressed, uint8_t button)
@@ -291,34 +343,40 @@ void InputThread::handleMouseUp(uint32_t x, uint32_t y, SDL_Keymod buttonState,
 	_NR<InteractiveObject> selected = getMouseTarget(x, y, DisplayObject::MOUSE_CLICK);
 	if (selected.isNull())
 		return;
-	Locker locker(mutexListeners);
 	number_t localX, localY;
 	selected->globalToLocal(x,y,localX,localY);
-	selected->incRef();
 	if (button == SDL_BUTTON_RIGHT)
 	{
 		m_sys->currentVm->addIdleEvent(selected,
-			_MR(Class<MouseEvent>::getInstanceS(m_sys,"contextMenu",localX,localY,true,buttonState,pressed)));
+			_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"contextMenu",localX,localY,true,buttonState,pressed)));
 		return;
 	}
 	m_sys->currentVm->addIdleEvent(selected,
-		_MR(Class<MouseEvent>::getInstanceS(m_sys,"mouseUp",localX,localY,true,buttonState,pressed)));
+		_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"mouseUp",localX,localY,true,buttonState,pressed)));
+	mutexListeners.lock();
+	lastMouseUpTarget=selected;
 	if(lastMouseDownTarget==selected)
 	{
+		lastMouseDownTarget=NullRef;
+		mutexListeners.unlock();
 		//Also send the click event
-		selected->incRef();
 		m_sys->currentVm->addIdleEvent(selected,
-			_MR(Class<MouseEvent>::getInstanceS(m_sys,"click",localX,localY,true,buttonState,pressed)));
+			_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"click",localX,localY,true,buttonState,pressed)));
 	}
 	else if (lastMouseDownTarget)
 	{
-		lastMouseDownTarget->incRef();
-		m_sys->currentVm->addIdleEvent(lastMouseDownTarget,
-			_MR(Class<MouseEvent>::getInstanceS(m_sys,"releaseOutside",localX,localY,true,buttonState,pressed)));
+		_NR<InteractiveObject> tmp = lastMouseDownTarget;
+		lastMouseDownTarget=NullRef;
+		mutexListeners.unlock();
+		m_sys->currentVm->addIdleEvent(tmp,
+			_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"releaseOutside",localX,localY,true,buttonState,pressed)));
+	}
+	else
+	{
+		lastMouseDownTarget=NullRef;
+		mutexListeners.unlock();
 	}
-	lastMouseDownTarget=NullRef;
 }
-
 void InputThread::handleMouseMove(uint32_t x, uint32_t y, SDL_Keymod buttonState, bool pressed)
 {
 	Locker locker(inputDataSpinlock);
@@ -328,8 +386,8 @@ void InputThread::handleMouseMove(uint32_t x, uint32_t y, SDL_Keymod buttonState
 	mousePos.y=y;
 	if (m_sys->getRenderThread()->inSettings)
 		return;
+	_NR<InteractiveObject> selected = getMouseTarget(x, y, DisplayObject::MOUSE_CLICK);
 	mutexDragged.lock();
-	// Handle current drag operation
 	if(curDragged)
 	{
 		Vector2f local;
@@ -348,92 +406,79 @@ void InputThread::handleMouseMove(uint32_t x, uint32_t y, SDL_Keymod buttonState
 
 		curDragged->setX(local.x);
 		curDragged->setY(local.y);
-		mutexDragged.unlock();
 	}
-	// Handle non-drag mouse movement
+	mutexDragged.unlock();
+	number_t localX, localY;
+	if(!currentMouseOver.isNull() && currentMouseOver != selected)
+	{
+		number_t clocalX, clocalY;
+		currentMouseOver->globalToLocal(x,y,clocalX,clocalY);
+		m_sys->currentVm->addIdleEvent(currentMouseOver,
+			_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"mouseOut",clocalX,clocalY,true,buttonState,pressed,selected)));
+		if (selected.isNull())
+			m_sys->currentVm->addIdleEvent(currentMouseOver,_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"rollOut",clocalX,clocalY,true,buttonState,pressed,selected)));
+		currentMouseOver.reset();
+	}
+	if (selected.isNull())
+		return;
+	selected->globalToLocal(x,y,localX,localY);
+	if(currentMouseOver == selected)
+	{
+		m_sys->currentVm->addIdleEvent(selected,
+			_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"mouseMove",localX,localY,true,buttonState,pressed)),true);
+	}
 	else
 	{
-		mutexDragged.unlock();
-		_NR<InteractiveObject> selected = getMouseTarget(x, y, DisplayObject::MOUSE_CLICK);
-		number_t localX, localY;
-		if(!currentMouseOver.isNull() && currentMouseOver != selected)
-		{
-			number_t clocalX, clocalY;
-			currentMouseOver->globalToLocal(x,y,clocalX,clocalY);
-			currentMouseOver->incRef();
-			m_sys->currentVm->addIdleEvent(currentMouseOver,
-				_MR(Class<MouseEvent>::getInstanceS(m_sys,"mouseOut",clocalX,clocalY,true,buttonState,pressed,selected)));
-			currentMouseOver->incRef();
-			m_sys->currentVm->addIdleEvent(currentMouseOver,
-				_MR(Class<MouseEvent>::getInstanceS(m_sys,"rollOut",clocalX,clocalY,true,buttonState,pressed,selected)));
-			currentMouseOver.reset();
-		}
-		if (selected.isNull())
-			return;
-		selected->globalToLocal(x,y,localX,localY);
-		if(currentMouseOver == selected)
-		{
-			selected->incRef();
-			m_sys->currentVm->addIdleEvent(selected,
-				_MR(Class<MouseEvent>::getInstanceS(m_sys,"mouseMove",localX,localY,true,buttonState,pressed)));
-		}
-		else
-		{
-			selected->incRef();
-			m_sys->currentVm->addIdleEvent(selected,
-				_MR(Class<MouseEvent>::getInstanceS(m_sys,"mouseOver",localX,localY,true,buttonState,pressed,currentMouseOver)));
-			selected->incRef();
-			m_sys->currentVm->addIdleEvent(selected,
-				_MR(Class<MouseEvent>::getInstanceS(m_sys,"rollOver",localX,localY,true,buttonState,pressed,currentMouseOver)));
-			currentMouseOver = selected;
-		}
+		m_sys->currentVm->addIdleEvent(selected,
+			_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"mouseOver",localX,localY,true,buttonState,pressed,currentMouseOver)),true);
+		currentMouseOver = selected;
+	}
+	if (selected != lastRolledOver)
+	{
+		if (lastRolledOver)
+			m_sys->currentVm->addIdleEvent(lastRolledOver,_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"rollOut",localX,localY,true,buttonState,pressed,selected)));
+		m_sys->currentVm->addIdleEvent(selected,
+			_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"rollOver",localX,localY,true,buttonState,pressed,lastRolledOver)));
+		lastRolledOver = selected;
 	}
 }
 
 void InputThread::handleScrollEvent(uint32_t x, uint32_t y, uint32_t direction, SDL_Keymod buttonState,bool pressed)
 {
-	if(m_sys->currentVm == NULL)
+	if(m_sys->currentVm == nullptr)
 		return;
 
 	int delta = 1;
 #if SDL_VERSION_ATLEAST(2, 0, 4)
- 	if(direction==SDL_MOUSEWHEEL_NORMAL)
- 		delta = 1;
- 	else if(direction==SDL_MOUSEWHEEL_FLIPPED)
- 		delta = -1;
- 	else
- 		return;
+	if(direction==SDL_MOUSEWHEEL_NORMAL)
+		delta = 1;
+	else if(direction==SDL_MOUSEWHEEL_FLIPPED)
+		delta = -1;
+	else
+		return;
 #endif
 
 	_NR<InteractiveObject> selected = getMouseTarget(x, y, DisplayObject::MOUSE_CLICK);
 	if (selected.isNull())
 		return;
-	Locker locker(mutexListeners);
 	number_t localX, localY;
 	selected->globalToLocal(x,y,localX,localY);
-	selected->incRef();
 	m_sys->currentVm->addIdleEvent(selected,
-		_MR(Class<MouseEvent>::getInstanceS(m_sys,"mouseWheel",localX,localY,true,buttonState,pressed,NullRef,delta)));
+		_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"mouseWheel",localX,localY,true,buttonState,pressed,NullRef,delta)));
 }
 
 void InputThread::handleMouseLeave()
 {
-	if(m_sys->currentVm == NULL)
+	if(m_sys->currentVm == nullptr)
 		return;
 
 	_NR<Stage> stage = _MR(m_sys->stage);
-	stage->incRef();
 	m_sys->currentVm->addIdleEvent(stage,
-		_MR(Class<Event>::getInstanceS(m_sys,"mouseLeave")));
+		_MR(Class<Event>::getInstanceS(m_sys->worker,"mouseLeave")));
 }
 
 bool InputThread::handleKeyboardShortcuts(const SDL_KeyboardEvent *keyevent)
 {
-	if (m_sys->getEngineData()->inFullScreenMode() && keyevent->keysym.sym == SDLK_ESCAPE)
-	{
-		m_sys->getEngineData()->setDisplayState("normal",m_sys);
-		return true;
-	}
 	if (keyevent->keysym.sym == SDLK_MENU)
 	{
 		int stageX,stageY;
@@ -443,12 +488,10 @@ bool InputThread::handleKeyboardShortcuts(const SDL_KeyboardEvent *keyevent)
 		_NR<InteractiveObject> selected = getMouseTarget(stageX,stageY, DisplayObject::MOUSE_CLICK);
 		if (!selected.isNull())
 		{
-			Locker locker(mutexListeners);
 			number_t localX, localY;
 			selected->globalToLocal(x,y,localX,localY);
-			selected->incRef();
 			m_sys->currentVm->addIdleEvent(selected,
-				_MR(Class<MouseEvent>::getInstanceS(m_sys,"contextMenu",localX,localY,true,(SDL_Keymod)keyevent->keysym.mod,false)));
+				_MR(Class<MouseEvent>::getInstanceS(m_sys->worker,"contextMenu",localX,localY,true,(SDL_Keymod)keyevent->keysym.mod,false)));
 			return true;
 		}
 	}
@@ -660,15 +703,127 @@ AS3KeyCode getAS3KeyCode(SDL_Keycode sdlkey)
 	//AS3KEYCODE_VOD
 	return AS3KEYCODE_UNKNOWN;
 }
+AS3KeyCode getAS3KeyCodeFromScanCode(SDL_Scancode sdlscancode)
+{
+	switch (sdlscancode)
+	{
+		case SDL_SCANCODE_RETURN: return AS3KEYCODE_ENTER;
+		case SDL_SCANCODE_ESCAPE: return AS3KEYCODE_ESCAPE;
+		case SDL_SCANCODE_BACKSPACE: return AS3KEYCODE_BACKSPACE;
+		case SDL_SCANCODE_TAB: return AS3KEYCODE_TAB;
+		case SDL_SCANCODE_SPACE: return AS3KEYCODE_SPACE;
+		case SDL_SCANCODE_COMMA: return AS3KEYCODE_COMMA;
+		case SDL_SCANCODE_MINUS: return AS3KEYCODE_MINUS;
+		case SDL_SCANCODE_PERIOD: return AS3KEYCODE_PERIOD;
+		case SDL_SCANCODE_SLASH: return AS3KEYCODE_SLASH;
+		case SDL_SCANCODE_0: return AS3KEYCODE_NUMBER_0;
+		case SDL_SCANCODE_1: return AS3KEYCODE_NUMBER_1;
+		case SDL_SCANCODE_2: return AS3KEYCODE_NUMBER_2;
+		case SDL_SCANCODE_3: return AS3KEYCODE_NUMBER_3;
+		case SDL_SCANCODE_4: return AS3KEYCODE_NUMBER_4;
+		case SDL_SCANCODE_5: return AS3KEYCODE_NUMBER_5;
+		case SDL_SCANCODE_6: return AS3KEYCODE_NUMBER_6;
+		case SDL_SCANCODE_7: return AS3KEYCODE_NUMBER_7;
+		case SDL_SCANCODE_8: return AS3KEYCODE_NUMBER_8;
+		case SDL_SCANCODE_9: return AS3KEYCODE_NUMBER_9;
+		case SDL_SCANCODE_SEMICOLON: return AS3KEYCODE_SEMICOLON;
+		case SDL_SCANCODE_EQUALS: return AS3KEYCODE_EQUAL;
+		case SDL_SCANCODE_LEFTBRACKET: return AS3KEYCODE_LEFTBRACKET;
+		case SDL_SCANCODE_BACKSLASH: return AS3KEYCODE_BACKSLASH;
+		case SDL_SCANCODE_RIGHTBRACKET: return AS3KEYCODE_RIGHTBRACKET;
+		case SDL_SCANCODE_A: return AS3KEYCODE_A;
+		case SDL_SCANCODE_B: return AS3KEYCODE_B;
+		case SDL_SCANCODE_C: return AS3KEYCODE_C;
+		case SDL_SCANCODE_D: return AS3KEYCODE_D;
+		case SDL_SCANCODE_E: return AS3KEYCODE_E;
+		case SDL_SCANCODE_F: return AS3KEYCODE_F;
+		case SDL_SCANCODE_G: return AS3KEYCODE_G;
+		case SDL_SCANCODE_H: return AS3KEYCODE_H;
+		case SDL_SCANCODE_I: return AS3KEYCODE_I;
+		case SDL_SCANCODE_J: return AS3KEYCODE_J;
+		case SDL_SCANCODE_K: return AS3KEYCODE_K;
+		case SDL_SCANCODE_L: return AS3KEYCODE_L;
+		case SDL_SCANCODE_M: return AS3KEYCODE_M;
+		case SDL_SCANCODE_N: return AS3KEYCODE_N;
+		case SDL_SCANCODE_O: return AS3KEYCODE_O;
+		case SDL_SCANCODE_P: return AS3KEYCODE_P;
+		case SDL_SCANCODE_Q: return AS3KEYCODE_Q;
+		case SDL_SCANCODE_R: return AS3KEYCODE_R;
+		case SDL_SCANCODE_S: return AS3KEYCODE_S;
+		case SDL_SCANCODE_T: return AS3KEYCODE_T;
+		case SDL_SCANCODE_U: return AS3KEYCODE_U;
+		case SDL_SCANCODE_V: return AS3KEYCODE_V;
+		case SDL_SCANCODE_W: return AS3KEYCODE_W;
+		case SDL_SCANCODE_X: return AS3KEYCODE_X;
+		case SDL_SCANCODE_Y: return AS3KEYCODE_Y;
+		case SDL_SCANCODE_Z: return AS3KEYCODE_Z;
+		case SDL_SCANCODE_CAPSLOCK:return AS3KEYCODE_CAPS_LOCK;
+		case SDL_SCANCODE_F1: return AS3KEYCODE_F1;
+		case SDL_SCANCODE_F2: return AS3KEYCODE_F2;
+		case SDL_SCANCODE_F3: return AS3KEYCODE_F3;
+		case SDL_SCANCODE_F4: return AS3KEYCODE_F4;
+		case SDL_SCANCODE_F5: return AS3KEYCODE_F5;
+		case SDL_SCANCODE_F6: return AS3KEYCODE_F6;
+		case SDL_SCANCODE_F7: return AS3KEYCODE_F7;
+		case SDL_SCANCODE_F8: return AS3KEYCODE_F8;
+		case SDL_SCANCODE_F9: return AS3KEYCODE_F9;
+		case SDL_SCANCODE_F10: return AS3KEYCODE_F10;
+		case SDL_SCANCODE_F11: return AS3KEYCODE_F11;
+		case SDL_SCANCODE_F12: return AS3KEYCODE_F12;
+		case SDL_SCANCODE_PAUSE: return AS3KEYCODE_PAUSE;
+		case SDL_SCANCODE_INSERT: return AS3KEYCODE_INSERT;
+		case SDL_SCANCODE_HOME: return AS3KEYCODE_HOME;
+		case SDL_SCANCODE_PAGEUP: return AS3KEYCODE_PAGE_UP;
+		case SDL_SCANCODE_DELETE: return AS3KEYCODE_DELETE;
+		case SDL_SCANCODE_END: return AS3KEYCODE_END;
+		case SDL_SCANCODE_PAGEDOWN: return AS3KEYCODE_PAGE_DOWN;
+		case SDL_SCANCODE_RIGHT: return AS3KEYCODE_RIGHT;
+		case SDL_SCANCODE_LEFT: return AS3KEYCODE_LEFT;
+		case SDL_SCANCODE_DOWN: return AS3KEYCODE_DOWN;
+		case SDL_SCANCODE_UP: return AS3KEYCODE_UP;
+		case SDL_SCANCODE_NUMLOCKCLEAR: return AS3KEYCODE_NUMPAD;
+		case SDL_SCANCODE_KP_DIVIDE: return AS3KEYCODE_NUMPAD_DIVIDE;
+		case SDL_SCANCODE_KP_MULTIPLY: return AS3KEYCODE_NUMPAD_MULTIPLY;
+		case SDL_SCANCODE_KP_MINUS:return AS3KEYCODE_NUMPAD_SUBTRACT;
+		case SDL_SCANCODE_KP_PLUS: return AS3KEYCODE_NUMPAD_ADD;
+		case SDL_SCANCODE_KP_ENTER: return AS3KEYCODE_NUMPAD_ENTER;
+		case SDL_SCANCODE_KP_1: return AS3KEYCODE_NUMPAD_1;
+		case SDL_SCANCODE_KP_2: return AS3KEYCODE_NUMPAD_2;
+		case SDL_SCANCODE_KP_3: return AS3KEYCODE_NUMPAD_3;
+		case SDL_SCANCODE_KP_4: return AS3KEYCODE_NUMPAD_4;
+		case SDL_SCANCODE_KP_5: return AS3KEYCODE_NUMPAD_5;
+		case SDL_SCANCODE_KP_6: return AS3KEYCODE_NUMPAD_6;
+		case SDL_SCANCODE_KP_7: return AS3KEYCODE_NUMPAD_7;
+		case SDL_SCANCODE_KP_8: return AS3KEYCODE_NUMPAD_8;
+		case SDL_SCANCODE_KP_9: return AS3KEYCODE_NUMPAD_9;
+		case SDL_SCANCODE_KP_0: return AS3KEYCODE_NUMPAD_0;
+		case SDL_SCANCODE_KP_PERIOD: return AS3KEYCODE_NUMPAD_DECIMAL;
+		case SDL_SCANCODE_F13: return AS3KEYCODE_F13;
+		case SDL_SCANCODE_F14: return AS3KEYCODE_F14;
+		case SDL_SCANCODE_F15: return AS3KEYCODE_F15;
+		case SDL_SCANCODE_HELP: return AS3KEYCODE_HELP;
+		case SDL_SCANCODE_MENU: return AS3KEYCODE_MENU;
+		case SDL_SCANCODE_LCTRL: return AS3KEYCODE_CONTROL;
+		case SDL_SCANCODE_LSHIFT: return AS3KEYCODE_SHIFT;
+		case SDL_SCANCODE_LALT: return AS3KEYCODE_ALTERNATE;
+		case SDL_SCANCODE_RCTRL: return AS3KEYCODE_CONTROL;
+		case SDL_SCANCODE_RSHIFT: return AS3KEYCODE_SHIFT;
+		case SDL_SCANCODE_RALT: return AS3KEYCODE_ALTERNATE;
+		case SDL_SCANCODE_AC_SEARCH: return AS3KEYCODE_SEARCH;
+		case SDL_SCANCODE_AC_BACK: return AS3KEYCODE_BACK ;
+		case SDL_SCANCODE_AC_STOP:return AS3KEYCODE_STOP;
+		default:
+			LOG(LOG_NOT_IMPLEMENTED,"scancode mapping for "<<sdlscancode);
+			return AS3KEYCODE_UNKNOWN;
+	}
+}
 
 
 void InputThread::sendKeyEvent(const SDL_KeyboardEvent *keyevent)
 {
-	if(m_sys->currentVm == NULL)
+	if(m_sys->currentVm == nullptr)
 		return;
 
-	Locker locker(mutexListeners);
-
 	_NR<DisplayObject> target = m_sys->stage->getFocusTarget();
 	if (target.isNull())
 		return;
@@ -679,41 +834,10 @@ void InputThread::sendKeyEvent(const SDL_KeyboardEvent *keyevent)
 	else
 		type = "keyUp";
 
-	target->incRef();
 	m_sys->currentVm->addIdleEvent(target,
-	    _MR(Class<KeyboardEvent>::getInstanceS(m_sys,type,keyevent->keysym.scancode,getAS3KeyCode(keyevent->keysym.sym), (SDL_Keymod)keyevent->keysym.mod,keyevent->keysym.sym)));
+	    _MR(Class<KeyboardEvent>::getInstanceS(m_sys->worker,type, keyevent->keysym.scancode,getAS3KeyCodeFromScanCode(keyevent->keysym.scancode),getAS3KeyCode(keyevent->keysym.sym), (SDL_Keymod)keyevent->keysym.mod,keyevent->keysym.sym)));
 }
 
-void InputThread::addListener(InteractiveObject* ob)
-{
-	Locker locker(mutexListeners);
-	assert(ob);
-
-#ifndef NDEBUG
-	vector<InteractiveObject*>::const_iterator it=find(listeners.begin(),listeners.end(),ob);
-	//Object is already register, should not happen
-	if(it != listeners.end())
-	{
-		LOG(LOG_ERROR, "Trying to addListener an InteractiveObject that's already added.");
-		return;
-	}
-#endif
-	
-	//Register the listener
-	listeners.push_back(ob);
-}
-
-void InputThread::removeListener(InteractiveObject* ob)
-{
-	Locker locker(mutexListeners);
-
-	vector<InteractiveObject*>::iterator it=find(listeners.begin(),listeners.end(),ob);
-	if(it==listeners.end()) //Listener not found
-		return;
-	
-	//Unregister the listener
-	listeners.erase(it);
-}
 
 void InputThread::startDrag(_R<Sprite> s, const lightspark::RECT* limit, Vector2f offset)
 {
@@ -785,5 +909,4 @@ void InputThread::setLastKeyUp(KeyboardEvent *e)
 	lastKeyUp = e->getSDLKeyCode();
 	keyDownSet.erase(getAS3KeyCode(e->getSDLKeyCode()));
 	lastKeyDown = 0;
-	
 }
