@@ -55,6 +55,8 @@ private:
 	_NR<Sprite> curDragged;
 	_NR<InteractiveObject> currentMouseOver;
 	_NR<InteractiveObject> lastMouseDownTarget;
+	_NR<InteractiveObject> lastMouseUpTarget;
+	_NR<InteractiveObject> lastRolledOver;
 	SDL_Keymod lastKeymod;
 	set<AS3KeyCode> keyDownSet;
 	SDL_Keycode lastKeyDown;
@@ -88,8 +90,6 @@ public:
 	~InputThread();
 	void wait();
 	void start(EngineData* data);
-	void addListener(InteractiveObject* ob);
-	void removeListener(InteractiveObject* ob);
 	void startDrag(_R<Sprite> s, const RECT* limit, Vector2f dragOffset);
 	void stopDrag(Sprite* s);
 
