@@ -26,6 +26,10 @@ using namespace lightspark;
 using namespace std;
 
 LocaleManager::LocaleManager()
+{
+	init();
+}
+void LocaleManager::init()
 {
 	locales["af-ZA"] = new LocaleItem("af-ZA", "af", "ZA", "Latn", "", false);
 	locales["am-ET"] = new LocaleItem("am-ET", "am", "ET", "Ethi", "", false);
@@ -379,4 +383,4 @@ std::vector<std::string> LocaleManager::getAvailableLocaleIDNames()
 		}
 	}
 	return values;
-}
\ No newline at end of file
+}
