@@ -21,6 +21,7 @@
 #include "backends/security.h"
 #include "scripting/abc.h"
 #include "scripting/class.h"
+#include "scripting/flash/net/flashnet.h"
 #include "swf.h"
 #include "backends/config.h"
 #include "backends/netutils.h"
@@ -170,28 +171,28 @@ StandaloneDownloadManager::~StandaloneDownloadManager()
 Downloader* StandaloneDownloadManager::download(const URLInfo& url, _R<StreamCache> cache, ILoadable* owner)
 {
 	bool cached = dynamic_cast<FileStreamCache *>(cache.getPtr()) != NULL;
-	LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager::download '") << url.getParsedURL()
-			<< "'" << (cached ? _(" - cached") : ""));
+	LOG(LOG_INFO, "NET: STANDALONE: DownloadManager::download '" << url.getParsedURL()
+			<< "'" << (cached ? " - cached" : ""));
 	ThreadedDownloader* downloader;
 	// empty URL means data is generated from calls to NetStream::appendBytes
 	if(url.isEmpty())
 	{
-		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: Data generation mode"));
+		LOG(LOG_INFO, "NET: STANDALONE: DownloadManager: Data generation mode");
 		downloader=new LocalDownloader(url.getPath(), cache, owner,true);
 	}
 	else if(url.getProtocol() == "file")
 	{
-		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: local file"));
+		LOG(LOG_INFO, "NET: STANDALONE: DownloadManager: local file");
 		downloader=new LocalDownloader(url.getPath(), cache, owner);
 	}
 	else if(url.getProtocol().substr(0, 4) == "rtmp")
 	{
-		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: RTMP stream"));
+		LOG(LOG_INFO, "NET: STANDALONE: DownloadManager: RTMP stream");
 		downloader=new RTMPDownloader(url.getParsedURL(), cache, url.getStream(), owner);
 	}
 	else
 	{
-		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: remote file"));
+		LOG(LOG_INFO, "NET: STANDALONE: DownloadManager: remote file");
 		downloader=new CurlDownloader(url.getParsedURL(), cache, owner);
 	}
 	downloader->enableFencingWaiting();
@@ -214,18 +215,18 @@ Downloader* StandaloneDownloadManager::downloadWithData(const URLInfo& url, _R<S
 		const std::vector<uint8_t>& data,
 		const std::list<tiny_string>& headers, ILoadable* owner)
 {
-	LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager::downloadWithData '") << url.getParsedURL());
+	LOG(LOG_INFO, "NET: STANDALONE: DownloadManager::downloadWithData '" << url.getParsedURL());
 	ThreadedDownloader* downloader;
 	if(url.getProtocol() == "file")
 	{
-		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: local file - Ignoring data field"));
+		LOG(LOG_INFO, "NET: STANDALONE: DownloadManager: local file - Ignoring data field");
 		downloader=new LocalDownloader(url.getPath(), cache, owner);
 	}
 	else if(url.getProtocol() == "rtmpe")
 		throw RunTimeException("RTMPE does not support additional data");
 	else
 	{
-		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: remote file"));
+		LOG(LOG_INFO, "NET: STANDALONE: DownloadManager: remote file");
 		downloader=new CurlDownloader(url.getParsedURL(), cache, data, headers, owner);
 	}
 	downloader->enableFencingWaiting();
@@ -360,7 +361,7 @@ void Downloader::append(uint8_t* buf, uint32_t added)
  */
 void Downloader::parseHeaders(const char* _headers, bool _setLength)
 {
-	if(_headers == NULL)
+	if(_headers == nullptr)
 		return;
 
 	std::string headersStr(_headers);
@@ -385,9 +386,9 @@ void Downloader::parseHeaders(const char* _headers, bool _setLength)
  */
 void Downloader::parseHeader(std::string header, bool _setLength)
 {
-	if(header.substr(0, 9) == "HTTP/1.1 " || header.substr(0, 9) == "HTTP/1.0 ") 
+	if(header.substr(0, 9) == "HTTP/1.1 " || header.substr(0, 9) == "HTTP/1.0 " || header.substr(0, 7) == "HTTP/2 ") 
 	{
-		std::string status = header.substr(9, 3);
+		std::string status = header.substr(0, 7) == "HTTP/2 " ? header.substr(7, 3) : header.substr(9, 3);
 		requestStatus = atoi(status.c_str());
 		//HTTP error or server error or proxy error, let's fail
 		//TODO: shouldn't we fetch the data anyway
@@ -425,7 +426,7 @@ void Downloader::parseHeader(std::string header, bool _setLength)
 			//Set the new real URL when we are being redirected
 			if(getRequestStatus()/100 == 3 && headerName == "location")
 			{
-				LOG(LOG_INFO, _("NET: redirect detected"));
+				LOG(LOG_INFO, "NET: redirect detected");
 				setRedirected(URLInfo(url).goToURL(tiny_string(headerValue)).getParsedURL());
 			}
 			if(headerName == "content-length")
@@ -576,7 +577,7 @@ void CurlDownloader::execute()
 		setFailed();
 		return;
 	}
-	LOG(LOG_INFO, _("NET: CurlDownloader::execute: reading remote file: ") << url.raw_buf());
+	LOG(LOG_INFO, "NET: CurlDownloader::execute: reading remote file: " << url.raw_buf());
 #ifdef ENABLE_CURL
 	CURL *curl;
 	CURLcode res;
@@ -596,13 +597,18 @@ void CurlDownloader::execute()
 		curl_easy_setopt(curl, CURLOPT_WRITEDATA, this);
 		curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, write_header);
 		curl_easy_setopt(curl, CURLOPT_HEADERDATA, this);
+#if LIBCURL_VERSION_NUM>= 0x072000
+		curl_easy_setopt(curl, CURLOPT_XFERINFOFUNCTION, progress_callback);
+#else
 		curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, progress_callback);
+#endif
 		curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, this);
 		curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0);
 		curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
 		//Its probably a good idea to limit redirections, 100 should be more than enough
 		curl_easy_setopt(curl, CURLOPT_MAXREDIRS, 100);
-		curl_easy_setopt(curl, CURLOPT_USERAGENT, "Mozilla/5.0");
+		// TODO use same useragent as adobe
+		//curl_easy_setopt(curl, CURLOPT_USERAGENT, "Mozilla/5.0");
 		// Empty string means that CURL will decompress if the
 		// server send a compressed file. (This has been
 		// renamed to CURLOPT_ACCEPT_ENCODING in newer CURL,
@@ -630,9 +636,6 @@ void CurlDownloader::execute()
 			//data is const, it would not be invalidated
 			curl_easy_setopt(curl, CURLOPT_POSTFIELDS, &data.front());
 			curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, data.size());
-
-			//For POST it's mandatory to set the Content-Type
-			assert(hasContentType);
 		}
 
 		if(headerList)
@@ -657,7 +660,7 @@ void CurlDownloader::execute()
 	}
 #else
 	//ENABLE_CURL not defined
-	LOG(LOG_ERROR,_("NET: CURL not enabled in this build. Downloader will always fail."));
+	LOG(LOG_ERROR,"NET: CURL not enabled in this build. Downloader will always fail.");
 	setFailed();
 	return;
 #endif
@@ -747,7 +750,7 @@ void LocalDownloader::execute()
 	}
 	else
 	{
-		LOG(LOG_INFO, _("NET: LocalDownloader::execute: reading local file: ") << url.raw_buf());
+		LOG(LOG_INFO, "NET: LocalDownloader::execute: reading local file: " << url.raw_buf());
 		//If the caching is selected, we override the normal behaviour and use the local file as the cache file
 		//This prevents unneeded copying of the file's data
 
@@ -789,7 +792,7 @@ void LocalDownloader::execute()
 				}
 				if(readFailed)
 				{
-					LOG(LOG_ERROR, _("NET: LocalDownloader::execute: reading from local file failed: ") << url.raw_buf());
+					LOG(LOG_ERROR, "NET: LocalDownloader::execute: reading from local file failed: " << url.raw_buf());
 					setFailed();
 					return;
 				}
@@ -797,7 +800,7 @@ void LocalDownloader::execute()
 			}
 			else
 			{
-				LOG(LOG_ERROR, _("NET: LocalDownloader::execute: could not open local file: ") << url.raw_buf());
+				LOG(LOG_ERROR, "NET: LocalDownloader::execute: could not open local file: " << url.raw_buf());
 				setFailed();
 				return;
 			}
@@ -832,8 +835,7 @@ bool DownloaderThreadBase::createDownloader(_R<StreamCache> cache,
 			return false;
 		if(evaluationResult == SecurityManager::NA_CROSSDOMAIN_POLICY)
 		{
-			dispatcher->incRef();
-			getVm(dispatcher->getSystemState())->addEvent(dispatcher,_MR(Class<SecurityErrorEvent>::getInstanceS(dispatcher->getSystemState(),"SecurityError: "
+			getVm(dispatcher->getSystemState())->addEvent(dispatcher,_MR(Class<SecurityErrorEvent>::getInstanceS(dispatcher->getInstanceWorker(),"SecurityError: "
 												 "connection to domain not allowed by securityManager")));
 			return false;
 		}
@@ -842,8 +844,7 @@ bool DownloaderThreadBase::createDownloader(_R<StreamCache> cache,
 	{
 		// url points to an adobe signed library which we cannot handle, so we abort here
 		LOG(LOG_NOT_IMPLEMENTED,"we don't handle adobe signed libraries");
-		dispatcher->incRef();
-		getVm(dispatcher->getSystemState())->addEvent(dispatcher,_MR(Class<IOErrorEvent>::getInstanceS(dispatcher->getSystemState())));
+		getVm(dispatcher->getSystemState())->addEvent(dispatcher,_MR(Class<IOErrorEvent>::getInstanceS(dispatcher->getInstanceWorker())));
 		return false;
 	}
 
@@ -869,11 +870,11 @@ void DownloaderThreadBase::jobFence()
 	//Get a copy of the downloader, do hold the lock less time.
 	//It's safe to set this->downloader to NULL, this is the last function that will
 	//be called over this thread job
-	Downloader* d=NULL;
+	Downloader* d=nullptr;
 	{
 		Locker l(downloaderLock);
 		d=downloader;
-		downloader=NULL;
+		downloader=nullptr;
 	}
 	if(d)
 		getSys()->downloadManager->destroy(d);
@@ -885,7 +886,7 @@ void DownloaderThreadBase::threadAbort()
 {
 	//We have to stop the downloader
 	Locker l(downloaderLock);
-	if(downloader != NULL)
+	if(downloader != nullptr)
 		downloader->stop();
 	threadAborting=true;
 }
