@@ -63,13 +63,13 @@ RenderThread::RenderThread(SystemState* s):GLRenderContext(),
 	offsetX(0),offsetY(0),tempBufferAcquired(false),frameCount(0),secsCount(0),initialized(0),refreshNeeded(false),screenshotneeded(false),inSettings(false),canrender(false),
 	cairoTextureContextSettings(nullptr),cairoTextureContext(nullptr)
 {
-	LOG(LOG_INFO,_("RenderThread this=") << this);
+	LOG(LOG_INFO,"RenderThread this=" << this);
 #ifdef _WIN32
 	fontPath = "TimesNewRoman.ttf";
 #else
 	fontPath = "Serif";
 #endif
-	gettimeofday(&time_s, NULL);
+	gettimeofday(&time_s, nullptr);
 }
 
 void RenderThread::start(EngineData* data)
@@ -87,7 +87,7 @@ void RenderThread::stop()
 RenderThread::~RenderThread()
 {
 	wait();
-	LOG(LOG_INFO,_("~RenderThread this=") << this);
+	LOG(LOG_INFO,"~RenderThread this=" << this);
 }
 
 void RenderThread::handleNewTexture()
@@ -107,8 +107,10 @@ void RenderThread::finalizeUpload()
 	ITextureUploadable* u=prevUploadJob;
 	uint32_t w,h;
 	u->sizeNeeded(w,h);
-	const TextureChunk& tex=u->getTexture();
-	loadChunkBGRA(tex, w, h, engineData->getCurrentPixBuf());
+	TextureChunk& tex=u->getTexture();
+	u->contentScale(tex.xContentScale, tex.yContentScale);
+	u->contentOffset(tex.xOffset, tex.yOffset);
+	loadChunkBGRA(tex, w, h, u->upload(false));
 	u->uploadFence();
 	prevUploadJob=nullptr;
 }
@@ -119,16 +121,9 @@ void RenderThread::handleUpload()
 	assert(u);
 	uint32_t w,h;
 	u->sizeNeeded(w,h);
-	engineData->resizePixelBuffers(w,h);
-
-	uint8_t* buf= engineData->switchCurrentPixBuf(w,h);
-	if (!buf)
-	{
-		handleGLErrors();
-		return;
-	}
-	u->upload(buf, w, h);
-
+	
+	//force creation of buffer if neccessary
+	u->upload(true);
 	//Get the texture to be sure it's allocated when the upload comes
 	u->getTexture();
 	prevUploadJob=u;
@@ -155,6 +150,7 @@ int RenderThread::worker(void* d)
 {
 	RenderThread *th = (RenderThread *)d;
 	setTLSSys(th->m_sys);
+	setTLSWorker(th->m_sys->worker);
 	/* set TLS variable for getRenderThread() */
 	tls_set(renderThread, th);
 
@@ -180,7 +176,7 @@ int RenderThread::worker(void* d)
 	}
 	catch(LightsparkException& e)
 	{
-		LOG(LOG_ERROR,_("Exception in RenderThread, stopping rendering: ") << e.what());
+		LOG(LOG_ERROR,"Exception in RenderThread, stopping rendering: " << e.what());
 		//TODO: add a comandline switch to disable rendering. Then add that commandline switch
 		//to the test runner script and uncomment the next line
 		//m_sys->setError(e.cause);
@@ -216,7 +212,7 @@ bool RenderThread::doRender(ThreadProfile* profile,Chronometer* chronometer)
 		newWidth=0;
 		newHeight=0;
 		//End of order critical part
-		LOG(LOG_INFO,_("Window resized to ") << windowWidth << 'x' << windowHeight);
+		LOG(LOG_INFO,"Window resized to " << windowWidth << 'x' << windowHeight);
 		commonGLResize();
 		m_sys->resizeCompleted();
 		if (profile && chronometer)
@@ -389,11 +385,6 @@ void RenderThread::renderSettingsPage()
 	renderText(cr, "allow local storage",10,height-25);
 
 	engineData->exec_glUniform1f(alphaUniform, 1);
-	engineData->exec_glUniform1f(rotateUniform, 0);
-	engineData->exec_glUniform2f(beforeRotateUniform, width,height);
-	engineData->exec_glUniform2f(afterRotateUniform, width,height);
-	engineData->exec_glUniform2f(startPositionUniform,(windowWidth-width)/2, (windowHeight-height)/2);
-	engineData->exec_glUniform2f(scaleUniform, 1.0,1.0);
 	engineData->exec_glUniform4f(colortransMultiplyUniform, 1.0,1.0,1.0,1.0);
 	engineData->exec_glUniform4f(colortransAddUniform, 0.0,0.0,0.0,0.0);
 	mapCairoTexture(width, height,true);
@@ -465,7 +456,7 @@ bool RenderThread::loadShaderPrograms()
 	const char *fs = 
 #include "lightspark.frag"
 ;
-	engineData->exec_glShaderSource(f, 1, &fs,NULL);
+	engineData->exec_glShaderSource(f, 1, &fs,nullptr);
 	uint32_t g = engineData->exec_glCreateShader_GL_VERTEX_SHADER();
 	
 	bool ret=true;
@@ -474,7 +465,7 @@ bool RenderThread::loadShaderPrograms()
 	int stat;
 	engineData->exec_glCompileShader(f);
 	engineData->exec_glGetShaderInfoLog(f,1024,&a,str);
-	LOG(LOG_INFO,_("Fragment shader compilation ") << str);
+	LOG(LOG_INFO,"Fragment shader compilation " << str);
 	engineData->exec_glGetShaderiv_GL_COMPILE_STATUS(f, &stat);
 	if (!stat)
 	{
@@ -485,10 +476,10 @@ bool RenderThread::loadShaderPrograms()
 	const char *fs2 = 
 #include "lightspark.vert"
 ;
-	engineData->exec_glShaderSource(g, 1, &fs2,NULL);
+	engineData->exec_glShaderSource(g, 1, &fs2,nullptr);
 
 	engineData->exec_glGetShaderInfoLog(g,1024,&a,str);
-	LOG(LOG_INFO,_("Vertex shader compilation ") << str);
+	LOG(LOG_INFO,"Vertex shader compilation " << str);
 
 	engineData->exec_glCompileShader(g);
 	engineData->exec_glGetShaderiv_GL_COMPILE_STATUS(g, &stat);
@@ -536,12 +527,13 @@ void RenderThread::commonGLInit(int width, int height)
 	//Load shaders
 	loadShaderPrograms();
 	engineData->driverInfoString = engineData->getGLDriverInfo();
+	engineData->getGlCompressedTextureFormats();
 
 	engineData->exec_glBlendFunc(BLEND_ONE,BLEND_ONE_MINUS_SRC_ALPHA);
 	engineData->exec_glEnable_GL_BLEND();
 
 	engineData->exec_glActiveTexture_GL_TEXTURE0(0);
-	//Viewport setup is left for GLResize	
+	//Viewport setup is left for GLResize
 
 	//Get the maximum allowed texture size, up to 8192
 	int maxTexSize;
@@ -570,12 +562,6 @@ void RenderThread::commonGLInit(int width, int height)
 	projectionMatrixUniform =engineData->exec_glGetUniformLocation(gpu_program,"ls_ProjectionMatrix");
 	modelviewMatrixUniform =engineData->exec_glGetUniformLocation(gpu_program,"ls_ModelViewMatrix");
 
-	fragmentTexScaleUniform=engineData->exec_glGetUniformLocation(gpu_program,"texScale");
-	rotateUniform =engineData->exec_glGetUniformLocation(gpu_program,"rotation");
-	beforeRotateUniform =engineData->exec_glGetUniformLocation(gpu_program,"beforeRotate");
-	afterRotateUniform=engineData->exec_glGetUniformLocation(gpu_program,"afterRotate");
-	startPositionUniform=engineData->exec_glGetUniformLocation(gpu_program,"startPosition");
-	scaleUniform=engineData->exec_glGetUniformLocation(gpu_program,"scale");
 	colortransMultiplyUniform=engineData->exec_glGetUniformLocation(gpu_program,"colorTransformMultiply");
 	colortransAddUniform=engineData->exec_glGetUniformLocation(gpu_program,"colorTransformAdd");
 	directColorUniform=engineData->exec_glGetUniformLocation(gpu_program,"directColor");
@@ -592,7 +578,7 @@ void RenderThread::commonGLInit(int width, int height)
 
 	if(handleGLErrors())
 	{
-		LOG(LOG_ERROR,_("GL errors during initialization"));
+		LOG(LOG_ERROR,"GL errors during initialization");
 	}
 }
 
@@ -650,15 +636,15 @@ void RenderThread::requestResize(uint32_t w, uint32_t h, bool force)
 	}
 	if (w != UINT32_MAX)
 		newWidth=w;
-	else
+	else if (newWidth == 0)
 		newWidth=windowWidth;
 	if (h != UINT32_MAX)
 		newHeight=h;
-	else
+	else if (newHeight == 0)
 		newHeight=windowHeight;
 	resizeNeeded=true;
 	m_sys->stage->incRef();
-	getVm(m_sys)->addEvent(_MR(m_sys->stage),_MR(Class<Event>::getInstanceS(m_sys,"resize")));
+	getVm(m_sys)->addEvent(_MR(m_sys->stage),_MR(Class<Event>::getInstanceS(m_sys->worker,"resize")));
 	
 	event.signal();
 }
@@ -764,11 +750,6 @@ void RenderThread::plotProfilingData()
 	for(;it!=m_sys->profilingData.end();++it)
 		(*it)->plot(1000000/m_sys->mainClip->getFrameRate(),cr);
 	engineData->exec_glUniform1f(directUniform, 0);
-	engineData->exec_glUniform1f(rotateUniform, 0);
-	engineData->exec_glUniform2f(beforeRotateUniform, windowWidth, windowHeight);
-	engineData->exec_glUniform2f(afterRotateUniform, windowWidth, windowHeight);
-	engineData->exec_glUniform2f(startPositionUniform,0,0);
-	engineData->exec_glUniform2f(scaleUniform, 1.0,1.0);
 	engineData->exec_glUniform4f(colortransMultiplyUniform, 1.0,1.0,1.0,1.0);
 	engineData->exec_glUniform4f(colortransAddUniform, 0.0,0.0,0.0,0.0);
 
@@ -788,10 +769,13 @@ bool RenderThread::coreRendering()
 	engineData->exec_glBindFramebuffer_GL_FRAMEBUFFER(0);
 	engineData->exec_glFrontFace(false);
 	engineData->exec_glDrawBuffer_GL_BACK();
-	//Clear the back buffer
-	RGB bg=m_sys->mainClip->getBackground();
-	engineData->exec_glClearColor(bg.Red/255.0F,bg.Green/255.0F,bg.Blue/255.0F,1);
-	engineData->exec_glClear_GL_COLOR_BUFFER_BIT();
+	if (!m_sys->stage->renderStage3D()) // no need to clear the backbuffer when using Stage3D
+	{
+		//Clear the back buffer
+		RGB bg=m_sys->mainClip->getBackground();
+		engineData->exec_glClearColor(bg.Red/255.0F,bg.Green/255.0F,bg.Blue/255.0F,1);
+		engineData->exec_glClear_GL_COLOR_BUFFER_BIT();
+	}
 	engineData->exec_glUseProgram(gpu_program);
 	lsglLoadIdentity();
 	setMatrixUniform(LSGL_MODELVIEW);
@@ -856,11 +840,6 @@ void RenderThread::renderErrorPage(RenderThread *th, bool standalone)
 
 	engineData->exec_glUniform1f(directUniform, 0);
 	engineData->exec_glUniform1f(alphaUniform, 1);
-	engineData->exec_glUniform1f(rotateUniform, 0);
-	engineData->exec_glUniform2f(beforeRotateUniform, windowWidth, windowHeight);
-	engineData->exec_glUniform2f(afterRotateUniform, windowWidth, windowHeight);
-	engineData->exec_glUniform2f(startPositionUniform,0,0);
-	engineData->exec_glUniform2f(scaleUniform, 1.0,1.0);
 	engineData->exec_glUniform4f(colortransMultiplyUniform, 1.0,1.0,1.0,1.0);
 	engineData->exec_glUniform4f(colortransAddUniform, 0.0,0.0,0.0,0.0);
 	mapCairoTexture(windowWidth, windowHeight);
@@ -870,12 +849,18 @@ void RenderThread::renderErrorPage(RenderThread *th, bool standalone)
 void RenderThread::addUploadJob(ITextureUploadable* u)
 {
 	mutexUploadJobs.lock();
+	if (u->getQueued())
+	{
+		mutexUploadJobs.unlock();
+		return;
+	}
 	if(m_sys->isShuttingDown() || status!=STARTED)
 	{
 		u->uploadFence();
 		mutexUploadJobs.unlock();
 		return;
 	}
+	u->setQueued();
 	uploadJobs.push_back(u);
 	uploadNeeded=true;
 	mutexUploadJobs.unlock();
@@ -901,12 +886,12 @@ void RenderThread::draw(bool force)
 	renderNeeded=true;
 	event.signal();
 
-	gettimeofday(&time_d, NULL);
+	gettimeofday(&time_d, nullptr);
 	int diff = time_d.tv_sec-time_s.tv_sec;
 	if(diff>0) /* is one seconds elapsed? */
 	{
 		time_s=time_d;
-		LOG(LOG_INFO,_("FPS: ") << dec << frameCount<<" "<<(getVm(m_sys) ? getVm(m_sys)->getEventQueueSize() : 0));
+		LOG(LOG_INFO,"FPS: " << dec << frameCount<<" "<<(getVm(m_sys) ? getVm(m_sys)->getEventQueueSize() : 0));
 		frameCount=0;
 		secsCount++;
 	}
@@ -952,7 +937,7 @@ uint32_t RenderThread::allocateNewGLTexture() const
 	engineData->exec_glTexImage2D_GL_TEXTURE_2D_GL_UNSIGNED_INT_8_8_8_8_HOST(0, largeTextureSize, largeTextureSize, 0, 0);
 	if(handleGLErrors())
 	{
-		LOG(LOG_ERROR,_("Can't allocate large texture... Aborting"));
+		LOG(LOG_ERROR,"Can't allocate large texture... Aborting");
 		::abort();
 	}
 	return tmp;
@@ -1062,8 +1047,8 @@ TextureChunk RenderThread::allocateTexture(uint32_t w, uint32_t h, bool compact)
 	Locker l(mutexLargeTexture);
 	//Find the number of blocks needed for the given w and h
 	TextureChunk ret(w, h);
-	uint32_t blocksW=(ret.width+CHUNKSIZE-1)/CHUNKSIZE;
-	uint32_t blocksH=(ret.height+CHUNKSIZE-1)/CHUNKSIZE;
+	uint32_t blocksW=(ret.width+CHUNKSIZE_REAL-1)/CHUNKSIZE_REAL;
+	uint32_t blocksH=(ret.height+CHUNKSIZE_REAL-1)/CHUNKSIZE_REAL;
 	//Try to find a good place in the available textures
 	uint32_t index=0;
 	for(index=0;index<largeTextures.size();index++)
@@ -1106,14 +1091,12 @@ TextureChunk RenderThread::allocateTexture(uint32_t w, uint32_t h, bool compact)
 void RenderThread::loadChunkBGRA(const TextureChunk& chunk, uint32_t w, uint32_t h, uint8_t* data)
 {
 	//Fast bailout if the TextureChunk is not valid
-	if(chunk.chunks==nullptr)
+	if(chunk.chunks==nullptr || data == nullptr)
 		return;
 	engineData->exec_glBindTexture_GL_TEXTURE_2D(largeTextures[chunk.texId].id);
 	//TODO: Detect continuos
 	//The size is ok if doesn't grow over the allocated size
 	//this allows some alignment freedom
-	assert(w<=((chunk.width+CHUNKSIZE-1)&0xffffff80));
-	assert(h<=((chunk.height+CHUNKSIZE-1)&0xffffff80));
 	const uint32_t numberOfChunks=chunk.getNumberOfChunks();
 	const uint32_t blocksPerSide=largeTextureSize/CHUNKSIZE;
 	const uint32_t blocksW=((w+CHUNKSIZE_REAL-1)/CHUNKSIZE_REAL);
