@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2013  Alessandro Pignotti (a.pignotti@sssup.it)
     Copyright (C) 2010-2011  Timon Van Overveldt (timonvo@gmail.com)
 
     This program is free software: you can redistribute it and/or modify
@@ -57,7 +57,7 @@ class SecurityManager
 public:
 	enum SANDBOXTYPE { REMOTE=1, LOCAL_WITH_FILE=2, LOCAL_WITH_NETWORK=4, LOCAL_TRUSTED=8 };
 private:
-	RecMutex mutex;
+	Mutex mutex;
 
 	const char* sandboxNames[4];
 	const char* sandboxTitles[4];
@@ -370,7 +370,7 @@ public:
 	bool getSecure() const { return secure; }
 
 	//Does this entry allow a given URL?
-	bool allowsAccessFrom(const URLInfo& url, uint16_t toPort=0) const;
+	bool allowsAccessFrom(const URLInfo& url, uint16_t toPort=0, bool bCheckHttps=true) const;
 };
 
 //Permit HTTP request header sending (only for HTTP)
