@@ -36,6 +36,10 @@
 #	ifndef PATH_MAX
 #		define PATH_MAX 260
 #	endif
+#elif _POSIX_C_SOURCE >= 199309L
+#include <time.h>   // for nanosleep
+#else
+#include <unistd.h> // for usleep
 #endif
 
 using namespace std;
@@ -62,6 +66,20 @@ int kill_child(GPid childPid)
 	return 0;
 }
 
+void compat_msleep(unsigned int time)
+{
+#ifdef WIN32
+	Sleep(time);
+#elif _POSIX_C_SOURCE >= 199309L
+	struct timespec ts;
+	ts.tv_sec = time / 1000;
+	ts.tv_nsec = (time % 1000) * 1000000;
+	nanosleep(&ts, NULL);
+#else
+	usleep(time * 1000);
+#endif
+}
+
 #ifndef _WIN32
 #include "timer.h"
 uint64_t timespecToUsecs(timespec t)
@@ -130,13 +148,13 @@ DEFDLLMAIN(cairo);
 extern "C"
 BOOL WINAPI DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
 {
-	RUNDLLMAIN(gio);
+	//RUNDLLMAIN(gio); //taken care of by patches from mxe
 	RUNDLLMAIN(glib);
 	//RUNDLLMAIN(cairo); //taken care of by patches from mxe
-	RUNDLLMAIN(atk);
-	RUNDLLMAIN(pango);
-	RUNDLLMAIN(gdk);
-	RUNDLLMAIN(gtk);
+	//RUNDLLMAIN(atk); //taken care of by patches from mxe
+	//RUNDLLMAIN(pango); //taken care of by patches from mxe
+	//RUNDLLMAIN(gdk); //taken care of by patches from mxe
+	//RUNDLLMAIN(gtk); //taken care of by patches from mxe
 	return TRUE;
 }
 
