@@ -42,6 +42,8 @@ using namespace lightspark;
 class StandaloneEngineData: public EngineData
 {
 	SDL_GLContext mSDLContext;
+	char* mBaseDir;
+	tiny_string mApplicationStoragePath;
 	void removedir(const char* dir)
 	{
 		GDir* d = g_dir_open(dir,0,nullptr);
@@ -51,7 +53,10 @@ class StandaloneEngineData: public EngineData
 		{
 			const char* filename = g_dir_read_name (d);
 			if (!filename)
+			{
+				g_dir_close(d);
 				return;
+			}
 			string path = dir;
 			path += G_DIR_SEPARATOR_S;
 			path += filename;
@@ -79,15 +84,26 @@ class StandaloneEngineData: public EngineData
 		return st.st_size;
 	}
 public:
-	StandaloneEngineData(const tiny_string& datapath)
+	StandaloneEngineData(const tiny_string& filedatapath)
 	{
-		sharedObjectDatapath=datapath;
+		sharedObjectDatapath = Config::getConfig()->getCacheDirectory();
+		sharedObjectDatapath += G_DIR_SEPARATOR_S;
+		sharedObjectDatapath += "data";
+		sharedObjectDatapath += filedatapath;
+
+		mApplicationStoragePath = Config::getConfig()->getCacheDirectory();
+		mApplicationStoragePath += filedatapath;
+		mApplicationStoragePath += G_DIR_SEPARATOR_S;
+		mApplicationStoragePath += "appstorage";
+		mBaseDir = g_get_current_dir();
 	}
 	~StandaloneEngineData()
 	{
 		removedir(Config::getConfig()->getDataDirectory().c_str());
+		if (mBaseDir)
+			g_free(mBaseDir);
 	}
-	SDL_Window* createWidget(uint32_t w, uint32_t h)
+	SDL_Window* createWidget(uint32_t w, uint32_t h) override
 	{
 		SDL_Window* window = SDL_CreateWindow("Lightspark",SDL_WINDOWPOS_UNDEFINED,SDL_WINDOWPOS_UNDEFINED,w,h,SDL_WINDOW_OPENGL | SDL_WINDOW_RESIZABLE);
 		if (window == 0)
@@ -99,7 +115,7 @@ public:
 		}
 		return window;
 	}
-	
+
 	uint32_t getWindowForGnash() override
 	{
 		/* passing and invalid window id to gnash makes
@@ -159,89 +175,154 @@ public:
 	{
 		SDL_GL_DeleteContext(mSDLContext);
 	}
-	bool FileExists(const tiny_string& filename) override
+
+	bool FileExists(SystemState* sys,const tiny_string& filename, bool isfullpath) override
 	{
 		if (!isvalidfilename(filename))
 			return false;
-		std::string p = Config::getConfig()->getDataDirectory();
+		tiny_string p = isfullpath ? filename : FileFullPath(sys,filename);
 		if (p.empty())
 			return false;
+		return g_file_test(p.raw_buf(),G_FILE_TEST_EXISTS);
+	}
+
+	uint32_t FileSize(SystemState* sys,const tiny_string& filename, bool isfullpath) override
+	{
+		if (!isvalidfilename(filename))
+			return 0;
+		tiny_string p = isfullpath ? filename : FileFullPath(sys,filename);
+		if (p.empty())
+			return 0;
+		return getfilesize(p.raw_buf());
+	}
+	
+	tiny_string FileFullPath(SystemState* sys, const tiny_string& filename) override
+	{
+		std::string p;
+		if (sys->flashMode == SystemState::FLASH_MODE::AIR && mBaseDir)
+		{
+			if (!g_path_is_absolute(filename.raw_buf()))
+				p = mBaseDir;
+		}
+		else
+			p = Config::getConfig()->getDataDirectory();
+		if (p.empty())
+			return "";
 		p += G_DIR_SEPARATOR_S;
 		p += filename.raw_buf();
-		return g_file_test(p.c_str(),G_FILE_TEST_EXISTS);
+		return p;
 	}
-	tiny_string FileRead(const tiny_string& filename) override
+
+	tiny_string FileRead(SystemState* sys,const tiny_string& filename, bool isfullpath) override
 	{
 		if (!isvalidfilename(filename))
 			return "";
-		std::string p = Config::getConfig()->getDataDirectory();
+		tiny_string p = isfullpath ? filename : FileFullPath(sys,filename);
 		if (p.empty())
 			return "";
 
-		p += G_DIR_SEPARATOR_S;
-		p += filename.raw_buf();
-		if (!g_file_test(p.c_str(),G_FILE_TEST_EXISTS))
+		if (!g_file_test(p.raw_buf(),G_FILE_TEST_EXISTS))
 			return "";
-		uint32_t len = getfilesize(p.c_str());
+		uint32_t len = getfilesize(p.raw_buf());
 		std::ifstream file;
 		
-		file.open(p, std::ios::in|std::ios::binary);
+		file.open(p.raw_buf(), std::ios::in|std::ios::binary);
 		
 		tiny_string res(file,len);
 		file.close();
 		return res;
 	}
-	void FileWrite(const tiny_string& filename, const tiny_string& data) override
+	void FileWrite(SystemState* sys,const tiny_string& filename, const tiny_string& data, bool isfullpath) override
 	{
 		if (!isvalidfilename(filename))
 			return;
-		std::string p = Config::getConfig()->getDataDirectory();
+		tiny_string p = isfullpath ? filename : FileFullPath(sys,filename);
 		if (p.empty())
 			return;
-		p += G_DIR_SEPARATOR_S;
-		p += filename.raw_buf();
 		std::ofstream file;
-		
-		file.open(p, std::ios::out|std::ios::binary);
+
+		file.open(p.raw_buf(), std::ios::out|std::ios::binary);
 		file << data;
 		file.close();
 	}
-	void FileReadByteArray(const tiny_string &filename,ByteArray* res) override
+	uint8_t FileReadUnsignedByte(SystemState* sys, const tiny_string& filename, uint32_t startpos, bool isfullpath) override
+	{
+		if (!isvalidfilename(filename))
+			return 0;
+		tiny_string p = isfullpath ? filename : FileFullPath(sys,filename);
+		if (p.empty())
+			return 0;
+		if (!g_file_test(p.raw_buf(),G_FILE_TEST_EXISTS))
+			return 0;
+		std::ifstream file;
+		file.open(p.raw_buf(), std::ios::in|std::ios::binary);
+		file.seekg(startpos);
+		uint8_t buf;
+		file.read((char*)&buf,1);
+		file.close();
+		return buf;
+	}
+	void FileReadByteArray(SystemState* sys,const tiny_string &filename,ByteArray* res, uint32_t startpos, uint32_t length, bool isfullpath) override
 	{
 		if (!isvalidfilename(filename))
 			return;
-		std::string p = Config::getConfig()->getDataDirectory();
+		tiny_string p = isfullpath ? filename : FileFullPath(sys,filename);
 		if (p.empty())
 			return;
-		p += G_DIR_SEPARATOR_S;
-		p += filename.raw_buf();
-		if (!g_file_test(p.c_str(),G_FILE_TEST_EXISTS))
+		if (!g_file_test(p.raw_buf(),G_FILE_TEST_EXISTS))
 			return;
-		uint32_t len = getfilesize(p.c_str());
+		uint32_t len = min(length,uint32_t(getfilesize(p.raw_buf())-startpos));
 		std::ifstream file;
-		uint8_t buf[len];
-		file.open(p, std::ios::in|std::ios::binary);
+		uint8_t* buf = new uint8_t[len];
+		file.open(p.raw_buf(), std::ios::in|std::ios::binary);
+		file.seekg(startpos);
 		file.read((char*)buf,len);
 		res->writeBytes(buf,len);
 		file.close();
+		delete[] buf;
 	}
 	
-	void FileWriteByteArray(const tiny_string &filename, ByteArray *data) override
+	void FileWriteByteArray(SystemState* sys,const tiny_string &filename, ByteArray *data, uint32_t startpos, uint32_t length, bool isfullpath) override
 	{
 		if (!isvalidfilename(filename))
 			return;
-		std::string p = Config::getConfig()->getDataDirectory();
+		tiny_string p = isfullpath ? filename : FileFullPath(sys,filename);
 		if (p.empty())
 			return;
-		p += G_DIR_SEPARATOR_S;
-		p += filename.raw_buf();
 		std::ofstream file;
 		
-		file.open(p, std::ios::out|std::ios::binary|std::ios::trunc);
+		file.open(p.raw_buf(), std::ios::out|std::ios::binary|std::ios::trunc);
 		uint8_t* buf = data->getBuffer(data->getLength(),false);
-		file.write((char*)buf,data->getLength());
+		uint32_t len = length;
+		if (length == UINT32_MAX || startpos+length > data->getLength())
+			len = data->getLength()-startpos;
+		file.write((char*)buf+startpos,len);
 		file.close();
 	}
+	bool FileCreateDirectory(SystemState* sys,const tiny_string& filename, bool isfullpath) override
+	{
+		if (!isvalidfilename(filename))
+			return false;
+		tiny_string p = isfullpath ? filename : FileFullPath(sys,filename);
+		if (p.empty())
+			return false;
+		if (!p.endsWith(G_DIR_SEPARATOR_S))
+		{
+			// if filename doesn't end with a directory separator, it is treated as a full path to a file
+			gchar* dir = g_path_get_dirname(p.raw_buf());
+			p=dir;
+			g_free(dir);
+		}
+		return g_mkdir_with_parents(p.raw_buf(),0755) == 0;
+	}
+	bool FilePathIsAbsolute(const tiny_string& filename) override
+	{
+		return g_path_is_absolute(filename.raw_buf());
+	}
+	tiny_string FileGetApplicationStorageDir() override
+	{
+		return mApplicationStoragePath;
+	}
 };
 
 int main(int argc, char* argv[])
@@ -258,18 +339,12 @@ int main(int argc, char* argv[])
 	bool useFastInterpreter=false;
 	bool useJit=false;
 	bool ignoreUnhandledExceptions = false;
+	bool startInFullScreenMode=false;
+	double startscalefactor=1.0;
 	SystemState::ERROR_TYPE exitOnError=SystemState::ERROR_PARSING;
 	LOG_LEVEL log_level=LOG_INFO;
 	SystemState::FLASH_MODE flashMode=SystemState::FLASH;
-
-	setlocale(LC_ALL, "");
-#ifdef _WIN32
-	const char* localedir = getExectuablePath();
-#else
-	const char* localedir = "/usr/share/locale";
-#endif
-	bindtextdomain("lightspark", localedir);
-	textdomain("lightspark");
+	std::vector<tiny_string> extensions;
 
 	LOG(LOG_INFO,"Lightspark version " << VERSION << " Copyright 2009-2013 Alessandro Pignotti and others");
 
@@ -299,6 +374,18 @@ int main(int argc, char* argv[])
 			useJit=true;
 		else if(strcmp(argv[i],"-ne")==0 || strcmp(argv[i],"--ignore-unhandled-exceptions")==0)
 			ignoreUnhandledExceptions=true;
+		else if(strcmp(argv[i],"-fs")==0 || strcmp(argv[i],"--fullscreen")==0)
+			startInFullScreenMode=true;
+		else if(strcmp(argv[i],"-sc")==0 || strcmp(argv[i],"--scale")==0)
+		{
+			i++;
+			if(i==argc)
+			{
+				fileName=nullptr;
+				break;
+			}
+			startscalefactor = max(1.0,atof(argv[i]));
+		}
 		else if(strcmp(argv[i],"-l")==0 || strcmp(argv[i],"--log-level")==0)
 		{
 			i++;
@@ -321,6 +408,18 @@ int main(int argc, char* argv[])
 			}
 			paramsFileName=argv[i];
 		}
+		else if (strcmp(argv[i],"-le")==0 || 
+				 strcmp(argv[i],"--load-extension")==0)
+		{
+			i++;
+			if(i==argc)
+			{
+				fileName=nullptr;
+				break;
+			}
+			tiny_string ext = argv[i];
+			extensions.push_back(ext);
+		}
 #ifdef PROFILING_SUPPORT
 		else if(strcmp(argv[i],"-o")==0 || 
 			strcmp(argv[i],"--profiling-output")==0)
@@ -401,6 +500,9 @@ int main(int argc, char* argv[])
 			" [--profiling-output|-o profiling-file]" <<
 #endif
 			" [--ignore-unhandled-exceptions|-ne]"
+			" [--fullscreen|-fs]"
+			" [--scale|-sc]"
+			" [--load-extension|-le extension-file"
 			" [--version|-v]" <<
 			" <file.swf>");
 		exit(1);
@@ -430,7 +532,10 @@ int main(int argc, char* argv[])
 	//NOTE: see SystemState declaration
 	SystemState* sys = new SystemState(fileSize, flashMode);
 	ParseThread* pt = new ParseThread(f, sys->mainClip);
+	pt->addExtensions(extensions);
 	setTLSSys(sys);
+	setTLSWorker(sys->worker);
+
 	sys->setDownloadedPath(fileName);
 	sys->allowFullscreen=true;
 	sys->allowFullscreenInteractive=true;
@@ -444,7 +549,7 @@ int main(int argc, char* argv[])
 		if (sandboxType != SecurityManager::REMOTE &&
 		    sys->mainClip->getOrigin().getProtocol() != "file")
 		{
-			LOG(LOG_INFO, _("Switching to remote sandbox because of remote url"));
+			LOG(LOG_INFO, "Switching to remote sandbox because of remote url");
 			sandboxType = SecurityManager::REMOTE;
 		}
 	}
@@ -462,13 +567,13 @@ int main(int argc, char* argv[])
 	else
 	{
 		sys->mainClip->setOrigin(string("file://") + fileName);
-		LOG(LOG_INFO, _("Warning: running with no origin URL set."));
+		LOG(LOG_INFO, "Warning: running with no origin URL set.");
 	}
 
 	//One of useInterpreter or useJit must be enabled
 	if(!(useInterpreter || useJit))
 	{
-		LOG(LOG_ERROR,_("No execution model enabled"));
+		LOG(LOG_ERROR,"No execution model enabled");
 		exit(1);
 	}
 	sys->useInterpreter=useInterpreter;
@@ -496,22 +601,22 @@ int main(int argc, char* argv[])
 	tiny_string filedatapath = absolutepath;
 	if (filedatapath.find(homedir) == 0) // remove home dir, if file is located below home dir
 		filedatapath = filedatapath.substr_bytes(homedir.numBytes(),UINT32_MAX);
-	tiny_string sharedobjectdatapath = Config::getConfig()->getCacheDirectory();
-	sharedobjectdatapath += G_DIR_SEPARATOR_S;
-	sharedobjectdatapath += "data";
-	sharedobjectdatapath += filedatapath;
 
-	sys->setParamsAndEngine(new StandaloneEngineData(sharedobjectdatapath), true);
+	sys->setParamsAndEngine(new StandaloneEngineData(filedatapath), true);
+	// on standalone local storage is always allowed
+	sys->getEngineData()->setLocalStorageAllowedMarker(true);
+	sys->getEngineData()->startInFullScreenMode=startInFullScreenMode;
+	sys->getEngineData()->startscalefactor=startscalefactor;
 
 	sys->securityManager->setSandboxType(sandboxType);
 	if(sandboxType == SecurityManager::REMOTE)
-		LOG(LOG_INFO, _("Running in remote sandbox"));
+		LOG(LOG_INFO, "Running in remote sandbox");
 	else if(sandboxType == SecurityManager::LOCAL_WITH_NETWORK)
-		LOG(LOG_INFO, _("Running in local-with-networking sandbox"));
+		LOG(LOG_INFO, "Running in local-with-networking sandbox");
 	else if(sandboxType == SecurityManager::LOCAL_WITH_FILE)
-		LOG(LOG_INFO, _("Running in local-with-filesystem sandbox"));
+		LOG(LOG_INFO, "Running in local-with-filesystem sandbox");
 	else if(sandboxType == SecurityManager::LOCAL_TRUSTED)
-		LOG(LOG_INFO, _("Running in local-trusted sandbox"));
+		LOG(LOG_INFO, "Running in local-trusted sandbox");
 
 	sys->downloadManager=new StandaloneDownloadManager();
 
