@@ -23,6 +23,9 @@
 #include "scripting/toplevel/Array.h"
 #include "scripting/toplevel/ASString.h"
 #include "scripting/toplevel/Date.h"
+#include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/Integer.h"
+#include "scripting/toplevel/UInteger.h"
 #include "scripting/toplevel/Vector.h"
 #include "scripting/class.h"
 #include "scripting/flash/xml/flashxml.h"
@@ -43,6 +46,43 @@ asAtom Amf3Deserializer::readObject() const
 	return parseValue(stringMap, objMap, traitsMap);
 }
 
+void Amf3Deserializer::readSharedObject(ASObject* ret)
+{
+	// skip the amf header of the file
+	// -> 2 byte always 0x00, 0xbf
+	// -> 4 byte filesize-6
+	// -> 10 byte always 'T', 'C', 'S', 'O', 0x00, 0x04, 0x00, 0x00, 0x00, 0x00
+	input->setPosition(16);
+	tiny_string name;
+	bool ok=false;
+	ok = input->readUTF(name);
+	uint32_t amfversion;
+	ok = ok && input->readUnsignedInt(amfversion);
+	if (ok)
+	{
+		vector<tiny_string> stringMap;
+		vector<asAtom> objMap;
+		vector<TraitsRef> traitsMap;
+		if (amfversion!= 3 && amfversion!=0)
+			LOG(LOG_ERROR,"invalid amf version for sharedObject:"<<name<<" "<<amfversion);
+		input->setCurrentObjectEncoding(amfversion==3 ? OBJECT_ENCODING::AMF3 : OBJECT_ENCODING::AMF0);
+		while (input->getPosition() < input->getLength())
+		{
+			tiny_string key = amfversion==3 ? parseStringVR(stringMap) : parseStringAMF0();
+			asAtom value = parseValue(stringMap,objMap,traitsMap);
+			if (asAtomHandler::isValid(value))
+			{
+				multiname m(nullptr);
+				m.name_type=multiname::NAME_STRING;
+				m.name_s_id=input->getSystemState()->getUniqueStringId(key);
+				ret->setVariableByMultiname(m,value,ASObject::CONST_ALLOWED,nullptr,input->getInstanceWorker());
+			}
+			uint8_t b;
+			input->readByte(b); // skip 0 byte
+		}
+	}
+}
+
 asAtom Amf3Deserializer::parseInteger() const
 {
 	uint32_t tmp;
@@ -67,7 +107,7 @@ asAtom Amf3Deserializer::parseDouble() const
 	}
 	tmp.dummy=GINT64_FROM_BE(tmp.dummy);
 	
-	return asAtomHandler::fromNumber(input->getSystemState(),tmp.val,false);
+	return asAtomHandler::fromNumber(input->getInstanceWorker(),tmp.val,false);
 }
 
 asAtom Amf3Deserializer::parseDate() const
@@ -85,7 +125,7 @@ asAtom Amf3Deserializer::parseDate() const
 			throw ParseException("Not enough data to parse date");
 	}
 	tmp.dummy=GINT64_FROM_BE(tmp.dummy);
-	Date* dt = Class<Date>::getInstanceS(input->getSystemState());
+	Date* dt = Class<Date>::getInstanceS(input->getInstanceWorker());
 	dt->MakeDateFromMilliseconds((int64_t)tmp.val);
 	return asAtomHandler::fromObject(dt);
 }
@@ -137,7 +177,7 @@ asAtom Amf3Deserializer::parseArray(std::vector<tiny_string>& stringMap,
 		return ret;
 	}
 
-	Array* ret=Class<lightspark::Array>::getInstanceS(input->getSystemState());
+	Array* ret=Class<lightspark::Array>::getInstanceS(input->getInstanceWorker());
 	//Add object to the map
 	objMap.push_back(asAtomHandler::fromObject(ret));
 
@@ -150,7 +190,6 @@ asAtom Amf3Deserializer::parseArray(std::vector<tiny_string>& stringMap,
 		if(varName=="")
 			break;
 		asAtom value=parseValue(stringMap, objMap, traitsMap);
-		ASATOM_INCREF(value);
 		ret->setVariableAtomByQName(varName,nsNameAndKind(),value, DYNAMIC_TRAIT);
 	}
 
@@ -200,13 +239,13 @@ asAtom Amf3Deserializer::parseVector(uint8_t marker, std::vector<tiny_string>& s
 		{
 			tiny_string vectypename;
 			vectypename = parseStringVR(stringMap);
-			multiname m(NULL);
+			multiname m(nullptr);
 			m.name_type=multiname::NAME_STRING;
 			m.name_s_id=input->getSystemState()->getUniqueStringId(vectypename);
 			m.ns.push_back(nsNameAndKind(input->getSystemState(),"",NAMESPACE));
 			m.isAttribute = false;
-			type = Type::getTypeFromMultiname(&m,getVm(input->getSystemState())->currentCallContext->mi->context);
-			if (type == NULL)
+			type = Type::getTypeFromMultiname(&m,input->getInstanceWorker()->currentCallContext->mi->context);
+			if (type == nullptr)
 			{
 				LOG(LOG_ERROR,"unknown vector type during deserialization:"<<m);
 				type = input->getSystemState()->getObjectClassRef();
@@ -219,7 +258,10 @@ asAtom Amf3Deserializer::parseVector(uint8_t marker, std::vector<tiny_string>& s
 			
 	}
 	asAtom v=asAtomHandler::invalidAtom;
-	Template<Vector>::getInstanceS(v,input->getSystemState(),type,ABCVm::getCurrentApplicationDomain(getVm(input->getSystemState())->currentCallContext));
+	Template<Vector>::getInstanceS(input->getInstanceWorker(),v,
+								   input->getInstanceWorker()->rootClip.getPtr(),
+								   type,
+								   ABCVm::getCurrentApplicationDomain(input->getInstanceWorker()->currentCallContext));
 	Vector* ret= asAtomHandler::as<Vector>(v);
 	//Add object to the map
 	objMap.push_back(asAtomHandler::fromObject(ret));
@@ -258,7 +300,6 @@ asAtom Amf3Deserializer::parseVector(uint8_t marker, std::vector<tiny_string>& s
 			case vector_object_marker:
 			{
 				asAtom value=parseValue(stringMap, objMap, traitsMap);
-				ASATOM_INCREF(value);
 				ret->append(value);
 				break;
 			}
@@ -293,7 +334,7 @@ asAtom Amf3Deserializer::parseDictionary(std::vector<tiny_string>& stringMap,
 		throw ParseException("Not enough data to parse AMF3 vector");
 	if (weakkeys)
 		LOG(LOG_NOT_IMPLEMENTED,"handling of weak keys in Dictionary");
-	Dictionary* ret=Class<Dictionary>::getInstanceS(input->getSystemState());
+	Dictionary* ret=Class<Dictionary>::getInstanceS(input->getInstanceWorker());
 	//Add object to the map
 	objMap.push_back(asAtomHandler::fromObject(ret));
 
@@ -304,13 +345,35 @@ asAtom Amf3Deserializer::parseDictionary(std::vector<tiny_string>& stringMap,
 	{
 		asAtom key=parseValue(stringMap, objMap, traitsMap);
 		asAtom value=parseValue(stringMap, objMap, traitsMap);
-		multiname name(NULL);
-		name.name_type=multiname::NAME_OBJECT;
-		name.name_o = asAtomHandler::toObject(key,input->getSystemState());
+		multiname name(nullptr);
+		if (asAtomHandler::isString(key))
+		{
+			name.name_type=multiname::NAME_STRING;
+			name.name_s_id=asAtomHandler::toStringId(key,input->getInstanceWorker());
+			ASATOM_DECREF(key);
+		}
+		else if (asAtomHandler::isInteger(key))
+		{
+			name.name_type=multiname::NAME_INT;
+			name.name_i=asAtomHandler::getInt(key);
+		}
+		else if (asAtomHandler::isUInteger(key))
+		{
+			name.name_type=multiname::NAME_UINT;
+			name.name_ui=asAtomHandler::getUInt(key);
+		}
+		else if (asAtomHandler::isNumber(key))
+		{
+			name.name_type=multiname::NAME_NUMBER;
+			name.name_d=asAtomHandler::toNumber(key);
+		}
+		else
+		{
+			name.name_type=multiname::NAME_OBJECT;
+			name.name_o = asAtomHandler::getObject(key);
+		}
 		name.ns.push_back(nsNameAndKind(input->getSystemState(),"",NAMESPACE));
-		ASATOM_INCREF(key);
-		ASATOM_INCREF(value);
-		ret->setVariableByMultiname(name,value,ASObject::CONST_ALLOWED);
+		ret->setVariableByMultiname(name,value,ASObject::CONST_ALLOWED,nullptr,input->getInstanceWorker());
 	}
 	return asAtomHandler::fromObject(ret);
 }
@@ -331,7 +394,7 @@ asAtom Amf3Deserializer::parseByteArray(std::vector<asAtom>& objMap) const
 		return ret;
 	}
 
-	ByteArray* ret=Class<ByteArray>::getInstanceS(input->getSystemState());
+	ByteArray* ret=Class<ByteArray>::getInstanceS(input->getInstanceWorker());
 	//Add object to the map
 	objMap.push_back(asAtomHandler::fromObject(ret));
 
@@ -369,31 +432,33 @@ asAtom Amf3Deserializer::parseObject(std::vector<tiny_string>& stringMap,
 		//Custom serialization
 		const tiny_string& className=parseStringVR(stringMap);
 		assert_and_throw(!className.empty());
-		const auto it=input->getSystemState()->aliasMap.find(className);
-		assert_and_throw(it!=input->getSystemState()->aliasMap.end());
+		RootMovieClip* root = input->getInstanceWorker()->rootClip.getPtr();
+		const auto it=root->aliasMap.find(className);
+		assert_and_throw(it!=root->aliasMap.end());
 
 		Class_base* type=it->second.getPtr();
 		traitsMap.push_back(TraitsRef(type));
 
 		asAtom ret=asAtomHandler::invalidAtom;
-		type->getInstance(ret,true, NULL, 0);
+		type->getInstance(input->getInstanceWorker(),ret,true, nullptr, 0);
 		//Invoke readExternal
-		multiname readExternalName(NULL);
+		multiname readExternalName(nullptr);
 		readExternalName.name_type=multiname::NAME_STRING;
 		readExternalName.name_s_id=input->getSystemState()->getUniqueStringId("readExternal");
 		readExternalName.ns.push_back(nsNameAndKind(input->getSystemState(),"",NAMESPACE));
 		readExternalName.isAttribute = false;
 
 		asAtom o=asAtomHandler::invalidAtom;
-		asAtomHandler::getObject(ret)->getVariableByMultiname(o,readExternalName,GET_VARIABLE_OPTION::SKIP_IMPL);
+		asAtomHandler::getObject(ret)->getVariableByMultiname(o,readExternalName,GET_VARIABLE_OPTION::SKIP_IMPL,input->getInstanceWorker());
 		assert_and_throw(asAtomHandler::isFunction(o));
 		asAtom tmpArg[1] = { asAtomHandler::fromObject(input) };
 		asAtom r=asAtomHandler::invalidAtom;
-		asAtomHandler::callFunction(o,r,ret, tmpArg, 1,false);
+		asAtomHandler::callFunction(o,input->getInstanceWorker(),r,ret, tmpArg, 1,false);
+		ASATOM_DECREF(o);
 		return ret;
 	}
 
-	TraitsRef traits(NULL);
+	TraitsRef traits(nullptr);
 	if((objRef&0x02)==0)
 	{
 		uint32_t traitsRef=objRef>>2;
@@ -410,32 +475,31 @@ asAtom Amf3Deserializer::parseObject(std::vector<tiny_string>& stringMap,
 		for(uint32_t i=0;i<traitsCount;i++)
 			traits.traitsNames.emplace_back(parseStringVR(stringMap));
 
-		const auto it=input->getSystemState()->aliasMap.find(className);
-		if(it!=input->getSystemState()->aliasMap.end())
+		RootMovieClip* root = input->getInstanceWorker()->rootClip.getPtr();
+		const auto it=root->aliasMap.find(className);
+		if(it!=root->aliasMap.end())
 			traits.type=it->second.getPtr();
 		traitsMap.emplace_back(traits);
 	}
 
 	asAtom ret=asAtomHandler::invalidAtom;
 	if (traits.type)
-		traits.type->getInstance(ret,true, NULL, 0);
+		traits.type->getInstance(input->getInstanceWorker(),ret,true, nullptr, 0);
 	else
-		ret =asAtomHandler::fromObject(Class<ASObject>::getInstanceS(input->getSystemState()));
+		ret =asAtomHandler::fromObject(Class<ASObject>::getInstanceS(input->getInstanceWorker()));
 	//Add object to the map
 	objMap.push_back(ret);
 
 	for(uint32_t i=0;i<traits.traitsNames.size();i++)
 	{
 		asAtom value=parseValue(stringMap, objMap, traitsMap);
-		ASATOM_INCREF(value);
 
-		multiname name(NULL);
+		multiname name(nullptr);
 		name.name_type=multiname::NAME_STRING;
 		name.name_s_id=input->getSystemState()->getUniqueStringId(traits.traitsNames[i]);
 		name.ns.push_back(nsNameAndKind(input->getSystemState(),"",NAMESPACE));
 		name.isAttribute=false;
-			
-		asAtomHandler::getObject(ret)->setVariableByMultiname_intern(name,value,ASObject::CONST_ALLOWED,traits.type,nullptr);
+		asAtomHandler::getObject(ret)->setVariableByMultiname_intern(name,value,ASObject::CONST_ALLOWED,traits.type,nullptr,input->getInstanceWorker());
 	}
 
 	//Read dynamic name, value pairs
@@ -445,7 +509,6 @@ asAtom Amf3Deserializer::parseObject(std::vector<tiny_string>& stringMap,
 		if(varName=="")
 			break;
 		asAtom value=parseValue(stringMap, objMap, traitsMap);
-		ASATOM_INCREF(value);
 		asAtomHandler::getObject(ret)->setVariableAtomByQName(varName,nsNameAndKind(),value,DYNAMIC_TRAIT);
 	}
 	return ret;
@@ -479,9 +542,9 @@ asAtom Amf3Deserializer::parseXML(std::vector<asAtom>& objMap, bool legacyXML) c
 
 	ASObject *xmlObj;
 	if(legacyXML)
-		xmlObj=Class<XMLDocument>::getInstanceS(input->getSystemState(),xmlStr);
+		xmlObj=Class<XMLDocument>::getInstanceS(input->getInstanceWorker(),xmlStr);
 	else
-		xmlObj=XML::createFromString(input->getSystemState(),xmlStr);
+		xmlObj=XML::createFromString(input->getInstanceWorker(),xmlStr);
 	objMap.push_back(asAtomHandler::fromObject(xmlObj));
 	return asAtomHandler::fromObject(xmlObj);
 }
@@ -495,7 +558,7 @@ asAtom Amf3Deserializer::parseValue(std::vector<tiny_string>& stringMap,
 	uint8_t marker;
 	if(!input->readByte(marker))
 		throw ParseException("Not enough data to parse AMF3 object");
-	if (input->getCurrentObjectEncoding() == ObjectEncoding::AMF3)
+	if (input->getCurrentObjectEncoding() == OBJECT_ENCODING::AMF3)
 	{
 		switch(marker)
 		{
@@ -514,7 +577,7 @@ asAtom Amf3Deserializer::parseValue(std::vector<tiny_string>& stringMap,
 			case date_marker:
 				return parseDate();
 			case string_marker:
-				return asAtomHandler::fromObject(abstract_s(input->getSystemState(),parseStringVR(stringMap)));
+				return asAtomHandler::fromObject(abstract_s(input->getInstanceWorker(),parseStringVR(stringMap)));
 			case xml_doc_marker:
 				return parseXML(objMap, true);
 			case array_marker:
@@ -551,7 +614,7 @@ asAtom Amf3Deserializer::parseValue(std::vector<tiny_string>& stringMap,
 				return asAtomHandler::fromBool((bool)b);
 			}
 			case amf0_string_marker:
-				return asAtomHandler::fromObject(abstract_s(input->getSystemState(),parseStringAMF0()));
+				return asAtomHandler::fromObject(abstract_s(input->getInstanceWorker(),parseStringAMF0()));
 			case amf0_object_marker:
 				return parseObjectAMF0(stringMap,objMap,traitsMap);
 			case amf0_null_marker:
@@ -579,7 +642,7 @@ asAtom Amf3Deserializer::parseValue(std::vector<tiny_string>& stringMap,
 				return parseObjectAMF0(stringMap,objMap,traitsMap, class_name);
 			}
 			case amf0_avmplus_object_marker:
-				input->setCurrentObjectEncoding(ObjectEncoding::AMF3);
+				input->setCurrentObjectEncoding(OBJECT_ENCODING::AMF3);
 				return parseValue(stringMap, objMap, traitsMap);
 			default:
 				LOG(LOG_ERROR,"Unsupported marker " << (uint32_t)marker);
@@ -611,7 +674,7 @@ asAtom Amf3Deserializer::parseECMAArrayAMF0(std::vector<tiny_string>& stringMap,
 	if(!input->readUnsignedInt(count))
 		throw ParseException("Not enough data to parse AMF3 array");
 
-	Array* ar = Class<Array>::getInstanceS(input->getSystemState());
+	Array* ar = Class<Array>::getInstanceS(input->getInstanceWorker());
 	ar->resize(count);
 	asAtom ret=asAtomHandler::fromObject(ar);
 
@@ -630,11 +693,10 @@ asAtom Amf3Deserializer::parseECMAArrayAMF0(std::vector<tiny_string>& stringMap,
 			throw ParseException("empty key in AMF0 ECMA array");
 		}
 		asAtom value=parseValue(stringMap, objMap, traitsMap);
-		ASATOM_INCREF(value);
 		// contrary to Adobe AMF specs integer names are treated as indexes inside the array
 		m.name_s_id = ar->getSystemState()->getUniqueStringId(varName);
 		m.isInteger=Array::isIntegerWithoutLeadingZeros(varName);
-		ar->setVariableByMultiname(m,value,ASObject::CONST_ALLOWED);
+		ar->setVariableByMultiname(m,value,ASObject::CONST_ALLOWED,nullptr,input->getInstanceWorker());
 		count--;
 	}
 	return ret;
@@ -647,14 +709,13 @@ asAtom Amf3Deserializer::parseStrictArrayAMF0(std::vector<tiny_string>& stringMa
 	if(!input->readUnsignedInt(count))
 		throw ParseException("Not enough data to parse AMF3 strict array");
 
-	lightspark::Array* ret=Class<lightspark::Array>::getInstanceS(input->getSystemState());
+	lightspark::Array* ret=Class<lightspark::Array>::getInstanceS(input->getInstanceWorker());
 	//Add object to the map
 	objMap.push_back(asAtomHandler::fromObject(ret));
 
 	while(count)
 	{
 		asAtom value=parseValue(stringMap, objMap, traitsMap);
-		ASATOM_INCREF(value);
 		ret->push(value);
 		count--;
 	}
@@ -668,10 +729,10 @@ asAtom Amf3Deserializer::parseObjectAMF0(std::vector<tiny_string>& stringMap,
 {
 	asAtom ret = asAtomHandler::invalidAtom;
 	if (clsname == "")
-		ret=asAtomHandler::fromObject(Class<ASObject>::getInstanceS(input->getSystemState()));
+		ret=asAtomHandler::fromObject(Class<ASObject>::getInstanceS(input->getInstanceWorker()));
 	else
 	{
-		input->getSystemState()->getClassInstanceByName(ret,clsname);
+		input->getSystemState()->getClassInstanceByName(input->getInstanceWorker(),ret,clsname);
 	}
 
 	while (true)
@@ -686,7 +747,6 @@ asAtom Amf3Deserializer::parseObjectAMF0(std::vector<tiny_string>& stringMap,
 			throw ParseException("empty key in AMF0 object");
 		}
 		asAtom value=parseValue(stringMap, objMap, traitsMap);
-		ASATOM_INCREF(value);
 
 		if (clsname == "")
 			asAtomHandler::getObjectNoCheck(ret)->setVariableAtomByQName(varName,nsNameAndKind(),value,DYNAMIC_TRAIT);
@@ -695,7 +755,7 @@ asAtom Amf3Deserializer::parseObjectAMF0(std::vector<tiny_string>& stringMap,
 			multiname m(nullptr);
 			m.name_type = multiname::NAME_STRING;
 			m.name_s_id = input->getSystemState()->getUniqueStringId(varName);
-			asAtomHandler::getObjectNoCheck(ret)->setVariableByMultiname(m,value,ASObject::CONST_ALLOWED);
+			asAtomHandler::getObjectNoCheck(ret)->setVariableByMultiname(m,value,ASObject::CONST_ALLOWED,nullptr,input->getInstanceWorker());
 		}
 	}
 	return ret;
