@@ -36,7 +36,8 @@ class DisplayObjectContainer;
 class DefineSpriteTag;
 class AdditionalDataTag;
 
-enum TAGTYPE {TAG=0,DISPLAY_LIST_TAG,SHOW_TAG,CONTROL_TAG,DICT_TAG,FRAMELABEL_TAG,SYMBOL_CLASS_TAG,ACTION_TAG,ABC_TAG,END_TAG,AVM1ACTION_TAG,AVM1INITACTION_TAG};
+enum TAGTYPE {TAG=0,DISPLAY_LIST_TAG,SHOW_TAG,CONTROL_TAG,DICT_TAG,FRAMELABEL_TAG,SYMBOL_CLASS_TAG,ACTION_TAG,ABC_TAG,END_TAG,
+			  AVM1ACTION_TAG,AVM1INITACTION_TAG,BUTTONSOUND_TAG, FILEATTRIBUTES_TAG,METADATA_TAG,BACKGROUNDCOLOR_TAG,ENABLEDEBUGGER_TAG,DEFINESCALINGGRID_TAG};
 
 void ignore(std::istream& i, int count);
 
@@ -114,7 +115,7 @@ public:
 	void execute(RootMovieClip* root) const override =0;
 };
 
-class AVM1ActionTag: public Tag
+class AVM1ActionTag: public DisplayListTag
 {
 private:
 	std::vector<uint8_t> actions;
@@ -122,8 +123,9 @@ private:
 public:
 	AVM1ActionTag(RECORDHEADER h, std::istream& s,RootMovieClip* root, AdditionalDataTag* datatag);
 	TAGTYPE getType() const override { return AVM1ACTION_TAG; }
-	void execute(MovieClip* clip, AVM1context *context);
+	void execute(DisplayObjectContainer* parent,bool inskipping) override;
 	bool empty() { return actions.empty(); }
+	void setActions(AVM1scriptToExecute& script) const;
 };
 class AVM1InitActionTag: public ControlTag
 {
@@ -189,6 +191,7 @@ public:
 class DefineMorphShapeTag: public DictionaryTag
 {
 	friend class TokenContainer;
+	friend class MorphShape;
 protected:
 	UI16_SWF CharacterId;
 	RECT StartBounds;
@@ -257,6 +260,7 @@ public:
 	int getId() const override { return CharacterID; }
 	ASObject* instance(Class_base* c=nullptr) override;
 	MATRIX MapToBounds(const MATRIX& mat) override;
+	bool isHTML() const { return HTML; }
 };
 
 class MemoryStreamCache;
@@ -282,7 +286,7 @@ public:
 	int getChannels() const;
 	_R<MemoryStreamCache> getSoundData() const;
 	std::streambuf *createSoundStream() const;
-	_NR<SoundChannel> soundchanel;
+	SoundChannel* createSoundChannel(const SOUNDINFO* soundinfo);
 	// indicates if this channel is attached to a Sound object
 	bool isAttached;
 };
@@ -292,11 +296,10 @@ class StartSoundTag: public DisplayListTag
 private:
 	UI16_SWF SoundId;
 	SOUNDINFO SoundInfo;
-
-	void play(DefineSoundTag *soundTag);
 public:
 	StartSoundTag(RECORDHEADER h, std::istream& s);
 	void execute(DisplayObjectContainer* parent,bool inskipping) override;
+	const SOUNDINFO* getSoundInfo() const { return &SoundInfo; }
 };
 
 class SoundStreamHeadTag: public DisplayListTag
@@ -311,8 +314,9 @@ private:
 	SI16_SWF LatencySeek;
 public:
 	SoundStreamHeadTag(RECORDHEADER h, std::istream& s, RootMovieClip* root,DefineSpriteTag* sprite);
-	_R<MemoryStreamCache> SoundData;
-	void setSoundChannel(Sprite* spr, bool autoplay);
+	~SoundStreamHeadTag();
+	_NR<MemoryStreamCache> SoundData;
+	void setSoundChannel(Sprite* spr);
 	void execute(DisplayObjectContainer *parent,bool inskipping) override {}
 };
 
@@ -417,6 +421,23 @@ private:
 public:
 	SetBackgroundColorTag(RECORDHEADER h, std::istream& in);
 	void execute(RootMovieClip* root) const override;
+	TAGTYPE getType() const override { return BACKGROUNDCOLOR_TAG; }
+};
+class DefineButtonTag;
+class DefineButtonSoundTag: public Tag
+{
+public:
+	UI16_SWF SoundID0_OverUpToIdle;
+	SOUNDINFO SoundInfo0_OverUpToIdle;
+	UI16_SWF SoundID1_IdleToOverUp;
+	SOUNDINFO SoundInfo1_IdleToOverUp;
+	UI16_SWF SoundID2_OverUpToOverDown;
+	SOUNDINFO SoundInfo2_OverUpToOverDown;
+	UI16_SWF SoundID3_OverDownToOverUp;
+	SOUNDINFO SoundInfo3_OverDownToOverUp;
+	DefineButtonTag* button;
+	DefineButtonSoundTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
+	TAGTYPE getType() const override { return BUTTONSOUND_TAG; }
 };
 
 class DefineButtonTag: public DictionaryTag
@@ -429,8 +450,10 @@ private:
 public:
 	std::vector<BUTTONCONDACTION> condactions;
 	DefineButtonTag(RECORDHEADER h, std::istream& in, int version, RootMovieClip* root, AdditionalDataTag* datatag);
+	~DefineButtonTag();
 	int getId() const override { return ButtonId; }
 	ASObject* instance(Class_base* c=nullptr) override;
+	DefineButtonSoundTag* sounds;
 };
 
 class KERNINGRECORD
@@ -480,7 +503,7 @@ public:
 	int getId() const override { return FontID; }
 	ASObject* instance(Class_base* c=nullptr) override;
 	const tiny_string getFontname() const { return fontname;}
-	virtual void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, FILLSTYLE& fillstyleColor, uint32_t leading,uint32_t startpos)=0;
+	virtual void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, const list<FILLSTYLE>& fillstyleColor, int32_t leading,int32_t startposx, int32_t startposy)=0;
 	virtual number_t getRenderCharAdvance(uint32_t index) const =0;
 	virtual void getTextBounds(const tiny_string& text, int fontpixelsize, number_t& width, number_t& height)=0;
 	const TextureChunk *getCharTexture(const CharIterator& chrIt, int fontpixelsize, uint32_t &codetableindex);
@@ -500,7 +523,7 @@ public:
 	DefineFontTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
 	number_t getRenderCharAdvance(uint32_t index) const override;
 	void getTextBounds(const tiny_string& text, int fontpixelsize, number_t& width, number_t& height) override;
-	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, FILLSTYLE& fillstyleColor, uint32_t leading,uint32_t startpos) override;
+	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, const list<FILLSTYLE>& fillstyleColor, int32_t leading, int32_t startposx, int32_t startposy) override;
 	int32_t getLeading() const override { return 1024; }
 	int32_t getAscent() const override { return 1024; }
 	int32_t getDescent() const override { return 1024; }
@@ -536,7 +559,7 @@ public:
 	DefineFont2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
 	number_t getRenderCharAdvance(uint32_t index) const override;
 	void getTextBounds(const tiny_string& text, int fontpixelsize, number_t& width, number_t& height) override;
-	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, FILLSTYLE& fillstyleColor, uint32_t leading,uint32_t startpos) override;
+	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, const list<FILLSTYLE>& fillstyleColor, int32_t leading,int32_t startposx, int32_t startposy) override;
 	int32_t getLeading() const override { return FontLeading; }
 	int32_t getAscent() const override { return FontAscent; }
 	int32_t getDescent() const override { return FontDescent; }
@@ -565,7 +588,7 @@ public:
 	DefineFont3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
 	number_t getRenderCharAdvance(uint32_t index) const override;
 	void getTextBounds(const tiny_string& text, int fontpixelsize, number_t& width, number_t& height) override;
-	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, FILLSTYLE& fillstyleColor, uint32_t leading,uint32_t startpos) override;
+	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, const list<FILLSTYLE>& fillstyleColor, int32_t leading, int32_t startposx, int32_t startposy) override;
 	int32_t getLeading() const override { return FontLeading/20; }
 	int32_t getAscent() const override { return FontAscent/20; }
 	int32_t getDescent() const override { return FontDescent/20; }
@@ -623,10 +646,10 @@ public:
 	~DefineSpriteTag();
 	int getId() const override { return SpriteID; }
 	ASObject* instance(Class_base* c=nullptr) override;
-	void setSoundStartFrame(uint32_t frame) 
+	void setSoundStartFrame()
 	{
 		if (soundstartframe == UINT32_MAX)
-			soundstartframe=frame;
+			soundstartframe=frames.size();
 	}
 };
 
@@ -642,12 +665,13 @@ class BitmapContainer;
 class BitmapTag: public DictionaryTag
 {
 protected:
-        _R<BitmapContainer> bitmap;
-    void loadBitmap(uint8_t* inData, int datasize, const uint8_t *tablesData=nullptr, int tablesLen=0);
+	_NR<BitmapContainer> bitmap;
+	void loadBitmap(uint8_t* inData, int datasize, const uint8_t *tablesData=nullptr, int tablesLen=0);
 public:
 	BitmapTag(RECORDHEADER h,RootMovieClip* root);
+	~BitmapTag();
 	ASObject* instance(Class_base* c=nullptr) override;
-	_R<BitmapContainer> getBitmap() const;
+	_NR<BitmapContainer> getBitmap() const;
 };
 
 class JPEGTablesTag: public Tag
@@ -704,12 +728,14 @@ public:
 	int getId() const override { return CharacterId; }
 };
 
-class DefineScalingGridTag: public Tag
+class DefineScalingGridTag: public ControlTag
 {
 public:
 	UI16_SWF CharacterId;
 	RECT Splitter;
 	DefineScalingGridTag(RECORDHEADER h, std::istream& in);
+	TAGTYPE getType() const override { return DEFINESCALINGGRID_TAG; }
+	void execute(RootMovieClip* root) const override;
 };
 
 class AdditionalDataTag: public Tag
@@ -775,6 +801,7 @@ public:
 	void setFrameData(VideoFrameTag* tag);
 	VideoFrameTag* getFrame(uint32_t frame) const { return frames[frame]; }
 };
+
 class VideoFrameTag: public DisplayListTag
 {
 private:
@@ -783,19 +810,21 @@ private:
 	uint8_t* framedata;
 	uint32_t numbytes;
 public:
-	VideoFrameTag(RECORDHEADER h, std::istream& in);
+	VideoFrameTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
 	~VideoFrameTag();
-	void execute(DisplayObjectContainer* parent,bool inskipping) override;
+	void execute(DisplayObjectContainer* parent, bool inskipping) override {}
 	uint8_t* getData() { return framedata; }
 	uint32_t getNumBytes() { return numbytes+AV_INPUT_BUFFER_PADDING_SIZE; }
 	uint32_t getFrameNumber() { return FrameNum; }
 };
+
 class MetadataTag: public Tag
 {
 private:
 	STRING XmlString;
 public:
 	MetadataTag(RECORDHEADER h, std::istream& in);
+	TAGTYPE getType() const override { return METADATA_TAG; }
 };
 
 class CSMTextSettingsTag: public Tag
@@ -838,6 +867,7 @@ public:
 	UB ActionScript3;
 	UB UseNetwork;
 	FileAttributesTag(RECORDHEADER h, std::istream& in);
+	TAGTYPE getType() const override { return FILEATTRIBUTES_TAG; }
 };
 
 class EnableDebuggerTag: public Tag
@@ -846,6 +876,7 @@ private:
 	STRING DebugPassword;
 public:
 	EnableDebuggerTag(RECORDHEADER h, std::istream& in);
+	TAGTYPE getType() const override { return ENABLEDEBUGGER_TAG; }
 };
 
 class EnableDebugger2Tag: public Tag
@@ -855,6 +886,7 @@ private:
 	STRING DebugPassword;
 public:
 	EnableDebugger2Tag(RECORDHEADER h, std::istream& in);
+	TAGTYPE getType() const override { return ENABLEDEBUGGER_TAG; }
 };
 
 class DebugIDTag: public Tag
@@ -891,6 +923,41 @@ public:
 	Tag* readTag(RootMovieClip* root,DefineSpriteTag* sprite=nullptr);
 };
 
+
+class DoABCTag: public ControlTag
+{
+private:
+	ABCContext* context;
+public:
+	DoABCTag(RECORDHEADER h, std::istream& in);
+	TAGTYPE getType() const override { return ABC_TAG; }
+	void execute(RootMovieClip* root) const override;
+};
+
+class DoABCDefineTag: public ControlTag
+{
+private:
+	UI32_SWF Flags;
+	STRING Name;
+	ABCContext* context;
+public:
+	DoABCDefineTag(RECORDHEADER h, std::istream& in);
+	TAGTYPE getType() const override { return ABC_TAG; }
+	void execute(RootMovieClip* root) const override;
+};
+
+class SymbolClassTag: public ControlTag
+{
+private:
+	UI16_SWF NumSymbols;
+	std::vector<UI16_SWF> Tags;
+	std::vector<STRING> Names;
+public:
+	SymbolClassTag(RECORDHEADER h, std::istream& in);
+	TAGTYPE getType() const override { return SYMBOL_CLASS_TAG; }
+	void execute(RootMovieClip* root) const override;
+};
+
 }
 
 #endif /* PARSING_TAGS_H */
