@@ -22,22 +22,33 @@
 #include "swf.h"
 #include <SDL2/SDL.h>
 #include <SDL2/SDL_mouse.h>
-#include <SDL2/SDL_mixer.h>
 #include "backends/input.h"
 #include "backends/rendering.h"
 #include "backends/lsopengl.h"
+#include "backends/audio.h"
 #include <pango/pangocairo.h>
 #include "version.h"
 #include "abc.h"
 #include "class.h"
 #include "scripting/flash/events/flashevents.h"
+#include "flash/display/NativeMenuItem.h"
+#include "flash/utils/ByteArray.h"
 #include <glib/gstdio.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
+extern "C" {
+#ifdef ENABLE_GLES2
+extern NVGcontext* nvgCreateGLES2(int flags);
+extern void nvgDeleteGLES2(NVGcontext* ctx);
+#else
+extern NVGcontext* nvgCreateGL2(int flags);
+extern void nvgDeleteGL2(NVGcontext* ctx);
+#endif
+}
 //The interpretation of texture data change with the endianness
-#if __BYTE_ORDER == __BIG_ENDIAN
+#if G_BYTE_ORDER == G_BIG_ENDIAN
 #define GL_UNSIGNED_INT_8_8_8_8_HOST GL_UNSIGNED_INT_8_8_8_8_REV
 #else
 #define GL_UNSIGNED_INT_8_8_8_8_HOST GL_UNSIGNED_BYTE
@@ -51,21 +62,25 @@ SDL_Thread* EngineData::mainLoopThread = nullptr;
 bool EngineData::mainthread_running = false;
 bool EngineData::sdl_needinit = true;
 bool EngineData::enablerendering = true;
+SDL_Cursor* EngineData::handCursor = nullptr;
 Semaphore EngineData::mainthread_initialized(0);
-EngineData::EngineData() : contextmenu(nullptr),contextmenurenderer(nullptr),sdleventtickjob(nullptr),incontextmenu(false),incontextmenupreparing(false),currentPixelBufPtr(nullptr),pixelBufferWidth(0),pixelBufferHeight(0),widget(0), width(0), height(0),needrenderthread(true),supportPackedDepthStencil(false),hasExternalFontRenderer(false)
+EngineData::EngineData() : contextmenu(nullptr),contextmenurenderer(nullptr),sdleventtickjob(nullptr),incontextmenu(false),incontextmenupreparing(false),widget(nullptr),nvgcontext(nullptr), width(0), height(0),needrenderthread(true),supportPackedDepthStencil(false),hasExternalFontRenderer(false),
+	startInFullScreenMode(false),startscalefactor(1.0)
 {
 }
 
 EngineData::~EngineData()
 {
-	if (currentPixelBufPtr) {
-#ifdef _WIN32
-		_aligned_free(currentPixelBufPtr);
+	if (handCursor)
+		SDL_FreeCursor(handCursor);
+	handCursor=nullptr;
+#ifdef ENABLE_GLES2
+	if (nvgcontext)
+		nvgDeleteGLES2(nvgcontext);
 #else
-		free(currentPixelBufPtr);
+	if (nvgcontext)
+		nvgDeleteGL2(nvgcontext);
 #endif
-		currentPixelBufPtr = nullptr;
-	}
 }
 bool EngineData::mainloop_handleevent(SDL_Event* event,SystemState* sys)
 {
@@ -75,6 +90,7 @@ bool EngineData::mainloop_handleevent(SDL_Event* event,SystemState* sys)
 	{
 		sys = (SystemState*)event->user.data1;
 		setTLSSys(sys);
+		setTLSWorker(sys->worker);
 	}
 	else if (event->type == LS_USEREVENT_EXEC)
 	{
@@ -127,12 +143,22 @@ bool EngineData::mainloop_handleevent(SDL_Event* event,SystemState* sys)
 					{
 						//Signal the renderThread
 						if (sys && sys->getRenderThread())
+						{
 							sys->getRenderThread()->draw(true);
+							// it seems that sometimes the systemstate thread stops ticking when in background, this ensures it starts ticking again...
+							sys->sendMainSignal();
+						}
 						break;
 					}
 					case SDL_WINDOWEVENT_FOCUS_LOST:
 						sys->getEngineData()->closeContextMenu();
 						break;
+					case SDL_WINDOWEVENT_ENTER:
+						sys->addBroadcastEvent("activate");
+						break;
+					case SDL_WINDOWEVENT_LEAVE:
+						sys->addBroadcastEvent("deactivate");
+						break;
 					default:
 						break;
 				}
@@ -145,12 +171,9 @@ bool EngineData::mainloop_handleevent(SDL_Event* event,SystemState* sys)
 	}
 	return false;
 }
-
-/* main loop handling */
-static int mainloop_runner(void*)
+bool initSDL()
 {
 	bool sdl_available = !EngineData::sdl_needinit;
-	
 	if (EngineData::sdl_needinit)
 	{
 		if (!EngineData::enablerendering)
@@ -166,9 +189,22 @@ static int mainloop_runner(void*)
 				sdl_available = !SDL_InitSubSystem ( SDL_INIT_VIDEO );
 			else
 				sdl_available = !SDL_Init ( SDL_INIT_VIDEO );
+			SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8); // needed for nanovg
+			SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);// needed for nanovg
+			SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 4);// needed for nanovg
+#ifdef ENABLE_GLES2
+			SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_ES);
+			SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 2);
+			SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 0);
+#endif
 		}
 	}
-	if (!sdl_available)
+	return sdl_available;
+}
+/* main loop handling */
+static int mainloop_runner(void*)
+{
+	if (!initSDL())
 	{
 		LOG(LOG_ERROR,"Unable to initialize SDL:"<<SDL_GetError());
 		EngineData::mainthread_initialized.signal();
@@ -194,6 +230,8 @@ static int mainloop_runner(void*)
 }
 void EngineData::mainloop_from_plugin(SystemState* sys)
 {
+	initSDL();
+	EngineData::sdl_needinit = false;
 	SDL_Event event;
 	setTLSSys(sys);
 	while (SDL_PollEvent(&event))
@@ -242,33 +280,69 @@ bool EngineData::startSDLMain()
 	return mainthread_running;
 }
 
-bool EngineData::FileExists(const tiny_string &filename)
+bool EngineData::FileExists(SystemState* sys, const tiny_string &filename, bool isfullpath)
 {
 	LOG(LOG_ERROR,"FileExists not implemented");
 	return false;
 }
 
-tiny_string EngineData::FileRead(const tiny_string &filename)
+uint32_t EngineData::FileSize(SystemState* sys, const tiny_string& filename, bool isfullpath)
+{
+	LOG(LOG_ERROR,"FileSize not implemented");
+	return 0;
+}
+
+tiny_string EngineData::FileFullPath(SystemState* sys, const tiny_string& filename)
+{
+	LOG(LOG_ERROR,"FileFullPath not implemented");
+	return "";
+}
+
+tiny_string EngineData::FileRead(SystemState* sys,const tiny_string &filename, bool isfullpath)
 {
 	LOG(LOG_ERROR,"FileRead not implemented");
 	return "";
 }
 
-void EngineData::FileWrite(const tiny_string &filename, const tiny_string &data)
+void EngineData::FileWrite(SystemState* sys,const tiny_string &filename, const tiny_string &data, bool isfullpath)
 {
 	LOG(LOG_ERROR,"FileWrite not implemented");
 }
 
-void EngineData::FileReadByteArray(const tiny_string &filename,ByteArray* res)
+uint8_t EngineData::FileReadUnsignedByte(SystemState* sys, const tiny_string& filename, uint32_t startpos, bool isfullpath)
+{
+	LOG(LOG_ERROR,"FileReadUnsignedByte not implemented");
+	return 0;
+}
+
+void EngineData::FileReadByteArray(SystemState* sys, const tiny_string &filename, ByteArray* res, uint32_t startpos, uint32_t length, bool isfullpath)
 {
 	LOG(LOG_ERROR,"FileReadByteArray not implemented");
 }
 
-void EngineData::FileWriteByteArray(const tiny_string &filename, ByteArray *data)
+void EngineData::FileWriteByteArray(SystemState* sys, const tiny_string &filename, ByteArray *data, uint32_t startpos, uint32_t length, bool isfullpath)
 {
 	LOG(LOG_ERROR,"FileWriteByteArray not implemented");
 }
 
+bool EngineData::FileCreateDirectory(SystemState* sys, const tiny_string &filename, bool isfullpath)
+{
+	LOG(LOG_ERROR,"FileCreateDirectory not implemented");
+	return false;
+}
+
+bool EngineData::FilePathIsAbsolute(const tiny_string& filename)
+{
+	LOG(LOG_ERROR,"FilePathIsAbsolute not implemented");
+	return false;
+}
+
+tiny_string EngineData::FileGetApplicationStorageDir()
+{
+	LOG(LOG_ERROR,"FileGetApplicationStorageDir not implemented");
+	return "";
+}
+
 void EngineData::initGLEW()
 {
 //For now GLEW does not work with GLES2
@@ -282,14 +356,14 @@ void EngineData::initGLEW()
 		if (!videodriver || strcmp(videodriver,"wayland")!= 0 || err != GLEW_ERROR_NO_GLX_DISPLAY)
 #endif
 		{
-			LOG(LOG_ERROR,_("Cannot initialize GLEW: cause ") << glewGetErrorString(err));
+			LOG(LOG_ERROR,"Cannot initialize GLEW: cause " << glewGetErrorString(err));
 			throw RunTimeException("Rendering: Cannot initialize GLEW!");
 		}
 	}
 
 	if(!GLEW_VERSION_2_0)
 	{
-		LOG(LOG_ERROR,_("Video card does not support OpenGL 2.0... Aborting"));
+		LOG(LOG_ERROR,"Video card does not support OpenGL 2.0... Aborting");
 		throw RunTimeException("Rendering: OpenGL driver does not support OpenGL 2.0");
 	}
 	if(!GLEW_ARB_framebuffer_object)
@@ -299,6 +373,18 @@ void EngineData::initGLEW()
 	}
 	supportPackedDepthStencil = GLEW_EXT_packed_depth_stencil;
 #endif
+	initNanoVG();
+}
+
+void EngineData::initNanoVG()
+{
+#ifdef ENABLE_GLES2
+	nvgcontext=nvgCreateGLES2(0);
+#else
+	nvgcontext=nvgCreateGL2(0);
+#endif
+	if (nvgcontext == nullptr)
+		LOG(LOG_ERROR,"couldn't initialize nanovg");
 }
 
 void EngineData::showWindow(uint32_t w, uint32_t h)
@@ -328,7 +414,6 @@ std::string EngineData::getsharedobjectfilename(const tiny_string& name)
 
 	std::string p = subdir.raw_buf();
 	p += G_DIR_SEPARATOR_S;
-	p += "shared_";
 	p += name.raw_buf();
 	p += ".sol";
 	return p;
@@ -342,7 +427,10 @@ void EngineData::setLocalStorageAllowedMarker(bool allowed)
 	p += G_DIR_SEPARATOR_S;
 	p += "localStorageAllowed";
 	if (allowed)
-		g_creat(p.c_str(),0600);
+	{
+		if (!g_file_test(p.c_str(),G_FILE_TEST_EXISTS))
+			g_creat(p.c_str(),0600);
+	}
 	else
 		g_unlink(p.c_str());
 }
@@ -490,6 +578,13 @@ void EngineData::selectContextMenuItem()
 	event.type = LS_USEREVENT_SELECTITEM_CONTEXTMENU;
 	SDL_PushEvent(&event);
 }
+void EngineData::InteractiveObjectRemovedFromStage()
+{
+	SDL_Event event;
+	SDL_zero(event);
+	event.type = LS_USEREVENT_INTERACTIVEOBJECT_REMOVED_FOM_STAGE;
+	SDL_PushEvent(&event);
+}
 void EngineData::selectContextMenuItemIntern()
 {
 	if (contextmenucurrentitem >=0)
@@ -532,7 +627,7 @@ void EngineData::selectContextMenuItemIntern()
 		else
 		{
 			item->incRef();
-			getVm(item->getSystemState())->addEvent(_MR(item),_MR(Class<ContextMenuEvent>::getInstanceS(item->getSystemState(),"menuItemSelect",contextmenuDispatcher,contextmenuOwner)));
+			getVm(item->getSystemState())->addEvent(_MR(item),_MR(Class<ContextMenuEvent>::getInstanceS(item->getInstanceWorker(),"menuItemSelect",contextmenuDispatcher,contextmenuOwner)));
 		}
 	}
 	closeContextMenu();
@@ -637,6 +732,17 @@ void EngineData::hideMouseCursor(SystemState* /*sys*/)
 	SDL_ShowCursor(SDL_DISABLE);
 }
 
+void EngineData::setMouseHandCursor(SystemState* /*sys*/)
+{
+	if (!handCursor)
+		handCursor = SDL_CreateSystemCursor(SDL_SystemCursor::SDL_SYSTEM_CURSOR_HAND);
+	SDL_SetCursor(handCursor);
+}
+void EngineData::resetMouseHandCursor(SystemState* /*sys*/)
+{
+	SDL_SetCursor(SDL_GetDefaultCursor());
+}
+
 void EngineData::setClipboardText(const std::string txt)
 {
 	int ret = SDL_SetClipboardText(txt.c_str());
@@ -657,31 +763,6 @@ bool EngineData::getGLError(uint32_t &errorCode) const
 	return errorCode!=GL_NO_ERROR;
 }
 
-uint8_t *EngineData::getCurrentPixBuf() const
-{
-	return currentPixelBufPtr;
-}
-
-uint8_t *EngineData::switchCurrentPixBuf(uint32_t w, uint32_t h)
-{
-	//TODO See if a more elegant way of handling the non-PBO case can be found.
-	//for now, each frame is uploaded one at a time synchronously to the server
-	if(!currentPixelBufPtr)
-#ifdef _WIN32
-		currentPixelBufPtr = (uint8_t*)_aligned_malloc(w*h*4, 16);
-		if (!currentPixelBufPtr) {
-			LOG(LOG_ERROR, "posix_memalign could not allocate memory");
-			return nullptr;
-		}
-#else
-		if(posix_memalign((void **)&currentPixelBufPtr, 16, w*h*4)) {
-			LOG(LOG_ERROR, "posix_memalign could not allocate memory");
-			return nullptr;
-		}
-#endif
-	return currentPixelBufPtr;
-}
-
 tiny_string EngineData::getGLDriverInfo()
 {
 	tiny_string res = "OpenGL Vendor=";
@@ -695,21 +776,21 @@ tiny_string EngineData::getGLDriverInfo()
 	return res;
 }
 
-void EngineData::resizePixelBuffers(uint32_t w, uint32_t h)
+void EngineData::getGlCompressedTextureFormats()
 {
-	if(w<=pixelBufferWidth && h<=pixelBufferHeight)
+	int32_t numformats;
+	glGetIntegerv(GL_NUM_COMPRESSED_TEXTURE_FORMATS,&numformats);
+	if (numformats == 0)
 		return;
-	
-	pixelBufferWidth=w;
-	pixelBufferHeight=h;
-	if (currentPixelBufPtr) {
-#ifdef _WIN32
-		_aligned_free(currentPixelBufPtr);
-#else
-		free(currentPixelBufPtr);
-#endif
-		currentPixelBufPtr = nullptr;
+	int32_t* formats = new int32_t[numformats];
+	glGetIntegerv(GL_COMPRESSED_TEXTURE_FORMATS,formats);
+	for (int32_t i = 0; i < numformats; i++)
+	{
+		LOG(LOG_INFO,"OpenGL supported compressed texture format:"<<hex<<formats[i]);
+		if (formats[i] == GL_COMPRESSED_RGBA_S3TC_DXT5_EXT)
+			compressed_texture_formats.push_back(TEXTUREFORMAT_COMPRESSED::DXT5);
 	}
+	delete [] formats;
 }
 
 void EngineData::exec_glUniform1f(int location,float v0)
@@ -793,7 +874,9 @@ void EngineData::exec_glBindBuffer_GL_ARRAY_BUFFER(uint32_t buffer)
 }
 void EngineData::exec_glEnable_GL_TEXTURE_2D()
 {
+#ifndef ENABLE_GLES2
 	glEnable(GL_TEXTURE_2D);
+#endif
 }
 void EngineData::exec_glEnable_GL_BLEND()
 {
@@ -850,7 +933,9 @@ void EngineData::exec_glEnable_GL_STENCIL_TEST()
 
 void EngineData::exec_glDisable_GL_TEXTURE_2D()
 {
+#ifndef ENABLE_GLES2
 	glDisable(GL_TEXTURE_2D);
+#endif
 }
 void EngineData::exec_glFlush()
 {
@@ -1165,8 +1250,8 @@ void EngineData::exec_glSetTexParameters(int32_t lodbias, uint32_t dimension, ui
 			break;
 	}
 	glTexParameteri(dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter ? GL_LINEAR : GL_NEAREST);
-	glTexParameteri(dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP_TO_EDGE);
-	glTexParameteri(dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP_TO_EDGE);
+	glTexParameteri(dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap&1 ? GL_REPEAT : GL_CLAMP_TO_EDGE);
+	glTexParameteri(dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap&2 ? GL_REPEAT : GL_CLAMP_TO_EDGE);
 #ifdef ENABLE_GLES2
 	if (lodbias != 0)
 		LOG(LOG_NOT_IMPLEMENTED,"Context3D: GL_TEXTURE_LOD_BIAS not available for OpenGL ES");
@@ -1184,6 +1269,58 @@ void EngineData::exec_glTexImage2D_GL_TEXTURE_2D_GL_UNSIGNED_INT_8_8_8_8_HOST(in
 {
 	glTexImage2D(GL_TEXTURE_2D, level, GL_RGBA8, width, height, border, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_HOST, pixels);
 }
+void EngineData::exec_glTexImage2D_GL_TEXTURE_2D(int32_t level,int32_t width, int32_t height,int32_t border, void* pixels, TEXTUREFORMAT format, TEXTUREFORMAT_COMPRESSED compressedformat,uint32_t compressedImageSize)
+{
+	switch (format)
+	{
+		case TEXTUREFORMAT::BGRA:
+			glTexImage2D(GL_TEXTURE_2D, level, GL_RGBA8, width, height, border, GL_BGRA, GL_UNSIGNED_BYTE, pixels);
+			break;
+		case TEXTUREFORMAT::BGR:
+#if ENABLE_GLES2
+			for (int i = 0; i < width*height*3; i += 3) {
+				uint8_t t = ((uint8_t*)pixels)[i];
+				((uint8_t*)pixels)[i] = ((uint8_t*)pixels)[i+2];
+				((uint8_t*)pixels)[i+2] = t;
+			}
+			glTexImage2D(GL_TEXTURE_2D, level, GL_RGB, width, height, border, GL_RGB, GL_UNSIGNED_BYTE, pixels);
+#else
+			glTexImage2D(GL_TEXTURE_2D, level, GL_RGB, width, height, border, GL_BGR, GL_UNSIGNED_BYTE, pixels);
+#endif
+			break;
+		case TEXTUREFORMAT::BGRA_PACKED:
+			glTexImage2D(GL_TEXTURE_2D, level, GL_RGBA8, width, height, border, GL_BGRA, GL_UNSIGNED_SHORT_4_4_4_4, pixels);
+			break;
+		case TEXTUREFORMAT::BGR_PACKED:
+#if ENABLE_GLES2
+			LOG(LOG_NOT_IMPLEMENTED,"textureformat BGR_PACKED for opengl es");
+			glTexImage2D(GL_TEXTURE_2D, level, GL_RGB, width, height, border, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, pixels);
+#else
+			glTexImage2D(GL_TEXTURE_2D, level, GL_RGB, width, height, border, GL_BGR, GL_UNSIGNED_SHORT_5_6_5, pixels);
+#endif
+			break;
+		case TEXTUREFORMAT::COMPRESSED:
+		case TEXTUREFORMAT::COMPRESSED_ALPHA:
+		{
+			switch (compressedformat)
+			{
+				case TEXTUREFORMAT_COMPRESSED::DXT5:
+					glCompressedTexImage2D(GL_TEXTURE_2D, level, GL_COMPRESSED_RGBA_S3TC_DXT5_EXT, width, height, border, compressedImageSize, pixels);
+					break;
+				default:
+					LOG(LOG_NOT_IMPLEMENTED,"upload texture in compressed format "<<compressedformat);
+					break;
+			}
+			break;
+		}
+		case TEXTUREFORMAT::RGBA_HALF_FLOAT:
+			LOG(LOG_NOT_IMPLEMENTED,"upload texture in format "<<format);
+			break;
+		default:
+			LOG(LOG_ERROR,"invalid format for upload texture:"<<format);
+			break;
+	}
+}
 
 void EngineData::exec_glDrawBuffer_GL_BACK()
 {
@@ -1233,6 +1370,7 @@ void EngineData::exec_glGetIntegerv_GL_MAX_TEXTURE_SIZE(int32_t* data)
 {
 	glGetIntegerv(GL_MAX_TEXTURE_SIZE,data);
 }
+
 void EngineData::exec_glGenerateMipmap_GL_TEXTURE_2D()
 {
 	glGenerateMipmap(GL_TEXTURE_2D);
@@ -1265,71 +1403,86 @@ void EngineData::exec_glScissor(int32_t x, int32_t y, int32_t width, int32_t hei
 	glEnable(GL_SCISSOR_TEST);
 	glScissor(x,y,width,height);
 }
+void EngineData::exec_glDisable_GL_SCISSOR_TEST()
+{
+	glDisable(GL_SCISSOR_TEST);
+}
 
 void EngineData::exec_glColorMask(bool red, bool green, bool blue, bool alpha)
 {
 	glColorMask(red,green,blue,alpha);
 }
 
-
-void mixer_effect_ffmpeg_cb(int chan, void * stream, int len, void * udata)
+void EngineData::exec_glStencilFunc_GL_ALWAYS()
 {
-	AudioStream *s = (AudioStream*)udata;
-	if (!s)
-		return;
-	s->startMixing();
-	memset(stream,0,len);
-	uint32_t readcount = 0;
-	while (readcount < ((uint32_t)len))
-	{
-		uint32_t ret = s->getDecoder()->copyFrame((int16_t *)(((unsigned char*)stream)+readcount), ((uint32_t)len)-readcount);
-		if (!ret)
-			break;
-		readcount += ret;
-	}
+	glStencilFunc(GL_ALWAYS, 0, 0xff);
 }
 
+void audioCallback(void * userdata, uint8_t * stream, int len)
+{
+	AudioManager* manager = (AudioManager*)userdata;
 
+    SDL_memset(stream, 0, len);
+
+	{
+		Locker l(manager->streamMutex);
+		for (auto it = manager->streams.begin(); it != manager->streams.end(); it++)
+		{
+			AudioStream* s = (*it);
+			if (s->ispaused())
+				continue;
+			s->startMixing();
+			const float fmaxvolume = 1.0f / ((float)SDL_MIX_MAXVOLUME);
+			const float fvolume = (float)s->getVolume()*(float)SDL_MIX_MAXVOLUME;
+			uint32_t readcount = 0;
+			uint8_t* buf = new uint8_t[len];
+			while (readcount < ((uint32_t)len))
+			{
+				uint32_t ret = s->getDecoder()->copyFrameF32((float *)(buf+readcount), ((uint32_t)len)-readcount);
+				if (!ret)
+					break;
+				float* src32=(float *)(buf+readcount);
+				float* dst32=(float *)(stream+readcount);
+				readcount += ret;
+
+				float src1, src2;
+				double dst_sample;
+
+				const double max_audioval = 3.402823466e+38F;
+				const double min_audioval = -3.402823466e+38F;
+				int curpanning=0;
+				while (ret)
+				{
+					src1 = *src32 * fvolume * fmaxvolume * s->getPanning()[curpanning];
+					curpanning = 1-curpanning;
+					src2 = *dst32;
+					src32++;
+					dst_sample = ((double)src1) + ((double)src2);
+					if (dst_sample > max_audioval) {
+						dst_sample = max_audioval;
+					} else if (dst_sample < min_audioval) {
+						dst_sample = min_audioval;
+					}
+					*(dst32++) = (float)dst_sample;
+					ret-=4;
+				}
+			}
+			delete[] buf;
+		}
+    }
+}
 
 int EngineData::audio_StreamInit(AudioStream* s)
 {
-	int mixer_channel = -1;
-
-	uint32_t len = LIGHTSPARK_AUDIO_BUFFERSIZE;
-
-	uint8_t *buf = new uint8_t[len];
-	memset(buf,0,len);
-	Mix_Chunk* chunk = Mix_QuickLoad_RAW(buf, len);
-	delete[] buf;
-
-
-	mixer_channel = Mix_PlayChannel(-1, chunk, -1);
-	Mix_RegisterEffect(mixer_channel, mixer_effect_ffmpeg_cb, NULL, s);
-	Mix_Resume(mixer_channel);
-	return mixer_channel;
+	return 0;
 }
 
 void EngineData::audio_StreamPause(int channel, bool dopause)
 {
-	if (channel == -1)
-		return;
-	if(dopause)
-		Mix_Pause(channel);
-	else
-		Mix_Resume(channel);
-}
-
-void EngineData::audio_StreamSetVolume(int channel, double volume)
-{
-	int curvolume = SDL_MIX_MAXVOLUME * volume;
-	if (channel != -1)
-		Mix_Volume(channel, curvolume);
 }
 
 void EngineData::audio_StreamDeinit(int channel)
 {
-	if (channel != -1)
-		Mix_HaltChannel(channel);
 }
 
 bool EngineData::audio_ManagerInit()
@@ -1342,18 +1495,30 @@ bool EngineData::audio_ManagerInit()
 	return sdl_available;
 }
 
-void EngineData::audio_ManagerCloseMixer()
+void EngineData::audio_ManagerCloseMixer(AudioManager* manager)
 {
-	Mix_CloseAudio();
+	if (manager->device)
+	{
+		SDL_PauseAudioDevice(manager->device, 1);
+		SDL_CloseAudioDevice(manager->device);
+		manager->device=0;
+	}
 }
 
-bool EngineData::audio_ManagerOpenMixer()
+bool EngineData::audio_ManagerOpenMixer(AudioManager* manager)
 {
-#if __BYTE_ORDER == __BIG_ENDIAN
-	return Mix_OpenAudio (audio_getSampleRate(), AUDIO_S16MSB, 2, LIGHTSPARK_AUDIO_BUFFERSIZE) >= 0;
-#else
-	return Mix_OpenAudio (audio_getSampleRate(), AUDIO_S16LSB, 2, LIGHTSPARK_AUDIO_BUFFERSIZE) >= 0;
-#endif
+	SDL_AudioSpec spec;
+	spec.freq=audio_getSampleRate();
+	spec.format=AUDIO_F32SYS;
+	spec.channels = 2;
+	spec.samples = LIGHTSPARK_AUDIO_BUFFERSIZE/2;
+	spec.callback = audioCallback;
+	spec.userdata = manager;
+
+	manager->device = SDL_OpenAudioDevice(nullptr, 0, &spec, nullptr, 0);
+	if (manager->device != 0)
+		SDL_PauseAudioDevice(manager->device, 0);
+	return manager->device!=0;
 }
 
 void EngineData::audio_ManagerDeinit()
@@ -1362,36 +1527,41 @@ void EngineData::audio_ManagerDeinit()
 	if (!SDL_WasInit(0))
 		SDL_Quit ();
 }
+bool EngineData::audio_useFloatSampleFormat()
+{
+	return true;
+}
 
 int EngineData::audio_getSampleRate()
 {
-	return MIX_DEFAULT_FREQUENCY;
+	return 44100;
 }
-
-IDrawable *EngineData::getTextRenderDrawable(const TextData &_textData, const MATRIX &_m, int32_t _x, int32_t _y, int32_t _w, int32_t _h, int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r, float _xs, float _ys, bool _im, bool _hm, float _s, float _a, const std::vector<IDrawable::MaskData> &_ms, float _redMultiplier, float _greenMultiplier, float _blueMultiplier, float _alphaMultiplier, float _redOffset, float _greenOffset, float _blueOffset, float _alphaOffset, bool smoothing)
+IDrawable *EngineData::getTextRenderDrawable(const TextData &_textData, const MATRIX &_m, int32_t _x, int32_t _y, int32_t _w, int32_t _h, int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r, float _xs, float _ys, bool _im, _NR<DisplayObject> _mask, float _s, float _a, const std::vector<IDrawable::MaskData> &_ms, float _redMultiplier, float _greenMultiplier, float _blueMultiplier, float _alphaMultiplier, float _redOffset, float _greenOffset, float _blueOffset, float _alphaOffset, SMOOTH_MODE smoothing)
 {
 	if (hasExternalFontRenderer)
-		return new externalFontRenderer(_textData,this, _x, _y, _w, _h, _rx,_ry,_rw,_rh,_r,_xs,_ys,_im,_hm, _a, _ms,
+		return new externalFontRenderer(_textData,this, _x, _y, _w, _h, _rx,_ry,_rw,_rh,_r,_xs,_ys,_im, _mask, _a, _ms,
 										_redMultiplier,_greenMultiplier,_blueMultiplier,_alphaMultiplier,
 										_redOffset,_greenOffset,_blueOffset,_alphaOffset,
-										smoothing);
+										smoothing,_m);
 	return nullptr;
 }
 
-externalFontRenderer::externalFontRenderer(const TextData &_textData, EngineData *engine, int32_t x, int32_t y, int32_t w, int32_t h, int32_t rx, int32_t ry, int32_t rw, int32_t rh, float r, float xs, float ys, bool im, bool hm, float a, const std::vector<IDrawable::MaskData> &m,
+externalFontRenderer::externalFontRenderer(const TextData &_textData, EngineData *engine, int32_t x, int32_t y, int32_t w, int32_t h, int32_t rx, int32_t ry, int32_t rw, int32_t rh, float r, float xs, float ys, bool im, _NR<DisplayObject> _mask, float a, const std::vector<IDrawable::MaskData> &m,
 										   float _redMultiplier,float _greenMultiplier,float _blueMultiplier,float _alphaMultiplier,
 										   float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset,
-										   bool smoothing)
-	: IDrawable(w, h, x, y,rw,rh,rx,ry,r,xs,ys,im,hm, a, m,
+										   SMOOTH_MODE smoothing, const MATRIX &_m)
+	: IDrawable(w, h, x, y,rw,rh,rx,ry,r,xs,ys, 1, 1, im, _mask, a, m,
 				_redMultiplier,_greenMultiplier,_blueMultiplier,_alphaMultiplier,
-				_redOffset,_greenOffset,_blueOffset,_alphaOffset,smoothing),m_engine(engine)
+				_redOffset,_greenOffset,_blueOffset,_alphaOffset,smoothing,_m),m_engine(engine)
 {
 	externalressource = engine->setupFontRenderer(_textData,a,smoothing);
 }
 
-uint8_t *externalFontRenderer::getPixelBuffer(float scalex, float scaley, bool *isBufferOwner)
+uint8_t *externalFontRenderer::getPixelBuffer(bool *isBufferOwner, uint32_t* bufsize)
 {
 	if (isBufferOwner)
 		*isBufferOwner = true;
+	if (bufsize)
+		*bufsize=width*height*4;
 	return m_engine->getFontPixelBuffer(externalressource,this->width,this->height);
 }
