@@ -27,10 +27,12 @@
 #include "threading.h"
 #include "tiny_string.h"
 #include "backends/graphics.h"
-#include "flash/utils/ByteArray.h"
-#include "flash/display/NativeMenuItem.h"
 
-#define LIGHTSPARK_AUDIO_BUFFERSIZE 8192
+#define LIGHTSPARK_AUDIO_BUFFERSIZE 512
+
+extern "C" {
+struct NVGcontext;
+}
 
 namespace lightspark
 {
@@ -41,16 +43,24 @@ namespace lightspark
 #define LS_USEREVENT_OPEN_CONTEXTMENU EngineData::userevent+3
 #define LS_USEREVENT_UPDATE_CONTEXTMENU EngineData::userevent+4
 #define LS_USEREVENT_SELECTITEM_CONTEXTMENU EngineData::userevent+5
+#define LS_USEREVENT_INTERACTIVEOBJECT_REMOVED_FOM_STAGE EngineData::userevent+6
+
 class SystemState;
 class StreamCache;
 class AudioStream;
+class AudioManager;
 class ITickJob;
+class ByteArray;
+class NativeMenuItem;
+class InteractiveObject;
 
 enum DEPTH_FUNCTION { ALWAYS, EQUAL, GREATER, GREATER_EQUAL, LESS, LESS_EQUAL, NEVER, NOT_EQUAL };
 enum TRIANGLE_FACE { FACE_BACK, FACE_FRONT, FACE_FRONT_AND_BACK, FACE_NONE };
 enum BLEND_FACTOR { BLEND_ONE,BLEND_ZERO,BLEND_SRC_ALPHA,BLEND_SRC_COLOR,BLEND_DST_ALPHA,BLEND_DST_COLOR,BLEND_ONE_MINUS_SRC_ALPHA,BLEND_ONE_MINUS_SRC_COLOR,BLEND_ONE_MINUS_DST_ALPHA,BLEND_ONE_MINUS_DST_COLOR };
-enum VERTEXBUFFER_FORMAT { BYTES_4, FLOAT_1, FLOAT_2, FLOAT_3, FLOAT_4 };
+enum VERTEXBUFFER_FORMAT { BYTES_4=0, FLOAT_1, FLOAT_2, FLOAT_3, FLOAT_4 };
 enum CLEARMASK { COLOR = 0x1, DEPTH = 0x2, STENCIL = 0x4 };
+enum TEXTUREFORMAT { BGRA, BGRA_PACKED, BGR_PACKED, COMPRESSED, COMPRESSED_ALPHA, RGBA_HALF_FLOAT,BGR };
+enum TEXTUREFORMAT_COMPRESSED { UNCOMPRESSED, DXT5 };
 
 // this is only used for font rendering in PPAPI plugin
 class externalFontRenderer : public IDrawable
@@ -58,13 +68,13 @@ class externalFontRenderer : public IDrawable
 	int32_t externalressource;
 	class EngineData* m_engine;
 public:
-	externalFontRenderer(const TextData &_textData, class EngineData* engine, int32_t x, int32_t y, int32_t w, int32_t h, int32_t rx, int32_t ry, int32_t rw, int32_t rh, float r, float xs, float ys, bool im, bool hm, float a, const std::vector<MaskData>& m,
+	externalFontRenderer(const TextData &_textData, class EngineData* engine, int32_t x, int32_t y, int32_t w, int32_t h, int32_t rx, int32_t ry, int32_t rw, int32_t rh, float r, float xs, float ys, bool im, _NR<DisplayObject> _mask, float a, const std::vector<MaskData>& m,
 						 float _redMultiplier,float _greenMultiplier,float _blueMultiplier,float _alphaMultiplier,
 						 float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset,
-						 bool smoothing);
+						 SMOOTH_MODE smoothing, const MATRIX &_m);
 	
-	uint8_t* getPixelBuffer(float scalex,float scaley, bool* isBufferOwner=nullptr) override;
-	void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY, float scalex, float scaley) const override {}
+	uint8_t* getPixelBuffer(bool* isBufferOwner=nullptr, uint32_t* bufsize=nullptr) override;
+	void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY) const override {}
 };
 
 #define CONTEXTMENUWIDTH 200
@@ -79,6 +89,7 @@ private:
 	SDL_Window* contextmenu;
 	SDL_Renderer* contextmenurenderer;
 	SDL_Texture* contextmenutexture;
+	static SDL_Cursor* handCursor;
 	uint8_t* contextmenupixels;
 	int32_t contextmenuheight;
 	void openContextMenuIntern(InteractiveObject *dispatcher);
@@ -95,10 +106,8 @@ protected:
 	virtual SDL_Window* createWidget(uint32_t w,uint32_t h)=0;
 public:
 	bool incontextmenupreparing; // needed for PPAPI plugin only
-	uint8_t* currentPixelBufPtr;
-	uint32_t pixelBufferWidth;
-	uint32_t pixelBufferHeight;
 	SDL_Window* widget;
+	NVGcontext* nvgcontext;
 	static uint32_t userevent;
 	static SDL_Thread* mainLoopThread;
 	uint32_t width;
@@ -108,7 +117,10 @@ public:
 	bool needrenderthread;
 	bool supportPackedDepthStencil;
 	bool hasExternalFontRenderer;
+	bool startInFullScreenMode;
+	double startscalefactor;
 	tiny_string driverInfoString;
+	std::vector<TEXTUREFORMAT_COMPRESSED> compressed_texture_formats;
 	EngineData();
 	virtual ~EngineData();
 	virtual bool isSizable() const = 0;
@@ -164,6 +176,7 @@ public:
 		return incontextmenupreparing;
 	}
 	void selectContextMenuItem();
+	void InteractiveObjectRemovedFromStage();
 
 	static bool sdl_needinit;
 	static bool enablerendering;
@@ -171,17 +184,26 @@ public:
 	static Semaphore mainthread_initialized;
 	static bool startSDLMain();
 
-	virtual bool FileExists(const tiny_string& filename);
-	virtual tiny_string FileRead(const tiny_string& filename);
-	virtual void FileWrite(const tiny_string& filename, const tiny_string& data);
-	virtual void FileReadByteArray(const tiny_string &filename,ByteArray* res);
-	virtual void FileWriteByteArray(const tiny_string& filename, ByteArray* data);
+	virtual bool FileExists(SystemState* sys,const tiny_string& filename, bool isfullpath);
+	virtual uint32_t FileSize(SystemState* sys,const tiny_string& filename, bool isfullpath);
+	virtual tiny_string FileFullPath(SystemState* sys,const tiny_string& filename);
+	virtual tiny_string FileRead(SystemState* sys,const tiny_string& filename, bool isfullpath);
+	virtual void FileWrite(SystemState* sys,const tiny_string& filename, const tiny_string& data, bool isfullpath);
+	virtual uint8_t FileReadUnsignedByte(SystemState* sys,const tiny_string &filename, uint32_t startpos, bool isfullpath);
+	virtual void FileReadByteArray(SystemState* sys,const tiny_string &filename,ByteArray* res, uint32_t startpos, uint32_t length, bool isfullpath);
+	virtual void FileWriteByteArray(SystemState* sys,const tiny_string& filename, ByteArray* data,uint32_t startpos, uint32_t length, bool isfullpath);
+	virtual bool FileCreateDirectory(SystemState* sys,const tiny_string& filename, bool isfullpath);
+	virtual bool FilePathIsAbsolute(const tiny_string& filename);
+	virtual tiny_string FileGetApplicationStorageDir();
+	
 	void initGLEW();
-	void resizePixelBuffers(uint32_t w, uint32_t h);
+	void initNanoVG();
 
 	/* show/hide mouse cursor, must be called from mainLoopThread */
 	static void showMouseCursor(SystemState *sys);
 	static void hideMouseCursor(SystemState *sys);
+	static void setMouseHandCursor(SystemState *sys);
+	static void resetMouseHandCursor(SystemState *sys);
 	virtual void setClipboardText(const std::string txt);
 	virtual bool getScreenData(SDL_DisplayMode* screen) = 0;
 	virtual double getScreenDPI() = 0;
@@ -192,9 +214,8 @@ public:
 	virtual void InitOpenGL() = 0;
 	virtual void DeinitOpenGL() = 0;
 	virtual bool getGLError(uint32_t& errorCode) const;
-	virtual uint8_t* getCurrentPixBuf() const;
-	virtual uint8_t* switchCurrentPixBuf(uint32_t w, uint32_t h);
 	virtual tiny_string getGLDriverInfo();
+	virtual void getGlCompressedTextureFormats();
 	virtual void exec_glUniform1f(int location,float v0);
 	virtual void exec_glUniform2f(int location,float v0, float v1);
 	virtual void exec_glUniform4f(int location,float v0, float v1,float v2, float v3);
@@ -270,6 +291,7 @@ public:
 	virtual void exec_glSetTexParameters(int32_t lodbias, uint32_t dimension, uint32_t filter, uint32_t mipmap, uint32_t wrap);
 	virtual void exec_glTexImage2D_GL_TEXTURE_2D_GL_UNSIGNED_BYTE(int32_t level, int32_t width, int32_t height, int32_t border, const void* pixels, bool hasalpha);
 	virtual void exec_glTexImage2D_GL_TEXTURE_2D_GL_UNSIGNED_INT_8_8_8_8_HOST(int32_t level,int32_t width, int32_t height,int32_t border, const void* pixels);
+	virtual void exec_glTexImage2D_GL_TEXTURE_2D(int32_t level, int32_t width, int32_t height, int32_t border, void* pixels, TEXTUREFORMAT format, TEXTUREFORMAT_COMPRESSED compressedformat, uint32_t compressedImageSize);
 	virtual void exec_glDrawBuffer_GL_BACK();
 	virtual void exec_glClearColor(float red,float green,float blue,float alpha);
 	virtual void exec_glClearStencil(uint32_t stencil);
@@ -286,26 +308,28 @@ public:
 	virtual void exec_glTexParameteri_GL_TEXTURE_CUBE_MAP_GL_TEXTURE_MAG_FILTER_GL_LINEAR();
 	virtual void exec_glTexImage2D_GL_TEXTURE_CUBE_MAP_POSITIVE_X_GL_UNSIGNED_BYTE(uint32_t side, int32_t level,int32_t width, int32_t height,int32_t border, const void* pixels);
 	virtual void exec_glScissor(int32_t x, int32_t y, int32_t width, int32_t height);
+	virtual void exec_glDisable_GL_SCISSOR_TEST();
 	virtual void exec_glColorMask(bool red, bool green, bool blue, bool alpha);
+	virtual void exec_glStencilFunc_GL_ALWAYS();
 
 	// Audio handling
 	virtual int audio_StreamInit(AudioStream* s);
 	virtual void audio_StreamPause(int channel, bool dopause);
-	virtual void audio_StreamSetVolume(int channel, double volume);
 	virtual void audio_StreamDeinit(int channel);
 	virtual bool audio_ManagerInit();
-	virtual void audio_ManagerCloseMixer();
-	virtual bool audio_ManagerOpenMixer();
+	virtual void audio_ManagerCloseMixer(AudioManager* manager);
+	virtual bool audio_ManagerOpenMixer(AudioManager* manager);
 	virtual void audio_ManagerDeinit();
 	virtual int audio_getSampleRate();
+	virtual bool audio_useFloatSampleFormat();
 	
 	// Text rendering
 	virtual uint8_t* getFontPixelBuffer(int32_t externalressource,int width,int height) { return nullptr; }
-	virtual int32_t setupFontRenderer(const TextData &_textData,float a, bool smoothing) { return 0; }
-	IDrawable* getTextRenderDrawable(const TextData& _textData, const MATRIX& _m, int32_t _x, int32_t _y, int32_t _w, int32_t _h, int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r, float _xs, float _ys, bool _im, bool _hm, float _s, float _a, const std::vector<IDrawable::MaskData>& _ms,
-									 float _redMultiplier,float _greenMultiplier,float _blueMultiplier,float _alphaMultiplier,
-									 float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset,
-									 bool smoothing);
+	virtual int32_t setupFontRenderer(const TextData &_textData,float a, SMOOTH_MODE smoothing) { return 0; }
+	IDrawable* getTextRenderDrawable(const TextData& _textData, const MATRIX& _m, int32_t _x, int32_t _y, int32_t _w, int32_t _h, int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r, float _xs, float _ys, bool _im, _NR<DisplayObject> _mask, float _s, float _a, const std::vector<IDrawable::MaskData>& _ms,
+									 float _redMultiplier, float _greenMultiplier, float _blueMultiplier, float _alphaMultiplier,
+									 float _redOffset, float _greenOffset, float _blueOffset, float _alphaOffset,
+									 SMOOTH_MODE smoothing);
 };
 
 }
