@@ -28,6 +28,9 @@
 #include "abc.h"
 #include "backends/security.h"
 #include "backends/rendering.h"
+#include "backends/audio.h"
+#include "scripting/flash/utils/ByteArray.h"
+#include "scripting/flash/display/NativeMenuItem.h"
 #include <string>
 #include <algorithm>
 #include <SDL2/SDL.h>
@@ -72,6 +75,7 @@
 #include "ppapi/c/ppb_message_loop.h"
 
 #include "GLES2/gl2.h"
+#include "GLES2/gl2ext.h"
 
 #ifdef _WIN32
 #define GL_UNSIGNED_INT_8_8_8_8_HOST GL_UNSIGNED_BYTE
@@ -593,8 +597,8 @@ lightspark::Downloader* ppDownloadManager::download(const lightspark::URLInfo& u
 	}
 
 	bool cached = false;
-	LOG(LOG_INFO, _("NET: PLUGIN: DownloadManager::download '") <<cache.getPtr()<<" "<< url.getParsedURL() << 
-			"'" << (cached ? _(" - cached") : ""));
+	LOG(LOG_INFO, "NET: PLUGIN: DownloadManager::download '" <<cache.getPtr()<<" "<< url.getParsedURL() << 
+			"'" << (cached ? " - cached" : ""));
 	//Register this download
 	ppDownloader* downloader=new ppDownloader(url.getParsedURL(), cache, m_instance, owner);
 	addDownloader(downloader);
@@ -610,7 +614,7 @@ lightspark::Downloader* ppDownloadManager::downloadWithData(const lightspark::UR
 		return StandaloneDownloadManager::downloadWithData(url, cache, data, headers, owner);
 	}
 
-	LOG(LOG_INFO, _("NET: PLUGIN: DownloadManager::downloadWithData '") << url.getParsedURL());
+	LOG(LOG_INFO, "NET: PLUGIN: DownloadManager::downloadWithData '" << url.getParsedURL());
 	//Register this download
 	ppDownloader* downloader=new ppDownloader(url.getParsedURL(), cache, data, headers, m_instance, owner);
 	addDownloader(downloader);
@@ -643,6 +647,7 @@ void ppDownloader::dlStartCallback(void* userdata,int result)
 {
 	ppDownloader* th = (ppDownloader*)userdata;
 	setTLSSys(th->m_sys);
+	setTLSWorker(th->m_sys->worker);
 	
 	if (result < 0)
 	{
@@ -681,6 +686,7 @@ void ppDownloader::dlReadResponseCallback(void* userdata,int result)
 {
 	ppDownloader* th = (ppDownloader*)userdata;
 	setTLSSys(th->m_sys);
+	setTLSWorker(th->m_sys->worker);
 	if (result < 0)
 	{
 		LOG(LOG_ERROR,"download failed:"<<result<<" "<<th->getURL()<<" "<<th->downloadedlength<<"/"<<th->getLength());
@@ -708,6 +714,7 @@ void ppDownloader::dlStartDownloadCallback(void* userdata,int result)
 {
 	ppDownloader* th = (ppDownloader*)userdata;
 	setTLSSys(th->m_sys);
+	setTLSWorker(th->m_sys->worker);
 	const tiny_string strurl = th->getURL();
 	LOG_CALL("dlStartDownloadCallback:"<<th->data.size()<<" "<<strurl);
 	th->ppurlloader = g_urlloader_interface->Create(th->m_pluginInstance->getppInstance());
@@ -817,19 +824,21 @@ ppPluginInstance::ppPluginInstance(PP_Instance instance, int16_t argc, const cha
 	inWriting(false)
 {
 	m_messageloop = g_messageloop_interface->Create(this->getppInstance());
-	m_ppLoopThread = SDL_CreateThread(ppPluginInstance::worker,"pploop",this);
 	m_cachefilesystem = g_filesystem_interface->Create(getppInstance(),PP_FileSystemType::PP_FILESYSTEMTYPE_LOCALTEMPORARY);
 	g_messageloop_interface->PostWork(m_messageloop,PP_MakeCompletionCallback(openfilesystem_callback,this),0);
-	
+
 	m_cachefilename = 0;
 	m_last_size.width = 0;
 	m_last_size.height = 0;
 	m_graphics = 0;
 	setTLSSys( nullptr );
+	setTLSWorker(nullptr);
 	m_sys=new lightspark::SystemState(0, lightspark::SystemState::FLASH);
 	//Files running in the plugin have REMOTE sandbox
 	m_sys->securityManager->setSandboxType(lightspark::SecurityManager::REMOTE);
 
+	m_ppLoopThread = SDL_CreateThread(ppPluginInstance::worker,"pploop",this);
+
 	m_sys->extScriptObject = new ppExtScriptObject(this,m_sys);
 	//Parse OBJECT/EMBED tag attributes
 	tiny_string swffile;
@@ -872,6 +881,7 @@ ppPluginInstance::ppPluginInstance(PP_Instance instance, int16_t argc, const cha
 	
 	//The sys var should be NULL in this thread
 	setTLSSys( nullptr );
+	setTLSWorker(nullptr);
 }
 int ppPluginInstance::worker(void* d)
 {
@@ -905,6 +915,7 @@ ppPluginInstance::~ppPluginInstance()
 {
 	//Shutdown the system
 	setTLSSys(m_sys);
+	setTLSWorker(m_sys->worker);
 	if(mainDownloader)
 		mainDownloader->stop();
 	if (mainDownloaderStreambuf)
@@ -920,16 +931,18 @@ ppPluginInstance::~ppPluginInstance()
 	m_sys->setShutdownFlag();
 
 	m_sys->destroy();
-	delete m_sys;
 	delete m_pt;
+	delete m_sys;
 	g_messageloop_interface->PostQuit(m_messageloop,PP_TRUE);
 	SDL_WaitThread(m_ppLoopThread,nullptr);
 	setTLSSys(nullptr);
+	setTLSWorker(nullptr);
 }
 
 void ppPluginInstance::handleResize(PP_Resource view)
 {
 	setTLSSys(m_sys);
+	setTLSWorker(m_sys->worker);
 	struct PP_Rect position;
 	if (g_view_interface->GetRect(view, &position) == PP_FALSE)
 	{
@@ -1118,6 +1131,7 @@ static uint16_t getppKeyModifier(PP_Resource input_event)
 PP_Bool ppPluginInstance::handleInputEvent(PP_Resource input_event)
 {
 	setTLSSys(m_sys);
+	setTLSWorker(m_sys->worker);
 	SDL_Event ev;
 	switch (g_inputevent_interface->GetType(input_event))
 	{
@@ -1241,10 +1255,11 @@ void ppPluginInstance::executeScriptAsync(ExtScriptObject::HOST_CALL_DATA *data)
 bool ppPluginInstance::executeScript(const std::string script, const ExtVariant **args, uint32_t argc, ASObject **result)
 {
 	setTLSSys(m_sys);
+	setTLSWorker(m_sys->worker);
 	PP_Var scr = g_var_interface->VarFromUtf8(script.c_str(),script.length());
 	PP_Var exception = PP_MakeUndefined();
 	PP_Var func = g_instance_private_interface->ExecuteScript(m_ppinstance,scr,&exception);
-	*result = NULL;
+	*result = nullptr;
 	uint32_t len;
 	if (exception.type == PP_VARTYPE_STRING)
 	{
@@ -1268,7 +1283,7 @@ bool ppPluginInstance::executeScript(const std::string script, const ExtVariant
 	std::map<int64_t, std::unique_ptr<ExtObject>> ppObjectsMap;
 	ppVariantObject tmp(ppObjectsMap, resultVariant);
 	std::map<const ExtObject*, ASObject*> asObjectsMap;
-	*(result) = tmp.getASObject(m_sys,asObjectsMap);
+	*(result) = tmp.getASObject(m_sys->worker,asObjectsMap);
 	return true;
 }
 
@@ -1325,6 +1340,7 @@ static void Messaging_HandleMessage(PP_Instance instance, struct PP_Var message)
 static bool PPP_Class_HasProperty(void* object,struct PP_Var name,struct PP_Var* exception)
 {
 	setTLSSys(((ppExtScriptObject*)object)->getSystemState());
+	setTLSWorker(((ppExtScriptObject*)object)->getSystemState()->worker);
 	LOG_CALL("PPP_Class_HasProperty");
 	uint32_t len;
 	switch (name.type)
@@ -1343,6 +1359,7 @@ static bool PPP_Class_HasProperty(void* object,struct PP_Var name,struct PP_Var*
 static bool PPP_Class_HasMethod(void* object,struct PP_Var name,struct PP_Var* exception)
 {
 	setTLSSys(((ppExtScriptObject*)object)->getSystemState());
+	setTLSWorker(((ppExtScriptObject*)object)->getSystemState()->worker);
 	LOG_CALL("PPP_Class_Method");
 	uint32_t len;
 	switch (name.type)
@@ -1360,6 +1377,7 @@ static bool PPP_Class_HasMethod(void* object,struct PP_Var name,struct PP_Var* e
 static struct PP_Var PPP_Class_GetProperty(void* object,struct PP_Var name,struct PP_Var* exception)
 {
 	setTLSSys(((ppExtScriptObject*)object)->getSystemState());
+	setTLSWorker(((ppExtScriptObject*)object)->getSystemState()->worker);
 	LOG_CALL("PPP_Class_GetProperty");
 	ExtVariant v;
 	uint32_t len;
@@ -1383,6 +1401,7 @@ static struct PP_Var PPP_Class_GetProperty(void* object,struct PP_Var name,struc
 static void PPP_Class_GetAllPropertyNames(void* object,uint32_t* property_count,struct PP_Var** properties,struct PP_Var* exception)
 {
 	setTLSSys(((ppExtScriptObject*)object)->getSystemState());
+	setTLSWorker(((ppExtScriptObject*)object)->getSystemState()->worker);
 	LOG_CALL("PPP_Class_GetAllPropertyNames");
 	ExtIdentifier** ids = NULL;
 	bool success = ((ppExtScriptObject*)object)->enumerate(&ids, property_count);
@@ -1405,17 +1424,18 @@ static void PPP_Class_GetAllPropertyNames(void* object,uint32_t* property_count,
 	}
 	else
 	{
-		properties = NULL;
+		properties = nullptr;
 		property_count = 0;
 	}
 
-	if(ids != NULL)
+	if(ids != nullptr)
 		delete ids;
 }
 
 static void PPP_Class_SetProperty(void* object,struct PP_Var name,struct PP_Var value,struct PP_Var* exception)
 {
 	setTLSSys(((ppExtScriptObject*)object)->getSystemState());
+	setTLSWorker(((ppExtScriptObject*)object)->getSystemState()->worker);
 	LOG_CALL("PPP_Class_SetProperty");
 	uint32_t len;
 	std::map<int64_t, std::unique_ptr<ExtObject>> objectsMap;
@@ -1436,6 +1456,7 @@ static void PPP_Class_SetProperty(void* object,struct PP_Var name,struct PP_Var
 static void PPP_Class_RemoveProperty(void* object,struct PP_Var name,struct PP_Var* exception)
 {
 	setTLSSys(((ppExtScriptObject*)object)->getSystemState());
+	setTLSWorker(((ppExtScriptObject*)object)->getSystemState()->worker);
 	LOG_CALL("PPP_Class_RemoveProperty");
 	uint32_t len;
 	switch (name.type)
@@ -1459,6 +1480,7 @@ static struct PP_Var PPP_Class_Call(void* object,struct PP_Var name,uint32_t arg
 	ppPluginInstance* instance = ((ppExtScriptObject*)object)->getInstance();
 	
 	setTLSSys(((ppExtScriptObject*)object)->getSystemState());
+	setTLSWorker(((ppExtScriptObject*)object)->getSystemState()->worker);
 	uint32_t len;
 	ExtIdentifier method_name;
 	switch (name.type)
@@ -1938,7 +1960,8 @@ void ppPluginEngineData::DoSwapBuffers()
 
 void ppPluginEngineData::InitOpenGL()
 {
-	
+	//TODO implement nanoVG renderer using EngineData openGL methods
+	//initNanoVG();
 }
 
 void ppPluginEngineData::DeinitOpenGL()
@@ -1965,6 +1988,23 @@ tiny_string ppPluginEngineData::getGLDriverInfo()
 	return res;
 }
 
+void ppPluginEngineData::getGlCompressedTextureFormats()
+{
+	int32_t numformats;
+	g_gles2_interface->GetIntegerv(instance->m_graphics,GL_NUM_COMPRESSED_TEXTURE_FORMATS,&numformats);
+	if (numformats == 0)
+		return;
+	int32_t* formats = new int32_t[numformats];
+	g_gles2_interface->GetIntegerv(instance->m_graphics,GL_COMPRESSED_TEXTURE_FORMATS,formats);
+	for (int32_t i = 0; i < numformats; i++)
+	{
+		LOG(LOG_INFO,"OpenGL supported compressed texture format:"<<hex<<formats[i]);
+		if (formats[i] == GL_COMPRESSED_RGBA_S3TC_DXT5_EXT)
+			compressed_texture_formats.push_back(TEXTUREFORMAT_COMPRESSED::DXT5);
+	}
+	delete [] formats;
+}
+
 void ppPluginEngineData::exec_glUniform1f(int location, float v0)
 {
 	g_gles2_interface->Uniform1f(instance->m_graphics,location,v0);
@@ -2417,8 +2457,8 @@ void ppPluginEngineData::exec_glSetTexParameters(int32_t lodbias, uint32_t dimen
 			g_gles2_interface->TexParameteri(instance->m_graphics,dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR_MIPMAP_NEAREST);
 			break;
 	}
-	g_gles2_interface->TexParameteri(instance->m_graphics,dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP_TO_EDGE);
-	g_gles2_interface->TexParameteri(instance->m_graphics,dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP_TO_EDGE);
+	g_gles2_interface->TexParameteri(instance->m_graphics,dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap&1 ? GL_REPEAT : GL_CLAMP_TO_EDGE);
+	g_gles2_interface->TexParameteri(instance->m_graphics,dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap&2 ? GL_REPEAT : GL_CLAMP_TO_EDGE);
 	if (lodbias != 0)
 		LOG(LOG_NOT_IMPLEMENTED,"Context3D: GL_TEXTURE_LOD_BIAS not available for PPAPI");
 	//g_gles2_interface->TexParameterf(instance->m_graphics,dimension ? GL_TEXTURE_CUBE_MAP : GL_TEXTURE_2D, GL_TEXTURE_LOD_BIAS,(float)(lodbias)/8.0);
@@ -2432,6 +2472,50 @@ void ppPluginEngineData::exec_glTexImage2D_GL_TEXTURE_2D_GL_UNSIGNED_INT_8_8_8_8
 {
 	g_gles2_interface->TexImage2D(instance->m_graphics,GL_TEXTURE_2D, level, GL_RGBA, width, height, border, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_HOST, pixels);
 }
+void ppPluginEngineData::exec_glTexImage2D_GL_TEXTURE_2D(int32_t level, int32_t width, int32_t height, int32_t border, void* pixels, TEXTUREFORMAT format, TEXTUREFORMAT_COMPRESSED compressedformat, uint32_t compressedImageSize)
+{
+	switch (format)
+	{
+		case TEXTUREFORMAT::BGRA:
+			g_gles2_interface->TexImage2D(instance->m_graphics,GL_TEXTURE_2D, level, GL_RGBA, width, height, border, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
+			break;
+		case TEXTUREFORMAT::BGR:
+			for (int i = 0; i < width*height*3; i += 3) {
+				uint8_t t = ((uint8_t*)pixels)[i];
+				((uint8_t*)pixels)[i] = ((uint8_t*)pixels)[i+2];
+				((uint8_t*)pixels)[i+2] = t;
+			}
+			g_gles2_interface->TexImage2D(instance->m_graphics,GL_TEXTURE_2D, level, GL_RGB, width, height, border, GL_RGB, GL_UNSIGNED_BYTE, pixels);
+			break;
+		case TEXTUREFORMAT::BGRA_PACKED:
+			g_gles2_interface->TexImage2D(instance->m_graphics,GL_TEXTURE_2D, level, GL_RGBA, width, height, border, GL_RGBA, GL_UNSIGNED_SHORT_4_4_4_4, pixels);
+			break;
+		case TEXTUREFORMAT::BGR_PACKED:
+			LOG(LOG_NOT_IMPLEMENTED,"textureformat BGR_PACKED for opengl es");
+			g_gles2_interface->TexImage2D(instance->m_graphics,GL_TEXTURE_2D, level, GL_RGB, width, height, border, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, pixels);
+			break;
+		case TEXTUREFORMAT::COMPRESSED:
+		case TEXTUREFORMAT::COMPRESSED_ALPHA:
+		{
+			switch (compressedformat)
+			{
+				case TEXTUREFORMAT_COMPRESSED::DXT5:
+					g_gles2_interface->CompressedTexImage2D(instance->m_graphics,GL_TEXTURE_2D, level, GL_COMPRESSED_RGBA_S3TC_DXT5_EXT, width, height, border, compressedImageSize, pixels);
+					break;
+				default:
+					LOG(LOG_NOT_IMPLEMENTED,"upload texture in compressed format "<<compressedformat);
+					break;
+			}
+			break;
+		}
+		case TEXTUREFORMAT::RGBA_HALF_FLOAT:
+			LOG(LOG_NOT_IMPLEMENTED,"upload texture in format "<<format);
+			break;
+		default:
+			LOG(LOG_ERROR,"invalid format for upload texture:"<<format);
+			break;
+	}
+}
 
 void ppPluginEngineData::exec_glDrawBuffer_GL_BACK()
 {
@@ -2482,6 +2566,7 @@ void ppPluginEngineData::exec_glGetIntegerv_GL_MAX_TEXTURE_SIZE(int32_t* data)
 {
 	g_gles2_interface->GetIntegerv(instance->m_graphics,GL_MAX_TEXTURE_SIZE,data);
 }
+
 void ppPluginEngineData::exec_glGenerateMipmap_GL_TEXTURE_2D()
 {
 	g_gles2_interface->GenerateMipmap(instance->m_graphics,GL_TEXTURE_2D);
@@ -2514,12 +2599,21 @@ void ppPluginEngineData::exec_glScissor(int32_t x, int32_t y, int32_t width, int
 	g_gles2_interface->Enable(instance->m_graphics,GL_SCISSOR_TEST);
 	g_gles2_interface->Scissor(instance->m_graphics,x,y,width,height);
 }
+void ppPluginEngineData::exec_glDisable_GL_SCISSOR_TEST()
+{
+	g_gles2_interface->Disable(instance->m_graphics,GL_SCISSOR_TEST);
+}
 
 void ppPluginEngineData::exec_glColorMask(bool red, bool green, bool blue, bool alpha)
 {
 	g_gles2_interface->ColorMask(instance->m_graphics,red,green,blue,alpha);
 }
 
+void ppPluginEngineData::exec_glStencilFunc_GL_ALWAYS()
+{
+	g_gles2_interface->StencilFunc(instance->m_graphics,GL_ALWAYS, 0, 0xff);
+}
+
 void audio_callback(void* sample_buffer,uint32_t buffer_size_in_bytes,PP_TimeDelta latency,void* user_data)
 {
 	AudioStream *s = (AudioStream*)user_data;
@@ -2529,7 +2623,7 @@ void audio_callback(void* sample_buffer,uint32_t buffer_size_in_bytes,PP_TimeDel
 	uint32_t readcount = 0;
 	while (readcount < buffer_size_in_bytes)
 	{
-		uint32_t ret = s->getDecoder()->copyFrame((int16_t *)(((unsigned char*)sample_buffer)+readcount), buffer_size_in_bytes-readcount);
+		uint32_t ret = s->getDecoder()->copyFrameS16((int16_t *)(((unsigned char*)sample_buffer)+readcount), buffer_size_in_bytes-readcount);
 		if (!ret)
 			break;
 		readcount += ret;
@@ -2537,9 +2631,11 @@ void audio_callback(void* sample_buffer,uint32_t buffer_size_in_bytes,PP_TimeDel
 	if (s->getVolume() != 1.0)
 	{
 		int16_t *p = (int16_t *)sample_buffer;
+		int curpanning=0;
 		for (uint32_t i = 0; i < readcount/2; i++)
 		{
-			*p = (*p)*s->getVolume();
+			*p = (*p)*s->getVolume()*s->getPanning()[curpanning];
+			curpanning =1-curpanning;
 			p++;
 		}
 	}
@@ -2563,10 +2659,6 @@ void ppPluginEngineData::audio_StreamPause(int channel, bool dopause)
 		g_audio_interface->StartPlayback(channel);
 }
 
-void ppPluginEngineData::audio_StreamSetVolume(int channel, double volume)
-{
-}
-
 void ppPluginEngineData::audio_StreamDeinit(int channel)
 {
 	g_audio_interface->StopPlayback(channel);
@@ -2579,11 +2671,11 @@ bool ppPluginEngineData::audio_ManagerInit()
 	return audioconfig != 0;
 }
 
-void ppPluginEngineData::audio_ManagerCloseMixer()
+void ppPluginEngineData::audio_ManagerCloseMixer(AudioManager* manager)
 {
 }
 
-bool ppPluginEngineData::audio_ManagerOpenMixer()
+bool ppPluginEngineData::audio_ManagerOpenMixer(AudioManager* manager)
 {
 	return true;
 }
@@ -2598,6 +2690,11 @@ int ppPluginEngineData::audio_getSampleRate()
 	return PP_AUDIOSAMPLERATE_44100;
 }
 
+bool ppPluginEngineData::audio_useFloatSampleFormat()
+{
+	return false;
+}
+
 uint8_t* ppPluginEngineData::getFontPixelBuffer(int32_t externalressource,int width,int height)
 {
 	uint8_t* data = new uint8_t[width*height*sizeof(uint32_t)];
@@ -2606,7 +2703,7 @@ uint8_t* ppPluginEngineData::getFontPixelBuffer(int32_t externalressource,int wi
 	return data;
 }
 
-int32_t ppPluginEngineData::setupFontRenderer(const TextData &_textData,float a, bool smoothing)
+int32_t ppPluginEngineData::setupFontRenderer(const TextData &_textData,float a, SMOOTH_MODE smoothing)
 {
 	PP_BrowserFont_Trusted_Description desc;
 	desc.face = g_var_interface->VarFromUtf8(_textData.font.raw_buf(),_textData.font.numBytes());
@@ -2623,7 +2720,7 @@ int32_t ppPluginEngineData::setupFontRenderer(const TextData &_textData,float a,
 	PP_Size size = PP_MakeSize(_textData.width, _textData.height);
 	
 	PP_BrowserFont_Trusted_TextRun text;
-	text.text = g_var_interface->VarFromUtf8(_textData.text.raw_buf(),_textData.text.numBytes());
+	text.text = g_var_interface->VarFromUtf8(_textData.getText().raw_buf(),_textData.getText().numBytes());
 	text.override_direction = PP_FALSE;
 	text.rtl = PP_FALSE;
 	
@@ -2633,6 +2730,6 @@ int32_t ppPluginEngineData::setupFontRenderer(const TextData &_textData,float a,
 	PP_Resource font = g_browserfont_interface->Create(instance->m_ppinstance,&desc);
 	if (font == 0)
 		LOG(LOG_ERROR,"couldn't create font:"<<_textData.font);
-	g_browserfont_interface->DrawTextAt(font,image_data,&text,&pos,color,NULL,smoothing ? PP_TRUE : PP_FALSE);
+	g_browserfont_interface->DrawTextAt(font,image_data,&text,&pos,color,nullptr,smoothing != SMOOTH_MODE::SMOOTH_NONE ? PP_TRUE : PP_FALSE);
 	return image_data;
 }
