@@ -22,7 +22,6 @@
 
 #include "compat.h"
 #include <cstddef>
-#include "parsing/tags.h"
 #include "logger.h"
 #include <vector>
 #include <deque>
@@ -31,8 +30,6 @@
 #include "swf.h"
 #include "scripting/abcutils.h"
 #include "scripting/abctypes.h"
-#include "scripting/flash/system/flashsystem.h"
-#include "scripting/toplevel/toplevel.h"
 
 #ifdef LLVM_ENABLED
 namespace llvm {
@@ -107,7 +104,7 @@ public:
 	bool validProfName;
 #endif
 
-	SyntheticFunction::synt_function f;
+	synt_function f;
 	ABCContext* context;
 	method_body_info* body;
 #ifdef LLVM_ENABLED
@@ -210,7 +207,7 @@ friend class method_info;
 private:
 	bool scriptsdeclared;
 public:
-	_R<RootMovieClip> root;
+	RootMovieClip* root;
 
 	method_info* get_method(unsigned int m);
 	uint32_t getString(unsigned int s) const;
@@ -289,7 +286,7 @@ public:
 			case 0x12: //RTQNameLA
 				return 2;
 			default:
-				LOG(LOG_ERROR,_("getMultinameRTData not yet implemented for this kind ") << m->kind);
+				LOG(LOG_ERROR,"getMultinameRTData not yet implemented for this kind " << m->kind);
 				throw UnsupportedException("kind not implemented for getMultinameRTData");
 		}
 	}
@@ -297,7 +294,7 @@ public:
 	multiname* getMultiname(unsigned int m, call_context* th);
 	multiname* getMultinameImpl(asAtom& rt1, ASObject* rt2, unsigned int m, bool isrefcounted = true);
 	void buildInstanceTraits(ASObject* obj, int class_index);
-	ABCContext(_R<RootMovieClip> r, std::istream& in, ABCVm* vm) DLL_PUBLIC;
+	ABCContext(RootMovieClip* r, std::istream& in, ABCVm* vm) DLL_PUBLIC;
 	~ABCContext();
 	void declareScripts();
 	void exec(bool lazy);
@@ -325,6 +322,7 @@ private:
 	enum STATUS { CREATED=0, STARTED, TERMINATED };
 	STATUS status;
 	bool isIdle;
+	bool canFlushInvalidationQueue;
 
 	void registerClassesFlashAccessibility(Global* builtin);
 	void registerClassesFlashConcurrent(Global* builtin);
@@ -359,55 +357,6 @@ private:
 	//If you change a definition here, update the opcode_table_* entry in abc_codesynth
 	static ASObject *hasNext(ASObject* obj, ASObject* cur_index); 
 	static bool hasNext2(call_context* th, int n, int m); 
-	template<class T>
-	static void loadIntN(call_context* th)
-	{
-		RUNTIME_STACK_POP_CREATE(th,arg1);
-		uint32_t addr=asAtomHandler::toUInt(*arg1);
-		ApplicationDomain* appDomain = th->mi->context->root->applicationDomain.getPtr();
-		T ret=appDomain->readFromDomainMemory<T>(addr);
-		ASATOM_DECREF_POINTER(arg1);
-		RUNTIME_STACK_PUSH(th,asAtomHandler::fromInt(ret));
-	}
-	template<class T>
-	static void storeIntN(call_context* th)
-	{
-		RUNTIME_STACK_POP_CREATE(th,arg1);
-		RUNTIME_STACK_POP_CREATE(th,arg2);
-		uint32_t addr=asAtomHandler::toUInt(*arg1);
-		ASATOM_DECREF_POINTER(arg1);
-		int32_t val=asAtomHandler::toInt(*arg2);
-		ASATOM_DECREF_POINTER(arg2);
-		ApplicationDomain* appDomain = th->mi->context->root->applicationDomain.getPtr();
-		appDomain->writeToDomainMemory<T>(addr, val);
-	}
-	template<class T>
-	static FORCE_INLINE void loadIntN(call_context* th,asAtom& ret, asAtom& arg1)
-	{
-		uint32_t addr=asAtomHandler::toUInt(arg1);
-		ByteArray* dm = th->mi->context->root->applicationDomain->currentDomainMemory;
-		if(dm->getLength() < (addr+sizeof(T)))
-			throwError<RangeError>(kInvalidRangeError);
-		ret = asAtomHandler::fromInt(*reinterpret_cast<T*>(dm->getBufferNoCheck()+addr));
-	}
-	template<class T>
-	static FORCE_INLINE void storeIntN(call_context* th, asAtom& arg1, asAtom& arg2)
-	{
-		uint32_t addr=asAtomHandler::toUInt(arg1);
-		int32_t val=asAtomHandler::toInt(arg2);
-		ByteArray* dm = th->mi->context->root->applicationDomain->currentDomainMemory;
-		if(dm->getLength() < (addr+sizeof(T)))
-			throwError<RangeError>(kInvalidRangeError);
-		*reinterpret_cast<T*>(dm->getBufferNoCheck()+addr)=val;
-	}
-	static void loadFloat(call_context* th);
-	static void loadFloat(call_context* th,asAtom& ret, asAtom& arg1);
-	static void loadDouble(call_context* th);
-	static void loadDouble(call_context* th,asAtom& ret, asAtom& arg1);
-	static void storeFloat(call_context* th);
-	static void storeFloat(call_context* th, asAtom& arg1, asAtom& arg2);
-	static void storeDouble(call_context* th);
-	static void storeDouble(call_context* th, asAtom& arg1, asAtom& arg2);
 
 	static void callStatic(call_context* th, int n, int m, method_info** called_mi, bool keepReturn);
 	static void callSuper(call_context* th, int n, int m, method_info** called_mi, bool keepReturn);
@@ -444,12 +393,12 @@ private:
 	static bool ifStrictNE(ASObject*, ASObject*); 
 	static bool ifFalse(ASObject*); 
 	static bool ifTrue(ASObject*); 
-	static ASObject* getSlot(ASObject* th, int n); 
-	static void setSlot(ASObject*, ASObject*, int n); 
+	static ASObject* getSlot(call_context* th, ASObject* obj, int n);
+	static void setSlot(call_context *th, ASObject*, ASObject*, int n);
 	static void kill(int n); 
-	static ASObject* pushString(call_context* th, int n); 
-	static bool getLex(call_context* th, int n); 
-	static bool getLex_multiname(call_context* th, multiname* n, uint32_t localresult); 
+	static ASObject* pushString(call_context* th, int n);
+	static bool getLex(call_context* th, int n);
+	static bool getLex_multiname(call_context* th, multiname* n, uint32_t localresult);
 	static ASObject* getScopeObject(call_context* th, int n); 
 	static bool deleteProperty(ASObject* obj, multiname* name);
 	static void initProperty(ASObject* obj, ASObject* val, multiname* name);
@@ -589,12 +538,13 @@ private:
 	//Synchronization
 	Mutex event_queue_mutex;
 	Cond sem_event_cond;
+	Mutex deletable_objects_mutex;
 
 	//Event handling
 	volatile bool shuttingdown;
 	typedef std::pair<_NR<EventDispatcher>,_R<Event>> eventType;
 	std::deque<eventType, reporter_allocator<eventType>> events_queue;
-	std::deque<eventType, reporter_allocator<eventType>> idleevents_queue;
+	std::list<eventType, reporter_allocator<eventType>> idleevents_queue;
 	void handleEvent(std::pair<_NR<EventDispatcher>,_R<Event> > e);
 	void handleFrontEvent();
 	void signalEventWaiters();
@@ -604,8 +554,8 @@ private:
 	//Profiling support
 	static uint64_t profilingCheckpoint(uint64_t& startTime);
 
-	static void checkPropertyException(ASObject *obj, multiname *name, asAtom &prop);
-	static void checkPropertyExceptionInteger(ASObject *obj, int index, asAtom &prop);
+	static bool checkPropertyException(ASObject *obj, multiname *name, asAtom &prop);
+	static bool checkPropertyExceptionInteger(ASObject *obj, int index, asAtom &prop);
 
 	// The base to assign to the next loaded context
 	ATOMIC_INT32(nextNamespaceBase);
@@ -625,7 +575,15 @@ private:
 	static void abc_ifnlt_constant_local(call_context* context);
 	static void abc_ifnlt_local_local(call_context* context);
 	static void abc_ifnle(call_context* context);
+	static void abc_ifnle_constant_constant(call_context* context);
+	static void abc_ifnle_local_constant(call_context* context);
+	static void abc_ifnle_constant_local(call_context* context);
+	static void abc_ifnle_local_local(call_context* context);
 	static void abc_ifngt(call_context* context);
+	static void abc_ifngt_constant_constant(call_context* context);
+	static void abc_ifngt_local_constant(call_context* context);
+	static void abc_ifngt_constant_local(call_context* context);
+	static void abc_ifngt_local_local(call_context* context);
 	static void abc_ifnge(call_context* context);
 	static void abc_ifnge_constant_constant(call_context* context);
 	static void abc_ifnge_local_constant(call_context* context);
@@ -684,6 +642,8 @@ private:
 	static void abc_ifstrictne_constant_local(call_context* context);
 	static void abc_ifstrictne_local_local(call_context* context);
 	static void abc_lookupswitch(call_context* context);
+	static void abc_lookupswitch_constant(call_context* context);
+	static void abc_lookupswitch_local(call_context* context);
 	static void abc_pushwith(call_context* context);
 	static void abc_popscope(call_context* context);
 	static void abc_nextname(call_context* context);
@@ -692,6 +652,14 @@ private:
 	static void abc_pushnull(call_context* context);// 0x20
 	static void abc_pushundefined(call_context* context);
 	static void abc_nextvalue(call_context* context);
+	static void abc_nextvalue_constant_constant(call_context* context);
+	static void abc_nextvalue_local_constant(call_context* context);
+	static void abc_nextvalue_constant_local(call_context* context);
+	static void abc_nextvalue_local_local(call_context* context);
+	static void abc_nextvalue_constant_constant_localresult(call_context* context);
+	static void abc_nextvalue_local_constant_localresult(call_context* context);
+	static void abc_nextvalue_constant_local_localresult(call_context* context);
+	static void abc_nextvalue_local_local_localresult(call_context* context);
 	static void abc_pushbyte(call_context* context);
 	static void abc_pushshort(call_context* context);
 	static void abc_pushtrue(call_context* context);
@@ -717,6 +685,8 @@ private:
 	static void abc_pushScope_local(call_context* context);// 0x30
 	static void abc_pushnamespace(call_context* context);
 	static void abc_hasnext2(call_context* context);
+	static void	abc_hasnext2_localresult(call_context* context);
+	static void	abc_hasnext2_iftrue(call_context* context);
 	static void abc_li8(call_context* context);
 	static void abc_li8_constant(call_context* context);
 	static void abc_li8_local(call_context* context);
@@ -780,6 +750,18 @@ private:
 
 	static void abc_newfunction(call_context* context);// 0x40
 	static void abc_call(call_context* context);
+	static void abc_callvoid_constant_constant(call_context* context);
+	static void abc_callvoid_local_constant(call_context* context);
+	static void abc_callvoid_constant_local(call_context* context);
+	static void abc_callvoid_local_local(call_context* context);
+	static void abc_call_constant_constant(call_context* context);
+	static void abc_call_local_constant(call_context* context);
+	static void abc_call_constant_local(call_context* context);
+	static void abc_call_local_local(call_context* context);
+	static void abc_call_constant_constant_localresult(call_context* context);
+	static void abc_call_local_constant_localresult(call_context* context);
+	static void abc_call_constant_local_localresult(call_context* context);
+	static void abc_call_local_local_localresult(call_context* context);
 	static void abc_construct(call_context* context);
 	static void abc_construct_constant(call_context* context);
 	static void abc_construct_local(call_context* context);
@@ -838,8 +820,20 @@ private:
 	
 	
 	static void abc_sxi1(call_context* context); // 0x50
+	static void abc_sxi1_constant(call_context* context);
+	static void abc_sxi1_local(call_context* context);
+	static void abc_sxi1_constant_localresult(call_context* context);
+	static void abc_sxi1_local_localresult(call_context* context);
 	static void abc_sxi8(call_context* context);
+	static void abc_sxi8_constant(call_context* context);
+	static void abc_sxi8_local(call_context* context);
+	static void abc_sxi8_constant_localresult(call_context* context);
+	static void abc_sxi8_local_localresult(call_context* context);
 	static void abc_sxi16(call_context* context);
+	static void abc_sxi16_constant(call_context* context);
+	static void abc_sxi16_local(call_context* context);
+	static void abc_sxi16_constant_localresult(call_context* context);
+	static void abc_sxi16_local_localresult(call_context* context);
 	static void abc_constructgenerictype(call_context* context);
 	static void abc_newobject(call_context* context);
 	static void abc_newarray(call_context* context);
@@ -966,6 +960,10 @@ private:
 	static void abc_checkfilter(call_context* context);
 
 	static void abc_coerce(call_context* context); // 0x80
+	static void abc_coerce_constant(call_context* context);
+	static void abc_coerce_local(call_context* context);
+	static void abc_coerce_constant_localresult(call_context* context);
+	static void abc_coerce_local_localresult(call_context* context);
 	static void abc_coerce_a(call_context* context);
 	static void abc_coerce_s(call_context* context);
 	static void abc_astype(call_context* context);
@@ -1218,8 +1216,10 @@ private:
 	static void abc_decrement_i(call_context* context);
 	static void abc_inclocal_i(call_context* context);
 	static void abc_inclocal_i_optimized(call_context* context);
+	static void abc_inclocal_i_postfix(call_context* context);
 	static void abc_declocal_i(call_context* context);
 	static void abc_declocal_i_optimized(call_context* context);
+	static void abc_declocal_i_postfix(call_context* context);
 	static void abc_negate_i(call_context* context);
 	static void abc_add_i(call_context* context);
 	static void abc_add_i_constant_constant(call_context* context);
@@ -1236,7 +1236,32 @@ private:
 	static void abc_add_i_local_local_setslotnocoerce(call_context* context);
 	
 	static void abc_subtract_i(call_context* context);
+	static void abc_subtract_i_constant_constant(call_context* context);
+	static void abc_subtract_i_local_constant(call_context* context);
+	static void abc_subtract_i_constant_local(call_context* context);
+	static void abc_subtract_i_local_local(call_context* context);
+	static void abc_subtract_i_constant_constant_localresult(call_context* context);
+	static void abc_subtract_i_local_constant_localresult(call_context* context);
+	static void abc_subtract_i_constant_local_localresult(call_context* context);
+	static void abc_subtract_i_local_local_localresult(call_context* context);
+	static void abc_subtract_i_constant_constant_setslotnocoerce(call_context* context);
+	static void abc_subtract_i_local_constant_setslotnocoerce(call_context* context);
+	static void abc_subtract_i_constant_local_setslotnocoerce(call_context* context);
+	static void abc_subtract_i_local_local_setslotnocoerce(call_context* context);
+
 	static void abc_multiply_i(call_context* context);
+	static void abc_multiply_i_constant_constant(call_context* context);
+	static void abc_multiply_i_local_constant(call_context* context);
+	static void abc_multiply_i_constant_local(call_context* context);
+	static void abc_multiply_i_local_local(call_context* context);
+	static void abc_multiply_i_constant_constant_localresult(call_context* context);
+	static void abc_multiply_i_local_constant_localresult(call_context* context);
+	static void abc_multiply_i_constant_local_localresult(call_context* context);
+	static void abc_multiply_i_local_local_localresult(call_context* context);
+	static void abc_multiply_i_constant_constant_setslotnocoerce(call_context* context);
+	static void abc_multiply_i_local_constant_setslotnocoerce(call_context* context);
+	static void abc_multiply_i_constant_local_setslotnocoerce(call_context* context);
+	static void abc_multiply_i_local_local_setslotnocoerce(call_context* context);
 
 	static void abc_getlocal_0(call_context* context); // 0xd0
 	static void abc_getlocal_1(call_context* context);
@@ -1323,7 +1348,6 @@ private:
 
 public:
 	static abc_function abcfunctions[];
-	call_context* currentCallContext;
 
 	MemoryAccount* vmDataMemory;
 
@@ -1354,12 +1378,10 @@ public:
 	void registerClassesAVM1();
 	static int Run(void* d);
 	static void executeFunction(call_context* context);
-#ifndef NDEBUG
 	static void dumpOpcodeCounters(uint32_t threshhold);
 	static void clearOpcodeCounters();
-#endif
 	
-	static void preloadFunction(SyntheticFunction *function);
+	static void preloadFunction(SyntheticFunction *function,ASWorker* wrk);
 	static ASObject* executeFunctionFast(const SyntheticFunction* function, call_context* context, ASObject *caller);
 	static void optimizeFunction(SyntheticFunction* function);
 	static void verifyBranch(std::set<uint32_t>& pendingBlock,std::map<uint32_t,BasicBlock>& basicBlocks,
@@ -1377,9 +1399,9 @@ public:
 			const std::vector<InferenceData>& scopeStack, const multiname* name, InferenceData& inferredData);
 	static const Type* getLocalType(const SyntheticFunction* f, unsigned localIndex);
 
-	bool addEvent(_NR<EventDispatcher>,_R<Event> ) DLL_PUBLIC;
+	bool addEvent(_NR<EventDispatcher>,_R<Event>, bool isGlobalMessage=false) DLL_PUBLIC;
 	bool prependEvent(_NR<EventDispatcher>, _R<Event> , bool force=false) DLL_PUBLIC;
-	void addIdleEvent(_NR<EventDispatcher>,_R<Event> ) DLL_PUBLIC;
+	bool addIdleEvent(_NR<EventDispatcher>, _R<Event> , bool removeprevious=false) DLL_PUBLIC;
 	int getEventQueueSize();
 	void shutdown();
 	bool hasEverStarted() const { return status!=CREATED; }
@@ -1391,83 +1413,13 @@ public:
 	static _R<ApplicationDomain> getCurrentApplicationDomain(call_context* th);
 	static _R<SecurityDomain> getCurrentSecurityDomain(call_context* th);
 
-	/* The current recursion level. Each call increases this by one,
-	 * each return from a call decreases this. */
-	uint32_t cur_recursion;
-	struct stacktrace_entry
-	{
-		asAtom object;
-		uint32_t name;
-		void set(asAtom o, uint32_t n) { object=o; name=n; }
-	};
-	stacktrace_entry* stacktrace;
-	FORCE_INLINE call_context* incStack(asAtom o, uint32_t f)
-	{
-		if(USUALLY_FALSE(cur_recursion == limits.max_recursion))
-		{
-			throwStackOverflow();
-		}
-		stacktrace[cur_recursion].set(o,f);
-		++cur_recursion; //increment current recursion depth
-		return currentCallContext;
-	}
-	FORCE_INLINE void decStack(call_context* saved_cc)
-	{
-		currentCallContext = saved_cc;
-		--cur_recursion; //decrement current recursion depth
-	}
-	void throwStackOverflow();
-
-	struct abc_limits {
-		/* maxmium number of recursion allowed. See ScriptLimitsTag */
-		uint32_t max_recursion;
-		/* maxmium number of seconds for script execution. See ScriptLimitsTag */
-		uint32_t script_timeout;
-	} limits;
-
 	uint32_t getAndIncreaseNamespaceBase(uint32_t nsNum);
 
-	tiny_string getDefaultXMLNamespace();
-	uint32_t getDefaultXMLNamespaceID();
-
 	bool buildClassAndBindTag(const std::string& s, DictionaryTag* t, Class_inherit *derived_cls=nullptr);
 	void checkExternalCallEvent() DLL_PUBLIC;
 	void setIdle(bool isidle) { isIdle = isidle; }
 };
 
-class DoABCTag: public ControlTag
-{
-private:
-	ABCContext* context;
-public:
-	DoABCTag(RECORDHEADER h, std::istream& in);
-	TAGTYPE getType() const{ return ABC_TAG; }
-	void execute(RootMovieClip* root) const;
-};
-
-class DoABCDefineTag: public ControlTag
-{
-private:
-	UI32_SWF Flags;
-	STRING Name;
-	ABCContext* context;
-public:
-	DoABCDefineTag(RECORDHEADER h, std::istream& in);
-	TAGTYPE getType() const{ return ABC_TAG; }
-	void execute(RootMovieClip* root) const;
-};
-
-class SymbolClassTag: public ControlTag
-{
-private:
-	UI16_SWF NumSymbols;
-	std::vector<UI16_SWF> Tags;
-	std::vector<STRING> Names;
-public:
-	SymbolClassTag(RECORDHEADER h, std::istream& in);
-	TAGTYPE getType() const{ return SYMBOL_CLASS_TAG; }
-	void execute(RootMovieClip* root) const;
-};
 
 inline ABCVm* getVm(SystemState* sys)
 {
