@@ -60,7 +60,6 @@ struct call_context
 	int32_t argarrayposition; // position of argument array in locals ( -1 if no argument array needed)
 	asAtom* scope_stack;
 	bool* scope_stack_dynamic;
-	std::vector<SyntheticFunction*> dynamicfunctions;
 	asAtom** localslots;
 	method_info* mi;
 	/* This is the function's inClass that is currently executing. It is used
@@ -68,17 +67,19 @@ struct call_context
 	 * */
 	Class_base* inClass;
 	SystemState* sys;
+	class ASWorker* worker;
 	/* Current namespace set by 'default xml namespace = ...'.
 	 * Defaults to empty string according to ECMA-357 13.1.1.1
 	 */
 	uint32_t defaultNamespaceUri;
+	ASObject* exceptionthrown;
 	call_context(method_info* _mi):
 		locals(nullptr),stack(nullptr),
 		stackp(nullptr),exec_pos(nullptr),
 		max_stackp(nullptr),
 		parent_scope_stack(nullptr),curr_scope_stack(0),argarrayposition(-1),
 		scope_stack(nullptr),scope_stack_dynamic(nullptr),localslots(nullptr),mi(_mi),
-		inClass(nullptr),sys(nullptr),defaultNamespaceUri(0)
+		inClass(nullptr),sys(nullptr),worker(nullptr),defaultNamespaceUri(0),exceptionthrown(nullptr)
 	{
 	}
 	static void handleError(int errorcode);
@@ -91,6 +92,34 @@ struct call_context
 		}
 	}
 };
+typedef ASObject* (*synt_function)(call_context* cc);
+
+class AVM1context
+{
+friend class AVM1Function;
+private:
+	std::vector<uint32_t> avm1strings;
+public:
+	AVM1context():keepLocals(true) {}
+	void AVM1ClearConstants()
+	{
+		avm1strings.clear();
+	}
+	void AVM1AddConstant(uint32_t nameID)
+	{
+		avm1strings.push_back(nameID);
+	}
+	asAtom AVM1GetConstant(uint16_t index)
+	{
+		if (index < avm1strings.size())
+			return asAtomHandler::fromStringID(avm1strings[index]);
+		LOG(LOG_ERROR,"AVM1:constant not found in pool:"<<index<<" "<<avm1strings.size());
+		return asAtomHandler::undefinedAtom;
+	}
+	bool keepLocals;
+};
+
+
 #define CONTEXT_GETLOCAL(context,pos) \
 	(*(context->localslots[pos]))
 
@@ -117,23 +146,23 @@ else asAtomHandler::set(*(context->stackp++),val)
 	context->stackp-=n; \
 	asAtom* ret= context->stackp;
 
-#define RUNTIME_STACK_POP_ASOBJECT(context,ret, sys) \
-	if(USUALLY_TRUE(context->stackp!=context->stack)) ret=asAtomHandler::toObject(*(--context->stackp),sys); \
+#define RUNTIME_STACK_POP_ASOBJECT(context,ret) \
+	if(USUALLY_TRUE(context->stackp!=context->stack)) ret=asAtomHandler::toObject(*(--context->stackp),context->worker); \
 	else context->handleError(kStackUnderflowError);
 
-#define RUNTIME_STACK_POP_CREATE_ASOBJECT(context,ret, sys) \
+#define RUNTIME_STACK_POP_CREATE_ASOBJECT(context,ret) \
 	if(USUALLY_FALSE(context->stackp==context->stack)) \
 		context->handleError(kStackUnderflowError); \
-	ASObject* ret = asAtomHandler::toObject(*(--context->stackp),sys);
+	ASObject* ret = asAtomHandler::toObject(*(--context->stackp),context->worker);
 
 #define RUNTIME_STACK_PEEK(context,ret) \
-	ret= USUALLY_TRUE(context->stackp != context->stack) ? (context->stackp-1) : NULL; 
+	ret= USUALLY_TRUE(context->stackp != context->stack) ? (context->stackp-1) : nullptr; 
 
-#define RUNTIME_STACK_PEEK_ASOBJECT(context,ret, sys) \
-	ret= USUALLY_TRUE(context->stackp != context->stack) ? asAtomHandler::toObject(*(context->stackp-1),sys) : NULL; 
+#define RUNTIME_STACK_PEEK_ASOBJECT(context,ret) \
+	ret= USUALLY_TRUE(context->stackp != context->stack) ? asAtomHandler::toObject(*(context->stackp-1),context->worker) : nullptr; 
 
 #define RUNTIME_STACK_PEEK_CREATE(context,ret) \
-	asAtom* ret = USUALLY_TRUE(context->stackp != context->stack) ? (context->stackp-1) : NULL; 
+	asAtom* ret = USUALLY_TRUE(context->stackp != context->stack) ? (context->stackp-1) : nullptr; 
 
 #define RUNTIME_STACK_POINTER_CREATE(context,ret) \
 	if(USUALLY_FALSE(context->stackp==context->stack)) \
