@@ -20,6 +20,7 @@
 #include "scripting/avm1/avm1display.h"
 #include "scripting/class.h"
 #include "scripting/argconv.h"
+#include "scripting/flash/net/flashnet.h"
 #include "parsing/tags.h"
 #include "scripting/abc.h"
 #include "backends/security.h"
@@ -30,11 +31,24 @@ using namespace lightspark;
 void AVM1MovieClip::afterConstruction()
 {
 	MovieClip::afterConstruction();
-	if (getSystemState()->getSwfVersion() >= 6)
-	{
-		this->incRef();
-		getVm(getSystemState())->prependEvent(_MR(this),_MR(Class<Event>::getInstanceS(getSystemState(),"load")));
-	}
+	getSystemState()->stage->AVM1AddEventListener(this);
+}
+
+bool AVM1MovieClip::destruct()
+{
+	avm1loader.reset();
+	color.reset();
+	getSystemState()->stage->AVM1RemoveEventListener(this);
+	return MovieClip::destruct();
+}
+
+void AVM1MovieClip::prepareShutdown()
+{
+	if (preparedforshutdown)
+		return;
+	MovieClip::prepareShutdown();
+	if (color)
+		color->prepareShutdown();
 }
 
 void AVM1MovieClip::sinit(Class_base* c)
@@ -45,46 +59,60 @@ void AVM1MovieClip::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("_currentframe","",Class<IFunction>::getFunction(c->getSystemState(),_getCurrentFrame),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("_framesloaded","",Class<IFunction>::getFunction(c->getSystemState(),_getFramesLoaded),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("_name","",Class<IFunction>::getFunction(c->getSystemState(),_getter_name),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("_name","",Class<IFunction>::getFunction(c->getSystemState(),_setter_name),SETTER_METHOD,true);
 	c->setDeclaredMethodByQName("startDrag","",Class<IFunction>::getFunction(c->getSystemState(),startDrag),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("stopDrag","",Class<IFunction>::getFunction(c->getSystemState(),stopDrag),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("attachAudio","",Class<IFunction>::getFunction(c->getSystemState(),attachAudio),NORMAL_METHOD,true);
+}
+
+void AVM1MovieClip::setColor(AVM1Color* c)
+{
+	c->incRef();
+	this->color = _MNR(c);
 }
 
 ASFUNCTIONBODY_ATOM(AVM1MovieClip,startDrag)
 {
 	bool lockcenter;
 	number_t x1, y1, x2, y2;
-	ARG_UNPACK_ATOM (lockcenter,false)(x1,0)(y1,0)(x2,0)(y2,0);
+	ARG_CHECK(ARG_UNPACK(lockcenter,false)(x1,0)(y1,0)(x2,0)(y2,0));
 
 	if (argslen > 1)
 	{
-		Rectangle* rect = Class<Rectangle>::getInstanceS(sys);
+		Rectangle* rect = Class<Rectangle>::getInstanceS(wrk);
 		asAtom fret = asAtomHandler::invalidAtom;
 		asAtom fobj = asAtomHandler::fromObject(rect);
-		asAtom fx1 = asAtomHandler::fromNumber(sys, x1, false);
-		asAtom fy1 = asAtomHandler::fromNumber(sys, y1, false);
-		asAtom fx2 = asAtomHandler::fromNumber(sys, x2, false);
-		asAtom fy2 = asAtomHandler::fromNumber(sys, y2, false);
-		Rectangle::_setLeft(fret,sys,fobj,&fx1,1);
-		Rectangle::_setTop(fret,sys,fobj,&fy1,1);
-		Rectangle::_setRight(fret,sys,fobj,&fx2,1);
-		Rectangle::_setBottom(fret,sys,fobj,&fy2,1);
+		asAtom fx1 = asAtomHandler::fromNumber(wrk, x1, false);
+		asAtom fy1 = asAtomHandler::fromNumber(wrk, y1, false);
+		asAtom fx2 = asAtomHandler::fromNumber(wrk, x2, false);
+		asAtom fy2 = asAtomHandler::fromNumber(wrk, y2, false);
+		Rectangle::_setLeft(fret,wrk,fobj,&fx1,1);
+		Rectangle::_setTop(fret,wrk,fobj,&fy1,1);
+		Rectangle::_setRight(fret,wrk,fobj,&fx2,1);
+		Rectangle::_setBottom(fret,wrk,fobj,&fy2,1);
 	
 		asAtom fargs[2];
 		fargs[0] = asAtomHandler::fromBool(lockcenter);
 		fargs[1] = asAtomHandler::fromObject(rect);
-		Sprite::_startDrag(ret,sys,obj,fargs,2);
+		Sprite::_startDrag(ret,wrk,obj,fargs,2);
 	}
 	else
 	{
 		asAtom fargs[1];
 		fargs[0] = asAtomHandler::fromBool(lockcenter);
-		Sprite::_startDrag(ret,sys,obj,fargs,1);
+		Sprite::_startDrag(ret,wrk,obj,fargs,1);
 	}
 }
 
 ASFUNCTIONBODY_ATOM(AVM1MovieClip,stopDrag)
 {
-	Sprite::_stopDrag(ret,sys,obj,nullptr,0);
+	Sprite::_stopDrag(ret,wrk,obj,nullptr,0);
+}
+
+ASFUNCTIONBODY_ATOM(AVM1MovieClip,attachAudio)
+{
+//	AVM1MovieClip* th=asAtomHandler::as<AVM1MovieClip>(obj);
+	LOG(LOG_NOT_IMPLEMENTED,"AVM1MovieClip.attachAudio");
 }
 
 void AVM1Shape::sinit(Class_base* c)
@@ -96,6 +124,7 @@ void AVM1Shape::sinit(Class_base* c)
 void AVM1SimpleButton::sinit(Class_base* c)
 {
 	SimpleButton::sinit(c);
+	c->isSealed = false;
 	DisplayObject::AVM1SetupMethods(c);
 }
 
@@ -104,7 +133,9 @@ void AVM1Stage::sinit(Class_base* c)
 	// in AVM1 Stage is no DisplayObject and all methods/properties are static
 	CLASS_SETUP_NO_CONSTRUCTOR(c, ASObject, CLASS_SEALED);
 	c->setDeclaredMethodByQName("width","",Class<IFunction>::getFunction(c->getSystemState(),_getStageWidth),GETTER_METHOD,false);
+	c->setDeclaredMethodByQName("width","",Class<IFunction>::getFunction(c->getSystemState(),_setStageWidth),SETTER_METHOD,false);
 	c->setDeclaredMethodByQName("height","",Class<IFunction>::getFunction(c->getSystemState(),_getStageHeight),GETTER_METHOD,false);
+	c->setDeclaredMethodByQName("height","",Class<IFunction>::getFunction(c->getSystemState(),_setStageHeight),SETTER_METHOD,false);
 	c->setDeclaredMethodByQName("displayState","",Class<IFunction>::getFunction(c->getSystemState(),_getDisplayState),GETTER_METHOD,false);
 	c->setDeclaredMethodByQName("displayState","",Class<IFunction>::getFunction(c->getSystemState(),_setDisplayState),SETTER_METHOD,false);
 	c->setDeclaredMethodByQName("scaleMode","",Class<IFunction>::getFunction(c->getSystemState(),_getScaleMode),GETTER_METHOD,false);
@@ -118,42 +149,44 @@ void AVM1Stage::sinit(Class_base* c)
 }
 ASFUNCTIONBODY_ATOM(AVM1Stage,_getDisplayState)
 {
-	ret = asAtomHandler::fromString(sys,sys->stage->displayState);
+	ret = asAtomHandler::fromString(wrk->getSystemState(),wrk->getSystemState()->stage->displayState);
 }
 ASFUNCTIONBODY_ATOM(AVM1Stage,_setDisplayState)
 {
-	ARG_UNPACK_ATOM(sys->stage->displayState);
-	sys->stage->onDisplayState(sys->stage->displayState);
+	ARG_CHECK(ARG_UNPACK(wrk->getSystemState()->stage->displayState));
+	wrk->getSystemState()->stage->onDisplayState(wrk->getSystemState()->stage->displayState);
 }
 ASFUNCTIONBODY_ATOM(AVM1Stage,_getShowMenu)
 {
-	ret = asAtomHandler::fromBool(sys->stage->showDefaultContextMenu);
+	ret = asAtomHandler::fromBool(wrk->getSystemState()->stage->showDefaultContextMenu);
 }
 ASFUNCTIONBODY_ATOM(AVM1Stage,_setShowMenu)
 {
-	ARG_UNPACK_ATOM(sys->stage->showDefaultContextMenu);
+	ARG_CHECK(ARG_UNPACK(wrk->getSystemState()->stage->showDefaultContextMenu));
 }
 ASFUNCTIONBODY_ATOM(AVM1Stage,getAlign)
 {
-	ret = asAtomHandler::fromString(sys,sys->stage->align);
+	ret = asAtomHandler::fromStringID(wrk->getSystemState()->stage->align);
 }
 ASFUNCTIONBODY_ATOM(AVM1Stage,setAlign)
 {
-	tiny_string oldalign=sys->stage->align;
-	ARG_UNPACK_ATOM(sys->stage->align);
-	sys->stage->onAlign(oldalign);
+	uint32_t oldalign=wrk->getSystemState()->stage->align;
+	tiny_string newalign;
+	ARG_CHECK(ARG_UNPACK(newalign));
+	wrk->getSystemState()->stage->align = wrk->getSystemState()->getUniqueStringId(newalign);
+	wrk->getSystemState()->stage->onAlign(oldalign);
 }
 ASFUNCTIONBODY_ATOM(AVM1Stage,addResizeListener)
 {
 	_NR<ASObject> listener;
-	ARG_UNPACK_ATOM(listener,NullRef);
-	sys->stage->AVM1AddResizeListener(listener.getPtr());
+	ARG_CHECK(ARG_UNPACK(listener,NullRef));
+	wrk->getSystemState()->stage->AVM1AddResizeListener(listener.getPtr());
 }
 ASFUNCTIONBODY_ATOM(AVM1Stage,removeResizeListener)
 {
 	_NR<ASObject> listener;
-	ARG_UNPACK_ATOM(listener,NullRef);
-	ret = asAtomHandler::fromBool(sys->stage->AVM1RemoveResizeListener(listener.getPtr()));
+	ARG_CHECK(ARG_UNPACK(listener,NullRef));
+	ret = asAtomHandler::fromBool(wrk->getSystemState()->stage->AVM1RemoveResizeListener(listener.getPtr()));
 }
 
 
@@ -173,11 +206,11 @@ ASFUNCTIONBODY_ATOM(AVM1MovieClipLoader,loadClip)
 	AVM1MovieClipLoader* th=asAtomHandler::as<AVM1MovieClipLoader>(obj);
 	tiny_string strurl;
 	asAtom target = asAtomHandler::invalidAtom;
-	ARG_UNPACK_ATOM (strurl)(target);
-	URLRequest* r = Class<URLRequest>::getInstanceS(sys,strurl);
+	ARG_CHECK(ARG_UNPACK(strurl)(target));
+	URLRequest* r = Class<URLRequest>::getInstanceS(wrk,strurl);
 	DisplayObjectContainer* parent = th->getParent();
 	if (!parent)
-		parent = sys->mainClip;
+		parent = wrk->rootClip.getPtr();
 	DisplayObject* t =nullptr;
 	if (asAtomHandler::isNumeric(target))
 	{
@@ -188,9 +221,15 @@ ASFUNCTIONBODY_ATOM(AVM1MovieClipLoader,loadClip)
 		t = asAtomHandler::getObject(target)->as<DisplayObject>();
 	}
 	else
-		throwError<ArgumentError>(kInvalidArgumentError,"target");
+	{
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"target");
+		return;
+	}
 	if (!t)
-		throwError<ArgumentError>(kInvalidArgumentError,"target");
+	{
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"target");
+		return;
+	}
 	
 	th->loaderInfo = th->getContentLoaderInfo();
 	t->incRef();
@@ -201,8 +240,8 @@ ASFUNCTIONBODY_ATOM(AVM1MovieClipLoader,addListener)
 {
 	AVM1MovieClipLoader* th=asAtomHandler::as<AVM1MovieClipLoader>(obj);
 
-	sys->stage->AVM1AddEventListener(th);
-	ASObject* o = asAtomHandler::toObject(args[0],sys);
+	wrk->getSystemState()->stage->AVM1AddEventListener(th);
+	ASObject* o = asAtomHandler::toObject(args[0],wrk);
 
 	o->incRef();
 	th->listeners.insert(_MR(o));
@@ -212,7 +251,7 @@ ASFUNCTIONBODY_ATOM(AVM1MovieClipLoader,removeListener)
 	AVM1MovieClipLoader* th=asAtomHandler::as<AVM1MovieClipLoader>(obj);
 
 	
-	ASObject* o = asAtomHandler::toObject(args[0],sys);
+	ASObject* o = asAtomHandler::toObject(args[0],wrk);
 
 	th->listeners.erase(_MR(o));
 }
@@ -220,6 +259,7 @@ void AVM1MovieClipLoader::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
 {
 	if (dispatcher == this->getContentLoaderInfo().getPtr())
 	{
+		ASWorker* wrk = getInstanceWorker();
 		auto it = listeners.begin();
 		while (it != listeners.end())
 		{
@@ -230,7 +270,7 @@ void AVM1MovieClipLoader::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
 				m.name_type=multiname::NAME_STRING;
 				m.isAttribute = false;
 				m.name_s_id=getSystemState()->getUniqueStringId("onLoadStart");
-				(*it)->getVariableByMultiname(func,m);
+				(*it)->getVariableByMultiname(func,m,GET_VARIABLE_OPTION::NONE,wrk);
 				if (asAtomHandler::is<AVM1Function>(func))
 				{
 					asAtom ret=asAtomHandler::invalidAtom;
@@ -240,11 +280,12 @@ void AVM1MovieClipLoader::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
 					if (this->getContent())
 					{
 						this->getContent()->incRef();
-						args[0] = asAtomHandler::fromObject(this->getContent().getPtr());
+						args[0] = asAtomHandler::fromObject(this->getContent());
 					}
 					else
 						args[0] = asAtomHandler::undefinedAtom;
 					asAtomHandler::as<AVM1Function>(func)->call(&ret,&obj,args,1);
+					asAtomHandler::as<AVM1Function>(func)->decRef();
 				}
 			}
 			else if (e->type == "init")
@@ -254,7 +295,7 @@ void AVM1MovieClipLoader::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
 				m.name_type=multiname::NAME_STRING;
 				m.isAttribute = false;
 				m.name_s_id=getSystemState()->getUniqueStringId("onLoadInit");
-				(*it)->getVariableByMultiname(func,m);
+				(*it)->getVariableByMultiname(func,m,GET_VARIABLE_OPTION::NONE,wrk);
 				if (asAtomHandler::is<AVM1Function>(func))
 				{
 					asAtom ret=asAtomHandler::invalidAtom;
@@ -263,11 +304,12 @@ void AVM1MovieClipLoader::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
 					if (this->getContent())
 					{
 						this->getContent()->incRef();
-						args[0] = asAtomHandler::fromObject(this->getContent().getPtr());
+						args[0] = asAtomHandler::fromObject(this->getContent());
 					}
 					else
 						args[0] = asAtomHandler::undefinedAtom;
 					asAtomHandler::as<AVM1Function>(func)->call(&ret,&obj,args,1);
+					asAtomHandler::as<AVM1Function>(func)->decRef();
 				}
 			}
 			else if (e->type == "progress")
@@ -277,7 +319,7 @@ void AVM1MovieClipLoader::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
 				m.name_type=multiname::NAME_STRING;
 				m.isAttribute = false;
 				m.name_s_id=getSystemState()->getUniqueStringId("onLoadProgress");
-				(*it)->getVariableByMultiname(func,m);
+				(*it)->getVariableByMultiname(func,m,GET_VARIABLE_OPTION::NONE,wrk);
 				if (asAtomHandler::is<AVM1Function>(func))
 				{
 					ProgressEvent* ev = e->as<ProgressEvent>();
@@ -287,13 +329,14 @@ void AVM1MovieClipLoader::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
 					if (this->getContent())
 					{
 						this->getContent()->incRef();
-						args[0] = asAtomHandler::fromObject(this->getContent().getPtr());
+						args[0] = asAtomHandler::fromObject(this->getContent());
 					}
 					else
 						args[0] = asAtomHandler::undefinedAtom;
 					args[1] = asAtomHandler::fromInt(ev->bytesLoaded);
 					args[2] = asAtomHandler::fromInt(ev->bytesTotal);
 					asAtomHandler::as<AVM1Function>(func)->call(&ret,&obj,args,3);
+					asAtomHandler::as<AVM1Function>(func)->decRef();
 				}
 			}
 			else if (e->type == "complete")
@@ -303,7 +346,7 @@ void AVM1MovieClipLoader::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
 				m.name_type=multiname::NAME_STRING;
 				m.isAttribute = false;
 				m.name_s_id=getSystemState()->getUniqueStringId("onLoadComplete");
-				(*it)->getVariableByMultiname(func,m);
+				(*it)->getVariableByMultiname(func,m,GET_VARIABLE_OPTION::NONE,wrk);
 				if (asAtomHandler::is<AVM1Function>(func))
 				{
 					asAtom ret=asAtomHandler::invalidAtom;
@@ -312,11 +355,12 @@ void AVM1MovieClipLoader::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
 					if (this->getContent())
 					{
 						this->getContent()->incRef();
-						args[0] = asAtomHandler::fromObject(this->getContent().getPtr());
+						args[0] = asAtomHandler::fromObject(this->getContent());
 					}
 					else
 						args[0] = asAtomHandler::undefinedAtom;
 					asAtomHandler::as<AVM1Function>(func)->call(&ret,&obj,args,1);
+					asAtomHandler::as<AVM1Function>(func)->decRef();
 				}
 			}
 			else if (e->type == "ioError")
@@ -326,12 +370,13 @@ void AVM1MovieClipLoader::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
 				m.name_type=multiname::NAME_STRING;
 				m.isAttribute = false;
 				m.name_s_id=getSystemState()->getUniqueStringId("onLoadError");
-				(*it)->getVariableByMultiname(func,m);
+				(*it)->getVariableByMultiname(func,m,GET_VARIABLE_OPTION::NONE,wrk);
 				if (asAtomHandler::is<AVM1Function>(func))
 				{
 					asAtom ret=asAtomHandler::invalidAtom;
 					asAtom obj = asAtomHandler::fromObject(this);
 					asAtomHandler::as<AVM1Function>(func)->call(&ret,&obj,nullptr,0);
+					asAtomHandler::as<AVM1Function>(func)->decRef();
 				}
 			}
 			it++;
@@ -346,6 +391,14 @@ bool AVM1MovieClipLoader::destruct()
 	return Loader::destruct();
 }
 
+void AVM1MovieClipLoader::load(const tiny_string& url, const tiny_string& method, AVM1MovieClip* target)
+{
+	URLRequest* r = Class<URLRequest>::getInstanceS(getInstanceWorker(),url,method);
+	target->incRef();
+	avm1target = _MR(target);
+	loadIntern(r,nullptr);
+}
+
 void AVM1Color::sinit(Class_base* c)
 {
 	CLASS_SETUP(c, ASObject, _constructor, CLASS_FINAL);
@@ -358,7 +411,28 @@ void AVM1Color::sinit(Class_base* c)
 ASFUNCTIONBODY_ATOM(AVM1Color,_constructor)
 {
 	AVM1Color* th=asAtomHandler::as<AVM1Color>(obj);
-	ARG_UNPACK_ATOM(th->target);
+	if(argslen == 0)
+	{
+		createError<ArgumentError>(wrk,kWrongArgumentCountError, "Color", "1", "0");
+		return;
+	}
+	DisplayObject* t = nullptr;
+	if (asAtomHandler::isString(args[0]))
+	{
+		tiny_string s = asAtomHandler::toString(args[0],wrk);
+		t = wrk->rootClip->AVM1GetClipFromPath(s);
+	}
+	else if (asAtomHandler::is<DisplayObject>(args[0]))
+		t = asAtomHandler::as<DisplayObject>(args[0]);
+	if (t)
+	{
+		if (t->is<AVM1MovieClip>())
+			t->as<AVM1MovieClip>()->setColor(th);
+		else
+			LOG(LOG_NOT_IMPLEMENTED,"constructing AVM1Color without MovieClip as target:"<<t->toDebugString());
+		th->target=t->as<AVM1MovieClip>();
+		t->addOwnedObject(th);
+	}
 }
 ASFUNCTIONBODY_ATOM(AVM1Color,getRGB)
 {
@@ -366,7 +440,7 @@ ASFUNCTIONBODY_ATOM(AVM1Color,getRGB)
 	if (th->target && th->target->colorTransform)
 	{
 		asAtom t = asAtomHandler::fromObject(th->target->colorTransform.getPtr());
-		ColorTransform::getColor(ret,sys,t,nullptr,0);
+		ColorTransform::getColor(ret,wrk,t,nullptr,0);
 	}
 	else
 		ret = asAtomHandler::undefinedAtom;
@@ -378,14 +452,16 @@ ASFUNCTIONBODY_ATOM(AVM1Color,setRGB)
 	if (th->target && th->target->colorTransform)
 	{
 		asAtom t = asAtomHandler::fromObject(th->target->colorTransform.getPtr());
-		ColorTransform::setColor(ret,sys,t,args,1);
+		ColorTransform::setColor(ret,wrk,t,args,1);
+		th->target->hasChanged=true;
+		th->target->requestInvalidation(wrk->getSystemState());
 	}
 }
 
 ASFUNCTIONBODY_ATOM(AVM1Color,getTransform)
 {
 	AVM1Color* th=asAtomHandler::as<AVM1Color>(obj);
-	ASObject* o = Class<ASObject>::getInstanceS(sys);
+	ASObject* o = Class<ASObject>::getInstanceS(wrk);
 	if (th->target && th->target->colorTransform)
 	{
 		asAtom a= asAtomHandler::undefinedAtom;
@@ -393,37 +469,37 @@ ASFUNCTIONBODY_ATOM(AVM1Color,getTransform)
 		m.isAttribute = false;
 		m.name_type=multiname::NAME_STRING;
 
-		m.name_s_id=sys->getUniqueStringId("ra");
-		a = asAtomHandler::fromNumber(sys,th->target->colorTransform->redMultiplier*100.0,false);
-		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED);
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("ra");
+		a = asAtomHandler::fromNumber(wrk,th->target->colorTransform->redMultiplier*100.0,false);
+		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED,nullptr,wrk);
 
-		m.name_s_id=sys->getUniqueStringId("rb");
-		a = asAtomHandler::fromNumber(sys,th->target->colorTransform->redOffset,false);
-		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED);
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("rb");
+		a = asAtomHandler::fromNumber(wrk,th->target->colorTransform->redOffset,false);
+		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED,nullptr,wrk);
 
-		m.name_s_id=sys->getUniqueStringId("ga");
-		a = asAtomHandler::fromNumber(sys,th->target->colorTransform->greenMultiplier*100.0,false);
-		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED);
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("ga");
+		a = asAtomHandler::fromNumber(wrk,th->target->colorTransform->greenMultiplier*100.0,false);
+		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED,nullptr,wrk);
 
-		m.name_s_id=sys->getUniqueStringId("gb");
-		a = asAtomHandler::fromNumber(sys,th->target->colorTransform->greenOffset,false);
-		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED);
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("gb");
+		a = asAtomHandler::fromNumber(wrk,th->target->colorTransform->greenOffset,false);
+		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED,nullptr,wrk);
 
-		m.name_s_id=sys->getUniqueStringId("ba");
-		a = asAtomHandler::fromNumber(sys,th->target->colorTransform->blueMultiplier*100.0,false);
-		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED);
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("ba");
+		a = asAtomHandler::fromNumber(wrk,th->target->colorTransform->blueMultiplier*100.0,false);
+		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED,nullptr,wrk);
 
-		m.name_s_id=sys->getUniqueStringId("bb");
-		a = asAtomHandler::fromNumber(sys,th->target->colorTransform->blueOffset,false);
-		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED);
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("bb");
+		a = asAtomHandler::fromNumber(wrk,th->target->colorTransform->blueOffset,false);
+		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED,nullptr,wrk);
 
-		m.name_s_id=sys->getUniqueStringId("aa");
-		a = asAtomHandler::fromNumber(sys,th->target->colorTransform->alphaMultiplier*100.0,false);
-		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED);
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("aa");
+		a = asAtomHandler::fromNumber(wrk,th->target->colorTransform->alphaMultiplier*100.0,false);
+		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED,nullptr,wrk);
 
-		m.name_s_id=sys->getUniqueStringId("ab");
-		a = asAtomHandler::fromNumber(sys,th->target->colorTransform->alphaOffset,false);
-		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED);
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("ab");
+		a = asAtomHandler::fromNumber(wrk,th->target->colorTransform->alphaOffset,false);
+		o->setVariableByMultiname(m,a,ASObject::CONST_ALLOWED,nullptr,wrk);
 
 	}
 	ret = asAtomHandler::fromObject(o);
@@ -433,68 +509,78 @@ ASFUNCTIONBODY_ATOM(AVM1Color,setTransform)
 {
 	AVM1Color* th=asAtomHandler::as<AVM1Color>(obj);
 	assert_and_throw(argslen==1);
-	ASObject* o = asAtomHandler::toObject(args[0],sys);
+	ASObject* o = asAtomHandler::toObject(args[0],wrk);
 	if (th->target)
 	{
 		if (!th->target->colorTransform)
-			th->target->colorTransform = _R<ColorTransform>(Class<ColorTransform>::getInstanceSNoArgs(sys));
+			th->target->colorTransform = _R<ColorTransform>(Class<ColorTransform>::getInstanceSNoArgs(wrk));
 		asAtom a = asAtomHandler::invalidAtom;
 		multiname m(nullptr);
 		m.isAttribute = false;
 		m.name_type=multiname::NAME_STRING;
 
-		m.name_s_id=sys->getUniqueStringId("ra");
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("ra");
 		a = asAtomHandler::invalidAtom;
-		o->getVariableByMultiname(a,m);
+		o->getVariableByMultiname(a,m,GET_VARIABLE_OPTION::NONE,wrk);
 		if (asAtomHandler::isValid(a))
 			th->target->colorTransform->redMultiplier = asAtomHandler::toNumber(a)/100.0;
+		ASATOM_DECREF(a);
 
-		m.name_s_id=sys->getUniqueStringId("rb");
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("rb");
 		a = asAtomHandler::invalidAtom;
-		o->getVariableByMultiname(a,m);
+		o->getVariableByMultiname(a,m,GET_VARIABLE_OPTION::NONE,wrk);
 		if (asAtomHandler::isValid(a))
 			th->target->colorTransform->redOffset = asAtomHandler::toNumber(a);
+		ASATOM_DECREF(a);
 
-		m.name_s_id=sys->getUniqueStringId("ga");
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("ga");
 		a = asAtomHandler::invalidAtom;
-		o->getVariableByMultiname(a,m);
+		o->getVariableByMultiname(a,m,GET_VARIABLE_OPTION::NONE,wrk);
 		if (asAtomHandler::isValid(a))
 			th->target->colorTransform->greenMultiplier = asAtomHandler::toNumber(a)/100.0;
+		ASATOM_DECREF(a);
 
-		m.name_s_id=sys->getUniqueStringId("gb");
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("gb");
 		a = asAtomHandler::invalidAtom;
-		o->getVariableByMultiname(a,m);
+		o->getVariableByMultiname(a,m,GET_VARIABLE_OPTION::NONE,wrk);
 		if (asAtomHandler::isValid(a))
 			th->target->colorTransform->greenOffset = asAtomHandler::toNumber(a);
+		ASATOM_DECREF(a);
 
-		m.name_s_id=sys->getUniqueStringId("ba");
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("ba");
 		a = asAtomHandler::invalidAtom;
-		o->getVariableByMultiname(a,m);
+		o->getVariableByMultiname(a,m,GET_VARIABLE_OPTION::NONE,wrk);
 		if (asAtomHandler::isValid(a))
 			th->target->colorTransform->blueMultiplier = asAtomHandler::toNumber(a)/100.0;
+		ASATOM_DECREF(a);
 
-		m.name_s_id=sys->getUniqueStringId("bb");
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("bb");
 		a = asAtomHandler::invalidAtom;
-		o->getVariableByMultiname(a,m);
+		o->getVariableByMultiname(a,m,GET_VARIABLE_OPTION::NONE,wrk);
 		if (asAtomHandler::isValid(a))
 			th->target->colorTransform->blueOffset = asAtomHandler::toNumber(a);
+		ASATOM_DECREF(a);
 
-		m.name_s_id=sys->getUniqueStringId("aa");
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("aa");
 		a = asAtomHandler::invalidAtom;
-		o->getVariableByMultiname(a,m);
+		o->getVariableByMultiname(a,m,GET_VARIABLE_OPTION::NONE,wrk);
 		if (asAtomHandler::isValid(a))
 			th->target->colorTransform->alphaMultiplier = asAtomHandler::toNumber(a)/100.0;
+		ASATOM_DECREF(a);
 
-		m.name_s_id=sys->getUniqueStringId("ab");
+		m.name_s_id=wrk->getSystemState()->getUniqueStringId("ab");
 		a = asAtomHandler::invalidAtom;
-		o->getVariableByMultiname(a,m);
+		o->getVariableByMultiname(a,m,GET_VARIABLE_OPTION::NONE,wrk);
 		if (asAtomHandler::isValid(a))
 			th->target->colorTransform->alphaOffset = asAtomHandler::toNumber(a);
+		ASATOM_DECREF(a);
+		th->target->hasChanged=true;
+		th->target->requestInvalidation(wrk->getSystemState());
 	}
 }
 bool AVM1Color::destruct()
 {
-	target.reset();
+	target=nullptr;
 	return ASObject::destruct();
 }
 
@@ -507,34 +593,31 @@ void AVM1Broadcaster::sinit(Class_base* c)
 ASFUNCTIONBODY_ATOM(AVM1Broadcaster,initialize)
 {
 	_NR<ASObject> listener;
-	ARG_UNPACK_ATOM(listener);
+	ARG_CHECK(ARG_UNPACK(listener));
 	if (!listener.isNull())
 	{
-		Array* listeners = Class<Array>::getInstanceSNoArgs(sys);
-		listener->setVariableAtomByQName("broadcastMessage",nsNameAndKind(),asAtomHandler::fromObjectNoPrimitive(Class<IFunction>::getFunction(sys,broadcastMessage)),DYNAMIC_TRAIT);
-		listener->setVariableAtomByQName("addListener",nsNameAndKind(),asAtomHandler::fromObjectNoPrimitive(Class<IFunction>::getFunction(sys,addListener)),DYNAMIC_TRAIT);
-		listener->setVariableAtomByQName("removeListener",nsNameAndKind(),asAtomHandler::fromObjectNoPrimitive(Class<IFunction>::getFunction(sys,removeListener)),DYNAMIC_TRAIT);
+		Array* listeners = Class<Array>::getInstanceSNoArgs(wrk);
+		listener->setVariableAtomByQName("broadcastMessage",nsNameAndKind(),asAtomHandler::fromObjectNoPrimitive(Class<IFunction>::getFunction(wrk->getSystemState(),broadcastMessage)),DYNAMIC_TRAIT);
+		listener->setVariableAtomByQName("addListener",nsNameAndKind(),asAtomHandler::fromObjectNoPrimitive(Class<IFunction>::getFunction(wrk->getSystemState(),addListener)),DYNAMIC_TRAIT);
+		listener->setVariableAtomByQName("removeListener",nsNameAndKind(),asAtomHandler::fromObjectNoPrimitive(Class<IFunction>::getFunction(wrk->getSystemState(),removeListener)),DYNAMIC_TRAIT);
 		listener->setVariableAtomByQName("_listeners",nsNameAndKind(),asAtomHandler::fromObjectNoPrimitive(listeners),DYNAMIC_TRAIT);
-		LOG(LOG_ERROR,"AVM1Broadcaster.initialize:"<<listener->toDebugString());
-		listener->dumpVariables();
 	}
 }
 ASFUNCTIONBODY_ATOM(AVM1Broadcaster,broadcastMessage)
 {
 	ASObject* th = asAtomHandler::getObject(obj);
 	tiny_string msg;
-	ARG_UNPACK_ATOM(msg);
-	LOG(LOG_ERROR,"AVM1Broadcaster.broadcastMessage:"<<msg);
+	ARG_CHECK(ARG_UNPACK(msg));
 	asAtom l = asAtomHandler::invalidAtom;
 	multiname m(nullptr);
 	m.name_type=multiname::NAME_STRING;
-	m.name_s_id=sys->getUniqueStringId("_listeners");
-	th->getVariableByMultiname(l,m);
+	m.name_s_id=wrk->getSystemState()->getUniqueStringId("_listeners");
+	th->getVariableByMultiname(l,m,GET_VARIABLE_OPTION::NONE,wrk);
 	if (asAtomHandler::isArray(l))
 	{
 		multiname mmsg(nullptr);
 		mmsg.name_type=multiname::NAME_STRING;
-		mmsg.name_s_id=sys->getUniqueStringId(msg);
+		mmsg.name_s_id=wrk->getSystemState()->getUniqueStringId(msg);
 		Array* listeners = asAtomHandler::as<Array>(l);
 		for (uint32_t i =0; i < listeners->size(); i++)
 		{
@@ -544,19 +627,21 @@ ASFUNCTIONBODY_ATOM(AVM1Broadcaster,broadcastMessage)
 			{
 				ASObject* listener = asAtomHandler::getObjectNoCheck(o);
 				asAtom f = asAtomHandler::invalidAtom;
-				listener->getVariableByMultiname(f,mmsg);
+				listener->getVariableByMultiname(f,mmsg,GET_VARIABLE_OPTION::NONE,wrk);
 				asAtom res;
 				if (asAtomHandler::is<Function>(f))
 				{
-					asAtomHandler::as<Function>(f)->call(res,o,nullptr,0);
+					asAtomHandler::as<Function>(f)->call(res,wrk,o,nullptr,0);
+					asAtomHandler::as<Function>(f)->decRef();
 				}
 				else if (asAtomHandler::is<SyntheticFunction>(f))
 				{
-					asAtomHandler::as<SyntheticFunction>(f)->call(res,o,nullptr,0,false,false);
+					asAtomHandler::as<SyntheticFunction>(f)->call(wrk,res,o,nullptr,0,false,false);
 				}
 				else if (asAtomHandler::is<AVM1Function>(f))
 				{
 					asAtomHandler::as<AVM1Function>(f)->call(&res,&o,nullptr,0);
+					asAtomHandler::as<AVM1Function>(f)->decRef();
 				}
 			}
 		}
@@ -566,7 +651,7 @@ ASFUNCTIONBODY_ATOM(AVM1Broadcaster,addListener)
 {
 	ASObject* th = asAtomHandler::getObject(obj);
 	_NR<ASObject> listener;
-	ARG_UNPACK_ATOM(listener);
+	ARG_CHECK(ARG_UNPACK(listener));
 	if (listener.isNull())
 	{
 		ret = asAtomHandler::falseAtom;
@@ -575,14 +660,13 @@ ASFUNCTIONBODY_ATOM(AVM1Broadcaster,addListener)
 	asAtom l = asAtomHandler::invalidAtom;
 	multiname m(nullptr);
 	m.name_type=multiname::NAME_STRING;
-	m.name_s_id=sys->getUniqueStringId("_listeners");
-	th->getVariableByMultiname(l,m);
-	LOG(LOG_ERROR,"AVM1Broadcaster.addListener"<<th->toDebugString());
-	th->dumpVariables();
+	m.name_s_id=wrk->getSystemState()->getUniqueStringId("_listeners");
+	th->getVariableByMultiname(l,m,GET_VARIABLE_OPTION::NONE,wrk);
 	if (asAtomHandler::isArray(l))
 	{
 		// TODO spec is not clear if listener can be added multiple times
 		Array* listeners = asAtomHandler::as<Array>(l);
+		listener->incRef();
 		listeners->push(asAtomHandler::fromObjectNoPrimitive(listener.getPtr()));
 	}
 	ret = asAtomHandler::trueAtom;
@@ -591,17 +675,15 @@ ASFUNCTIONBODY_ATOM(AVM1Broadcaster,removeListener)
 {
 	ASObject* th = asAtomHandler::getObject(obj);
 	_NR<ASObject> listener;
-	ARG_UNPACK_ATOM(listener);
+	ARG_CHECK(ARG_UNPACK(listener));
 	ret = asAtomHandler::falseAtom;
 	if (listener.isNull())
 		return;
 	asAtom l = asAtomHandler::invalidAtom;
 	multiname m(nullptr);
 	m.name_type=multiname::NAME_STRING;
-	m.name_s_id=sys->getUniqueStringId("_listeners");
-	th->getVariableByMultiname(l,m);
-	LOG(LOG_ERROR,"AVM1Broadcaster.removeListener "<<th->toDebugString());
-	th->dumpVariables();
+	m.name_s_id=wrk->getSystemState()->getUniqueStringId("_listeners");
+	th->getVariableByMultiname(l,m,GET_VARIABLE_OPTION::NONE,wrk);
 	if (asAtomHandler::isArray(l))
 	{
 		Array* listeners = asAtomHandler::as<Array>(l);
@@ -613,7 +695,7 @@ ASFUNCTIONBODY_ATOM(AVM1Broadcaster,removeListener)
 			{
 				asAtom res;
 				asAtom index = asAtomHandler::fromUInt(i);
-				listeners->removeAt(res,sys,obj,&index,1);
+				listeners->removeAt(res,wrk,obj,&index,1);
 				ret=asAtomHandler::trueAtom;
 				break;
 			}
@@ -625,12 +707,13 @@ void AVM1BitmapData::sinit(Class_base *c)
 {
 	BitmapData::sinit(c);
 	c->setDeclaredMethodByQName("loadBitmap","",Class<IFunction>::getFunction(c->getSystemState(),loadBitmap),NORMAL_METHOD,false);
+	c->setDeclaredMethodByQName("rectangle","",Class<IFunction>::getFunction(c->getSystemState(),getRect,0,Class<Rectangle>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
 }
 ASFUNCTIONBODY_ATOM(AVM1BitmapData,loadBitmap)
 {
 	tiny_string name;
-	ARG_UNPACK_ATOM(name);
-	BitmapTag* tag = dynamic_cast<BitmapTag*>( sys->mainClip->dictionaryLookupByName(sys->getUniqueStringId(name)));
+	ARG_CHECK(ARG_UNPACK(name));
+	BitmapTag* tag = dynamic_cast<BitmapTag*>( wrk->rootClip->dictionaryLookupByName(wrk->getSystemState()->getUniqueStringId(name)));
 	if (tag)
 		ret = asAtomHandler::fromObjectNoPrimitive(tag->instance());
 	else
@@ -641,4 +724,6 @@ void AVM1Bitmap::sinit(Class_base *c)
 {
 	Bitmap::sinit(c);
 	DisplayObject::AVM1SetupMethods(c);
+	c->setDeclaredMethodByQName("forceSmoothing","",Class<IFunction>::getFunction(c->getSystemState(),_getter_smoothing),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("forceSmoothing","",Class<IFunction>::getFunction(c->getSystemState(),_setter_smoothing),SETTER_METHOD,true);
 }
