@@ -30,42 +30,49 @@
 
 namespace lightspark
 {
+class AVM1Color;
 
 class AVM1MovieClip: public MovieClip
 {
+private:
+	// this is only needed to make this movieclip the owner of the color (avoids circular references if color is also set as variable)
+	_NR<AVM1Color> color;
 public:
-	AVM1MovieClip(Class_base* c):MovieClip(c){}
-	AVM1MovieClip(Class_base* c, const FrameContainer& f, uint32_t defineSpriteTagID,uint32_t nameID=BUILTIN_STRINGS::EMPTY):MovieClip(c,f,defineSpriteTagID) 
+	AVM1MovieClip(ASWorker* wrk,Class_base* c):MovieClip(wrk,c){}
+	AVM1MovieClip(ASWorker* wrk,Class_base* c, const FrameContainer& f, uint32_t defineSpriteTagID,uint32_t nameID=BUILTIN_STRINGS::EMPTY):MovieClip(wrk,c,f,defineSpriteTagID) 
 	{
 		name=nameID;
 	}
-	void afterConstruction();
+	void afterConstruction() override;
+	bool destruct() override;
+	void prepareShutdown() override;
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(startDrag);
 	ASFUNCTION_ATOM(stopDrag);
+	ASFUNCTION_ATOM(attachAudio);
+	void setColor(AVM1Color* c);
 };
 
 class AVM1Shape: public Shape
 {
 public:
-	AVM1Shape(Class_base* c):Shape(c){}
-	AVM1Shape(Class_base* c, float scaling, DefineShapeTag* tag):Shape(c,scaling,tag) {}
+	AVM1Shape(ASWorker* wrk,Class_base* c):Shape(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
 class AVM1SimpleButton: public SimpleButton
 {
 public:
-	AVM1SimpleButton(Class_base* c, DisplayObject *dS = nullptr, DisplayObject *hTS = nullptr,
+	AVM1SimpleButton(ASWorker* wrk,Class_base* c, DisplayObject *dS = nullptr, DisplayObject *hTS = nullptr,
 				 DisplayObject *oS = nullptr, DisplayObject *uS = nullptr, DefineButtonTag* tag = nullptr)
-		:SimpleButton(c,dS,hTS,oS,uS,tag) {}
+		:SimpleButton(wrk,c,dS,hTS,oS,uS,tag) {}
 	
 	static void sinit(Class_base* c);
 };
 class AVM1Stage: public Stage
 {
 public:
-	AVM1Stage(Class_base* c):Stage(c) {}
+	AVM1Stage(ASWorker* wrk,Class_base* c):Stage(wrk,c) {}
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_getDisplayState);
 	ASFUNCTION_ATOM(_setDisplayState);
@@ -84,7 +91,7 @@ private:
 	std::list<IThreadJob *> jobs;
 	std::set<_R<ASObject> > listeners;
 public:
-	AVM1MovieClipLoader(Class_base* c):Loader(c){}
+	AVM1MovieClipLoader(ASWorker* wrk,Class_base* c):Loader(wrk,c){}
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(loadClip);
@@ -92,13 +99,14 @@ public:
 	ASFUNCTION_ATOM(removeListener);
 	void AVM1HandleEvent(EventDispatcher* dispatcher, Event* e) override;
 	bool destruct() override;
+	void load(const tiny_string& url, const tiny_string& method, AVM1MovieClip* target);
 };
 
 class AVM1Color: public ASObject
 {
-	_NR<DisplayObject> target;
+	AVM1MovieClip* target;
 public:
-	AVM1Color(Class_base* c):ASObject(c) {}
+	AVM1Color(ASWorker* wrk,Class_base* c):ASObject(wrk,c),target(nullptr) {}
 	static void sinit(Class_base* c);
 	bool destruct() override;
 	ASFUNCTION_ATOM(_constructor);
@@ -111,7 +119,7 @@ public:
 class AVM1Broadcaster: public ASObject
 {
 public:
-	AVM1Broadcaster(Class_base* c):ASObject(c) {}
+	AVM1Broadcaster(ASWorker* wrk,Class_base* c):ASObject(wrk,c) {}
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(initialize);
 	ASFUNCTION_ATOM(broadcastMessage);
@@ -122,10 +130,10 @@ public:
 class AVM1BitmapData: public BitmapData
 {
 public:
-	AVM1BitmapData(Class_base* c):BitmapData(c) {}
-	AVM1BitmapData(Class_base* c, _R<BitmapContainer> b):BitmapData(c,b) {}
-	AVM1BitmapData(Class_base* c, const BitmapData& other):BitmapData(c,other) {}
-	AVM1BitmapData(Class_base* c, uint32_t width, uint32_t height):BitmapData(c,width,height) {}
+	AVM1BitmapData(ASWorker* wrk,Class_base* c):BitmapData(wrk,c) {}
+	AVM1BitmapData(ASWorker* wrk,Class_base* c, _R<BitmapContainer> b):BitmapData(wrk,c,b) {}
+	AVM1BitmapData(ASWorker* wrk,Class_base* c, const BitmapData& other):BitmapData(wrk,c,other) {}
+	AVM1BitmapData(ASWorker* wrk,Class_base* c, uint32_t width, uint32_t height):BitmapData(wrk,c,width,height) {}
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(loadBitmap);
 };
@@ -133,8 +141,8 @@ public:
 class AVM1Bitmap: public Bitmap
 {
 public:
-	AVM1Bitmap(Class_base* c, _NR<LoaderInfo> li=NullRef, std::istream *s = nullptr, FILE_TYPE type=FT_UNKNOWN):Bitmap(c,li,s,type) {}
-	AVM1Bitmap(Class_base* c, _R<AVM1BitmapData> data):Bitmap(c,_R<BitmapData>(data)) {}
+	AVM1Bitmap(ASWorker* wrk,Class_base* c, _NR<LoaderInfo> li=NullRef, std::istream *s = nullptr, FILE_TYPE type=FT_UNKNOWN):Bitmap(wrk,c,li,s,type) {}
+	AVM1Bitmap(ASWorker* wrk,Class_base* c, _R<AVM1BitmapData> data):Bitmap(wrk,c,_R<BitmapData>(data)) {}
 	static void sinit(Class_base* c);
 };
 
