@@ -30,8 +30,8 @@ namespace lightspark
 class AVM1Video: public Video
 {
 public:
-	AVM1Video(Class_base* c):Video(c){}
-	AVM1Video(Class_base* c, uint32_t w, uint32_t h, DefineVideoStreamTag* v):Video(c,w,h,v) {}
+	AVM1Video(ASWorker* wrk,Class_base* c):Video(wrk,c){}
+	AVM1Video(ASWorker* wk,Class_base* c, uint32_t w, uint32_t h, DefineVideoStreamTag* v):Video(wk,c,w,h,v) {}
 	static void sinit(Class_base* c);
 };
 
