@@ -24,59 +24,80 @@
 using namespace std;
 using namespace lightspark;
 
+void AVM1XMLDocument::finalize()
+{
+	XMLDocument::finalize();
+	loader.reset();
+}
+bool AVM1XMLDocument::destruct()
+{
+	loader.reset();
+	getSystemState()->stage->AVM1RemoveEventListener(this);
+	return XMLDocument::destruct();
+}
+
+void AVM1XMLDocument::prepareShutdown()
+{
+	if (this->preparedforshutdown)
+		return;
+	XMLDocument::prepareShutdown();
+	if (loader)
+		loader->prepareShutdown();
+}
+
 void AVM1XMLDocument::sinit(Class_base* c)
 {
 	XMLDocument::sinit(c);
+	c->isSealed = false;
 	c->setDeclaredMethodByQName("load","",Class<IFunction>::getFunction(c->getSystemState(),load),NORMAL_METHOD,true);
-	
+	REGISTER_GETTER(c, status);
 }
 
+ASFUNCTIONBODY_GETTER(AVM1XMLDocument,status)
+
 ASFUNCTIONBODY_ATOM(AVM1XMLDocument,load)
 {
 	AVM1XMLDocument* th=asAtomHandler::as<AVM1XMLDocument>(obj);
 	if (th->loader.isNull())
-		th->loader = _MR(Class<URLLoader>::getInstanceS(sys));
+		th->loader = _MR(Class<URLLoader>::getInstanceS(wrk));
 	tiny_string url;
-	ARG_UNPACK_ATOM(url);
-	URLRequest* req = Class<URLRequest>::getInstanceS(sys,url);
+	ARG_CHECK(ARG_UNPACK(url));
+	URLRequest* req = Class<URLRequest>::getInstanceS(wrk,url);
 	asAtom urlarg = asAtomHandler::fromObjectNoPrimitive(req);
 	asAtom loaderobj = asAtomHandler::fromObjectNoPrimitive(th->loader.getPtr());
-	URLLoader::load(ret,sys,loaderobj,&urlarg,1);
+	URLLoader::load(ret,wrk,loaderobj,&urlarg,1);
+	req->decRef();
 }
 
-multiname* AVM1XMLDocument::setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool *alreadyset)
+multiname* AVM1XMLDocument::setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool *alreadyset,ASWorker* wrk)
 {
-	multiname* res = XMLDocument::setVariableByMultiname(name,o,allowConst,alreadyset);
-	if (!getSystemState()->mainClip->usesActionScript3)
+	multiname* res = XMLDocument::setVariableByMultiname(name,o,allowConst,alreadyset,wrk);
+	if (name.name_s_id == BUILTIN_STRINGS::STRING_ONLOAD)
 	{
-		if (name.name_s_id == BUILTIN_STRINGS::STRING_ONLOAD)
-		{
-			if (loader.isNull())
-				loader = _MR(Class<URLLoader>::getInstanceS(getSystemState()));
-			this->incRef();
-			getSystemState()->stage->AVM1AddEventListener(this);
-			setIsEnumerable(name, false);
-		}
+		if (loader.isNull())
+			loader = _MR(Class<URLLoader>::getInstanceS(wrk));
+		getSystemState()->stage->AVM1AddEventListener(this);
+		setIsEnumerable(name, false);
 	}
 	return res;
 }
 void AVM1XMLDocument::AVM1HandleEvent(EventDispatcher* dispatcher, Event* e)
 {
-	if (!getSystemState()->mainClip->usesActionScript3 && dispatcher == loader.getPtr())
+	if (dispatcher == loader.getPtr())
 	{
 		if (e->type == "complete" || e->type=="ioError")
 		{
 			if (e->type == "complete" && loader->getData())
 			{
 				std::string str = loader->getData()->toString().raw_buf();
-				this->parseXMLImpl(str);
+				this->status = this->parseXMLImpl(str);
 			}
 			asAtom func=asAtomHandler::invalidAtom;
 			multiname m(nullptr);
 			m.name_type=multiname::NAME_STRING;
 			m.isAttribute = false;
 			m.name_s_id=BUILTIN_STRINGS::STRING_ONLOAD;
-			getVariableByMultiname(func,m);
+			getVariableByMultiname(func,m,GET_VARIABLE_OPTION::NONE,getInstanceWorker());
 			if (asAtomHandler::is<AVM1Function>(func))
 			{
 				asAtom ret=asAtomHandler::invalidAtom;
@@ -84,6 +105,7 @@ void AVM1XMLDocument::AVM1HandleEvent(EventDispatcher* dispatcher, Event* e)
 				asAtom args[1];
 				args[0] = asAtomHandler::fromBool(e->type=="complete");
 				asAtomHandler::as<AVM1Function>(func)->call(&ret,&obj,args,1);
+				asAtomHandler::as<AVM1Function>(func)->decRef();
 			}
 		}
 	}
