@@ -18,16 +18,20 @@
 **************************************************************************/
 
 #include "scripting/avmplus/avmplus.h"
+#include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/Integer.h"
 #include "scripting/flash/utils/ByteArray.h"
+#include "scripting/flash/system/flashsystem.h"
 #include "scripting/class.h"
 #include "scripting/argconv.h"
 #include "threading.h"
 #include "abc.h"
+#include "platforms/engineutils.h"
 
 using namespace lightspark;
 
-avmplusFile::avmplusFile(Class_base* c):
-	ASObject(c)
+avmplusFile::avmplusFile(ASWorker* wrk, Class_base* c):
+	ASObject(wrk,c)
 {
 }
 
@@ -44,44 +48,50 @@ void avmplusFile::sinit(Class_base* c)
 ASFUNCTIONBODY_ATOM(avmplusFile,exists)
 {
 	tiny_string filename;
-	ARG_UNPACK_ATOM(filename);
-	asAtomHandler::setBool(ret,sys->getEngineData()->FileExists(filename));
+	ARG_CHECK(ARG_UNPACK(filename));
+	asAtomHandler::setBool(ret,wrk->getSystemState()->getEngineData()->FileExists(wrk->getSystemState(),filename,false));
 }
 ASFUNCTIONBODY_ATOM(avmplusFile,read)
 {
 	tiny_string filename;
-	ARG_UNPACK_ATOM(filename);
-	if (!sys->getEngineData()->FileExists(filename))
-		throwError<ASError>(kFileOpenError,filename);
-	ret = asAtomHandler::fromObject(abstract_s(sys,sys->getEngineData()->FileRead(filename)));
+	ARG_CHECK(ARG_UNPACK(filename));
+	if (!wrk->getSystemState()->getEngineData()->FileExists(wrk->getSystemState(),filename,false))
+	{
+		createError<ASError>(wrk,kFileOpenError,filename);
+		return;
+	}
+	ret = asAtomHandler::fromObject(abstract_s(wrk,wrk->getSystemState()->getEngineData()->FileRead(wrk->getSystemState(),filename,false)));
 }
 ASFUNCTIONBODY_ATOM(avmplusFile,write)
 {
 	tiny_string filename;
 	tiny_string data;
-	ARG_UNPACK_ATOM(filename)(data);
-	sys->getEngineData()->FileWrite(filename,data);
+	ARG_CHECK(ARG_UNPACK(filename)(data));
+	wrk->getSystemState()->getEngineData()->FileWrite(wrk->getSystemState(),filename,data,false);
 }
 ASFUNCTIONBODY_ATOM(avmplusFile,readByteArray)
 {
 	tiny_string filename;
-	ARG_UNPACK_ATOM(filename);
-	if (!sys->getEngineData()->FileExists(filename))
-		throwError<ASError>(kFileOpenError,filename);
-	ByteArray* res = Class<ByteArray>::getInstanceS(sys);
-	sys->getEngineData()->FileReadByteArray(filename,res);
+	ARG_CHECK(ARG_UNPACK(filename));
+	if (!wrk->getSystemState()->getEngineData()->FileExists(wrk->getSystemState(),filename,false))
+	{
+		createError<ASError>(wrk,kFileOpenError,filename);
+		return;
+	}
+	ByteArray* res = Class<ByteArray>::getInstanceS(wrk);
+	wrk->getSystemState()->getEngineData()->FileReadByteArray(wrk->getSystemState(),filename,res,0,UINT32_MAX,false);
 	ret = asAtomHandler::fromObject(res);
 }
 ASFUNCTIONBODY_ATOM(avmplusFile,writeByteArray)
 {
 	tiny_string filename;
 	_NR<ByteArray> data;
-	ARG_UNPACK_ATOM(filename)(data);
-	sys->getEngineData()->FileWriteByteArray(filename,data.getPtr());
+	ARG_CHECK(ARG_UNPACK(filename)(data));
+	wrk->getSystemState()->getEngineData()->FileWriteByteArray(wrk->getSystemState(),filename,data.getPtr(),0,UINT32_MAX,false);
 }
 
-avmplusSystem::avmplusSystem(Class_base* c):
-	ASObject(c)
+avmplusSystem::avmplusSystem(ASWorker* wrk, Class_base* c):
+	ASObject(wrk,c)
 {
 }
 
@@ -111,54 +121,54 @@ void avmplusSystem::sinit(Class_base* c)
 
 ASFUNCTIONBODY_ATOM(avmplusSystem,getFeatures)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.getFeatures is unimplemented."));
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.getFeatures is unimplemented.");
 	ret = asAtomHandler::fromStringID(BUILTIN_STRINGS::EMPTY);
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,getRunmode)
 {
-	ret = asAtomHandler::fromString(sys,sys->useJit ? "jit":"interpreted");
+	ret = asAtomHandler::fromString(wrk->getSystemState(),wrk->getSystemState()->useJit ? "jit":"interpreted");
 }
 
 ASFUNCTIONBODY_ATOM(avmplusSystem,queueCollection)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.queueCollection is unimplemented."));
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.queueCollection is unimplemented.");
 	asAtomHandler::setUndefined(ret);
 }
 
 ASFUNCTIONBODY_ATOM(avmplusSystem,forceFullCollection)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.forceFullCollection is unimplemented."));
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.forceFullCollection is unimplemented.");
 }
 
 ASFUNCTIONBODY_ATOM(avmplusSystem,getAvmplusVersion)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.getAvmplusVersion is unimplemented."));
-	ret = asAtomHandler::fromString(sys,"0");
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.getAvmplusVersion is unimplemented.");
+	ret = asAtomHandler::fromString(wrk->getSystemState(),"0");
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,pauseForGCIfCollectionImminent)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.pauseForGCIfCollectionImminent is unimplemented."));
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.pauseForGCIfCollectionImminent is unimplemented.");
 }
 
 ASFUNCTIONBODY_ATOM(avmplusSystem,isDebugger)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.isDebugger is unimplemented."));
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.isDebugger is unimplemented.");
 	asAtomHandler::setBool(ret,false);
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,isGlobal)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.isGlobal is unimplemented."));
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.isGlobal is unimplemented.");
 	asAtomHandler::setBool(ret,false);
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,_freeMemory)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.freeMemory is unimplemented."));
-	asAtomHandler::setUInt(ret,sys,1024);
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.freeMemory is unimplemented.");
+	asAtomHandler::setUInt(ret,wrk,1024);
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,_totalMemory)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.totalMemory is unimplemented."));
-	asAtomHandler::setUInt(ret,sys,1024);
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.totalMemory is unimplemented.");
+	asAtomHandler::setUInt(ret,wrk,1024);
 }
 //#include <sys/resource.h>
 //#include <stdio.h>
@@ -182,44 +192,44 @@ ASFUNCTIONBODY_ATOM(avmplusSystem,_totalMemory)
 //}
 ASFUNCTIONBODY_ATOM(avmplusSystem,_privateMemory)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.privateMemory is unimplemented."));
-	asAtomHandler::setUInt(ret,sys,1024);
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.privateMemory is unimplemented.");
+	asAtomHandler::setUInt(ret,wrk,1024);
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,_swfVersion)
 {
-	asAtomHandler::setUInt(ret,sys,sys->getSwfVersion());
+	asAtomHandler::setUInt(ret,wrk,wrk->getSystemState()->getSwfVersion());
 }
 
 ASFUNCTIONBODY_ATOM(avmplusSystem,argv)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.argv is unimplemented."));
-	ret = asAtomHandler::fromObject(Class<Array>::getInstanceS(sys));
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.argv is unimplemented.");
+	ret = asAtomHandler::fromObject(Class<Array>::getInstanceS(wrk));
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,exec)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.exec is unimplemented."));
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.exec is unimplemented.");
 	if (argslen == 0)
-		throwError<ArgumentError>(kWrongArgumentCountError,"exec",">0",Integer::toString(argslen));
+		createError<ArgumentError>(wrk,kWrongArgumentCountError,"exec",">0",Integer::toString(argslen));
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,write)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.write is unimplemented."));
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.write is unimplemented.");
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,sleep)
 {
 	uint32_t ms;
-	ARG_UNPACK_ATOM(ms);
+	ARG_CHECK(ARG_UNPACK(ms));
 	compat_msleep(ms);
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,exit)
 {
-	LOG(LOG_NOT_IMPLEMENTED, _("avmplus.System.exit is unimplemented."));
+	LOG(LOG_NOT_IMPLEMENTED, "avmplus.System.exit is unimplemented.");
 	ret = asAtomHandler::undefinedAtom;
 }
 ASFUNCTIONBODY_ATOM(avmplusSystem,canonicalizeNumber)
 {
 	_NR<ASObject> o;
-	ARG_UNPACK_ATOM(o);
+	ARG_CHECK(ARG_UNPACK(o));
 	switch(o->getObjectType())
 	{
 		case T_NUMBER:
@@ -228,11 +238,11 @@ ASFUNCTIONBODY_ATOM(avmplusSystem,canonicalizeNumber)
 		case T_UINTEGER:
 		case T_NULL:
 		case T_UNDEFINED:
-			asAtomHandler::setNumber(ret,sys,o->toNumber());
+			asAtomHandler::setNumber(ret,wrk,o->toNumber());
 			break;
 		case T_QNAME:
 		case T_NAMESPACE:
-			asAtomHandler::setNumber(ret,sys,Number::NaN);
+			asAtomHandler::setNumber(ret,wrk,Number::NaN);
 			break;
 		default:
 			o->incRef();
@@ -241,8 +251,8 @@ ASFUNCTIONBODY_ATOM(avmplusSystem,canonicalizeNumber)
 	}
 }
 
-avmplusDomain::avmplusDomain(Class_base* c):
-	ASObject(c)
+avmplusDomain::avmplusDomain(ASWorker* wrk, Class_base* c):
+	ASObject(wrk,c)
 {
 }
 
@@ -260,47 +270,49 @@ void avmplusDomain::sinit(Class_base* c)
 ASFUNCTIONBODY_ATOM(avmplusDomain,_constructor)
 {
 	_NR<avmplusDomain> parentDomain;
-	ARG_UNPACK_ATOM(parentDomain);
+	ARG_CHECK(ARG_UNPACK(parentDomain));
 	avmplusDomain* th = asAtomHandler::as<avmplusDomain>(obj);
 	if (parentDomain.isNull())
-		th->appdomain = ABCVm::getCurrentApplicationDomain(getVm(sys)->currentCallContext);
+		th->appdomain = ABCVm::getCurrentApplicationDomain(wrk->currentCallContext);
 	else
-		th->appdomain = _NR<ApplicationDomain>(Class<ApplicationDomain>::getInstanceS(sys,parentDomain->appdomain));
+		th->appdomain = _NR<ApplicationDomain>(Class<ApplicationDomain>::getInstanceS(wrk,parentDomain->appdomain));
 }
 
 ASFUNCTIONBODY_ATOM(avmplusDomain,_getCurrentDomain)
 {
-	avmplusDomain* res = Class<avmplusDomain>::getInstanceSNoArgs(sys);
-	res->appdomain = ABCVm::getCurrentApplicationDomain(getVm(sys)->currentCallContext);
+	avmplusDomain* res = Class<avmplusDomain>::getInstanceSNoArgs(wrk);
+	res->appdomain = ABCVm::getCurrentApplicationDomain(wrk->currentCallContext);
 	ret = asAtomHandler::fromObject(res);
 }
 ASFUNCTIONBODY_ATOM(avmplusDomain,_getMinDomainMemoryLength)
 {
-	asAtomHandler::setUInt(ret,sys,MIN_DOMAIN_MEMORY_LIMIT);
+	asAtomHandler::setUInt(ret,wrk,MIN_DOMAIN_MEMORY_LIMIT);
 }
 ASFUNCTIONBODY_ATOM(avmplusDomain,load)
 {
 	avmplusDomain* th = asAtomHandler::as<avmplusDomain>(obj);
 	tiny_string filename;
 	uint32_t swfVersion;
-	ARG_UNPACK_ATOM(filename)(swfVersion, 0);
+	ARG_CHECK(ARG_UNPACK(filename)(swfVersion, 0));
 	if (swfVersion != 0)
 		LOG(LOG_NOT_IMPLEMENTED, "avmplus.Domain.load is unimplemented for swfVersion "<<swfVersion);
-	if (!sys->getEngineData()->FileExists(filename))
-		throwError<ASError>(kFileOpenError,filename);
-	_NR<ByteArray> bytes = _NR<ByteArray>(Class<ByteArray>::getInstanceS(sys));
-	sys->getEngineData()->FileReadByteArray(filename,bytes.getPtr());
+	if (!wrk->getSystemState()->getEngineData()->FileExists(wrk->getSystemState(),filename,false))
+	{
+		createError<ASError>(wrk,kFileOpenError,filename);
+		return;
+	}
+	_NR<ByteArray> bytes = _NR<ByteArray>(Class<ByteArray>::getInstanceS(wrk));
+	wrk->getSystemState()->getEngineData()->FileReadByteArray(wrk->getSystemState(),filename,bytes.getPtr(),0,UINT32_MAX,false);
 
 	// execute loaded abc file
-	MemoryStreamCache mc(sys);
+	MemoryStreamCache mc(wrk->getSystemState());
 	mc.append(bytes->getBuffer(bytes->getLength(),false),bytes->getLength());
 	std::streambuf *sbuf = mc.createReader();
 	std::istream s(sbuf);
-	RootMovieClip* root=getVm(sys)->currentCallContext->mi->context->root.getPtr();
+	RootMovieClip* root=wrk->currentCallContext->mi->context->root;
 	_NR<ApplicationDomain> origdomain = root->applicationDomain;
 	root->applicationDomain = th->appdomain;
-	root->incRef();
-	ABCContext* context = new ABCContext(_MR(root), s, getVm(root->getSystemState()));
+	ABCContext* context = new ABCContext(root, s, getVm(root->getSystemState()));
 	context->exec(false);
 	root->applicationDomain = origdomain;
 	delete sbuf;
@@ -310,28 +322,27 @@ ASFUNCTIONBODY_ATOM(avmplusDomain,loadBytes)
 	avmplusDomain* th = asAtomHandler::as<avmplusDomain>(obj);
 	_NR<ByteArray> bytes;
 	uint32_t swfversion;
-	ARG_UNPACK_ATOM (bytes)(swfversion, 0);
+	ARG_CHECK(ARG_UNPACK(bytes)(swfversion, 0));
 
 	if (swfversion != 0)
 		LOG(LOG_NOT_IMPLEMENTED,"Domain.loadBytes ignores parameter swfVersion");
-	MemoryStreamCache mc(sys);
+	MemoryStreamCache mc(wrk->getSystemState());
 	mc.append(bytes->getBuffer(bytes->getLength(),false),bytes->getLength());
 	std::streambuf *sbuf = mc.createReader();
 	std::istream s(sbuf);
 	
 	// execute loaded abc bytes
-	RootMovieClip* root=getVm(sys)->currentCallContext->mi->context->root.getPtr();
+	RootMovieClip* root=wrk->currentCallContext->mi->context->root;
 	_NR<ApplicationDomain> origdomain = root->applicationDomain;
 	root->applicationDomain = th->appdomain;
-	root->incRef();
-	ABCContext* context = new ABCContext(_MR(root), s, getVm(root->getSystemState()));
+	ABCContext* context = new ABCContext(root, s, getVm(root->getSystemState()));
 	context->exec(false);
 	root->applicationDomain = origdomain;
 	delete sbuf;
 }
 ASFUNCTIONBODY_ATOM(avmplusDomain,getClass)
 {
-	getDefinitionByName(ret,sys,obj,args,argslen);
+	getDefinitionByName(ret,wrk,obj,args,argslen);
 }
 ASFUNCTIONBODY_ATOM(avmplusDomain,_getDomainMemory)
 {
@@ -347,7 +358,7 @@ ASFUNCTIONBODY_ATOM(avmplusDomain,_getDomainMemory)
 ASFUNCTIONBODY_ATOM(avmplusDomain,_setDomainMemory)
 {
 	_NR<ByteArray> b;
-	ARG_UNPACK_ATOM(b);
+	ARG_CHECK(ARG_UNPACK(b));
 	avmplusDomain* th = asAtomHandler::as<avmplusDomain>(obj);
 	
 	if (b.isNull())
@@ -358,20 +369,23 @@ ASFUNCTIONBODY_ATOM(avmplusDomain,_setDomainMemory)
 	}
 		
 	if (b->getLength() < MIN_DOMAIN_MEMORY_LIMIT)
-		throwError<RangeError>(kEndOfFileError);
+	{
+		createError<RangeError>(wrk,kEndOfFileError);
+		return;
+	}
 	th->appdomain->domainMemory = b;
 	th->appdomain->checkDomainMemory();
 }
 
 ASFUNCTIONBODY_ATOM(lightspark,casi32)
 {
-	if (sys->mainClip 
-			&& sys->mainClip->applicationDomain
-			&& sys->mainClip->applicationDomain->domainMemory)
+	if (wrk->rootClip
+			&& wrk->rootClip->applicationDomain
+			&& wrk->rootClip->applicationDomain->domainMemory)
 	{
-		asAtom a = asAtomHandler::fromObject(sys->mainClip->applicationDomain->domainMemory.getPtr());
-		ByteArray::atomicCompareAndSwapIntAt(ret,sys,a,args,argslen);
+		asAtom a = asAtomHandler::fromObject(wrk->rootClip->applicationDomain->domainMemory.getPtr());
+		ByteArray::atomicCompareAndSwapIntAt(ret,wrk,a,args,argslen);
 	}
 	else
-		asAtomHandler::setInt(ret,sys,0);
+		asAtomHandler::setInt(ret,wrk,0);
 }
