@@ -25,47 +25,48 @@
 
 using namespace lightspark;
 
-ASObject* lightspark::new_asobject(SystemState* sys)
+ASObject* lightspark::new_asobject(ASWorker* wrk)
 {
-	return Class<ASObject>::getInstanceS(sys);
+	return Class<ASObject>::getInstanceS(wrk);
 }
 
-Prototype* lightspark::new_objectPrototype(SystemState* sys)
+Prototype* lightspark::new_objectPrototype(ASWorker* wrk)
 {
 	//Create a Prototype object, the class should be ASObject
-	Class_base* c=Class<ASObject>::getClass(sys);
-	return new (c->memoryAccount) ObjectPrototype(c);
+	Class_base* c=Class<ASObject>::getClass(wrk->getSystemState());
+	Prototype* p = new (c->memoryAccount) ObjectPrototype(wrk,c);
+	p->getObj()->setRefConstant();
+	return p;
 }
 
-Prototype* lightspark::new_functionPrototype(Class_base* functionClass, _NR<Prototype> p)
+Prototype* lightspark::new_functionPrototype(ASWorker* wrk,Class_base* functionClass, _NR<Prototype> p)
 {
 	//Create a Prototype object, the class should be ASObject
-	return new (functionClass->memoryAccount) FunctionPrototype(functionClass, p);
+	return new (functionClass->memoryAccount) FunctionPrototype(wrk,functionClass, p);
 }
 
 Function_object* lightspark::new_functionObject(_NR<ASObject> p)
 {
 	Class_base* c=Class<ASObject>::getClass(p->getSystemState());
-	return new (c->memoryAccount) Function_object(c, p);
+	return new (c->memoryAccount) Function_object(p->getInstanceWorker(), c, p);
 }
 
 ObjectConstructor* lightspark::new_objectConstructor(Class_base* cls,uint32_t length)
 {
-	return new (cls->memoryAccount) ObjectConstructor(cls, length);
+	return new (cls->memoryAccount) ObjectConstructor(cls->getInstanceWorker(), cls, length);
 }
 
-Activation_object* lightspark::new_activationObject(SystemState* sys)
+Activation_object* lightspark::new_activationObject(ASWorker* wrk)
 {
-	Class_base* c=Class<ASObject>::getClass(sys);
-	return new (c->memoryAccount) Activation_object(c);
+	Class_base* c=Class<ASObject>::getClass(wrk->getSystemState());
+	return new (c->memoryAccount) Activation_object(wrk,c);
 }
 
 
-Class_inherit::Class_inherit(const QName& name, MemoryAccount* m, const traits_info *_classtrait, Global *_global):Class_base(name, m),tag(nullptr),bindedToRoot(false),classtrait(_classtrait)
+Class_inherit::Class_inherit(const QName& name, MemoryAccount* m, const traits_info *_classtrait, Global *_global):Class_base(name, UINT32_MAX,m),tag(nullptr),bindedToRoot(false),bindingchecked(false),inScriptInit(false),classtrait(_classtrait)
 {
 	this->global=_global;
 	this->incRef(); //create on reference for the classes map
-	this->getSystemState()->customClasses.insert(make_pair(name.nameId,this));
 	isReusable = true;
 	subtype = SUBTYPE_INHERIT;
 }
@@ -73,6 +74,7 @@ Class_inherit::Class_inherit(const QName& name, MemoryAccount* m, const traits_i
 void Class_inherit::finalize()
 {
 	instancefactory.reset();
+	tag=nullptr;
 	auto it = class_scope.begin();
 	while (it != class_scope.end())
 	{
@@ -84,30 +86,55 @@ void Class_inherit::finalize()
 	
 }
 
-void Class_inherit::getInstance(asAtom& ret,bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass)
+void Class_inherit::prepareShutdown()
+{
+	if (this->preparedforshutdown)
+		return;
+	Class_base::prepareShutdown();
+	if (instancefactory)
+		instancefactory->prepareShutdown();
+	auto it = class_scope.begin();
+	while (it != class_scope.end())
+	{
+		ASObject* o = asAtomHandler::getObject((*it).object);
+		if (o)
+			o->prepareShutdown();
+		it++;
+	}
+}
+
+void Class_inherit::getInstance(ASWorker* worker, asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass)
 {
 	checkScriptInit();
 	//We override the classdef
 	if(realClass==nullptr)
 		realClass=this;
-	if (!this->isBinded()) // it seems possible that an instance of a class is constructed before the binding of the class is available, so we have to check for a binding here
-		getSystemState()->mainClip->bindClass(this->class_name,this);
+	if (this->needsBindingCheck()) // it seems possible that an instance of a class is constructed before the binding of the class is available, so we have to check for a binding here
+	{
+		worker->rootClip->bindClass(this->class_name,this);
+		if (worker->rootClip->hasFinishedLoading())
+			this->bindingchecked=true;
+	}
 	if(tag)
 	{
 		ret=asAtomHandler::fromObject(tag->instance(realClass));
-		if (instancefactory.isNull())
+		if (instancefactory.isNull() && this->bindingchecked)
+		{
 			instancefactory = _MR(tag->instance(realClass));
+			instancefactory->setRefConstant();
+		}
 	}
 	else
 	{
 		assert_and_throw(super);
 		//Our super should not construct, we are going to do it ourselves
-		super->getInstance(ret,false,nullptr,0,realClass);
-		if (instancefactory.isNull())
+		super->getInstance(worker,ret,false,nullptr,0,realClass);
+		if (instancefactory.isNull() && this->bindingchecked)
 		{
 			asAtom instance=asAtomHandler::invalidAtom;
-			super->getInstance(instance,false,nullptr,0,realClass);
+			super->getInstance(worker,instance,false,nullptr,0,realClass);
 			instancefactory = _MR(asAtomHandler::getObject(instance));
+			instancefactory->setRefConstant();
 		}
 	}
 	if(construct)
@@ -137,9 +164,9 @@ void Class_inherit::setupDeclaredTraits(ASObject *target, bool checkclone)
 {
 	if (!target->traitsInitialized)
 	{
-	#ifndef NDEBUG
-		assert_and_throw(!target->initialized);
-	#endif
+#ifndef NDEBUG
+		assert(!target->initialized);
+#endif
 		bool cloneable = checkclone && !instancefactory.isNull();
 		if (cloneable)
 		{
@@ -156,6 +183,7 @@ void Class_inherit::setupDeclaredTraits(ASObject *target, bool checkclone)
 				instancefactory->implEnable=bak;
 
 				instancefactory->traitsInitialized = true;
+				instancefactory->setRefConstant();
 				cloneable = instancefactory->cloneInstance(target);
 			}
 			else
@@ -171,10 +199,9 @@ void Class_inherit::setupDeclaredTraits(ASObject *target, bool checkclone)
 			//And restore it
 			target->implEnable=bak;
 		}
-
-	#ifndef NDEBUG
+#ifndef NDEBUG
 		target->initialized=true;
-	#endif
+#endif
 		target->traitsInitialized = true;
 	}
 }
@@ -191,19 +218,38 @@ bool Class_inherit::hasoverriddenmethod(multiname *name) const
 {
 	// TODO we currently only check for names, not for namespaces as there are some issues regarding protected namespaces
 	// so we may get some false positives here...
-	return class_index == -1 ? true : this->context->instances[this->class_index].overriddenmethods && this->context->instances[this->class_index].overriddenmethods->find(name->name_s_id) != this->context->instances[this->class_index].overriddenmethods->end();
+	if (class_index == -1)
+		return true;
+	const instance_info& c = this->context->instances[this->class_index];
+	if (c.overriddenmethods && c.overriddenmethods->find(name->name_s_id) != c.overriddenmethods->end())
+		return true;
+	return Class_base::hasoverriddenmethod(name);
 }
 
-GET_VARIABLE_RESULT Class_inherit::getVariableByMultiname(asAtom& ret, const multiname& name, GET_VARIABLE_OPTION opt)
+GET_VARIABLE_RESULT Class_inherit::getVariableByMultiname(asAtom& ret, const multiname& name, GET_VARIABLE_OPTION opt,ASWorker* wrk)
 {
 	checkScriptInit();
-	return Class_base::getVariableByMultiname(ret,name,opt);
+	return Class_base::getVariableByMultiname(ret,name,opt,wrk);
 }
 
-template<>
-void Class<Global>::getInstance(asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass)
+variable* Class_inherit::findVariableByMultiname(const multiname& name, Class_base* cls, uint32_t* nsRealID, bool* isborrowed, bool considerdynamic, ASWorker* wrk)
 {
-	throwError<TypeError>(kConstructOfNonFunctionError);
+	checkScriptInit();
+	return Class_base::findVariableByMultiname(name, cls, nsRealID, isborrowed, considerdynamic, wrk);
+}
+string Class_inherit::toDebugString() const
+{
+	std::string res = Class_base::toDebugString();
+	res += "tag=";
+	char buf[100];
+	sprintf(buf,"%u",tag ? tag->getId():UINT32_MAX);
+	res += buf;
+	return res;
+}
+template<>
+void Class<Global>::getInstance(ASWorker* worker,asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass)
+{
+	createError<TypeError>(worker,kConstructOfNonFunctionError);
 }
 
 void lightspark::lookupAndLink(Class_base* c, const tiny_string& name, const tiny_string& interfaceNs)
@@ -272,7 +318,7 @@ void lightspark::lookupAndLink(Class_base* c, uint32_t nameID, uint32_t interfac
 	}
 }
 
-void Class<ASObject>::getInstance(asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass)
+void Class<ASObject>::getInstance(ASWorker* worker, asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass)
 {
 	if (construct && args && argslen == 1 && this == Class<ASObject>::getClass(this->getSystemState()))
 	{
@@ -296,7 +342,7 @@ void Class<ASObject>::getInstance(asAtom& ret, bool construct, asAtom* args, con
 	}
 	if(realClass==nullptr)
 		realClass=this;
-	ret=asAtomHandler::fromObjectNoPrimitive(new (realClass->memoryAccount) ASObject(realClass));
+	ret=asAtomHandler::fromObjectNoPrimitive(new (realClass->memoryAccount) ASObject(worker,realClass));
 	if(construct)
 		handleConstruction(ret,args,argslen,true);
 }
@@ -311,11 +357,12 @@ Class<ASObject>* Class<ASObject>::getClass(SystemState* sys)
 		//Create the class
 		QName name(s->getUniqueStringId(ClassName<ASObject>::name),s->getUniqueStringId(ClassName<ASObject>::ns));
 		MemoryAccount* m = s->allocateMemoryAccount(ClassName<ASObject>::name);
-		ret=new (m) Class<ASObject>(name, m);
+		ret=new (m) Class<ASObject>(name, ClassName<ASObject>::id, m);
+		ret->setWorker(s->worker);
 		ret->setSystemState(s);
 		ret->incRef();
 		*retAddr=ret;
-		ret->prototype = _MNR(new_objectPrototype(sys));
+		ret->prototype = _MNR(new_objectPrototype(sys->worker));
 		ASObject::sinit(ret);
 		ret->initStandardProps();
 	}
