@@ -31,7 +31,7 @@ private:
 	int lockcount;
 
 public:
-	ASMutex(Class_base* c);
+	ASMutex(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(_lock);
