@@ -18,6 +18,7 @@
 **************************************************************************/
 
 #include "scripting/flash/crypto/flashcrypto.h"
+#include "scripting/flash/utils/ByteArray.h"
 #include "scripting/argconv.h"
 #include <random>
 
@@ -26,9 +27,9 @@ using namespace lightspark;
 ASFUNCTIONBODY_ATOM(lightspark,generateRandomBytes)
 {
 	uint32_t numbytes;
-	ARG_UNPACK_ATOM (numbytes);
+	ARG_CHECK(ARG_UNPACK(numbytes));
 	std::independent_bits_engine<std::default_random_engine, CHAR_BIT, unsigned char> engine;
-	ByteArray *res = Class<ByteArray>::getInstanceS(sys);
+	ByteArray *res = Class<ByteArray>::getInstanceS(wrk);
 
     for (uint32_t i = 0; i < numbytes; i++)
         res->writeByte((uint8_t)engine());
