@@ -20,28 +20,132 @@
 #include <stack>
 #include "scripting/flash/display/BitmapContainer.h"
 #include "scripting/flash/display/flashdisplay.h"
+#include "scripting/flash/filters/flashfilters.h"
+#include "scripting/flash/geom/flashgeom.h"
 #include "backends/rendering.h"
 #include "backends/image.h"
+#include "backends/decoder.h"
 #include "swf.h"
 
 using namespace std;
 using namespace lightspark;
 
+extern void nanoVGDeleteImage(int image);
 BitmapContainer::BitmapContainer(MemoryAccount* m):stride(0),width(0),height(0),
-	data(reporter_allocator<uint8_t>(m))
+	data(reporter_allocator<uint8_t>(m)),
+	redMultiplier(1.0),greenMultiplier(1.0),blueMultiplier(1.0),alphaMultiplier(1.0),
+	redOffset(0),greenOffset(0),blueOffset(0),alphaOffset(0),
+	nanoVGImageHandle(-1),cachedCairoPattern(nullptr)
 {
 }
 
 BitmapContainer::~BitmapContainer()
 {
-        if (bitmaptexture.isValid())
-        {
-                RenderThread* rt = getSys()->getRenderThread();
-                if (rt)
-                {
-                        getSys()->getRenderThread()->releaseTexture(bitmaptexture);
-                }
-        }
+	if (bitmaptexture.isValid())
+	{
+		RenderThread* rt = getSys()->getRenderThread();
+		if (rt)
+		{
+			getSys()->getRenderThread()->releaseTexture(bitmaptexture);
+		}
+	}
+	if (nanoVGImageHandle != -1)
+		nanoVGDeleteImage(nanoVGImageHandle);
+	if (cachedCairoPattern)
+		cairo_pattern_destroy(cachedCairoPattern);
+		
+}
+
+uint8_t* BitmapContainer::applyColorTransform(ColorTransform *ctransform)
+{
+	if (ctransform->redMultiplier==1.0 &&
+		ctransform->greenMultiplier==1.0 &&
+		ctransform->blueMultiplier==1.0 &&
+		ctransform->alphaMultiplier==1.0 &&
+		ctransform->redOffset==0.0 &&
+		ctransform->greenOffset==0.0 &&
+		ctransform->blueOffset==0.0 &&
+		ctransform->alphaOffset==0.0)
+		return getData();
+	if (ctransform->redMultiplier==this->redMultiplier &&
+		ctransform->greenMultiplier==this->greenMultiplier &&
+		ctransform->blueMultiplier==this->blueMultiplier &&
+		ctransform->alphaMultiplier==this->alphaMultiplier &&
+		ctransform->redOffset==this->redOffset &&
+		ctransform->greenOffset==this->greenOffset &&
+		ctransform->blueOffset==this->blueOffset &&
+		ctransform->alphaOffset==this->alphaOffset)
+		return getDataColorTransformed();
+	redMultiplier=ctransform->redMultiplier;
+	greenMultiplier=ctransform->greenMultiplier;
+	blueMultiplier=ctransform->blueMultiplier;
+	alphaMultiplier=ctransform->alphaMultiplier;
+	redOffset=ctransform->redOffset;
+	greenOffset=ctransform->greenOffset;
+	blueOffset=ctransform->blueOffset;
+	alphaOffset=ctransform->alphaOffset;
+	return ctransform->applyTransformation(this);
+}
+
+uint8_t* BitmapContainer::applyColorTransform(number_t redMulti, number_t greenMulti, number_t blueMulti, number_t alphaMulti, number_t redOff, number_t greenOff, number_t blueOff, number_t alphaOff)
+{
+	if (redMulti==1.0 &&
+		greenMulti==1.0 &&
+		blueMulti==1.0 &&
+		alphaMulti==1.0 &&
+		redOff==0.0 &&
+		greenOff==0.0 &&
+		blueOff==0.0 &&
+		alphaOff==0.0)
+		return getData();
+	if (redMulti==this->redMultiplier &&
+		greenMulti==this->greenMultiplier &&
+		blueMulti==this->blueMultiplier &&
+		alphaMulti==this->alphaMultiplier &&
+		redOff==this->redOffset &&
+		greenOff==this->greenOffset &&
+		blueOff==this->blueOffset &&
+		alphaOff==this->alphaOffset)
+		return getDataColorTransformed();
+	redMultiplier=redMulti;
+	greenMultiplier=greenMulti;
+	blueMultiplier=blueMulti;
+	alphaMultiplier=alphaMulti;
+	redOffset=redOff;
+	greenOffset=greenOff;
+	blueOffset=blueOff;
+	alphaOffset=alphaOff;
+	uint8_t* src = getData();
+	uint8_t* dst = getDataColorTransformed();
+	uint32_t size = getWidth()*getHeight()*4;
+	for (uint32_t i = 0; i < size; i+=4)
+	{
+		dst[i+3] = max(0,min(255,int(((number_t(src[i+3]) * alphaMultiplier) + alphaOffset))));
+		dst[i+2] = max(0,min(255,int(((number_t(src[i+2]) *  blueMultiplier) +  blueOffset)*(number_t(dst[i+3])/255.0))));
+		dst[i+1] = max(0,min(255,int(((number_t(src[i+1]) * greenMultiplier) + greenOffset)*(number_t(dst[i+3])/255.0))));
+		dst[i  ] = max(0,min(255,int(((number_t(src[i  ]) *   redMultiplier) +   redOffset)*(number_t(dst[i+3])/255.0))));
+	}
+	return getDataColorTransformed();
+}
+
+uint8_t* BitmapContainer::getRectangleData(const RECT& sourceRect)
+{
+	RECT clippedSourceRect;
+	clipRect(sourceRect, clippedSourceRect);
+
+	int copyWidth = clippedSourceRect.Xmax - clippedSourceRect.Xmin;
+	int copyHeight = clippedSourceRect.Ymax - clippedSourceRect.Ymin;
+
+	int sx = clippedSourceRect.Xmin;
+	int sy = clippedSourceRect.Ymin;
+	uint8_t* res = new uint8_t[copyWidth * copyHeight * 4];
+	for (int i=0; i<copyHeight; i++)
+	{
+		memcpy(&res[i*copyWidth*4],
+				&data[(sy+i)*stride + 4*sx],
+				4*copyWidth);
+	}
+	return res;
 }
 
 bool BitmapContainer::fromRGB(uint8_t* rgb, uint32_t w, uint32_t h, BITMAP_FORMAT format, bool frompng)
@@ -98,7 +202,7 @@ bool BitmapContainer::fromPNG(std::istream &s)
 	bool hasAlpha;
 	uint8_t *rgb=ImageDecoder::decodePNG(s, &w, &h,&hasAlpha);
 	assert_and_throw((int32_t)w >= 0 && (int32_t)h >= 0);
-	BITMAP_FORMAT format=ARGB32; // libpng also returns ARGB32 for images without alpha channel
+	BITMAP_FORMAT format=hasAlpha ? ARGB32 : RGB24;
 	return fromRGB(rgb, (int32_t)w, (int32_t)h, format,true);
 }
 bool BitmapContainer::fromPNG(uint8_t* data, int len)
@@ -108,9 +212,36 @@ bool BitmapContainer::fromPNG(uint8_t* data, int len)
 	bool hasAlpha;
 	uint8_t *rgb=ImageDecoder::decodePNG(data,len, &w, &h,&hasAlpha);
 	assert_and_throw((int32_t)w >= 0 && (int32_t)h >= 0);
-	BITMAP_FORMAT format=ARGB32; // libpng also returns ARGB32 for images without alpha channel
+	BITMAP_FORMAT format=hasAlpha ? ARGB32 : RGB24;
 	return fromRGB(rgb, (int32_t)w, (int32_t)h, format,true);
 }
+bool BitmapContainer::fromGIF(uint8_t* data, int len, SystemState* sys)
+{
+#ifdef ENABLE_LIBAVCODEC
+	MemoryStreamCache gifdata(sys);
+	gifdata.append(data, len);
+	gifdata.markFinished();
+	std::streambuf *sbuf = gifdata.createReader();
+	istream s(sbuf);
+	FFMpegStreamDecoder* streamDecoder=new FFMpegStreamDecoder(nullptr,sys->getEngineData(),s,0,nullptr,gifdata.getReceivedLength());
+	if(streamDecoder->videoDecoder)
+	{
+		uint32_t w,h;
+		streamDecoder->videoDecoder->sizeNeeded(w,h);
+		// TODO how are GIFs with multiple frames handled?
+		if (streamDecoder->decodeNextFrame())
+		{
+			uint8_t* frame =streamDecoder->videoDecoder->upload(true);
+			return fromRGB(frame, (int32_t)w, (int32_t)h, ARGB32,true);
+		}
+	}
+	delete streamDecoder;
+	delete sbuf;
+#else
+	LOG(LOG_ERROR,"can't decode gif image because ffmpeg is not available");
+#endif
+	return false;
+}
 
 bool BitmapContainer::fromPalette(uint8_t* inData, uint32_t w, uint32_t h, uint32_t inStride, uint8_t* palette, unsigned numColors, unsigned paletteBPP)
 {
@@ -124,6 +255,17 @@ bool BitmapContainer::fromPalette(uint8_t* inData, uint32_t w, uint32_t h, uint3
 	return fromRGB(rgb, (int32_t)w, (int32_t)h, paletteBPP == 4 ? ARGB32 : RGB24);
 }
 
+void BitmapContainer::fromRawData(uint8_t* data, uint32_t width, uint32_t height)
+{
+	this->stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
+	this->width=width;
+	this->height=height;
+	uint32_t dataSize = stride * height;
+	this->data.resize(dataSize);
+	memcpy(this->data.data(),data,dataSize);
+	
+}
+
 void BitmapContainer::clear()
 {
 	data.clear();
@@ -134,39 +276,40 @@ void BitmapContainer::clear()
 	bitmaptexture.makeEmpty();
 }
 
-void BitmapContainer::upload(uint8_t *data, uint32_t w, uint32_t h)
+uint8_t* BitmapContainer::upload(bool refresh)
 {
-	memcpy(data, getData(), w*h*4);
+	return getData();
 }
 
-const TextureChunk &BitmapContainer::getTexture()
+TextureChunk& BitmapContainer::getTexture()
 {
 	return bitmaptexture;
 }
 
 void BitmapContainer::uploadFence()
 {
+	ITextureUploadable::uploadFence();
 	decRef();// is increffed in checkTexture
 }
 bool BitmapContainer::checkTexture()
 {
-    if (isEmpty()) {
-        return false;
-    }
+	if (isEmpty()) {
+		return false;
+	}
 
 	if (!bitmaptexture.isValid())
 	{
 		bitmaptexture=getSys()->getRenderThread()->allocateTexture(width, height, true);
 	}
 	incRef();// is decreffed in uploadFence
-    return true;
+	return true;
 }
 
 void BitmapContainer::setAlpha(int32_t x, int32_t y, uint8_t alpha)
 {
 	if (x < 0 || x >= width || y < 0 || y >= height)
 		return;
-
+	resetColorTransform();
 	uint32_t *p=reinterpret_cast<uint32_t *>(&data[y*stride + 4*x]);
 	*p = ((uint32_t)alpha << 24) + (*p & 0xFFFFFF);
 }
@@ -176,6 +319,7 @@ void BitmapContainer::setPixel(int32_t x, int32_t y, uint32_t color, bool setAlp
 	if (x < 0 || x >= width || y < 0 || y >= height)
 		return;
 
+	resetColorTransform();
 	uint32_t *p=reinterpret_cast<uint32_t *>(&data[y*stride + 4*x]);
 	if(setAlpha)
 	{
@@ -238,6 +382,7 @@ void BitmapContainer::copyRectangle(_R<BitmapContainer> source,
 
 	if (copyWidth <= 0 || copyHeight <= 0)
 		return;
+	resetColorTransform();
 	int sx = clippedSourceRect.Xmin;
 	int sy = clippedSourceRect.Ymin;
 	if (mergeAlpha==false)
@@ -271,22 +416,39 @@ void BitmapContainer::copyRectangle(_R<BitmapContainer> source,
 	}
 }
 
+void BitmapContainer::applyFilter(_R<BitmapContainer> source,
+				    const RECT& sourceRect,
+				    int32_t destX, int32_t destY,
+				    BitmapFilter* filter)
+{
+	RECT clippedSourceRect;
+	int32_t clippedX;
+	int32_t clippedY;
+	clipRect(source, sourceRect, destX, destY, clippedSourceRect, clippedX, clippedY);
+	filter->applyFilter(this,source.getPtr(),clippedSourceRect,destX,destY,1.0,1.0);
+}
+
 void BitmapContainer::fillRectangle(const RECT& inputRect, uint32_t color, bool useAlpha)
 {
 	RECT clippedRect;
 	clipRect(inputRect, clippedRect);
-
-	for(int32_t y=clippedRect.Ymin;y<clippedRect.Ymax;y++)
+	if (clippedRect.Ymin>=clippedRect.Ymax || clippedRect.Xmin>=clippedRect.Xmax)
+		return;
+	
+	resetColorTransform();
+	uint32_t realcolor = useAlpha ? color : (0xFF000000 | (color & 0xFFFFFF));
+	// fill first line
+	for(int32_t x=clippedRect.Xmin;x<clippedRect.Xmax;x++)
 	{
-		for(int32_t x=clippedRect.Xmin;x<clippedRect.Xmax;x++)
-		{
-			uint32_t offset=y*stride + x*4;
-			uint32_t* ptr=(uint32_t*)(getData()+offset);
-			if (useAlpha)
-				*ptr = color;
-			else
-				*ptr = 0xFF000000 | (color & 0xFFFFFF);
-		}
+		uint32_t offset=clippedRect.Ymin*stride + x*4;
+		uint32_t* ptr=(uint32_t*)(getData()+offset);
+		*ptr = realcolor;
+	}
+	// use memcpy to fill all other lines
+	for(int32_t y=clippedRect.Ymin+1;y<clippedRect.Ymax;y++)
+	{
+		uint32_t offset=y*stride + clippedRect.Xmin*4;
+		memcpy(getData()+offset,getData()+clippedRect.Ymin*stride+clippedRect.Xmin*4,(clippedRect.Xmax-clippedRect.Xmin)*4);
 	}
 }
 
@@ -327,6 +489,17 @@ inline uint32_t *BitmapContainer::getDataNoBoundsChecking(int32_t x, int32_t y)
 {
 	return (uint32_t*)&data[y*stride + 4*x];
 }
+void BitmapContainer::resetColorTransform()
+{
+	redMultiplier=1.0;
+	greenMultiplier=1.0;
+	blueMultiplier=1.0;
+	alphaMultiplier=1.0;
+	redOffset=0.0;
+	greenOffset=0.0;
+	blueOffset=0.0;
+	alphaOffset=0.0;
+}
 
 /*
  * Fill a connected area around (startX, startY) with the given color.
@@ -338,7 +511,7 @@ void BitmapContainer::floodFill(int32_t startX, int32_t startY, uint32_t color)
 {
 	struct LineSegment {
 		LineSegment(int32_t _x1, int32_t _x2, int32_t _y, int32_t _dy) 
-			: x1(_x1), x2(_x2), y(_y), dy(_dy) {};
+			: x1(_x1), x2(_x2), y(_y), dy(_dy) {}
 		int32_t x1; // leftmost filled point on last line
 		int32_t x2; // rightmost filled point on last line
 		int32_t y;  // y coordinate (may be invalid!)
@@ -350,6 +523,7 @@ void BitmapContainer::floodFill(int32_t startX, int32_t startY, uint32_t color)
 	if (startX < 0 || startX >= width || startY < 0 || startY >= height)
 		return;
 
+	resetColorTransform();
 	uint32_t seedColor = getPixel(startX, startY);
 
 	// Comment on the codeproject.com: "needed in some cases" ???
