@@ -22,10 +22,14 @@
 #include "scripting/argconv.h"
 #include "scripting/toplevel/toplevel.h"
 #include "scripting/flash/geom/flashgeom.h"
+#include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/Integer.h"
+#include "scripting/toplevel/UInteger.h"
 #include "scripting/toplevel/Vector.h"
 #include "scripting/flash/errors/flasherrors.h"
 #include "scripting/flash/utils/ByteArray.h"
 #include "scripting/flash/filters/flashfilters.h"
+#include "scripting/flash/system/flashsystem.h"
 #include "backends/rendering.h"
 #include "3rdparty/perlinnoise/PerlinNoise.hpp"
 
@@ -34,27 +38,27 @@
 using namespace lightspark;
 using namespace std;
 
-BitmapData::BitmapData(Class_base* c):ASObject(c,T_OBJECT,SUBTYPE_BITMAPDATA),pixels(_MR(new BitmapContainer(c->memoryAccount))),locked(0),transparent(true)
+BitmapData::BitmapData(ASWorker* wrk, Class_base* c):ASObject(wrk,c,T_OBJECT,SUBTYPE_BITMAPDATA),pixels(_MR(new BitmapContainer(c->memoryAccount))),locked(0),needsupload(true),transparent(true)
 {
 }
 
-BitmapData::BitmapData(Class_base* c, _R<BitmapContainer> b):ASObject(c,T_OBJECT,SUBTYPE_BITMAPDATA),pixels(b),locked(0),transparent(true)
+BitmapData::BitmapData(ASWorker* wrk,Class_base* c, _R<BitmapContainer> b):ASObject(wrk,c,T_OBJECT,SUBTYPE_BITMAPDATA),pixels(b),locked(0),needsupload(true),transparent(true)
 {
 	traitsInitialized = true;
 	constructIndicator = true;
 	constructorCallComplete = true;
 }
 
-BitmapData::BitmapData(Class_base* c, const BitmapData& other)
-  : ASObject(c,T_OBJECT,SUBTYPE_BITMAPDATA),pixels(other.pixels),locked(other.locked),transparent(other.transparent)
+BitmapData::BitmapData(ASWorker* wrk,Class_base* c, const BitmapData& other)
+  : ASObject(wrk,c,T_OBJECT,SUBTYPE_BITMAPDATA),pixels(other.pixels),locked(other.locked),needsupload(other.needsupload),transparent(other.transparent)
 {
 	traitsInitialized = other.traitsInitialized;
 	constructIndicator = other.constructIndicator;
 	constructorCallComplete = other.constructorCallComplete;
 }
 
-BitmapData::BitmapData(Class_base* c, uint32_t width, uint32_t height)
- : ASObject(c,T_OBJECT,SUBTYPE_BITMAPDATA),pixels(_MR(new BitmapContainer(c->memoryAccount))),locked(0),transparent(true)
+BitmapData::BitmapData(ASWorker* wrk,Class_base* c, uint32_t width, uint32_t height)
+ : ASObject(wrk,c,T_OBJECT,SUBTYPE_BITMAPDATA),pixels(_MR(new BitmapContainer(c->memoryAccount))),locked(0),needsupload(true),transparent(true)
 {
 	if (width!=0 && height!=0)
 	{
@@ -67,11 +71,12 @@ BitmapData::BitmapData(Class_base* c, uint32_t width, uint32_t height)
 void BitmapData::sinit(Class_base* c)
 {
 	CLASS_SETUP(c, ASObject, _constructor, CLASS_SEALED);
+	c->isReusable=true;
 	c->addImplementedInterface(InterfaceClass<IBitmapDrawable>::getClass(c->getSystemState()));
 	c->setDeclaredMethodByQName("draw","",Class<IFunction>::getFunction(c->getSystemState(),draw),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("dispose","",Class<IFunction>::getFunction(c->getSystemState(),dispose),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("getPixel","",Class<IFunction>::getFunction(c->getSystemState(),getPixel),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("getPixel32","",Class<IFunction>::getFunction(c->getSystemState(),getPixel32),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("getPixel","",Class<IFunction>::getFunction(c->getSystemState(),getPixel,2,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("getPixel32","",Class<IFunction>::getFunction(c->getSystemState(),getPixel32,2,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("setPixel","",Class<IFunction>::getFunction(c->getSystemState(),setPixel),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("setPixel32","",Class<IFunction>::getFunction(c->getSystemState(),setPixel32),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("copyPixels","",Class<IFunction>::getFunction(c->getSystemState(),copyPixels),NORMAL_METHOD,true);
@@ -107,16 +112,38 @@ void BitmapData::sinit(Class_base* c)
 	IBitmapDrawable::linkTraits(c);
 }
 
-void BitmapData::addUser(Bitmap* b)
+bool BitmapData::destruct()
+{
+	users.clear();
+	if (preparedforshutdown)
+		pixels.reset();
+	else
+		pixels = _MR(new BitmapContainer(getClass()->memoryAccount));
+	locked = 0;
+	transparent = true;
+	return ASObject::destruct();
+}
+
+void BitmapData::finalize()
+{
+	users.clear();
+	pixels.reset();
+	ASObject::finalize();
+}
+
+void BitmapData::prepareShutdown()
+{
+	if (preparedforshutdown)
+		return;
+	pixels.reset();
+	ASObject::prepareShutdown();
+}
+void BitmapData::addUser(Bitmap* b, bool startupload)
 {
 	users.insert(b);
-	if (!pixels.isNull())
-	{
-		if (pixels->checkTexture())
-		{
-			getSystemState()->getRenderThread()->addUploadJob(this->pixels.getPtr());
-		}
-	}
+	if (!startupload)
+		return;
+	needsupload=true;
 	b->updatedData();
 }
 
@@ -125,18 +152,21 @@ void BitmapData::removeUser(Bitmap* b)
 	users.erase(b);
 }
 
-void BitmapData::notifyUsers() const
+void BitmapData::checkForUpload()
+{
+	if (!pixels.isNull() && needsupload)
+	{
+		if (pixels->checkTexture())
+			getSystemState()->getRenderThread()->addUploadJob(this->pixels.getPtr());
+		needsupload=false;
+	}
+}
+
+void BitmapData::notifyUsers()
 {
 	if (locked > 0 || users.empty())
 		return;
-
-	if (!pixels.isNull())
-	{
-		if (pixels->checkTexture())
-        {
-		    getSystemState()->getRenderThread()->addUploadJob(this->pixels.getPtr());
-        }
-	}
+	needsupload=true;
 	for(auto it=users.begin();it!=users.end();it++)
 		(*it)->updatedData();
 }
@@ -149,16 +179,25 @@ ASFUNCTIONBODY_ATOM(BitmapData,_constructor)
 	uint32_t fillColor;
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
-	ARG_UNPACK_ATOM(width, 0)(height, 0)(transparent, true)(fillColor, 0xFFFFFFFF);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
+	ARG_CHECK(ARG_UNPACK(width, 0)(height, 0)(transparent, true)(fillColor, 0xFFFFFFFF));
 
 	//If the bitmap is already initialized, just return
 	if(width==0 || height==0 || !th->pixels->isEmpty())
 		return;
 	if(width<0 || height<0)
-		throw Class<ArgumentError>::getInstanceS(sys,"invalid height or width", kInvalidArgumentError);
+	{
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"invalid height or width");
+		return;
+	}
 	if(width>8191 || height>8191)
-		throw Class<ArgumentError>::getInstanceS(sys,"invalid height or width", kInvalidArgumentError);
+	{
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"invalid height or width");
+		return;
+	}
 
 	uint32_t *pixelArray=new uint32_t[width*height];
 	uint32_t c=GUINT32_TO_BE(fillColor); // fromRGB expects big endian data
@@ -187,7 +226,7 @@ ASFUNCTIONBODY_ATOM(BitmapData,_constructor)
 	th->transparent=transparent;
 }
 
-ASFUNCTIONBODY_GETTER(BitmapData, transparent);
+ASFUNCTIONBODY_GETTER(BitmapData, transparent)
 
 ASFUNCTIONBODY_ATOM(BitmapData,dispose)
 {
@@ -196,49 +235,58 @@ ASFUNCTIONBODY_ATOM(BitmapData,dispose)
 	th->notifyUsers();
 }
 
-void BitmapData::drawDisplayObject(DisplayObject* d, const MATRIX& initialMatrix, bool smoothing)
+void BitmapData::drawDisplayObject(DisplayObject* d, const MATRIX& initialMatrix, bool smoothing, bool forCachedBitmap, AS_BLENDMODE blendMode, ColorTransform* ct)
 {
+	if (forCachedBitmap)
+		d->incRef();
 	//Create an InvalidateQueue to store all the hierarchy of objects that must be drawn
-	SoftwareInvalidateQueue queue;
+	SoftwareInvalidateQueue queue(forCachedBitmap ? _MNR(d):NullRef);
 	d->hasChanged=true;
 	d->requestInvalidation(&queue);
+	if (forCachedBitmap)
+	{
+		uint8_t* p = pixels->getData();
+		memset(p,0,pixels->getWidth()*pixels->getHeight()*4);
+	}
 	CairoRenderContext ctxt(pixels->getData(), pixels->getWidth(), pixels->getHeight(),smoothing);
-	map<uint32_t,pair<IDrawable*,uint8_t*>> drawablecache;
 	for(auto it=queue.queue.begin();it!=queue.queue.end();it++)
 	{
 		DisplayObject* target=(*it).getPtr();
-		if (target->legacy)
-		{
-			auto it2 = drawablecache.find(target->getTagID());
-			if (it2 != drawablecache.end())
-			{
-				CachedSurface& surface=ctxt.allocateCustomSurface(target,(*it2).second.second,false);
-				IDrawable* dr = (*it2).second.first;
-				surface.tex->width=dr->getWidth();
-				surface.tex->height=dr->getHeight();
-				surface.xOffset=dr->getXOffset();
-				surface.yOffset=dr->getYOffset();
-				surface.xOffsetTransformed=dr->getXOffsetTransformed();
-				surface.yOffsetTransformed=dr->getYOffsetTransformed();
-				surface.widthTransformed=dr->getWidthTransformed();
-				surface.heightTransformed=dr->getHeightTransformed();
-				surface.rotation=dr->getRotation();
-				surface.xscale = dr->getXScale();
-				surface.yscale = dr->getYScale();
-				continue;
-			}
-		}
 		//Get the drawable from each of the added objects
-		IDrawable* drawable=target->invalidate(d, initialMatrix,smoothing);
+		IDrawable* drawable=target->invalidate(d, initialMatrix,smoothing,&queue, nullptr);
 		if(drawable==nullptr)
 			continue;
-
+		if (forCachedBitmap && !target->isMask())
+			target->hasChanged=false;
 		//Compute the matrix for this object
 		bool isBufferOwner=true;
-		uint8_t* buf=drawable->getPixelBuffer(initialMatrix.getScaleX(),initialMatrix.getScaleY(),&isBufferOwner);
-		if (target->legacy)
-			drawablecache[target->getTagID()]=make_pair(drawable,buf);
-		
+		uint32_t bufsize=0;
+		uint8_t* buf=drawable->getPixelBuffer(&isBufferOwner,&bufsize);
+		if (!forCachedBitmap && !target->filters.isNull())
+		{
+			BitmapContainer bc(nullptr);
+			bc.fromRawData(buf,drawable->getWidth(),drawable->getHeight());
+			target->applyFilters(&bc,nullptr,RECT(0,bc.getWidth(),0,bc.getHeight()),0,0, drawable->getXScale(), drawable->getYScale());
+			memcpy(buf,bc.getData(),bufsize);
+		}
+		ColorTransform* colortransform = target->colorTransform.getPtr();
+		DisplayObjectContainer* p = target->getParent();
+		while (!colortransform && p && p!= d)
+		{
+			colortransform = p->colorTransform.getPtr();
+			p = p->getParent();
+		}
+		if (colortransform && !colortransform->isIdentity())
+		{
+			if (!isBufferOwner)
+			{
+				isBufferOwner=true;
+				uint8_t* buf2 = new uint8_t[bufsize];
+				memcpy(buf2,buf,bufsize);
+				buf=buf2;
+			}
+			colortransform->applyTransformation(buf,bufsize);
+		}
 		//Construct a CachedSurface using the data
 		CachedSurface& surface=ctxt.allocateCustomSurface(target,buf,isBufferOwner);
 		surface.tex->width=drawable->getWidth();
@@ -250,68 +298,111 @@ void BitmapData::drawDisplayObject(DisplayObject* d, const MATRIX& initialMatrix
 		surface.widthTransformed=drawable->getWidthTransformed();
 		surface.heightTransformed=drawable->getHeightTransformed();
 		surface.rotation=drawable->getRotation();
+		surface.alpha = drawable->getAlpha();
 		surface.xscale = drawable->getXScale();
 		surface.yscale = drawable->getYScale();
-		if (!target->legacy)
-			delete drawable;
-	}
-	auto it3 = drawablecache.begin();
-	while (it3 != drawablecache.end())
-	{
-		delete (*it3).second.first;
-		it3++;
+		surface.tex->xContentScale = drawable->getXContentScale();
+		surface.tex->yContentScale = drawable->getYContentScale();
+		surface.tex->xOffset = surface.xOffset;
+		surface.tex->yOffset = surface.yOffset;
+		surface.isMask=drawable->getIsMask();
+		surface.mask=drawable->getMask();
+		surface.matrix = drawable->getMatrix();
+		if (!forCachedBitmap)
+			surface.blendmode=blendMode;
+		surface.isValid=true;
+		surface.isInitialized=true;
+		delete drawable;
 	}
+	if (d->getMask())
+		d->getMask()->Render(ctxt,true);
 	d->Render(ctxt,true);
+	if (ct && !ct->isIdentity())
+		ct->applyTransformation(pixels.getPtr());
 }
 
 ASFUNCTIONBODY_ATOM(BitmapData,draw)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
 	_NR<ASObject> drawable;
 	_NR<Matrix> matrix;
 	_NR<ColorTransform> ctransform;
-	tiny_string blendMode;
+	asAtom blendMode;
 	_NR<Rectangle> clipRect;
 	bool smoothing;
-	ARG_UNPACK_ATOM (drawable) (matrix, NullRef) (ctransform, NullRef) (blendMode, "")
-					(clipRect, NullRef) (smoothing, false);
+	ARG_CHECK(ARG_UNPACK(drawable) (matrix, NullRef) (ctransform, NullRef) (blendMode, asAtomHandler::fromStringID(BUILTIN_STRINGS::EMPTY))
+					(clipRect, NullRef) (smoothing, false));
 
-	if(!drawable->getClass() || !drawable->getClass()->isSubClass(InterfaceClass<IBitmapDrawable>::getClass(sys)) )
-		throwError<TypeError>(kCheckTypeFailedError, 
-				      drawable->getClassName(),
-				      "IBitmapDrawable");
-
-	if(!ctransform.isNull() || !(blendMode.empty() || blendMode == "null") || !clipRect.isNull())
-		LOG(LOG_NOT_IMPLEMENTED,"BitmapData.draw does not support many parameters:"<<ctransform.isNull()<<" "<<clipRect.isNull()<<" "<<blendMode);
+	if(!drawable->getClass() || !drawable->getClass()->isSubClass(InterfaceClass<IBitmapDrawable>::getClass(wrk->getSystemState())) )
+	{
+		if (wrk->rootClip->needsActionScript3())
+			createError<TypeError>(wrk,kCheckTypeFailedError, 
+								   drawable->getClassName(),
+								   "IBitmapDrawable");
+		return;
+	}
 
+	uint32_t blendModeID = BUILTIN_STRINGS::EMPTY;
+	if (asAtomHandler::isStringID(blendMode))
+		blendModeID = asAtomHandler::toStringId(blendMode,wrk);
 	if(drawable->is<BitmapData>())
 	{
+		if(blendModeID != BUILTIN_STRINGS::EMPTY)
+			LOG(LOG_NOT_IMPLEMENTED,"BitmapData.draw does not support blendMode parameter:"<<drawable->toDebugString()<<" "<<wrk->getSystemState()->getStringFromUniqueId(blendModeID));
 		BitmapData* data=drawable->as<BitmapData>();
 		//Compute the initial matrix, if any
 		MATRIX initialMatrix;
 		if(!matrix.isNull())
 			initialMatrix=matrix->getMATRIX();
-		CairoRenderContext ctxt(th->pixels->getData(), th->pixels->getWidth(), th->pixels->getHeight(),smoothing);
+		CairoRenderContext ctxt(th->pixels->getData(), th->pixels->getWidth(), th->pixels->getHeight(),false);
 		//Blit the data while transforming it
-		ctxt.transformedBlit(initialMatrix, data->pixels->getData(),
-				data->pixels->getWidth(), data->pixels->getHeight(),
-				CairoRenderContext::FILTER_NONE);
+		if (clipRect.isNull())
+		{
+			ctxt.transformedBlit(initialMatrix, data->pixels.getPtr(),ctransform.getPtr(),
+								smoothing ? CairoRenderContext::FILTER_SMOOTH : CairoRenderContext::FILTER_NONE,0,0,data->pixels->getWidth(), data->pixels->getHeight());
+		}
+		else
+		{
+			ctxt.transformedBlit(initialMatrix, data->pixels.getPtr(),ctransform.getPtr(),
+								smoothing ? CairoRenderContext::FILTER_SMOOTH : CairoRenderContext::FILTER_NONE,clipRect->x,clipRect->y,clipRect->width,clipRect->height);
+		}
 	}
 	else if(drawable->is<DisplayObject>())
 	{
+		if(!clipRect.isNull())
+			LOG(LOG_NOT_IMPLEMENTED,"BitmapData.draw does not support clipRect parameter:"<<drawable->toDebugString()<<" "<<clipRect->x<<"/"<<clipRect->y<<" "<<clipRect->width<<"/"<<clipRect->height);
 		DisplayObject* d=drawable->as<DisplayObject>();
 		//Compute the initial matrix, if any
 		MATRIX initialMatrix;
 		if(!matrix.isNull())
 			initialMatrix=matrix->getMATRIX();
-		th->drawDisplayObject(d, initialMatrix,smoothing);
+		AS_BLENDMODE bl = BLENDMODE_NORMAL;
+		switch(blendModeID)
+		{
+			case BUILTIN_STRINGS::STRING_ADD: bl = BLENDMODE_ADD; break;
+			case BUILTIN_STRINGS::STRING_ALPHA: bl = BLENDMODE_ALPHA; break;
+			case BUILTIN_STRINGS::STRING_DARKEN: bl = BLENDMODE_DARKEN; break;
+			case BUILTIN_STRINGS::STRING_DIFFERENCE: bl = BLENDMODE_DIFFERENCE; break;
+			case BUILTIN_STRINGS::STRING_ERASE: bl = BLENDMODE_ERASE; break;
+			case BUILTIN_STRINGS::STRING_HARDLIGHT: bl = BLENDMODE_HARDLIGHT; break;
+			case BUILTIN_STRINGS::STRING_INVERT: bl = BLENDMODE_INVERT; break;
+			case BUILTIN_STRINGS::STRING_LAYER: bl = BLENDMODE_LAYER; break;
+			case BUILTIN_STRINGS::STRING_LIGHTEN: bl = BLENDMODE_LIGHTEN; break;
+			case BUILTIN_STRINGS::STRING_MULTIPLY: bl = BLENDMODE_MULTIPLY; break;
+			case BUILTIN_STRINGS::STRING_OVERLAY: bl = BLENDMODE_OVERLAY; break;
+			case BUILTIN_STRINGS::STRING_SCREEN: bl = BLENDMODE_SCREEN; break;
+			case BUILTIN_STRINGS::STRING_SUBTRACT: bl = BLENDMODE_SUBTRACT; break;
+		}
+		d->DrawToBitmap(th,initialMatrix,true,false,bl,ctransform.getPtr());
 	}
 	else
 		LOG(LOG_NOT_IMPLEMENTED,"BitmapData.draw does not support " << drawable->toDebugString());
-
 	th->notifyUsers();
 }
 
@@ -319,37 +410,46 @@ ASFUNCTIONBODY_ATOM(BitmapData,getPixel)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	int32_t x;
 	int32_t y;
-	ARG_UNPACK_ATOM(x)(y);
+	ARG_CHECK(ARG_UNPACK(x)(y));
 
 	uint32_t pix=th->pixels->getPixel(x, y,false);
-	asAtomHandler::setUInt(ret,sys,pix & 0xffffff);
+	asAtomHandler::setUInt(ret,wrk,pix & 0xffffff);
 }
 
 ASFUNCTIONBODY_ATOM(BitmapData,getPixel32)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	int32_t x;
 	int32_t y;
-	ARG_UNPACK_ATOM(x)(y);
+	ARG_CHECK(ARG_UNPACK(x)(y));
 
 	uint32_t pix=th->pixels->getPixel(x, y,false);
-	asAtomHandler::setUInt(ret,sys,pix);
+	asAtomHandler::setUInt(ret,wrk,pix);
 }
 
 ASFUNCTIONBODY_ATOM(BitmapData,setPixel)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	int32_t x;
 	int32_t y;
 	uint32_t color;
-	ARG_UNPACK_ATOM(x)(y)(color);
+	ARG_CHECK(ARG_UNPACK(x)(y)(color));
 
 	th->pixels->setPixel(x, y, color, false,false);
 	th->notifyUsers();
@@ -359,11 +459,14 @@ ASFUNCTIONBODY_ATOM(BitmapData,setPixel32)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	int32_t x;
 	int32_t y;
 	uint32_t color;
-	ARG_UNPACK_ATOM(x)(y)(color);
+	ARG_CHECK(ARG_UNPACK(x)(y)(color));
 
 	th->pixels->setPixel(x, y, color, th->transparent,false);
 	th->notifyUsers();
@@ -373,8 +476,11 @@ ASFUNCTIONBODY_ATOM(BitmapData,getRect)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
-	Rectangle *rect=Class<Rectangle>::getInstanceS(sys);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
+	Rectangle *rect=Class<Rectangle>::getInstanceS(wrk);
 	rect->width=th->pixels->getWidth();
 	rect->height=th->pixels->getHeight();
 	ret = asAtomHandler::fromObject(rect);
@@ -384,16 +490,22 @@ ASFUNCTIONBODY_ATOM(BitmapData,_getHeight)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
-	asAtomHandler::setInt(ret,sys,th->getHeight());
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
+	asAtomHandler::setInt(ret,wrk,th->getHeight());
 }
 
 ASFUNCTIONBODY_ATOM(BitmapData,_getWidth)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
-	asAtomHandler::setInt(ret,sys,th->getWidth());
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
+	asAtomHandler::setInt(ret,wrk,th->getWidth());
 }
 
 ASFUNCTIONBODY_ATOM(BitmapData,fillRect)
@@ -401,12 +513,18 @@ ASFUNCTIONBODY_ATOM(BitmapData,fillRect)
 	BitmapData* th=asAtomHandler::as<BitmapData>(obj);
 	_NR<Rectangle> rect;
 	uint32_t color;
-	ARG_UNPACK_ATOM(rect)(color);
+	ARG_CHECK(ARG_UNPACK(rect)(color));
 
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	if (rect.isNull())
-		throwError<TypeError>(kNullPointerError, "rect");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "rect");
+		return;
+	}
 
 	if (th->transparent)
 	{
@@ -435,16 +553,28 @@ ASFUNCTIONBODY_ATOM(BitmapData,copyPixels)
 	_NR<BitmapData> alphaBitmapData;
 	_NR<Point> alphaPoint;
 	bool mergeAlpha;
-	ARG_UNPACK_ATOM(source)(sourceRect)(destPoint)(alphaBitmapData, NullRef)(alphaPoint, NullRef)(mergeAlpha,false);
+	ARG_CHECK(ARG_UNPACK(source)(sourceRect)(destPoint)(alphaBitmapData, NullRef)(alphaPoint, NullRef)(mergeAlpha,false));
 
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	if (source.isNull())
-		throwError<TypeError>(kNullPointerError, "source");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "source");
+		return;
+	}
 	if (sourceRect.isNull())
-		throwError<TypeError>(kNullPointerError, "sourceRect");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "sourceRect");
+		return;
+	}
 	if (destPoint.isNull())
-		throwError<TypeError>(kNullPointerError, "destPoint");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "destPoint");
+		return;
+	}
 
 	if(!alphaBitmapData.isNull())
 		LOG(LOG_NOT_IMPLEMENTED, "BitmapData.copyPixels doesn't support alpha bitmap");
@@ -460,8 +590,11 @@ ASFUNCTIONBODY_ATOM(BitmapData,generateFilterRect)
 	LOG(LOG_NOT_IMPLEMENTED,"BitmapData::generateFilterRect is just a stub");
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
-	Rectangle *rect=Class<Rectangle>::getInstanceS(sys);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
+	Rectangle *rect=Class<Rectangle>::getInstanceS(wrk);
 	rect->width=th->pixels->getWidth();
 	rect->height=th->pixels->getHeight();
 	ret = asAtomHandler::fromObject(rect);
@@ -471,15 +604,18 @@ ASFUNCTIONBODY_ATOM(BitmapData,hitTest)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
 	_NR<Point> firstPoint;
 	uint32_t firstAlphaThreshold;
 	_NR<ASObject> secondObject;
 	_NR<Point> secondBitmapDataPoint;
 	uint32_t secondAlphaThreshold;
-	ARG_UNPACK_ATOM (firstPoint) (firstAlphaThreshold) (secondObject) (secondBitmapDataPoint, NullRef)
-					(secondAlphaThreshold,1);
+	ARG_CHECK(ARG_UNPACK(firstPoint) (firstAlphaThreshold) (secondObject) (secondBitmapDataPoint, NullRef)
+					(secondAlphaThreshold,1));
 
 	asAtomHandler::setBool(ret,false);
 
@@ -551,7 +687,7 @@ ASFUNCTIONBODY_ATOM(BitmapData,hitTest)
 		}
 	}
 	else
-		throwError<TypeError>(kCheckTypeFailedError, 
+		createError<TypeError>(wrk,kCheckTypeFailedError, 
 				      secondObject->getClassName(),
 				      " Point, Rectangle, Bitmap, or BitmapData");
 	asAtomHandler::setBool(ret,false);
@@ -561,11 +697,14 @@ ASFUNCTIONBODY_ATOM(BitmapData,scroll)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
 	int x;
 	int y;
-	ARG_UNPACK_ATOM (x) (y);
+	ARG_CHECK(ARG_UNPACK(x) (y));
 
 	if (th->pixels->scroll(x, y))
 		th->notifyUsers();
@@ -576,30 +715,48 @@ ASFUNCTIONBODY_ATOM(BitmapData,clone)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
-	ret = asAtomHandler::fromObject(Class<BitmapData>::getInstanceS(sys,*th));
+	BitmapData* clone = Class<BitmapData>::getInstanceS(wrk,th->getWidth(),th->getHeight());
+	if (th->getBitmapContainer())
+		memcpy (clone->getBitmapContainer()->getData(),th->getBitmapContainer()->getData(),th->getWidth()*th->getHeight()*4);
+	ret = asAtomHandler::fromObject(clone);
 }
 
 ASFUNCTIONBODY_ATOM(BitmapData,copyChannel)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	
 	_NR<BitmapData> source;
 	_NR<Rectangle> sourceRect;
 	_NR<Point> destPoint;
 	uint32_t sourceChannel;
 	uint32_t destChannel;
-	ARG_UNPACK_ATOM (source) (sourceRect) (destPoint) (sourceChannel) (destChannel);
+	ARG_CHECK(ARG_UNPACK(source) (sourceRect) (destPoint) (sourceChannel) (destChannel));
 
 	if (source.isNull())
-		throwError<TypeError>(kNullPointerError, "source");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "source");
+		return;
+	}
 	if (sourceRect.isNull())
-		throwError<TypeError>(kNullPointerError, "sourceRect");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "sourceRect");
+		return;
+	}
 	if (destPoint.isNull())
-		throwError<TypeError>(kNullPointerError, "destPoint");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "destPoint");
+		return;
+	}
 
 	unsigned int sourceShift = BitmapDataChannel::channelShift(sourceChannel);
 	unsigned int destShift = BitmapDataChannel::channelShift(destChannel);
@@ -642,7 +799,10 @@ ASFUNCTIONBODY_ATOM(BitmapData,lock)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
 	th->locked++;
 }
@@ -651,7 +811,10 @@ ASFUNCTIONBODY_ATOM(BitmapData,unlock)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
 	if (th->locked > 0)
 	{
@@ -665,12 +828,15 @@ ASFUNCTIONBODY_ATOM(BitmapData,floodFill)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
 	int32_t x;
 	int32_t y;
 	uint32_t color;
-	ARG_UNPACK_ATOM (x) (y) (color);
+	ARG_CHECK(ARG_UNPACK(x) (y) (color));
 
 	if (!th->transparent)
 		color = 0xFF000000 | color;
@@ -683,10 +849,13 @@ ASFUNCTIONBODY_ATOM(BitmapData,histogram)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
 	_NR<Rectangle> inputRect;
-	ARG_UNPACK_ATOM (inputRect);
+	ARG_CHECK(ARG_UNPACK(inputRect));
 
 	RECT rect;
 	if (inputRect.isNull()) {
@@ -709,12 +878,13 @@ ASFUNCTIONBODY_ATOM(BitmapData,histogram)
 	}
 
 	asAtom v=asAtomHandler::invalidAtom;
-	Template<Vector>::getInstanceS(v,sys,Template<Vector>::getTemplateInstance(sys,Class<Number>::getClass(sys),NullRef).getPtr(),NullRef);
+	RootMovieClip* root = wrk->rootClip.getPtr();
+	Template<Vector>::getInstanceS(wrk,v,root,Template<Vector>::getTemplateInstance(root,Class<Number>::getClass(wrk->getSystemState()),NullRef).getPtr(),NullRef);
 	Vector *result = asAtomHandler::as<Vector>(v);
 	int channelOrder[4] = {2, 1, 0, 3}; // red, green, blue, alpha
 	for (int j=0; j<4; j++)
 	{
-		Template<Vector>::getInstanceS(v,sys,Class<Number>::getClass(sys),NullRef);
+		Template<Vector>::getInstanceS(wrk,v,root,Class<Number>::getClass(wrk->getSystemState()),NullRef);
 		Vector *histogram = asAtomHandler::as<Vector>(v);
 		for (int level=0; level<256; level++)
 		{
@@ -732,12 +902,15 @@ ASFUNCTIONBODY_ATOM(BitmapData,getColorBoundsRect)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
 	uint32_t mask;
 	uint32_t color;
 	bool findColor;
-	ARG_UNPACK_ATOM (mask) (color) (findColor, true);
+	ARG_CHECK(ARG_UNPACK(mask) (color) (findColor, true));
 
 	int xmin = th->getWidth();
 	int xmax = 0;
@@ -763,7 +936,7 @@ ASFUNCTIONBODY_ATOM(BitmapData,getColorBoundsRect)
 		}
 	}
 
-	Rectangle *bounds = Class<Rectangle>::getInstanceS(sys);
+	Rectangle *bounds = Class<Rectangle>::getInstanceS(wrk);
 	if ((xmin <= xmax) && (ymin <= ymax))
 	{
 		bounds->x = xmin;
@@ -778,15 +951,21 @@ ASFUNCTIONBODY_ATOM(BitmapData,getPixels)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	
 	_NR<Rectangle> rect;
-	ARG_UNPACK_ATOM (rect);
+	ARG_CHECK(ARG_UNPACK(rect));
 
 	if (rect.isNull())
-		throwError<TypeError>(kNullPointerError, "rect");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "rect");
+		return;
+	}
 
-	ByteArray *ba = Class<ByteArray>::getInstanceS(sys);
+	ByteArray *ba = Class<ByteArray>::getInstanceS(wrk);
 	vector<uint32_t> pixelvec = th->pixels->getPixelVector(rect->getRect());
 	vector<uint32_t>::const_iterator it;
 	for (it=pixelvec.begin(); it!=pixelvec.end(); ++it)
@@ -798,16 +977,23 @@ ASFUNCTIONBODY_ATOM(BitmapData,getVector)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	
 	_NR<Rectangle> rect;
-	ARG_UNPACK_ATOM (rect);
+	ARG_CHECK(ARG_UNPACK(rect));
 
 	if (rect.isNull())
-		throwError<TypeError>(kNullPointerError, "rect");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "rect");
+		return;
+	}
 
 	asAtom v=asAtomHandler::invalidAtom;
-	Template<Vector>::getInstanceS(v,sys,Class<UInteger>::getClass(sys),NullRef);
+	RootMovieClip* root = wrk->rootClip.getPtr();
+	Template<Vector>::getInstanceS(wrk,v,root,Class<UInteger>::getClass(wrk->getSystemState()),NullRef);
 	Vector *result = asAtomHandler::as<Vector>(v);
 	vector<uint32_t> pixelvec = th->pixels->getPixelVector(rect->getRect());
 	vector<uint32_t>::const_iterator it;
@@ -823,16 +1009,25 @@ ASFUNCTIONBODY_ATOM(BitmapData,setPixels)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	
 	_NR<Rectangle> inputRect;
 	_NR<ByteArray> inputByteArray;
-	ARG_UNPACK_ATOM (inputRect) (inputByteArray);
+	ARG_CHECK(ARG_UNPACK(inputRect) (inputByteArray));
 
 	if (inputRect.isNull())
-		throwError<TypeError>(kNullPointerError, "rect");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "rect");
+		return;
+	}
 	if (inputByteArray.isNull())
-		throwError<TypeError>(kNullPointerError, "inputByteArray");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "inputByteArray");
+		return;
+	}
 
 	RECT rect;
 	th->pixels->clipRect(inputRect->getRect(), rect);
@@ -843,26 +1038,39 @@ ASFUNCTIONBODY_ATOM(BitmapData,setPixels)
 		{
 			uint32_t pixel;
 			if (!inputByteArray->readUnsignedInt(pixel))
-				throwError<EOFError>(kEOFError);
+			{
+				createError<EOFError>(wrk,kEOFError);
+				return;
+			}
 			th->pixels->setPixel(x, y, pixel, th->transparent);
 		}
 	}
+	th->notifyUsers();
 }
 
 ASFUNCTIONBODY_ATOM(BitmapData,setVector)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 	
 	_NR<Rectangle> inputRect;
 	_NR<Vector> inputVector;
-	ARG_UNPACK_ATOM (inputRect) (inputVector);
+	ARG_CHECK(ARG_UNPACK(inputRect) (inputVector));
 
 	if (inputRect.isNull())
-		throwError<TypeError>(kNullPointerError, "rect");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "rect");
+		return;
+	}
 	if (inputVector.isNull())
-		throwError<TypeError>(kNullPointerError, "inputVector");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "inputVector");
+		return;
+	}
 
 	RECT rect;
 	th->pixels->clipRect(inputRect->getRect(), rect);
@@ -873,7 +1081,10 @@ ASFUNCTIONBODY_ATOM(BitmapData,setVector)
 		for (int32_t x=rect.Xmin; x<rect.Xmax; x++)
 		{
 			if (i >= inputVector->size())
-				throwError<RangeError>(kParamRangeError);
+			{
+				createError<RangeError>(wrk,kParamRangeError);
+				return;
+			}
 
 			asAtom v = inputVector->at(i);
 			uint32_t pixel = asAtomHandler::toUInt(v);
@@ -881,6 +1092,7 @@ ASFUNCTIONBODY_ATOM(BitmapData,setVector)
 			i++;
 		}
 	}
+	th->notifyUsers();
 }
 
 ASFUNCTIONBODY_ATOM(BitmapData,colorTransform)
@@ -889,25 +1101,27 @@ ASFUNCTIONBODY_ATOM(BitmapData,colorTransform)
 	
 	_NR<Rectangle> inputRect;
 	_NR<ColorTransform> inputColorTransform;
-	ARG_UNPACK_ATOM (inputRect) (inputColorTransform);
+	ARG_CHECK(ARG_UNPACK(inputRect) (inputColorTransform));
 
 	if (inputRect.isNull())
-		throwError<TypeError>(kNullPointerError, "rect");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "rect");
+		return;
+	}
 	if (inputColorTransform.isNull())
-		throwError<TypeError>(kNullPointerError, "inputVector");
-
+	{
+		createError<TypeError>(wrk,kNullPointerError, "inputColor");
+		return;
+	}
 	RECT rect;
 	th->pixels->clipRect(inputRect->getRect(), rect);
 	
-	vector<uint32_t> pixelvec = th->pixels->getPixelVector(rect);
-
 	unsigned int i = 0;
 	for (int32_t y=rect.Ymin; y<rect.Ymax; y++)
 	{
 		for (int32_t x=rect.Xmin; x<rect.Xmax; x++)
 		{
-
-			uint32_t pixel = pixelvec[i];
+			uint32_t pixel = th->pixels->getPixel(x, y);;
 
 			int a, r, g, b;
 			a = ((pixel >> 24 )&0xff) * inputColorTransform->alphaMultiplier + inputColorTransform->alphaOffset;
@@ -935,19 +1149,22 @@ ASFUNCTIONBODY_ATOM(BitmapData,compare)
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	
 	_NR<BitmapData> otherBitmapData;
-	ARG_UNPACK_ATOM (otherBitmapData);
+	ARG_CHECK(ARG_UNPACK(otherBitmapData));
 
 	if (otherBitmapData.isNull())
-		throwError<TypeError>(kNullPointerError, "otherBitmapData");
+	{
+		createError<TypeError>(wrk,kNullPointerError, "otherBitmapData");
+		return;
+	}
 
 	if (th->getWidth() != otherBitmapData->getWidth())
 	{
-		asAtomHandler::setInt(ret,sys,-3);
+		asAtomHandler::setInt(ret,wrk,-3);
 		return;
 	}
 	if (th->getHeight() != otherBitmapData->getHeight())
 	{
-		asAtomHandler::setInt(ret,sys,-4);
+		asAtomHandler::setInt(ret,wrk,-4);
 		return;
 	}
 	RECT rect;
@@ -959,7 +1176,7 @@ ASFUNCTIONBODY_ATOM(BitmapData,compare)
 	vector<uint32_t> pixelvec = th->pixels->getPixelVector(rect);
 	vector<uint32_t> otherpixelvec = otherBitmapData->pixels->getPixelVector(rect);
 	
-	BitmapData* res = Class<BitmapData>::getInstanceS(sys,rect.Xmax,rect.Ymax);
+	BitmapData* res = Class<BitmapData>::getInstanceS(wrk,rect.Xmax,rect.Ymax);
 	unsigned int i = 0;
 	bool different = false;
 	for (int32_t y=rect.Ymin; y<rect.Ymax; y++)
@@ -985,33 +1202,51 @@ ASFUNCTIONBODY_ATOM(BitmapData,compare)
 		}
 	}
 	if (!different)
-		asAtomHandler::setInt(ret,sys,0);
+		asAtomHandler::setInt(ret,wrk,0);
 	else
 		ret = asAtomHandler::fromObject(res);
 }
 
 ASFUNCTIONBODY_ATOM(BitmapData,applyFilter)
 {
+	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	_NR<BitmapData> sourceBitmapData;
 	_NR<Rectangle> sourceRect;
 	_NR<Point> destPoint;
 	_NR<BitmapFilter> filter;
-	ARG_UNPACK_ATOM (sourceBitmapData)(sourceRect)(destPoint)(filter);
-	LOG(LOG_NOT_IMPLEMENTED,"BitmapData.applyFilter not implemented");
+	ARG_CHECK(ARG_UNPACK(sourceBitmapData)(sourceRect)(destPoint)(filter));
+	if (sourceBitmapData.isNull())
+		createError<TypeError>(wrk,kNullPointerError,"sourceBitmapData");
+	else if (sourceRect.isNull())
+		createError<TypeError>(wrk,kNullPointerError,"sourceRect");
+	else if (destPoint.isNull())
+		createError<TypeError>(wrk,kNullPointerError,"destPoint");
+	else if (filter.isNull())
+		createError<TypeError>(wrk,kNullPointerError,"filter");
+	else
+	{
+		th->pixels->applyFilter(sourceBitmapData->pixels, sourceRect->getRect(),
+				  destPoint->getX(), destPoint->getY(),
+				  filter.getPtr());
+		th->notifyUsers();
+	}
 }
 
 ASFUNCTIONBODY_ATOM(BitmapData,noise)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
 	int randomSeed;
 	unsigned int low;
 	unsigned int high;
 	unsigned int channelOptions;
 	bool grayScale;
-	ARG_UNPACK_ATOM(randomSeed)(low, 0) (high, 255) (channelOptions, 7) (grayScale, false);
+	ARG_CHECK(ARG_UNPACK(randomSeed)(low, 0) (high, 255) (channelOptions, 7) (grayScale, false));
 	
 	srand(randomSeed);
 
@@ -1047,7 +1282,10 @@ ASFUNCTIONBODY_ATOM(BitmapData,perlinNoise)
 {
 	BitmapData* th = asAtomHandler::as<BitmapData>(obj);
 	if(th->pixels.isNull())
-		throw Class<ArgumentError>::getInstanceS(sys,"Disposed BitmapData", 2015);
+	{
+		createError<ArgumentError>(wrk,2015,"Disposed BitmapData");
+		return;
+	}
 
 	number_t baseX;
 	number_t baseY;
@@ -1058,7 +1296,7 @@ ASFUNCTIONBODY_ATOM(BitmapData,perlinNoise)
 	unsigned int channelOptions;
 	bool grayScale;
 	_NR<Array> offsets;
-	ARG_UNPACK_ATOM(baseX)(baseY)(numOctaves)(randomSeed)(stitch) (fractalNoise) (channelOptions, 7) (grayScale, false) (offsets, NullRef);
+	ARG_CHECK(ARG_UNPACK(baseX)(baseY)(numOctaves)(randomSeed)(stitch) (fractalNoise) (channelOptions, 7) (grayScale, false) (offsets, NullRef));
 
 	if (stitch)
 		LOG(LOG_NOT_IMPLEMENTED,"perlinNoise: parameter stitch is ignored");
@@ -1117,7 +1355,7 @@ ASFUNCTIONBODY_ATOM(BitmapData,threshold)
 	uint32_t color;
 	uint32_t mask;
 	bool copySource;
-	ARG_UNPACK_ATOM(sourceBitmapData)(sourceRect)(destPoint)(operation)(threshold) (color,0) (mask, 0xFFFFFFFF) (copySource, false);
+	ARG_CHECK(ARG_UNPACK(sourceBitmapData)(sourceRect)(destPoint)(operation)(threshold) (color,0) (mask, 0xFFFFFFFF) (copySource, false));
 
 	LOG(LOG_NOT_IMPLEMENTED,"BitmapData.threshold not implemented");
 }
@@ -1130,7 +1368,7 @@ ASFUNCTIONBODY_ATOM(BitmapData,merge)
 	uint32_t greenMultiplier;
 	uint32_t blueMultiplier;
 	uint32_t alphaMultiplier;
-	ARG_UNPACK_ATOM(sourceBitmapData)(sourceRect) (destPoint) (redMultiplier) (greenMultiplier) (blueMultiplier) (alphaMultiplier);
+	ARG_CHECK(ARG_UNPACK(sourceBitmapData)(sourceRect) (destPoint) (redMultiplier) (greenMultiplier) (blueMultiplier) (alphaMultiplier));
 
 	LOG(LOG_NOT_IMPLEMENTED,"BitmapData.merge not implemented");
 }
@@ -1145,7 +1383,7 @@ ASFUNCTIONBODY_ATOM(BitmapData,paletteMap)
 	_NR<Array> greenArray;
 	_NR<Array> blueArray;
 	_NR<Array> alphaArray;
-	ARG_UNPACK_ATOM(sourceBitmapData)(sourceRect) (destPoint) (redArray, NullRef) (greenArray, NullRef) (blueArray, NullRef) (alphaArray, NullRef);
+	ARG_CHECK(ARG_UNPACK(sourceBitmapData)(sourceRect) (destPoint) (redArray, NullRef) (greenArray, NullRef) (blueArray, NullRef) (alphaArray, NullRef));
 
 	LOG(LOG_NOT_IMPLEMENTED,"BitmapData.paletteMap not implemented");
 }
