@@ -24,6 +24,7 @@
 #include "scripting/flash/display/IBitmapDrawable.h"
 #include "asobject.h"
 #include "scripting/flash/events/flashevents.h"
+#include "backends/graphics.h"
 
 namespace lightspark
 {
@@ -48,13 +49,15 @@ friend class Loader;
 friend class TextField;
 friend class Shape;
 friend class Bitmap;
+friend class CairoRenderer;
+friend class Graphics;
 friend std::ostream& operator<<(std::ostream& s, const DisplayObject& r);
 public:
 	enum HIT_TYPE { GENERIC_HIT, // point is over the object
-                        GENERIC_HIT_INVISIBLE, // ...even if the object is invisible
-			MOUSE_CLICK, // point over the object and mouseEnabled
-			DOUBLE_CLICK // point over the object and doubleClickEnabled
-		      };
+					GENERIC_HIT_INVISIBLE, // ...even if the object is invisible
+					MOUSE_CLICK, // point over the object and mouseEnabled
+					DOUBLE_CLICK // point over the object and doubleClickEnabled
+				  };
 private:
 	ASPROPERTY_GETTER_SETTER(_NR<AccessibilityProperties>,accessibilityProperties);
 	static ATOMIC_INT32(instanceCount);
@@ -66,16 +69,13 @@ private:
 	AS_BLENDMODE blendMode;
 	// if true, this displayobject is the root object of a loaded file (swf or image)
 	bool isLoadedRoot;
+	bool ismask;
 public:
 	UI16_SWF Ratio;
 	int ClipDepth;
+	DisplayObject* avm1PrevDisplayObject;
+	DisplayObject* avm1NextDisplayObject;
 private:
-	/**
-	  	The object we are masking, if any
-	*/
-	_NR<DisplayObject> maskOf;
-	void becomeMaskOf(_NR<DisplayObject> m);
-
 	// the parent is not handled as a _NR<DisplayObjectContainer> because that will lead to circular dependencies in refcounting
 	// and the parent can never be destructed
 	DisplayObjectContainer* parent;
@@ -91,14 +91,19 @@ private:
 	 * Also used by Transform
 	 */
 	void setMatrix(_NR<Matrix> m);
+	void setMatrix3D(_NR<Matrix3D> m);
 	ACQUIRE_RELEASE_FLAG(constructed);
 	bool useLegacyMatrix;
 	bool needsTextureRecalculation;
+	bool needsNeedsCachedBitmapRecalculation;
 	bool textureRecalculationSkippable;
-	void gatherMaskIDrawables(std::vector<IDrawable::MaskData>& masks) const;
+	void gatherMaskIDrawables(std::vector<IDrawable::MaskData>& masks);
 	std::map<uint32_t,asAtom> avm1variables;
-	std::map<uint32_t,_NR<AVM1Function>> avm1functions;
+	uint32_t avm1mouselistenercount;
+	uint32_t avm1framelistenercount;
 protected:
+	_NR<Bitmap> cachedBitmap;
+	_NR<Rectangle> scalingGrid;
 	std::multimap<uint32_t,_NR<DisplayObject>> variablebindings;
 	bool onStage;
 	bool visible;
@@ -109,7 +114,7 @@ protected:
 	_NR<DisplayObject> mask;
 	mutable Mutex spinlock;
 	void computeBoundsForTransformedRect(number_t xmin, number_t xmax, number_t ymin, number_t ymax,
-			int32_t& outXMin, int32_t& outYMin, uint32_t& outWidth, uint32_t& outHeight,
+			number_t& outXMin, number_t& outYMin, number_t& outWidth, number_t& outHeight,
 			const MATRIX& m) const;
 	/*
 	 * Assume the lock is held and the matrix will not change
@@ -117,23 +122,24 @@ protected:
 	void extractValuesFromMatrix();
 	number_t computeWidth();
 	number_t computeHeight();
+	void geometryChanged();
 	bool skipRender() const;
 
-	bool defaultRender(RenderContext& ctxt) const;
-	virtual bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const
+	bool defaultRender(RenderContext& ctxt);
+	virtual bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax)
 	{
 		throw RunTimeException("DisplayObject::boundsRect: Derived class must implement this!");
 	}
-	virtual bool boundsRectWithoutChildren(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const
+	virtual bool boundsRectWithoutChildren(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax)
 	{
 		return boundsRect(xmin, xmax, ymin, ymax);
 	}
-	bool boundsRectGlobal(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const;
-	virtual bool renderImpl(RenderContext& ctxt) const
+	bool boundsRectGlobal(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax);
+	virtual bool renderImpl(RenderContext& ctxt)
 	{
 		throw RunTimeException("DisplayObject::renderImpl: Derived class must implement this!");
 	}
-	virtual _NR<DisplayObject> hitTestImpl(_NR<DisplayObject> last, number_t x, number_t y, HIT_TYPE type,bool interactiveObjectsOnly)
+	virtual _NR<DisplayObject> hitTestImpl(number_t x, number_t y, HIT_TYPE type,bool interactiveObjectsOnly)
 	{
 		throw RunTimeException("DisplayObject::hitTestImpl: Derived class must implement this!");
 	}
@@ -144,14 +150,26 @@ public:
 	AS_BLENDMODE getBlendMode() const { return blendMode; }
 	void constructionComplete() override;
 	void afterConstruction() override;
+	void prepareDestruction()
+	{
+		destroyContents();
+		setParent(nullptr);
+		removeAVM1Listeners();
+	}
+	void applyFilters(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos,number_t scalex,number_t scaley);
 	_NR<DisplayObject> invalidateQueueNext;
 	_NR<LoaderInfo> loaderInfo;
 	ASPROPERTY_GETTER_SETTER(_NR<Array>,filters);
 	ASPROPERTY_GETTER_SETTER(_NR<Rectangle>,scrollRect);
 	_NR<ColorTransform> colorTransform;
+	// pointer to the ancestor of this DisplayObject that is cached as Bitmap
+	DisplayObject* cachedAsBitmapOf;
+	void invalidateCachedAsBitmapOf();
 	void setNeedsTextureRecalculation(bool skippable=false);
 	void resetNeedsTextureRecalculation() { needsTextureRecalculation=false; }
 	bool getNeedsTextureRecalculation() const { return needsTextureRecalculation; }
+	void setNeedsCachedBitmapRecalculation()  { needsNeedsCachedBitmapRecalculation=true; }
+	void resetNeedsCachedBitmapRecalculation() { needsNeedsCachedBitmapRecalculation=false; }
 	bool getTextureRecalculationSkippable() const { return textureRecalculationSkippable; }
 	// this may differ from the main clip if this instance was generated from a loaded swf, not from the main clip
 	RootMovieClip* loadedFrom;
@@ -159,21 +177,29 @@ public:
 	bool hasChanged;
 	// this is set to true for DisplayObjects that are placed from a tag
 	bool legacy;
+	bool markedForLegacyDeletion;
 	/**
 	 * cacheAsBitmap is true also if any filter is used
 	 */
-	bool computeCacheAsBitmap() const;
-	void computeMasksAndMatrix(const DisplayObject *target, std::vector<IDrawable::MaskData>& masks, MATRIX& totalMatrix, bool includeRotation, bool &isMask, bool &hasMask) const;
+	bool computeCacheAsBitmap(bool checksize=true);
+	bool requestInvalidationForCacheAsBitmap(InvalidateQueue* q);
+	void computeMasksAndMatrix(const DisplayObject *target, std::vector<IDrawable::MaskData>& masks, MATRIX& totalMatrix, bool includeRotation, bool &isMask, _NR<DisplayObject>& mask) const;
 	ASPROPERTY_GETTER_SETTER(bool,cacheAsBitmap);
+	IDrawable* getCachedBitmapDrawable(DisplayObject* target, const MATRIX& initialMatrix, _NR<DisplayObject>* pcachedBitmap);
+	_NR<DisplayObject> getCachedBitmap() const { return cachedBitmap; }
 	DisplayObjectContainer* getParent() const { return parent; }
 	bool findParent(DisplayObject* d) const;
 	void setParent(DisplayObjectContainer* p);
+	void setScalingGrid();
 	/*
 	   Used to link DisplayObjects the invalidation queue
 	*/
-	DisplayObject(Class_base* c);
+	DisplayObject(ASWorker* wrk, Class_base* c);
+	virtual void markAsChanged();
 	void finalize() override;
 	bool destruct() override;
+	void prepareShutdown() override;
+	bool countCylicMemberReferences(garbagecollectorstate& gcstate) override;
 	MATRIX getMatrix(bool includeRotation = true) const;
 	bool isConstructed() const override { return ACQUIRE_READ(constructed); }
 	/**
@@ -182,10 +208,10 @@ public:
 	 * _must_ be on the parent chain of this
 	 * @param initialMatrix A matrix that will be prepended to all transformations
 	 */
-	virtual IDrawable* invalidate(DisplayObject* target, const MATRIX& initialMatrix, bool smoothing);
+	virtual IDrawable* invalidate(DisplayObject* target, const MATRIX& initialMatrix, bool smoothing, InvalidateQueue* q, _NR<DisplayObject>* cachedBitmap);
 	virtual void requestInvalidation(InvalidateQueue* q, bool forceTextureRefresh=false);
 	void updateCachedSurface(IDrawable* d);
-	MATRIX getConcatenatedMatrix() const;
+	MATRIX getConcatenatedMatrix(bool includeRoot=false) const;
 	void localToGlobal(number_t xin, number_t yin, number_t& xout, number_t& yout) const;
 	void globalToLocal(number_t xin, number_t yin, number_t& xout, number_t& yout) const;
 	float getConcatenatedAlpha() const;
@@ -193,8 +219,8 @@ public:
 	{
 		throw RunTimeException("DisplayObject::getScaleFactor");
 	}
-	multiname* setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool* alreadyset=nullptr) override;
-	bool deleteVariableByMultiname(const multiname& name) override;
+	multiname* setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool* alreadyset, ASWorker* wrk) override;
+	bool deleteVariableByMultiname(const multiname& name, ASWorker* wrk) override;
 	virtual void removeAVM1Listeners();
 	void AVM1registerPrototypeListeners();
 
@@ -206,19 +232,18 @@ public:
 	virtual void UpdateVariableBinding(asAtom v) {}
 	
 	tiny_string AVM1GetPath();
-	virtual void afterLegacyInsert() {}
-	virtual void afterLegacyDelete(DisplayObjectContainer* parent) {}
-	virtual uint32_t getTagID() const { return 0;}
-	virtual void resetLegacyState() {}
+	virtual void afterLegacyInsert();
+	virtual void afterLegacyDelete(DisplayObjectContainer* parent,bool inskipping) {}
+	virtual uint32_t getTagID() const { return UINT32_MAX;}
 	virtual void startDrawJob() {}
 	virtual void endDrawJob() {}
 	
 	bool Render(RenderContext& ctxt,bool force=false);
-	bool getBounds(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax, const MATRIX& m) const;
-	_NR<DisplayObject> hitTest(_NR<DisplayObject> last, number_t x, number_t y, HIT_TYPE type,bool interactiveObjectsOnly);
-	virtual void setOnStage(bool staged, bool force = false);
+	bool getBounds(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax, const MATRIX& m);
+	_NR<DisplayObject> hitTest(number_t x, number_t y, HIT_TYPE type,bool interactiveObjectsOnly);
+	virtual void setOnStage(bool staged, bool force, bool inskipping=false);
 	bool isOnStage() const { return onStage; }
-	bool isMask() const { return !maskOf.isNull(); }
+	bool isMask() const { return ismask; }
 	// checks for visibility depending on parent visibility 
 	bool isVisible() const;
 	bool isLoadedRootObject() const { return isLoadedRoot; }
@@ -227,11 +252,14 @@ public:
 	virtual _NR<RootMovieClip> getRoot();
 	virtual _NR<Stage> getStage();
 	void setLegacyMatrix(const MATRIX& m);
-	virtual void advanceFrame() {}
-	virtual void declareFrame() {}
+	void setFilters(const FILTERLIST& filterlist);
+	virtual void advanceFrame(bool implicit) {}
+	virtual void declareFrame(bool implicit) {}
 	virtual void initFrame();
 	virtual void executeFrameScript();
 	virtual bool needsActionScript3() const;
+	virtual void handleMouseCursor(bool rollover) {}
+	virtual bool hasGraphics() const { return false; }
 	Vector2f getLocalMousePos();
 	Vector2f getXY();
 	void setX(number_t x);
@@ -244,8 +272,8 @@ public:
 	// Nominal width and heigt are the size before scaling and rotation
 	number_t getNominalWidth();
 	number_t getNominalHeight();
+	DisplayObject* getMask() const { return mask.getPtr(); }
 	static void sinit(Class_base* c);
-	static void buildTraits(ASObject* o);
 	ASFUNCTION_ATOM(_getVisible);
 	ASFUNCTION_ATOM(_setVisible);
 	ASFUNCTION_ATOM(_getStage);
@@ -315,12 +343,16 @@ public:
 	static void AVM1SetupMethods(Class_base* c);
 	DisplayObject* AVM1GetClipFromPath(tiny_string& path);
 	void AVM1SetVariable(tiny_string& name, asAtom v, bool setMember=true);
-	asAtom AVM1GetVariable(const tiny_string &name);
+	void AVM1SetVariableDirect(uint32_t nameId, asAtom v);
+	asAtom AVM1GetVariable(const tiny_string &name, bool checkrootvars=true);
 	void AVM1UpdateVariableBindings(uint32_t nameID, asAtom &value);
 	asAtom getVariableBindingValue(const tiny_string &name) override;
 	void setVariableBinding(tiny_string& name, _NR<DisplayObject> obj);
-	void AVM1SetFunction(uint32_t nameID, _NR<AVM1Function> obj);
+	void AVM1SetFunction(const tiny_string& name, _NR<AVM1Function> obj);
 	AVM1Function *AVM1GetFunction(uint32_t nameID);
+	virtual void AVM1AfterAdvance() {}
+	void DrawToBitmap(BitmapData* bm, const MATRIX& initialMatrix, bool smoothing, bool forcachedbitmap, AS_BLENDMODE blendMode, ColorTransform* ct);
+	std::string toDebugString() const override;
 };
 }
 #endif /* SCRIPTING_FLASH_DISPLAY_DISPLAYOBJECT_H */
