@@ -30,7 +30,7 @@ namespace lightspark
 class GraphicsEndFill: public ASObject, public IGraphicsFill, public IGraphicsData
 {
 public:
-	GraphicsEndFill(Class_base* c);
+	GraphicsEndFill(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base* c);
 	FILLSTYLE toFillStyle();
 	void appendToTokens(std::vector<uint64_t>& tokens,Graphics* graphics);
