@@ -17,16 +17,19 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
+#include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/Integer.h"
 #include "scripting/toplevel/Vector.h"
 #include "scripting/flash/display/GraphicsPath.h"
 #include "scripting/flash/display/Graphics.h"
+#include "scripting/flash/system/flashsystem.h"
 #include "scripting/argconv.h"
 #include "scripting/class.h"
 
 using namespace lightspark;
 
-GraphicsPath::GraphicsPath(Class_base* c):
-	ASObject(c), winding("evenOdd")
+GraphicsPath::GraphicsPath(ASWorker* wrk, Class_base* c):
+	ASObject(wrk,c), winding("evenOdd")
 {
 }
 
@@ -48,16 +51,16 @@ void GraphicsPath::sinit(Class_base* c)
 	IGraphicsData::linkTraits(c);
 }
 
-ASFUNCTIONBODY_GETTER_SETTER(GraphicsPath, commands);
-ASFUNCTIONBODY_GETTER_SETTER(GraphicsPath, data);
-ASFUNCTIONBODY_GETTER_SETTER(GraphicsPath, winding);
+ASFUNCTIONBODY_GETTER_SETTER(GraphicsPath, commands)
+ASFUNCTIONBODY_GETTER_SETTER(GraphicsPath, data)
+ASFUNCTIONBODY_GETTER_SETTER(GraphicsPath, winding)
 
 ASFUNCTIONBODY_ATOM(GraphicsPath,_constructor)
 {
 	_NR<Vector> commands;
 	_NR<Vector> data;
 	GraphicsPath* th = asAtomHandler::as<GraphicsPath>(obj);
-	ARG_UNPACK_ATOM(commands, NullRef)(data, NullRef)(th->winding, "evenOdd");
+	ARG_CHECK(ARG_UNPACK(commands, NullRef)(data, NullRef)(th->winding, "evenOdd"));
 
 	if (!commands.isNull())
 		th->commands = commands;
@@ -74,16 +77,17 @@ void GraphicsPath::finalize()
 
 void GraphicsPath::ensureValid()
 {
+	RootMovieClip* root = getInstanceWorker()->rootClip.getPtr();
 	if (commands.isNull())
 	{
 		asAtom v=asAtomHandler::invalidAtom;
-		Template<Vector>::getInstanceS(v,getSystemState(),Class<Integer>::getClass(getSystemState()),NullRef);
+		Template<Vector>::getInstanceS(getInstanceWorker(),v,root,Class<Integer>::getClass(getSystemState()),NullRef);
 		commands = _MNR(asAtomHandler::as<Vector>(v));
 	}
 	if (data.isNull())
 	{
 		asAtom v=asAtomHandler::invalidAtom;
-		Template<Vector>::getInstanceS(v,getSystemState(),Class<Number>::getClass(getSystemState()),NullRef);
+		Template<Vector>::getInstanceS(getInstanceWorker(),v,root,Class<Number>::getClass(getSystemState()),NullRef);
 		data = _MNR(asAtomHandler::as<Vector>(v));
 	}
 }
@@ -95,7 +99,7 @@ ASFUNCTIONBODY_ATOM(GraphicsPath, curveTo)
 	asAtom cy = asAtomHandler::fromType(T_NUMBER);
 	asAtom ax = asAtomHandler::fromType(T_NUMBER);
 	asAtom ay = asAtomHandler::fromType(T_NUMBER);
-	ARG_UNPACK_ATOM (cx) (cy) (ax) (ay);
+	ARG_CHECK(ARG_UNPACK (cx) (cy) (ax) (ay));
 
 	th->ensureValid();
 	asAtom v = asAtomHandler::fromInt((int32_t)GraphicsPathCommand::CURVE_TO);
@@ -115,7 +119,7 @@ ASFUNCTIONBODY_ATOM(GraphicsPath, lineTo)
 	GraphicsPath* th=asAtomHandler::as<GraphicsPath>(obj);
 	asAtom x = asAtomHandler::fromType(T_NUMBER);
 	asAtom y = asAtomHandler::fromType(T_NUMBER);
-	ARG_UNPACK_ATOM (x) (y);
+	ARG_CHECK(ARG_UNPACK (x) (y));
 
 	th->ensureValid();
 	asAtom v = asAtomHandler::fromInt((int32_t)(GraphicsPathCommand::LINE_TO));
@@ -131,7 +135,7 @@ ASFUNCTIONBODY_ATOM(GraphicsPath, moveTo)
 	GraphicsPath* th=asAtomHandler::as<GraphicsPath>(obj);
 	asAtom x = asAtomHandler::fromType(T_NUMBER);
 	asAtom y = asAtomHandler::fromType(T_NUMBER);
-	ARG_UNPACK_ATOM (x) (y);
+	ARG_CHECK(ARG_UNPACK (x) (y));
 
 	th->ensureValid();
 	asAtom v = asAtomHandler::fromInt((int32_t)(GraphicsPathCommand::MOVE_TO));
@@ -147,12 +151,12 @@ ASFUNCTIONBODY_ATOM(GraphicsPath, wideLineTo)
 	GraphicsPath* th=asAtomHandler::as<GraphicsPath>(obj);
 	asAtom x = asAtomHandler::fromType(T_NUMBER);
 	asAtom y = asAtomHandler::fromType(T_NUMBER);
-	ARG_UNPACK_ATOM (x) (y);
+	ARG_CHECK(ARG_UNPACK (x) (y));
 
 	th->ensureValid();
 	asAtom v = asAtomHandler::fromInt((int32_t)(GraphicsPathCommand::LINE_TO));
 	th->commands->append(v);
-	asAtom n = asAtomHandler::fromNumber(sys,0.0,false);
+	asAtom n = asAtomHandler::fromNumber(wrk,0.0,false);
 	ASATOM_INCREF(n);
 	th->data->append(n);
 	ASATOM_INCREF(n);
@@ -168,12 +172,12 @@ ASFUNCTIONBODY_ATOM(GraphicsPath, wideMoveTo)
 	GraphicsPath* th=asAtomHandler::as<GraphicsPath>(obj);
 	asAtom x = asAtomHandler::fromType(T_NUMBER);
 	asAtom y = asAtomHandler::fromType(T_NUMBER);
-	ARG_UNPACK_ATOM (x) (y);
+	ARG_CHECK(ARG_UNPACK (x) (y));
 
 	th->ensureValid();
 	asAtom v = asAtomHandler::fromInt((int32_t)(GraphicsPathCommand::MOVE_TO));
 	th->commands->append(v);
-	asAtom n = asAtomHandler::fromNumber(sys,0.0,false);
+	asAtom n = asAtomHandler::fromNumber(wrk,0.0,false);
 	ASATOM_INCREF(n);
 	th->data->append(n);
 	ASATOM_INCREF(n);
