@@ -25,8 +25,8 @@
 
 using namespace lightspark;
 
-GraphicsShaderFill::GraphicsShaderFill(Class_base* c):
-	ASObject(c)
+GraphicsShaderFill::GraphicsShaderFill(ASWorker* wrk, Class_base* c):
+	ASObject(wrk,c)
 {
 }
 
@@ -52,11 +52,11 @@ void GraphicsShaderFill::finalize()
 ASFUNCTIONBODY_ATOM(GraphicsShaderFill,_constructor)
 {
 	GraphicsShaderFill* th = asAtomHandler::as<GraphicsShaderFill>(obj);
-	ARG_UNPACK_ATOM (th->shader, NullRef) (th->matrix, NullRef);
+	ARG_CHECK(ARG_UNPACK (th->shader, NullRef) (th->matrix, NullRef));
 }
 
-ASFUNCTIONBODY_GETTER_SETTER(GraphicsShaderFill, matrix);
-ASFUNCTIONBODY_GETTER_SETTER(GraphicsShaderFill, shader);
+ASFUNCTIONBODY_GETTER_SETTER(GraphicsShaderFill, matrix)
+ASFUNCTIONBODY_GETTER_SETTER(GraphicsShaderFill, shader)
 
 FILLSTYLE GraphicsShaderFill::toFillStyle()
 {
