@@ -32,9 +32,9 @@ class Vector;
 class GraphicsTrianglePath: public ASObject, public IGraphicsPath, public IGraphicsData
 {
 public:
-	GraphicsTrianglePath(Class_base* c);
+	GraphicsTrianglePath(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base* c);
-	void finalize();
+	void finalize() override;
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(tiny_string, culling);
 	ASPROPERTY_GETTER_SETTER(_NR<Vector>, indices);
