@@ -27,6 +27,7 @@ namespace lightspark
 {
 
 class Class_base;
+class Graphics;
 
 class IGraphicsData
 {
@@ -35,8 +36,8 @@ protected:
 public:
 	static void linkTraits(Class_base* c) {}
 	// Appends GeomTokens for drawing this object into tokens
-	virtual void appendToTokens(std::vector<_NR<GeomToken>, reporter_allocator<_NR<GeomToken>>>& tokens) = 0;
+	virtual void appendToTokens(std::vector<uint64_t>& tokens,Graphics* graphics) = 0;
 };
 
-};
+}
 #endif /* SCRIPTING_FLASH_DISPLAY_IGRAPHICSDATA_H */
