@@ -22,16 +22,16 @@
 
 #include "compat.h"
 #include "asobject.h"
-#include "backends/extscriptobject.h"
+#include "threading.h"
 #include <string>
-#include <SDL2/SDL_keyboard.h>
+#include <SDL2/SDL_keycode.h>
 #undef MOUSE_EVENT
 
 namespace lightspark
 {
 
 enum EVENT_TYPE { EVENT=0, BIND_CLASS, SHUTDOWN, SYNC, MOUSE_EVENT,
-	FUNCTION, EXTERNAL_CALL, CONTEXT_INIT, INIT_FRAME,
+	FUNCTION,FUNCTION_ASYNC, EXTERNAL_CALL, CONTEXT_INIT, INIT_FRAME,
 	FLUSH_INVALIDATION_QUEUE, ADVANCE_FRAME, PARSE_RPC_MESSAGE,EXECUTE_FRAMESCRIPT,TEXTINPUT_EVENT,IDLE_EVENT,AVM1INITACTION_EVENT,ROOTCONSTRUCTEDEVENT };
 
 class ABCContext;
@@ -42,12 +42,13 @@ class DisplayObject;
 class Responder;
 class GameInputDevice;
 class DefineSpriteTag;
+class ByteArray;
 
 class Event: public ASObject
 {
 public:
-	Event(Class_base* cb, const tiny_string& t = "Event", bool b=false, bool c=false, CLASS_SUBTYPE st=SUBTYPE_EVENT);
-	void finalize();
+	Event(ASWorker* wrk, Class_base* cb, const tiny_string& t = "Event", bool b=false, bool c=false, CLASS_SUBTYPE st=SUBTYPE_EVENT);
+	void finalize() override;
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
 	virtual void setTarget(asAtom t) {target = t; }
@@ -60,6 +61,8 @@ public:
 	ASPROPERTY_GETTER(bool,bubbles);
 	ASPROPERTY_GETTER(bool,cancelable);
 	bool defaultPrevented;
+	bool propagationStopped;
+	bool immediatePropagationStopped;
 	ACQUIRE_RELEASE_FLAG(queued); // indicates that this event was added to the event queue
 	ASPROPERTY_GETTER(uint32_t,eventPhase);
 	ASPROPERTY_GETTER(tiny_string,type);
@@ -83,7 +86,7 @@ private:
 	bool handled;
 public:
 	WaitableEvent(const tiny_string& t = "Event", bool b=false, bool c=false)
-		: Event(NULL,t,b,c,SUBTYPE_WAITABLE_EVENT), handled(false) {}
+		: Event(nullptr,nullptr,t,b,c,SUBTYPE_WAITABLE_EVENT), handled(false) {}
 	void wait();
 	void signal();
 	void finalize() override ;
@@ -92,7 +95,7 @@ public:
 class EventPhase: public ASObject
 {
 public:
-	EventPhase(Class_base* c):ASObject(c){}
+	EventPhase(ASWorker* wrk, Class_base* c):ASObject(wrk,c){}
 	enum
 	{
 		CAPTURING_PHASE = 1,
@@ -100,7 +103,6 @@ public:
 		BUBBLING_PHASE = 3
 	};
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o) {}
 };
 
 class KeyboardEvent: public Event
@@ -109,23 +111,21 @@ private:
 	Event* cloneImpl() const override ;
 
 	uint32_t modifiers;
+	uint32_t sdlScanCode;
 	ASPROPERTY_GETTER_SETTER(uint32_t, charCode);
 	ASPROPERTY_GETTER_SETTER(uint32_t, keyCode);
 	ASPROPERTY_GETTER_SETTER(uint32_t, keyLocation);
 	SDL_Keycode sdlkeycode;
 public:
-	KeyboardEvent(Class_base* c, tiny_string _type="", uint32_t _charcode=0, uint32_t _keycode=0, SDL_Keymod modifiers=KMOD_NONE, SDL_Keycode _sdlkeycode=SDLK_UNKNOWN);
+	KeyboardEvent(ASWorker* wrk, Class_base* c, tiny_string _type="", uint32_t _sdlcharcode=0, uint32_t _charcode=0, uint32_t _keycode=0, SDL_Keymod modifiers=KMOD_NONE, SDL_Keycode _sdlkeycode=SDLK_UNKNOWN);
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_GETTER_SETTER(altKey);
 	ASFUNCTION_GETTER_SETTER(commandKey);
 	ASFUNCTION_GETTER_SETTER(controlKey);
 	ASFUNCTION_GETTER_SETTER(ctrlKey);
 	ASFUNCTION_GETTER_SETTER(shiftKey);
-	uint32_t getCharCode() const { return charCode; }
+	uint32_t getSDLScanCode() const { return sdlScanCode; }
 	uint32_t getKeyCode() const { return keyCode; }
 	uint32_t getModifiers() const { return modifiers; }
 	SDL_Keycode getSDLKeyCode() const { return sdlkeycode; }
@@ -134,22 +134,16 @@ public:
 class FocusEvent: public Event
 {
 public:
-	FocusEvent(Class_base* c);
+	FocusEvent(ASWorker* wrk,Class_base* c, tiny_string _type="focusEvent");
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 };
 
 class FullScreenEvent: public Event
 {
 public:
-	FullScreenEvent(Class_base* c);
+	FullScreenEvent(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 };
 
@@ -158,11 +152,8 @@ class NetStatusEvent: public Event
 private:
 	Event* cloneImpl() const override ;
 public:
-	NetStatusEvent(Class_base* cb, const tiny_string& l="", const tiny_string& c="");
+	NetStatusEvent(ASWorker* wrk,Class_base* cb, const tiny_string& l="", const tiny_string& c="");
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 	tiny_string statuscode;
 };
@@ -170,11 +161,8 @@ public:
 class HTTPStatusEvent: public Event
 {
 public:
-	HTTPStatusEvent(Class_base* c):Event(c){}
+	HTTPStatusEvent(ASWorker* wrk, Class_base* c):Event(wrk,c){}
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 };
 
@@ -183,11 +171,8 @@ class TextEvent: public Event
 protected:
 	ASPROPERTY_GETTER_SETTER(tiny_string,text);
 public:
-	TextEvent(Class_base* c, const tiny_string& t = "textEvent");
+	TextEvent(ASWorker* wrk, Class_base* c, const tiny_string& t = "textEvent");
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 };
 
@@ -198,11 +183,8 @@ protected:
 	Event* cloneImpl() const override;
 	ASPROPERTY_GETTER(int32_t,errorID);
 public:
-	ErrorEvent(Class_base* c, const tiny_string& t = "error", const std::string& e = "", int id = 0);
+	ErrorEvent(ASWorker* wrk, Class_base* c, const tiny_string& t = "error", const std::string& e = "", int id = 0);
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 };
 
@@ -211,42 +193,30 @@ class IOErrorEvent: public ErrorEvent
 private:
 	Event* cloneImpl() const override;
 public:
-	IOErrorEvent(Class_base* c, const tiny_string& t = "ioError", const std::string& e = "", int id = 0);
+	IOErrorEvent(ASWorker* wrk, Class_base* c, const tiny_string& t = "ioError", const std::string& e = "", int id = 0);
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 };
 
 class SecurityErrorEvent: public ErrorEvent
 {
 public:
-	SecurityErrorEvent(Class_base* c, const std::string& e = "");
+	SecurityErrorEvent(ASWorker* wrk, Class_base* c, const std::string& e = "");
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 };
 
 class AsyncErrorEvent: public ErrorEvent
 {
 public:
-	AsyncErrorEvent(Class_base* c);
+	AsyncErrorEvent(ASWorker* wrk, Class_base* c);
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 };
 
 class UncaughtErrorEvent: public ErrorEvent
 {
 public:
-	UncaughtErrorEvent(Class_base* c);
+	UncaughtErrorEvent(ASWorker* wrk, Class_base* c);
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 };
 
@@ -259,22 +229,18 @@ public:
 	Mutex accesmutex;
 	ASPROPERTY_GETTER_SETTER(int32_t,bytesLoaded);
 	ASPROPERTY_GETTER_SETTER(int32_t,bytesTotal);
-	ProgressEvent(Class_base* c);
-	ProgressEvent(Class_base* c, uint32_t loaded, uint32_t total, const tiny_string& t="progress");
+	ProgressEvent(ASWorker* wrk, Class_base* c);
+	ProgressEvent(ASWorker* wrk, Class_base* c, uint32_t loaded, uint32_t total, const tiny_string& t="progress");
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o);
 	ASFUNCTION_ATOM(_constructor);
 };
 
 class TimerEvent: public Event
 {
 public:
-	TimerEvent(Class_base* c):Event(c, "DEPRECATED"){}
-	TimerEvent(Class_base* c,const tiny_string& t):Event(c,t,true){}
+	TimerEvent(ASWorker* wrk, Class_base* c):Event(wrk, c, "DEPRECATED"){}
+	TimerEvent(ASWorker* wrk, Class_base* c,const tiny_string& t):Event(wrk,c,t,true){}
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(updateAfterEvent);
 };
 
@@ -284,12 +250,11 @@ private:
 	uint32_t modifiers; 
 	Event* cloneImpl() const override;
 public:
-	MouseEvent(Class_base* c);
-	MouseEvent(Class_base* c, const tiny_string& t, number_t lx, number_t ly,
+	MouseEvent(ASWorker* wrk, Class_base* c);
+	MouseEvent(ASWorker* wrk, Class_base* c, const tiny_string& t, number_t lx, number_t ly,
 		   bool b=true, SDL_Keymod _modifiers=KMOD_NONE,bool _buttonDown = false,
 		   _NR<InteractiveObject> relObj = NullRef, int32_t delta=1);
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o);
 	void setTarget(asAtom t) override;
 	EVENT_TYPE getEventType() const override { return MOUSE_EVENT;}
 	ASFUNCTION_ATOM(_constructor);
@@ -311,7 +276,7 @@ public:
 class NativeDragEvent: public MouseEvent
 {
 public:
-	NativeDragEvent(Class_base* c);
+	NativeDragEvent(ASWorker* wrk, Class_base* c);
 	
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
@@ -320,9 +285,8 @@ public:
 class InvokeEvent: public Event
 {
 public:
-	InvokeEvent(Class_base* c) : Event(c, "invoke"){}
+	InvokeEvent(ASWorker* wrk, Class_base* c) : Event(wrk,c, "invoke"){}
 	static void sinit(Class_base* c);
-	static void buildTraits(ASObject* o){}
 	ASFUNCTION_ATOM(_constructor);
 };
 
@@ -336,21 +300,17 @@ private:
 	 * false: get events in the bubble phase
 	 */
 	bool use_capture;
+	ASWorker* worker;
 public:
-	explicit listener(asAtom _f, int32_t _p, bool _c)
-		:f(_f),priority(_p),use_capture(_c){}
-	bool operator==(std::pair<asAtom,bool> r)
-	{
-		/* One can register the same handle for the same event with
-		 * different values of use_capture
-		 */
-		return (use_capture == r.second) && asAtomHandler::isEqual(f,getSys(),r.first);
-	}
+	explicit listener(asAtom _f, int32_t _p, bool _c,ASWorker* _w)
+		:f(_f),priority(_p),use_capture(_c),worker(_w){}
+	bool operator==(const listener& r);
 	bool operator<(const listener& r) const
 	{
 		//The higher the priority the earlier this must be executed
 		return priority>r.priority;
 	}
+	void resetClosure();
 };
 
 class IEventDispatcher
@@ -371,15 +331,17 @@ private:
 protected:
 	virtual void eventListenerAdded(const tiny_string& eventName) {}
 public:
-	EventDispatcher(Class_base* c);
+	EventDispatcher(ASWorker* wrk, Class_base* c);
 	void finalize() override;
 	bool destruct() override;
+	bool countCylicMemberReferences(garbagecollectorstate& gcstate) override;
+	void prepareShutdown() override;
+	void clearEventListeners();
 	// is called when a new event is added to the event queue
 	virtual void onNewEvent(Event* ev){}
 	// is called after an event was handled by the event queue
 	virtual void afterHandleEvent(Event* ev) {}
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o);
 	void handleEvent(_R<Event> e);
 	void dumpHandlers();
 	bool hasEventListener(const tiny_string& eventName);
@@ -395,9 +357,8 @@ public:
 class StatusEvent: public Event
 {
 public:
-	StatusEvent(Class_base* c) : Event(c, "StatusEvent") {}
+	StatusEvent(ASWorker* wrk, Class_base* c) : Event(wrk,c, "StatusEvent") {}
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o) {}
 };
 
 class DataEvent: public TextEvent
@@ -405,9 +366,8 @@ class DataEvent: public TextEvent
 private:
 	Event* cloneImpl() const override;
 public:
-	DataEvent(Class_base* c, const tiny_string& _data="") : TextEvent(c, "data"), data(_data) {}
+	DataEvent(ASWorker* wrk, Class_base* c, const tiny_string& _data="") : TextEvent(wrk,c, "data"), data(_data) {}
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o) {}
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(tiny_string, data);
 };
@@ -446,12 +406,27 @@ private:
 	asAtom* args;
 	unsigned int numArgs;
 public:
-	FunctionEvent(asAtom _f, asAtom _obj=asAtomHandler::invalidAtom, asAtom* _args=NULL, uint32_t _numArgs=0);
+	FunctionEvent(asAtom _f, asAtom _obj=asAtomHandler::invalidAtom, asAtom* _args=nullptr, uint32_t _numArgs=0);
 	~FunctionEvent();
 	static void sinit(Class_base*);
 	EVENT_TYPE getEventType() const override { return FUNCTION; }
 };
 
+class FunctionAsyncEvent: public Event
+{
+friend class ABCVm;
+private:
+	asAtom f=asAtomHandler::invalidAtom;
+	asAtom obj=asAtomHandler::invalidAtom;
+	asAtom* args;
+	unsigned int numArgs;
+public:
+	FunctionAsyncEvent(asAtom _f, asAtom _obj=asAtomHandler::invalidAtom, asAtom* _args=nullptr, uint32_t _numArgs=0);
+	~FunctionAsyncEvent();
+	static void sinit(Class_base*);
+	EVENT_TYPE getEventType() const override { return FUNCTION_ASYNC; }
+};
+
 class ExternalCallEvent: public WaitableEvent
 {
 friend class ABCVm;
@@ -505,7 +480,7 @@ friend class ABCVm;
 private:
 	_NR<DisplayObject> clip;
 public:
-	InitFrameEvent(_NR<DisplayObject> m) : Event(nullptr, "InitFrameEvent"),clip(m) {}
+	InitFrameEvent(_NR<DisplayObject> m) : Event(nullptr,nullptr, "InitFrameEvent"),clip(m) {}
 	EVENT_TYPE getEventType() const override { return INIT_FRAME; }
 };
 
@@ -515,7 +490,7 @@ friend class ABCVm;
 private:
 	_NR<DisplayObject> clip;
 public:
-	ExecuteFrameScriptEvent(_NR<DisplayObject> m):Event(nullptr, "ExecuteFrameScriptEvent"),clip(m) {}
+	ExecuteFrameScriptEvent(_NR<DisplayObject> m):Event(nullptr,nullptr, "ExecuteFrameScriptEvent"),clip(m) {}
 	static void sinit(Class_base*);
 	EVENT_TYPE getEventType() const override { return EXECUTE_FRAMESCRIPT; }
 };
@@ -526,7 +501,7 @@ friend class ABCVm;
 private:
 	_NR<DisplayObject> clip;
 public:
-	AdvanceFrameEvent(_NR<DisplayObject> m=NullRef): Event(nullptr,"AdvanceFrameEvent"),clip(m) {}
+	AdvanceFrameEvent(_NR<DisplayObject> m=NullRef): Event(nullptr,nullptr,"AdvanceFrameEvent"),clip(m) {}
 	EVENT_TYPE getEventType() const override { return ADVANCE_FRAME; }
 };
 class RootConstructedEvent: public Event
@@ -535,7 +510,7 @@ friend class ABCVm;
 private:
 	_NR<DisplayObject> clip;
 public:
-	RootConstructedEvent(_NR<DisplayObject> m): Event(nullptr,"RootConstructedEvent"),clip(m) {}
+	RootConstructedEvent(_NR<DisplayObject> m): Event(nullptr,nullptr,"RootConstructedEvent"),clip(m) {}
 	EVENT_TYPE getEventType() const override { return ROOTCONSTRUCTEDEVENT; }
 };
 
@@ -550,7 +525,7 @@ public:
 class FlushInvalidationQueueEvent: public Event
 {
 public:
-	FlushInvalidationQueueEvent():Event(NULL, "FlushInvalidationQueueEvent"){}
+	FlushInvalidationQueueEvent():Event(nullptr,nullptr, "FlushInvalidationQueueEvent"){}
 	EVENT_TYPE getEventType() const override { return FLUSH_INVALIDATION_QUEUE; }
 };
 
@@ -571,29 +546,23 @@ private:
 	_NR<InteractiveObject> target;
 	tiny_string text;
 public:
-	TextInputEvent(_NR<InteractiveObject> m, const tiny_string& s) : Event(nullptr, "TextInputEvent"),target(m),text(s) {}
+	TextInputEvent(_NR<InteractiveObject> m, const tiny_string& s) : Event(nullptr,nullptr, "TextInputEvent"),target(m),text(s) {}
 	EVENT_TYPE getEventType() const override { return TEXTINPUT_EVENT; }
 };
 
 class DRMErrorEvent: public ErrorEvent
 {
 public:
-	DRMErrorEvent(Class_base* c);
+	DRMErrorEvent(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 };
 
 class DRMStatusEvent: public Event
 {
 public:
-	DRMStatusEvent(Class_base* c);
+	DRMStatusEvent(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o)
-	{
-	}
 	ASFUNCTION_ATOM(_constructor);
 };
 
@@ -602,7 +571,7 @@ class VideoEvent: public Event
 private:
 	Event* cloneImpl() const override;
 public:
-	VideoEvent(Class_base* c);
+	VideoEvent(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER(tiny_string,status);
@@ -613,7 +582,7 @@ class StageVideoEvent: public Event
 private:
 	Event* cloneImpl() const override;
 public:
-	StageVideoEvent(Class_base* c);
+	StageVideoEvent(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER(tiny_string,colorSpace);
@@ -625,7 +594,7 @@ class StageVideoAvailabilityEvent: public Event
 private:
 	Event* cloneImpl() const override;
 public:
-	StageVideoAvailabilityEvent(Class_base* c);
+	StageVideoAvailabilityEvent(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER(tiny_string,availability);
@@ -636,8 +605,8 @@ class ContextMenuEvent: public Event
 private:
 	Event* cloneImpl() const override;
 public:
-	ContextMenuEvent(Class_base* c) : Event(c, "ContextMenuEvent") {}
-	ContextMenuEvent(Class_base* c, tiny_string t, _NR<InteractiveObject> target, _NR<InteractiveObject> owner) : Event(c, t,false,false,SUBTYPE_CONTEXTMENUEVENT),mouseTarget(target),contextMenuOwner(owner) {}
+	ContextMenuEvent(ASWorker* wrk, Class_base* c) : Event(wrk,c, "ContextMenuEvent") {}
+	ContextMenuEvent(ASWorker* wrk, Class_base* c, tiny_string t, _NR<InteractiveObject> target, _NR<InteractiveObject> owner) : Event(wrk,c, t,false,false,SUBTYPE_CONTEXTMENUEVENT),mouseTarget(target),contextMenuOwner(owner) {}
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(_NR<InteractiveObject>,mouseTarget);
@@ -647,34 +616,34 @@ public:
 class TouchEvent: public Event
 {
 public:
-	TouchEvent(Class_base* c) : Event(c, "TouchEvent") {}
+	TouchEvent(ASWorker* wrk, Class_base* c) : Event(wrk,c, "TouchEvent") {}
 	static void sinit(Class_base*);
 };
 
 class GestureEvent: public Event
 {
 public:
-	GestureEvent(Class_base* c, const tiny_string& t = "GestureEvent") : Event(c, t) {}
+	GestureEvent(ASWorker* wrk, Class_base* c, const tiny_string& t = "GestureEvent") : Event(wrk,c, t) {}
 	static void sinit(Class_base*);
 };
 
 class PressAndTapGestureEvent: public GestureEvent
 {
 public:
-	PressAndTapGestureEvent(Class_base* c) : GestureEvent(c, "PressAndTapGestureEvent") {}
+	PressAndTapGestureEvent(ASWorker* wrk, Class_base* c) : GestureEvent(wrk,c, "PressAndTapGestureEvent") {}
 	static void sinit(Class_base*);
 };
 class TransformGestureEvent: public GestureEvent
 {
 public:
-	TransformGestureEvent(Class_base* c) : GestureEvent(c, "TransformGestureEvent") {}
+	TransformGestureEvent(ASWorker* wrk, Class_base* c) : GestureEvent(wrk,c, "TransformGestureEvent") {}
 	static void sinit(Class_base*);
 };
 
 class UncaughtErrorEvents: public EventDispatcher
 {
 public:
-	UncaughtErrorEvents(Class_base* c);
+	UncaughtErrorEvents(ASWorker* wrk, Class_base* c);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 };
@@ -684,8 +653,9 @@ class SampleDataEvent: public Event
 private:
 	Event* cloneImpl() const override;
 public:
-	SampleDataEvent(Class_base* c) : Event(c, "sampleData",false,false,SUBTYPE_SAMPLEDATA_EVENT),position(0) {}
-	SampleDataEvent(Class_base* c,_NR<ByteArray> _data,number_t _pos) : Event(c, "sampleData",false,false,SUBTYPE_SAMPLEDATA_EVENT),data(_data),position(_pos) {}
+	SampleDataEvent(ASWorker* wrk, Class_base* c);
+	SampleDataEvent(ASWorker* wrk, Class_base* c,_NR<ByteArray> _data,number_t _pos);
+	bool destruct() override;
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(_toString);
@@ -698,8 +668,8 @@ class ThrottleEvent: public Event
 private:
 	Event* cloneImpl() const override;
 public:
-	ThrottleEvent(Class_base* c) : Event(c, "throttle",false,false,SUBTYPE_THROTTLE_EVENT),targetFrameRate(0) {}
-	ThrottleEvent(Class_base* c,const tiny_string _state,number_t _targetFrameRate) : Event(c, "throttle",false,false,SUBTYPE_THROTTLE_EVENT),state(_state),targetFrameRate(_targetFrameRate) {}
+	ThrottleEvent(ASWorker* wrk, Class_base* c) : Event(wrk,c, "throttle",false,false,SUBTYPE_THROTTLE_EVENT),targetFrameRate(0) {}
+	ThrottleEvent(ASWorker* wrk, Class_base* c,const tiny_string _state,number_t _targetFrameRate) : Event(wrk,c, "throttle",false,false,SUBTYPE_THROTTLE_EVENT),state(_state),targetFrameRate(_targetFrameRate) {}
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(_toString);
@@ -709,7 +679,7 @@ public:
 class ThrottleType: public ASObject
 {
 public:
-	ThrottleType(Class_base* c):ASObject(c){}
+	ThrottleType(ASWorker* wrk, Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
@@ -718,8 +688,8 @@ class GameInputEvent: public Event
 private:
 	Event* cloneImpl() const override;
 public:
-	GameInputEvent(Class_base* c);
-	GameInputEvent(Class_base* c,_NR<GameInputDevice> _device);
+	GameInputEvent(ASWorker* wrk, Class_base* c);
+	GameInputEvent(ASWorker* wrk, Class_base* c,_NR<GameInputDevice> _device);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER(_NR<GameInputDevice>,device);
