@@ -37,75 +37,73 @@ void ExternalInterface::sinit(Class_base* c)
 
 ASFUNCTIONBODY_ATOM(ExternalInterface,_getAvailable)
 {
-	asAtomHandler::setBool(ret,sys->extScriptObject != NULL);
+	asAtomHandler::setBool(ret,wrk->getSystemState()->extScriptObject != nullptr);
 }
 
 ASFUNCTIONBODY_ATOM(ExternalInterface,_getObjectID)
 {
-	if(sys->extScriptObject == NULL)
+	if(wrk->getSystemState()->extScriptObject == nullptr)
 	{
-		ret = asAtomHandler::fromString(sys,"");
+		ret = asAtomHandler::fromString(wrk->getSystemState(),"");
 		return;
 	}
 
-	ExtScriptObject* so=sys->extScriptObject;
+	ExtScriptObject* so=wrk->getSystemState()->extScriptObject;
 	if(so->hasProperty("name")==false)
 	{
-		ret = asAtomHandler::fromString(sys,"");
+		ret = asAtomHandler::fromString(wrk->getSystemState(),"");
 		return;
 	}
 
 	const ExtVariant& object = so->getProperty("name");
 	std::string result = object.getString();
-	ret = asAtomHandler::fromObject(abstract_s(sys,result));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,result));
 }
 
 ASFUNCTIONBODY_ATOM(ExternalInterface, _getMarshallExceptions)
 {
-	if(sys->extScriptObject == NULL)
+	if(wrk->getSystemState()->extScriptObject == nullptr)
 		asAtomHandler::setBool(ret,false);
 	else
-		asAtomHandler::setBool(ret,sys->extScriptObject->getMarshallExceptions());
+		asAtomHandler::setBool(ret,wrk->getSystemState()->extScriptObject->getMarshallExceptions());
 }
 
 ASFUNCTIONBODY_ATOM(ExternalInterface, _setMarshallExceptions)
 {
-	if(sys->extScriptObject != NULL)
-		sys->extScriptObject->setMarshallExceptions(asAtomHandler::Boolean_concrete(args[0]));
+	if(wrk->getSystemState()->extScriptObject != nullptr)
+		wrk->getSystemState()->extScriptObject->setMarshallExceptions(asAtomHandler::Boolean_concrete(args[0]));
 }
 
 
 ASFUNCTIONBODY_ATOM(ExternalInterface,addCallback)
 {
-	if(sys->extScriptObject == NULL)
+	if(wrk->getSystemState()->extScriptObject == nullptr)
 	{
 		asAtomHandler::setBool(ret,false);
 		return;
 	}
-//		throw Class<ASError>::getInstanceS("Container doesn't support callbacks");
 
 	assert_and_throw(argslen == 2);
 
 	if(asAtomHandler::isNull(args[1]))
-		sys->extScriptObject->removeMethod(asAtomHandler::toString(args[0],sys).raw_buf());
+		wrk->getSystemState()->extScriptObject->removeMethod(asAtomHandler::toString(args[0],wrk).raw_buf());
 	else
 	{
-		sys->extScriptObject->setMethod(asAtomHandler::toString(args[0],sys).raw_buf(), new ExtASCallback(args[1]));
+		wrk->getSystemState()->extScriptObject->setMethod(asAtomHandler::toString(args[0],wrk).raw_buf(), new ExtASCallback(args[1]));
 	}
 	asAtomHandler::setBool(ret,true);
 }
 
 ASFUNCTIONBODY_ATOM(ExternalInterface,call)
 {
-	if(sys->extScriptObject == NULL)
+	if(wrk->getSystemState()->extScriptObject == nullptr)
 	{
 		asAtomHandler::setNull(ret);
 		return;
 	}
-//		throw Class<ASError>::getInstanceS("Container doesn't support callbacks");
 
 	assert_and_throw(argslen >= 1);
-	const tiny_string& arg0=asAtomHandler::toString(args[0],sys);
+	const tiny_string& arg0=asAtomHandler::toString(args[0],wrk);
 
 	// TODO: Check security constraints & throw SecurityException
 
@@ -115,12 +113,12 @@ ASFUNCTIONBODY_ATOM(ExternalInterface,call)
 	for(uint32_t i = 0; i < argslen-1; i++)
 	{
 		ASATOM_INCREF(args[i+1]);
-		callArgs[i] = new ExtVariant(objectsMap,_MR(asAtomHandler::toObject(args[i+1],sys)));
+		callArgs[i] = new ExtVariant(objectsMap,_MR(asAtomHandler::toObject(args[i+1],wrk)));
 	}
 
-	ASObject* asobjResult = NULL;
+	ASObject* asobjResult = nullptr;
 	// Let the external script object call the external method
-	bool callSuccess = sys->extScriptObject->callExternal(arg0.raw_buf(), callArgs, argslen-1, &asobjResult);
+	bool callSuccess = wrk->getSystemState()->extScriptObject->callExternal(arg0.raw_buf(), callArgs, argslen-1, &asobjResult);
 
 	// Delete converted arguments
 	for(uint32_t i = 0; i < argslen-1; i++)
@@ -128,7 +126,7 @@ ASFUNCTIONBODY_ATOM(ExternalInterface,call)
 
 	if(!callSuccess)
 	{
-		assert(asobjResult==NULL);
+		assert(asobjResult==nullptr);
 		LOG(LOG_INFO, "External function failed, returning null: " << arg0);
 		// If the call fails, return null
 		asAtomHandler::setNull(ret);
