@@ -30,10 +30,17 @@ class BitmapFilter: public ASObject
 {
 private:
 	virtual BitmapFilter* cloneImpl() const;
+protected:
+	static void applyBlur(uint8_t* data, uint32_t width, uint32_t height, number_t blurx, number_t blury, int quality, number_t scalex, number_t scaley);
+	static void applyDropShadowFilter(BitmapContainer* target, uint8_t* tmpdata, const RECT& sourceRect, int xpos, int ypos, number_t strength, number_t alpha, uint32_t color, bool inner, bool knockout, number_t scalex, number_t scaley);
+	static void fillGradientColors(number_t* gradientalphas, uint32_t* gradientcolors, Array* ratios, Array* alphas, Array* colors);
+	static void applyGradientFilter(BitmapContainer* target, uint8_t* tmpdata, const RECT& sourceRect, int xpos, int ypos, number_t strength, number_t* alphas, uint32_t* colors, bool inner, bool knockout, number_t scalex, number_t scaley);
 public:
-	BitmapFilter(Class_base* c, CLASS_SUBTYPE st=SUBTYPE_BITMAPFILTER):ASObject(c,T_OBJECT,st){}
+	BitmapFilter(ASWorker* wrk,Class_base* c, CLASS_SUBTYPE st=SUBTYPE_BITMAPFILTER):ASObject(wrk,c,T_OBJECT,st){}
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(clone);
+	virtual void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley);
+	virtual uint32_t getMaxFilterBorder() const { return 0; }
 };
 
 class GlowFilter: public BitmapFilter
@@ -49,10 +56,12 @@ private:
 	ASPROPERTY_GETTER_SETTER(number_t, strength);
 	BitmapFilter* cloneImpl() const override;
 public:
-	GlowFilter(Class_base* c);
-	GlowFilter(Class_base* c,const GLOWFILTER& filter);
+	GlowFilter(ASWorker* wrk,Class_base* c);
+	GlowFilter(ASWorker* wrk,Class_base* c,const GLOWFILTER& filter);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
+	void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley) override;
+	uint32_t getMaxFilterBorder() const override { return ceil(max(blurX,blurY)); }
 };
 
 class DropShadowFilter: public BitmapFilter
@@ -71,10 +80,12 @@ private:
 	ASPROPERTY_GETTER_SETTER(number_t, strength);
 	BitmapFilter* cloneImpl() const override;
 public:
-	DropShadowFilter(Class_base* c);
-	DropShadowFilter(Class_base* c,const DROPSHADOWFILTER& filter);
+	DropShadowFilter(ASWorker* wrk,Class_base* c);
+	DropShadowFilter(ASWorker* wrk,Class_base* c,const DROPSHADOWFILTER& filter);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
+	void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley) override;
+	uint32_t getMaxFilterBorder() const override { return ceil(max(blurX,blurY)); }
 };
 
 class GradientGlowFilter: public BitmapFilter
@@ -82,8 +93,8 @@ class GradientGlowFilter: public BitmapFilter
 private:
 	BitmapFilter* cloneImpl() const override;
 public:
-	GradientGlowFilter(Class_base* c);
-	GradientGlowFilter(Class_base* c, const GRADIENTGLOWFILTER& filter);
+	GradientGlowFilter(ASWorker* wrk,Class_base* c);
+	GradientGlowFilter(ASWorker* wrk,Class_base* c, const GRADIENTGLOWFILTER& filter);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(number_t,distance);
@@ -97,6 +108,9 @@ public:
 	ASPROPERTY_GETTER_SETTER(int32_t,quality);
 	ASPROPERTY_GETTER_SETTER(tiny_string,type);
 	ASPROPERTY_GETTER_SETTER(bool,knockout);
+	void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley) override;
+	uint32_t getMaxFilterBorder() const override { return ceil(max(blurX,blurY)); }
+	void prepareShutdown() override;
 };
 
 class BevelFilter: public BitmapFilter
@@ -104,8 +118,8 @@ class BevelFilter: public BitmapFilter
 private:
 	BitmapFilter* cloneImpl() const override;
 public:
-	BevelFilter(Class_base* c);
-	BevelFilter(Class_base* c,const BEVELFILTER& filter);
+	BevelFilter(ASWorker* wrk,Class_base* c);
+	BevelFilter(ASWorker* wrk,Class_base* c,const BEVELFILTER& filter);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(number_t, angle);
@@ -120,38 +134,44 @@ public:
 	ASPROPERTY_GETTER_SETTER(uint32_t,shadowColor);
 	ASPROPERTY_GETTER_SETTER(number_t,strength);
 	ASPROPERTY_GETTER_SETTER(tiny_string,type);
+	void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley) override;
+	uint32_t getMaxFilterBorder() const override { return ceil(max(blurX,blurY)); }
 };
 class ColorMatrixFilter: public BitmapFilter
 {
 private:
 	BitmapFilter* cloneImpl() const override;
 public:
-	ColorMatrixFilter(Class_base* c);
-	ColorMatrixFilter(Class_base* c,const COLORMATRIXFILTER& filter);
+	ColorMatrixFilter(ASWorker* wrk,Class_base* c);
+	ColorMatrixFilter(ASWorker* wrk,Class_base* c,const COLORMATRIXFILTER& filter);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(_NR<Array>, matrix);
+	void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley) override;
+	void prepareShutdown() override;
 };
 class BlurFilter: public BitmapFilter
 {
 private:
 	BitmapFilter* cloneImpl() const override;
 public:
-	BlurFilter(Class_base* c);
-	BlurFilter(Class_base* c,const BLURFILTER& filter);
+	BlurFilter(ASWorker* wrk,Class_base* c);
+	BlurFilter(ASWorker* wrk,Class_base* c,const BLURFILTER& filter);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(number_t, blurX);
 	ASPROPERTY_GETTER_SETTER(number_t, blurY);
 	ASPROPERTY_GETTER_SETTER(int, quality);
+	void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley) override;
+	uint32_t getMaxFilterBorder() const override { return ceil(max(blurX,blurY)); }
 };
 class ConvolutionFilter: public BitmapFilter
 {
 private:
 	BitmapFilter* cloneImpl() const override;
 public:
-	ConvolutionFilter(Class_base* c);
-	ConvolutionFilter(Class_base* c,const CONVOLUTIONFILTER& filter);
+	ConvolutionFilter(ASWorker* wrk,Class_base* c);
+	ConvolutionFilter(ASWorker* wrk,Class_base* c,const CONVOLUTIONFILTER& filter);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(number_t,alpha);
@@ -163,13 +183,15 @@ public:
 	ASPROPERTY_GETTER_SETTER(number_t, matrixX);
 	ASPROPERTY_GETTER_SETTER(number_t, matrixY);
 	ASPROPERTY_GETTER_SETTER(bool, preserveAlpha);
+	void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley) override;
+	void prepareShutdown() override;
 };
 class DisplacementMapFilter: public BitmapFilter
 {
 private:
 	BitmapFilter* cloneImpl() const override;
 public:
-	DisplacementMapFilter(Class_base* c);
+	DisplacementMapFilter(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(number_t,alpha);
@@ -181,14 +203,16 @@ public:
 	ASPROPERTY_GETTER_SETTER(tiny_string,mode);
 	ASPROPERTY_GETTER_SETTER(number_t,scaleX);
 	ASPROPERTY_GETTER_SETTER(number_t,scaleY);
+	void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley) override;
+	void prepareShutdown() override;
 };
 class GradientBevelFilter: public BitmapFilter
 {
 private:
 	BitmapFilter* cloneImpl() const override;
 public:
-	GradientBevelFilter(Class_base* c);
-	GradientBevelFilter(Class_base* c, const GRADIENTBEVELFILTER& filter);
+	GradientBevelFilter(ASWorker* wrk,Class_base* c);
+	GradientBevelFilter(ASWorker* wrk,Class_base* c, const GRADIENTBEVELFILTER& filter);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(_NR<Array>, alphas);
@@ -202,35 +226,39 @@ public:
 	ASPROPERTY_GETTER_SETTER(_NR<Array>, ratios);
 	ASPROPERTY_GETTER_SETTER(number_t, strength);
 	ASPROPERTY_GETTER_SETTER(tiny_string, type);
+	void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley) override;
+	uint32_t getMaxFilterBorder() const override { return ceil(max(blurX,blurY)); }
+	void prepareShutdown() override;
 };
 class ShaderFilter: public BitmapFilter
 {
 private:
 	BitmapFilter* cloneImpl() const override;
 public:
-	ShaderFilter(Class_base* c);
+	ShaderFilter(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
+	void applyFilter(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos, number_t scalex, number_t scaley) override;
 };
 
 class BitmapFilterQuality: public ASObject
 {
 public:
-	BitmapFilterQuality(Class_base* c):ASObject(c) {}
+	BitmapFilterQuality(ASWorker* wrk,Class_base* c):ASObject(wrk,c) {}
 	static void sinit(Class_base* c);
 };
 
 class DisplacementMapFilterMode: public ASObject
 {
 public:
-	DisplacementMapFilterMode(Class_base* c):ASObject(c) {}
+	DisplacementMapFilterMode(ASWorker* wrk,Class_base* c):ASObject(wrk,c) {}
 	static void sinit(Class_base* c);
 };
 
 class BitmapFilterType: public ASObject
 {
 public:
-	BitmapFilterType(Class_base* c):ASObject(c) {}
+	BitmapFilterType(ASWorker* wrk,Class_base* c):ASObject(wrk,c) {}
 	static void sinit(Class_base* c);
 };
 
