@@ -23,12 +23,13 @@
 #include "scripting/class.h"
 #include "scripting/argconv.h"
 #include "currencyparseresult.h"
+#include "scripting/toplevel/Number.h"
 #include <string>
 
 using namespace lightspark;
 
-CurrencyFormatter::CurrencyFormatter(Class_base* c):
-	ASObject(c)
+CurrencyFormatter::CurrencyFormatter(ASWorker* wrk, Class_base* c):
+	ASObject(wrk,c)
 {
 }
 
@@ -68,7 +69,7 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,_constructor)
 {
 	CurrencyFormatter* th =asAtomHandler::as<CurrencyFormatter>(obj);
 	th->fractionalDigits = 0;
-	ARG_UNPACK_ATOM(th->requestedLocaleIDName);
+	ARG_CHECK(ARG_UNPACK(th->requestedLocaleIDName));
 
 	std::string localeId = th->requestedLocaleIDName;
 	std::string isoCurrencyCode = "";
@@ -77,19 +78,19 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,_constructor)
 		isoCurrencyCode = localeId.substr(3, localeId.size());
 	}
 
-	if (sys->localeManager->isLocaleAvailableOnSystem(th->requestedLocaleIDName))
+	if (wrk->getSystemState()->localeManager->isLocaleAvailableOnSystem(th->requestedLocaleIDName))
 	{
 		LOG(LOG_INFO,"unknown locale:"<<th->requestedLocaleIDName);
 		th->lastOperationStatus="usingDefaultWarning";
 	}
 
-	std::string localeName = sys->localeManager->getSystemLocaleName(th->requestedLocaleIDName);
-	th->currlocale = std::locale(localeName);
+	std::string localeName = wrk->getSystemState()->localeManager->getSystemLocaleName(th->requestedLocaleIDName);
+	th->currlocale = std::locale(localeName.c_str());
 	th->actualLocaleIDName = th->requestedLocaleIDName;
 	th->lastOperationStatus="noError";
 
-	std::string currencyIsoCode = sys->currencyManager->getCountryISOSymbol(isoCurrencyCode);
-	std::string currencySymbol = sys->currencyManager->getCurrencySymbol(currencyIsoCode);
+	std::string currencyIsoCode = wrk->getSystemState()->currencyManager->getCountryISOSymbol(isoCurrencyCode);
+	std::string currencySymbol = wrk->getSystemState()->currencyManager->getCurrencySymbol(currencyIsoCode);
 
 	th->currencyISOCode = currencyIsoCode;
 	th->currencySymbol = currencySymbol;
@@ -111,22 +112,22 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,_constructor)
 	std::locale::global(l);
 }
 
-ASFUNCTIONBODY_GETTER(CurrencyFormatter, actualLocaleIDName);
-ASFUNCTIONBODY_GETTER(CurrencyFormatter, currencyISOCode);
-ASFUNCTIONBODY_GETTER(CurrencyFormatter, currencySymbol);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, decimalSeparator);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, digitsType);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, fractionalDigits);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, groupingPattern);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, groupingSeparator);
-ASFUNCTIONBODY_GETTER(CurrencyFormatter, lastOperationStatus);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, leadingZero);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, negativeCurrencyFormat);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, negativeSymbol);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, positiveCurrencyFormat);
-ASFUNCTIONBODY_GETTER(CurrencyFormatter, requestedLocaleIDName);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, trailingZeros);
-ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, useGrouping);
+ASFUNCTIONBODY_GETTER(CurrencyFormatter, actualLocaleIDName)
+ASFUNCTIONBODY_GETTER(CurrencyFormatter, currencyISOCode)
+ASFUNCTIONBODY_GETTER(CurrencyFormatter, currencySymbol)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, decimalSeparator)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, digitsType)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, fractionalDigits)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, groupingPattern)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, groupingSeparator)
+ASFUNCTIONBODY_GETTER(CurrencyFormatter, lastOperationStatus)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, leadingZero)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, negativeCurrencyFormat)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, negativeSymbol)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, positiveCurrencyFormat)
+ASFUNCTIONBODY_GETTER(CurrencyFormatter, requestedLocaleIDName)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, trailingZeros)
+ASFUNCTIONBODY_GETTER_SETTER(CurrencyFormatter, useGrouping)
 
 ASFUNCTIONBODY_ATOM(CurrencyFormatter,format)
 {
@@ -160,7 +161,7 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,format)
 	double value;
 	bool withCurrencySymbol = false; // withCurrencySymbol argument not implemented
 	std::stringstream res;
-	ARG_UNPACK_ATOM(value);
+	ARG_CHECK(ARG_UNPACK(value));
 	value *= 100;
 	res.imbue(th->currlocale);
 	res << std::showbase << std::setprecision(th->fractionalDigits) << std::put_money(value,!withCurrencySymbol) << std::fixed;
@@ -170,7 +171,7 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,format)
 	// Now we format the output
 
 	// Replace spaces in input, get rid of the spaces generated by put_money
-	std::unordered_map<string, string> currencies = sys->currencyManager->getCurrencySymbols();
+	std::unordered_map<string, string> currencies = wrk->getSystemState()->currencyManager->getCurrencySymbols();
 	for (std::unordered_map<string, string>::iterator it = currencies.begin(); it != currencies.end(); ++it)
 	{
 		std::string key = it->first;
@@ -246,7 +247,7 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,format)
 		}
 	}
 
-	ret = asAtomHandler::fromString(sys,output);
+	ret = asAtomHandler::fromString(wrk->getSystemState(),output);
 }
 
 ASFUNCTIONBODY_ATOM(CurrencyFormatter,formattingWithCurrencySymbolIsSafe)
@@ -255,11 +256,12 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,formattingWithCurrencySymbolIsSafe)
 	LOG(LOG_NOT_IMPLEMENTED,"CurrencyFormatter.formattingWithCurrencySymbolIsSafe is not really tested for all formats");
 	tiny_string requestedISOCode;
 	std::string currencyISOCode = th->currencyISOCode;
-	ARG_UNPACK_ATOM(requestedISOCode);
+	ARG_CHECK(ARG_UNPACK(requestedISOCode));
 
 	if (requestedISOCode.raw_buf() == nullptr)
 	{
-		throwError<TypeError>(kNullArgumentError);
+		createError<TypeError>(wrk,kNullArgumentError);
+		return;
 	}
 
 	bool result = (requestedISOCode == currencyISOCode);
@@ -270,12 +272,12 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,formattingWithCurrencySymbolIsSafe)
 ASFUNCTIONBODY_ATOM(CurrencyFormatter,getAvailableLocaleIDNames)
 {
 	CurrencyFormatter* th =asAtomHandler::as<CurrencyFormatter>(obj);
-	Array* res=Class<Array>::getInstanceSNoArgs(sys);
-	std::vector<std::string> localeIds = sys->localeManager->getAvailableLocaleIDNames();
+	Array* res=Class<Array>::getInstanceSNoArgs(wrk);
+	std::vector<std::string> localeIds = wrk->getSystemState()->localeManager->getAvailableLocaleIDNames();
 	for (std::vector<std::string>::iterator it = localeIds.begin(); it != localeIds.end(); ++it)
 	{
 		tiny_string value = (*it);
-		res->push(asAtomHandler::fromObject(abstract_s(sys, value)));
+		res->push(asAtomHandler::fromObject(abstract_s(wrk, value)));
 	}
 	th->lastOperationStatus="noError";
 	ret = asAtomHandler::fromObject(res);
@@ -287,11 +289,12 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,parse)
 	LOG(LOG_NOT_IMPLEMENTED,"CurrencyFormatter.parse is not really tested for all formats");
 	std::string inputString;
 	tiny_string input;
-	ARG_UNPACK_ATOM(input);
+	ARG_CHECK(ARG_UNPACK(input));
 
 	if (input.raw_buf() == nullptr)
 	{
-		throwError<TypeError>(kNullArgumentError);
+		createError<TypeError>(wrk,kNullArgumentError);
+		return;
 	}
 
 	inputString = input;
@@ -322,9 +325,9 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,parse)
 	inputString.erase(0, inputString.find_first_not_of(' '));
 	inputString.erase(inputString.find_last_not_of(' ') + 1);
 
-	CurrencyParseResult* cpr = Class<CurrencyParseResult>::getInstanceS(sys);
+	CurrencyParseResult* cpr = Class<CurrencyParseResult>::getInstanceS(wrk);
 
-	std::unordered_map<string, string> currencies = sys->currencyManager->getCurrencySymbols();
+	std::unordered_map<string, string> currencies = wrk->getSystemState()->currencyManager->getCurrencySymbols();
 	for (std::unordered_map<string, string>::iterator it = currencies.begin(); it != currencies.end(); ++it)
 	{
 		std::string key = it->first;
@@ -400,6 +403,6 @@ ASFUNCTIONBODY_ATOM(CurrencyFormatter,parse)
 ASFUNCTIONBODY_ATOM(CurrencyFormatter,setCurrency)
 {
 	CurrencyFormatter* th =asAtomHandler::as<CurrencyFormatter>(obj);
-	ARG_UNPACK_ATOM(th->currencyISOCode)(th->currencySymbol);
+	ARG_CHECK(ARG_UNPACK(th->currencyISOCode)(th->currencySymbol));
 	th->lastOperationStatus="noError";
 }
