@@ -25,6 +25,10 @@
 #include "argconv.h"
 #include "swf.h"
 #include "flash/errors/flasherrors.h"
+#include "scripting/flash/utils/ByteArray.h"
+#include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/Integer.h"
+#include "scripting/toplevel/UInteger.h"
 #include <sys/types.h>
 #ifdef _WIN32
 #ifndef _WIN32_WINNT
@@ -48,6 +52,23 @@ const char SOCKET_COMMAND_CLOSE = '-';
 using namespace std;
 using namespace lightspark;
 
+struct socketbuf
+{
+	uint8_t* buf;
+	size_t len;
+	socketbuf(uint8_t* data, size_t l)
+	{
+		buf = new uint8_t[l];
+		len = l;
+		memcpy(buf,data,len);
+	}
+	~socketbuf()
+	{
+		delete[] buf;
+	}
+};
+
+
 SocketIO::SocketIO() : fd(-1)
 {
 #ifdef _WIN32
@@ -90,7 +111,7 @@ bool SocketIO::connect(const tiny_string& hostname, int port, int timeoutseconds
 		return false;
 	}
 
-	for(p = servinfo; p != NULL; p = p->ai_next)
+	for(p = servinfo; p != nullptr; p = p->ai_next)
 	{
 		if ((fd = socket(p->ai_family, p->ai_socktype, p->ai_protocol)) == -1)
 			continue;
@@ -186,38 +207,47 @@ void ASSocket::sinit(Class_base* c)
 	CLASS_SETUP(c, EventDispatcher, _constructor, CLASS_SEALED);
 	c->setDeclaredMethodByQName("close","",Class<IFunction>::getFunction(c->getSystemState(),_close),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("connect","",Class<IFunction>::getFunction(c->getSystemState(),_connect),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("connected","",Class<IFunction>::getFunction(c->getSystemState(),_connected),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("connected","",Class<IFunction>::getFunction(c->getSystemState(),_connected,0,Class<Boolean>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("flush","",Class<IFunction>::getFunction(c->getSystemState(),_flush),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("bytesAvailable","",Class<IFunction>::getFunction(c->getSystemState(),bytesAvailable),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("bytesAvailable","",Class<IFunction>::getFunction(c->getSystemState(),bytesAvailable,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("endian","",Class<IFunction>::getFunction(c->getSystemState(),_getEndian),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("endian","",Class<IFunction>::getFunction(c->getSystemState(),_setEndian),SETTER_METHOD,true);
-//	c->setDeclaredMethodByQName("readBoolean","",Class<IFunction>::getFunction(c->getSystemState(),readBoolean),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("objectEncoding","",Class<IFunction>::getFunction(c->getSystemState(),_getObjectEncoding,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("objectEncoding","",Class<IFunction>::getFunction(c->getSystemState(),_setObjectEncoding),SETTER_METHOD,true);
+	c->setDeclaredMethodByQName("readBoolean","",Class<IFunction>::getFunction(c->getSystemState(),readBoolean,0,Class<Boolean>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readByte","",Class<IFunction>::getFunction(c->getSystemState(),readByte,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("readBytes","",Class<IFunction>::getFunction(c->getSystemState(),readBytes),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readByte","",Class<IFunction>::getFunction(c->getSystemState(),readByte),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readDouble","",Class<IFunction>::getFunction(c->getSystemState(),readDouble),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readFloat","",Class<IFunction>::getFunction(c->getSystemState(),readFloat),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readInt","",Class<IFunction>::getFunction(c->getSystemState(),readInt),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readMultiByte","",Class<IFunction>::getFunction(c->getSystemState(),readMultiByte),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readShort","",Class<IFunction>::getFunction(c->getSystemState(),readShort),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readUnsignedByte","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedByte),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readUnsignedInt","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedInt),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readUnsignedShort","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedShort),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readObject","",Class<IFunction>::getFunction(c->getSystemState(),readObject),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("readUTF","",Class<IFunction>::getFunction(c->getSystemState(),readUTF),NORMAL_METHOD,true);
+
+	c->setDeclaredMethodByQName("readDouble","",Class<IFunction>::getFunction(c->getSystemState(),readDouble,0,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readFloat","",Class<IFunction>::getFunction(c->getSystemState(),readFloat,0,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readInt","",Class<IFunction>::getFunction(c->getSystemState(),readInt,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readMultiByte","",Class<IFunction>::getFunction(c->getSystemState(),readMultiByte,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readShort","",Class<IFunction>::getFunction(c->getSystemState(),readShort,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readUnsignedByte","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedByte,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readUnsignedInt","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedInt,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readUnsignedShort","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedShort,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readObject","",Class<IFunction>::getFunction(c->getSystemState(),readObject),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readUTF","",Class<IFunction>::getFunction(c->getSystemState(),readUTF,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("readUTFBytes","",Class<IFunction>::getFunction(c->getSystemState(),readUTFBytes),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("writeBoolean","",Class<IFunction>::getFunction(c->getSystemState(),writeBoolean),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("writeUTF","",Class<IFunction>::getFunction(c->getSystemState(),writeUTF),NORMAL_METHOD,true);
+
+	c->setDeclaredMethodByQName("writeBoolean","",Class<IFunction>::getFunction(c->getSystemState(),writeBoolean),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeUTF","",Class<IFunction>::getFunction(c->getSystemState(),writeUTF),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("writeUTFBytes","",Class<IFunction>::getFunction(c->getSystemState(),writeUTFBytes),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("writeBytes","",Class<IFunction>::getFunction(c->getSystemState(),writeBytes),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("writeByte","",Class<IFunction>::getFunction(c->getSystemState(),writeByte),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("writeDouble","",Class<IFunction>::getFunction(c->getSystemState(),writeDouble),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("writeFloat","",Class<IFunction>::getFunction(c->getSystemState(),writeFloat),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("writeInt","",Class<IFunction>::getFunction(c->getSystemState(),writeInt),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("writeMultiByte","",Class<IFunction>::getFunction(c->getSystemState(),writeMultiByte),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("writeUnsignedInt","",Class<IFunction>::getFunction(c->getSystemState(),writeUnsignedInt),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("writeObject","",Class<IFunction>::getFunction(c->getSystemState(),writeObject),NORMAL_METHOD,true);
-//	c->setDeclaredMethodByQName("writeShort","",Class<IFunction>::getFunction(c->getSystemState(),writeShort),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeByte","",Class<IFunction>::getFunction(c->getSystemState(),writeByte),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeDouble","",Class<IFunction>::getFunction(c->getSystemState(),writeDouble),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeFloat","",Class<IFunction>::getFunction(c->getSystemState(),writeFloat),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeInt","",Class<IFunction>::getFunction(c->getSystemState(),writeInt),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeMultiByte","",Class<IFunction>::getFunction(c->getSystemState(),writeMultiByte),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeUnsignedInt","",Class<IFunction>::getFunction(c->getSystemState(),writeUnsignedInt),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeObject","",Class<IFunction>::getFunction(c->getSystemState(),writeObject),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeShort","",Class<IFunction>::getFunction(c->getSystemState(),writeShort),NORMAL_METHOD,true);
 	REGISTER_GETTER_SETTER(c,timeout);
+
+	c->addImplementedInterface(InterfaceClass<IDataInput>::getClass(c->getSystemState()));
+	IDataInput::linkTraits(c);
+	c->addImplementedInterface(InterfaceClass<IDataOutput>::getClass(c->getSystemState()));
+	IDataOutput::linkTraits(c);
 }
 
 void ASSocket::finalize()
@@ -230,7 +260,7 @@ void ASSocket::finalize()
 		job->threadAborting = true;
 		job->requestClose();
 		job->threadAbort();
-		job = NULL;
+		job = nullptr;
 	}
 	timeout = 20000;
 }
@@ -242,9 +272,9 @@ ASFUNCTIONBODY_ATOM(ASSocket,_constructor)
 	tiny_string host;
 	bool host_is_null;
 	int port;
-	ARG_UNPACK_ATOM (host, "") (port, 0);
+	ARG_CHECK(ARG_UNPACK (host, "") (port, 0));
 
-	EventDispatcher::_constructor(ret,sys,obj,NULL,0);
+	EventDispatcher::_constructor(ret,wrk,obj,nullptr,0);
 
 	ASSocket* th=asAtomHandler::as<ASSocket>(obj);
 	host_is_null = argslen > 0 && asAtomHandler::is<Null>(args[0]);
@@ -268,20 +298,50 @@ ASFUNCTIONBODY_ATOM(ASSocket, _close)
 	}
 }
 
+ASFUNCTIONBODY_ATOM(ASSocket,_getObjectEncoding)
+{
+	ASSocket* th=asAtomHandler::as<ASSocket>(obj);
+	asAtomHandler::setUInt(ret,wrk,th->objectEncoding);
+}
+
+ASFUNCTIONBODY_ATOM(ASSocket,_setObjectEncoding)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"setting Socket.objectEncoding has no effect");
+	ASSocket* th=asAtomHandler::as<ASSocket>(obj);
+	uint32_t value;
+	ARG_CHECK(ARG_UNPACK(value));
+	if(value!=OBJECT_ENCODING::AMF0 && value!=OBJECT_ENCODING::AMF3)
+	{
+		createError<ArgumentError>(wrk,kInvalidEnumError, "objectEncoding");
+		return;
+	}
+
+	th->objectEncoding=value;
+}
+
 void ASSocket::connect(tiny_string host, int port)
 {
-	if (port <= 0 || port > 65535)
-		throw Class<SecurityError>::getInstanceS(getSystemState(),"Invalid port");
+	if ((port <= 0) || (port > 65535))
+	{
+		createError<SecurityError>(getInstanceWorker(),0,"Invalid port");
+		return;
+	}
 
 	if (host.empty())
 		host = getSys()->mainClip->getOrigin().getHostname();
 
 	if (isConnected())
-		throw Class<IOError>::getInstanceS(getSystemState(),"Already connected");
+	{
+		createError<IOError>(getInstanceWorker(),0,"Already connected");
+		return;
+	}
 
 	// Host shouldn't contain scheme or port
-	if (host.strchr(':') != NULL)
-		throw Class<SecurityError>::getInstanceS(getSystemState(),"Invalid hostname");
+	if (host.strchr(':') != nullptr)
+	{
+		createError<SecurityError>(getInstanceWorker(),0,"Invalid hostname");
+		return;
+	}
 
 	// Check sandbox and policy file
 	size_t buflen = host.numBytes() + 22;
@@ -294,13 +354,15 @@ void ASSocket::connect(tiny_string host, int port)
 		~(SecurityManager::LOCAL_WITH_FILE),
 		SecurityManager::LOCAL_WITH_FILE | SecurityManager::LOCAL_TRUSTED,
 		true);
+	if (getInstanceWorker()->currentCallContext && getInstanceWorker()->currentCallContext->exceptionthrown)
+		return;
 
 	SecurityManager::EVALUATIONRESULT evaluationResult;
 	evaluationResult = getSys()->securityManager->evaluateSocketConnection(url, true);
 	if(evaluationResult != SecurityManager::ALLOWED)
 	{
 		incRef();
-		getVm(getSystemState())->addEvent(_MR(this), _MR(Class<SecurityErrorEvent>::getInstanceS(getSystemState(),"No policy file allows socket connection")));
+		getVm(getSystemState())->addEvent(_MR(this), _MR(Class<SecurityErrorEvent>::getInstanceS(getInstanceWorker(),"No policy file allows socket connection")));
 		return;
 	}
 
@@ -316,7 +378,7 @@ ASFUNCTIONBODY_ATOM(ASSocket, _connect)
 	tiny_string host;
 	bool host_is_null;
 	int port;
-	ARG_UNPACK_ATOM (host) (port);
+	ARG_CHECK(ARG_UNPACK (host) (port));
 	host_is_null = argslen > 0 && asAtomHandler::is<Null>(args[0]);
 
 	if (host_is_null)
@@ -332,11 +394,11 @@ ASFUNCTIONBODY_ATOM(ASSocket, bytesAvailable)
 	if (th->job)
 	{
 		th->job->datareceive->lock();
-		asAtomHandler::setUInt(ret,sys,th->job->datareceive->getLength());
+		asAtomHandler::setUInt(ret,wrk,th->job->datareceive->getLength());
 		th->job->datareceive->unlock();
 	}
 	else
-		asAtomHandler::setUInt(ret,sys,0);
+		asAtomHandler::setUInt(ret,wrk,0);
 }
 
 ASFUNCTIONBODY_ATOM(ASSocket,_getEndian)
@@ -346,24 +408,27 @@ ASFUNCTIONBODY_ATOM(ASSocket,_getEndian)
 	if (th->job)
 	{
 		if(th->job->datasend->getLittleEndian())
-			ret = asAtomHandler::fromString(sys,Endian::littleEndian);
+			ret = asAtomHandler::fromString(wrk->getSystemState(),Endian::littleEndian);
 		else
-			ret = asAtomHandler::fromString(sys,Endian::bigEndian);
+			ret = asAtomHandler::fromString(wrk->getSystemState(),Endian::bigEndian);
 	}
 	else
-		ret = asAtomHandler::fromString(sys,Endian::bigEndian);
+		ret = asAtomHandler::fromString(wrk->getSystemState(),Endian::bigEndian);
 }
 
 ASFUNCTIONBODY_ATOM(ASSocket,_setEndian)
 {
 	ASSocket* th=asAtomHandler::as<ASSocket>(obj);
 	bool v = false;
-	if(asAtomHandler::toString(args[0],sys) == Endian::littleEndian)
+	if(asAtomHandler::toString(args[0],wrk) == Endian::littleEndian)
 		v = true;
-	else if(asAtomHandler::toString(args[0],sys) == Endian::bigEndian)
+	else if(asAtomHandler::toString(args[0],wrk) == Endian::bigEndian)
 		v = false;
 	else
-		throwError<ArgumentError>(kInvalidEnumError, "endian");
+	{
+		createError<ArgumentError>(wrk,kInvalidEnumError, "endian");
+		return;
+	}
 	Locker l(th->joblock);
 	if (th->job)
 	{
@@ -371,6 +436,45 @@ ASFUNCTIONBODY_ATOM(ASSocket,_setEndian)
 		th->job->datareceive->setLittleEndian(v);
 	}
 }
+ASFUNCTIONBODY_ATOM(ASSocket,readBoolean)
+{
+	ASSocket* th=asAtomHandler::as<ASSocket>(obj);
+
+	uint8_t res=0;
+	Locker l(th->joblock);
+	if (th->job)
+	{
+		th->job->datareceive->lock();
+		th->job->datareceive->readByte(res);
+		th->job->datareceive->unlock();
+	}
+	else
+	{
+		createError<IOError>(wrk,0,"Socket is not connected");
+		return;
+	}
+	asAtomHandler::setBool(ret,res!=0);
+}
+
+ASFUNCTIONBODY_ATOM(ASSocket,readByte)
+{
+	ASSocket* th=asAtomHandler::as<ASSocket>(obj);
+
+	uint8_t res=0;
+	Locker l(th->joblock);
+	if (th->job)
+	{
+		th->job->datareceive->lock();
+		th->job->datareceive->readByte(res);
+		th->job->datareceive->unlock();
+	}
+	else
+	{
+		createError<IOError>(wrk,0,"Socket is not connected");
+		return;
+	}
+	asAtomHandler::setInt(ret,wrk,int32_t(res));
+}
 
 ASFUNCTIONBODY_ATOM(ASSocket,readBytes)
 {
@@ -378,7 +482,7 @@ ASFUNCTIONBODY_ATOM(ASSocket,readBytes)
 	_NR<ByteArray> data;
 	uint32_t offset;
 	uint32_t length;
-	ARG_UNPACK_ATOM (data)(offset,0)(length,0);
+	ARG_CHECK(ARG_UNPACK (data)(offset,0)(length,0));
 	if (data.isNull())
 		return;
 	Locker l(th->joblock);
@@ -389,6 +493,8 @@ ASFUNCTIONBODY_ATOM(ASSocket,readBytes)
 			length = th->job->datareceive->getLength();
 		uint8_t buf[length];
 		th->job->datareceive->readBytes(0,length,buf);
+		th->job->datareceive->setPosition(length);
+		th->job->datareceive->removeFrontBytes(length);
 		th->job->datareceive->unlock();
 		uint32_t pos = data->getPosition();
 		data->setPosition(offset);
@@ -397,15 +503,98 @@ ASFUNCTIONBODY_ATOM(ASSocket,readBytes)
 	}
 	else
 	{
-		throw Class<IOError>::getInstanceS(sys,"Socket is not connected");
+		createError<IOError>(wrk,0,"Socket is not connected");
+		return;
 	}
 }
 
+ASFUNCTIONBODY_ATOM(ASSocket,readDouble)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.readDouble");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,readFloat)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.readFloat");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,readInt)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.readInt");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,readMultiByte)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.readMultiByte");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,readObject)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.readObject");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,readShort)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.readShort");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,readUnsignedByte)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.readUnsignedByte");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,readUnsignedInt)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.readUnsignedInt");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,readUnsignedShort)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.readUnsignedShort");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,readUTF)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.readUTF");
+}
+
+ASFUNCTIONBODY_ATOM(ASSocket,writeBoolean)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.writeBoolean");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,writeByte)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.writeByte");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,writeDouble)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.writeDouble");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,writeFloat)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.writeFloat");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,writeInt)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.writeInt");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,writeUnsignedInt)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.writeUnsignedInt");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,writeMultiByte)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.writeMultiByte");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,writeObject)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.writeObject");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,writeShort)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.writeShort");
+}
+ASFUNCTIONBODY_ATOM(ASSocket,writeUTF)
+{
+	LOG(LOG_NOT_IMPLEMENTED,"Socket.writeUTF");
+}
+
 ASFUNCTIONBODY_ATOM(ASSocket,writeUTFBytes)
 {
 	ASSocket* th=asAtomHandler::as<ASSocket>(obj);
 	tiny_string data;
-	ARG_UNPACK_ATOM (data);
+	ARG_CHECK(ARG_UNPACK (data));
 	Locker l(th->joblock);
 	if (th->job)
 	{
@@ -415,7 +604,8 @@ ASFUNCTIONBODY_ATOM(ASSocket,writeUTFBytes)
 	}
 	else
 	{
-		throw Class<IOError>::getInstanceS(sys,"Socket is not connected");
+		createError<IOError>(wrk,0,"Socket is not connected");
+		return;
 	}
 }
 ASFUNCTIONBODY_ATOM(ASSocket,writeBytes)
@@ -424,13 +614,19 @@ ASFUNCTIONBODY_ATOM(ASSocket,writeBytes)
 	_NR<ByteArray> data;
 	uint32_t offset;
 	uint32_t length;
-	ARG_UNPACK_ATOM (data)(offset,0)(length,0);
+	ARG_CHECK(ARG_UNPACK (data)(offset,0)(length,0));
 	if (data.isNull())
 		return;
 	if (offset >= data->getLength())
-		throwError<RangeError>(kParamRangeError);
+	{
+		createError<RangeError>(wrk,kParamRangeError);
+		return;
+	}
 	if (offset+length > data->getLength())
-		throwError<RangeError>(kParamRangeError);
+	{
+		createError<RangeError>(wrk,kParamRangeError);
+		return;
+	}
 	Locker l(th->joblock);
 	if (th->job)
 	{
@@ -444,7 +640,8 @@ ASFUNCTIONBODY_ATOM(ASSocket,writeBytes)
 	}
 	else
 	{
-		throw Class<IOError>::getInstanceS(sys,"Socket is not connected");
+		createError<IOError>(wrk,0,"Socket is not connected");
+		return;
 	}
 }
 
@@ -452,7 +649,7 @@ ASFUNCTIONBODY_ATOM(ASSocket,readUTFBytes)
 {
 	ASSocket* th=asAtomHandler::as<ASSocket>(obj);
 	uint32_t length;
-	ARG_UNPACK_ATOM (length);
+	ARG_CHECK(ARG_UNPACK (length));
 	tiny_string data;
 	Locker l(th->joblock);
 	if (th->job)
@@ -461,11 +658,12 @@ ASFUNCTIONBODY_ATOM(ASSocket,readUTFBytes)
 		th->job->datareceive->readUTFBytes(length,data);
 		th->job->datareceive->removeFrontBytes(length);
 		th->job->datareceive->unlock();
-		asAtomHandler::set(ret,asAtomHandler::fromString(sys,data));
+		asAtomHandler::set(ret,asAtomHandler::fromString(wrk->getSystemState(),data));
 	}
 	else
 	{
-		throw Class<IOError>::getInstanceS(sys,"Socket is not connected");
+		createError<IOError>(wrk,0,"Socket is not connected");
+		return;
 	}
 }
 ASFUNCTIONBODY_ATOM(ASSocket,_flush)
@@ -478,7 +676,7 @@ ASFUNCTIONBODY_ATOM(ASSocket,_flush)
 	}
 	else
 	{
-		throw Class<IOError>::getInstanceS(sys,"Socket is not connected");
+		createError<IOError>(wrk,0,"Socket is not connected");
 	}
 }
 
@@ -488,6 +686,10 @@ bool ASSocket::isConnected()
 	return job && job->isConnected();
 }
 
+ASSocket::ASSocket(ASWorker* wrk, Class_base* c) : EventDispatcher(wrk,c), job(nullptr), objectEncoding(OBJECT_ENCODING::AMF3), timeout(20000)
+{
+}
+
 ASFUNCTIONBODY_ATOM(ASSocket, _connected)
 {
 	ASSocket* th=asAtomHandler::as<ASSocket>(obj);
@@ -497,18 +699,18 @@ ASFUNCTIONBODY_ATOM(ASSocket, _connected)
 void ASSocket::threadFinished()
 {
 	Locker l(joblock);
-	job = NULL;
+	job = nullptr;
 }
 
 ASSocketThread::ASSocketThread(_R<ASSocket> _owner, const tiny_string& _hostname, int _port, int _timeout)
 : owner(_owner), hostname(_hostname), port(_port), timeout(_timeout)
 {
 	sendQueue = g_async_queue_new();
-	datasend = _MR(Class<ByteArray>::getInstanceS(owner->getSystemState()));
-	datareceive = _MR(Class<ByteArray>::getInstanceS(owner->getSystemState()));
+	datasend = _MR(Class<ByteArray>::getInstanceS(owner->getInstanceWorker()));
+	datareceive = _MR(Class<ByteArray>::getInstanceS(owner->getInstanceWorker()));
 #ifdef _WIN32
 	HANDLE readPipe, writePipe;
-	if (!CreatePipe(&readPipe,&writePipe,NULL,0))
+	if (!CreatePipe(&readPipe,&writePipe,nullptr,0))
 	{
 		signalListener = -1;
 		signalEmitter = -1;
@@ -537,7 +739,7 @@ ASSocketThread::~ASSocketThread()
 		::close(signalEmitter);
 
 	void *data;
-	while ((data = g_async_queue_try_pop(sendQueue)) != NULL)
+	while ((data = g_async_queue_try_pop(sendQueue)) != nullptr)
 	{
 		tiny_string *s = (tiny_string *)data;
 		delete s;
@@ -550,15 +752,19 @@ void ASSocketThread::execute()
 	if (!sock.connect(hostname, port))
 	{
 		owner->incRef();
-		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getInstanceWorker())));
 		return;
 	}
+	if (!threadAborting)
+	{
+		owner->incRef();
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getInstanceWorker(),"connect")));
+	}
 
-	bool first=true;
+//	bool first=true;
 	struct timeval timeout;
 	int maxfd;
 	fd_set readfds;
-	       
 	while (!threadAborting)
 	{
 		FD_ZERO(&readfds);
@@ -570,11 +776,11 @@ void ASSocketThread::execute()
 		timeout.tv_sec = 10;
 		timeout.tv_usec = 0;
 
-		int status = select(maxfd+1, &readfds, NULL, NULL, &timeout);
+		int status = select(maxfd+1, &readfds, nullptr, nullptr, &timeout);
 		if (status  < 0)
 		{
 			owner->incRef();
-			getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
+			getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getInstanceWorker())));
 			return;
 		}
 
@@ -586,7 +792,7 @@ void ASSocketThread::execute()
 			if (nbytes < 0)
 			{
 				owner->incRef();
-				getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
+				getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getInstanceWorker())));
 				return;
 			}
 			else if (nbytes == 0)
@@ -600,13 +806,13 @@ void ASSocketThread::execute()
 			{
 				executeCommand(cmd, sock);
 			}
-			if (first && !threadAborting)
-			{
-				// send connect event only after first succesful communication
-				first = false;
-				owner->incRef();
-				getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getSystemState(),"connect")));
-			}
+//			if (first && !threadAborting)
+//			{
+//				// send connect event only after first succesful communication
+//				first = false;
+//				owner->incRef();
+//				getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getInstanceWorker(),"connect")));
+//			}
 		}
 		else if (FD_ISSET(sock.fileDescriptor(), &readfds))
 		{
@@ -625,20 +831,20 @@ void ASSocketThread::readSocket(const SocketIO& sock)
 		buf[nbytes] = '\0';
 		datareceive->writeBytes(buf,nbytes);
 		owner->incRef();
-		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<ProgressEvent>::getInstanceS(owner->getSystemState(),nbytes,0,"socketData")));
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<ProgressEvent>::getInstanceS(owner->getInstanceWorker(),nbytes,0,"socketData")));
 	}
 	else if (nbytes == 0)
 	{
 		// The server has closed the socket
 		owner->incRef();
-		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getSystemState(),"close")));
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getInstanceWorker(),"close")));
 		threadAborting = true;
 	}
 	else
 	{
 		// Error
 		owner->incRef();
-		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getInstanceWorker())));
 		threadAborting = true;
 	}
 }
@@ -649,14 +855,11 @@ void ASSocketThread::executeCommand(char cmd, SocketIO& sock)
 	{
 		case SOCKET_COMMAND_SEND:
 		{
-			datasend->lock();
-			sock.sendAll(datasend->getBuffer(datasend->getLength(),false),datasend->getLength());
-			datasend->setLength(0);
-			datasend->unlock();
 			void *data;
-			while ((data = g_async_queue_try_pop(sendQueue)) != NULL)
+			while ((data = g_async_queue_try_pop(sendQueue)) != nullptr)
 			{
-				tiny_string *s = (tiny_string *)data;
+				socketbuf *s = (socketbuf *)data;
+				sock.sendAll(s->buf, s->len);
 				delete s;
 			}
 			break;
@@ -665,7 +868,7 @@ void ASSocketThread::executeCommand(char cmd, SocketIO& sock)
 		{
 			sock.close();
 			owner->incRef();
-			getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getSystemState(),"close")));
+			getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getInstanceWorker(),"close")));
 			threadAborting = true;
 			break;
 		}
@@ -693,7 +896,10 @@ void ASSocketThread::flushData()
 	if (threadAborting)
 		return;
 
-	tiny_string *packet = new tiny_string("");
+	datasend->lock();
+	socketbuf* packet = new socketbuf(datasend->getBuffer(datasend->getLength(),false),datasend->getLength());
+	datasend->setLength(0);
+	datasend->unlock();
 	g_async_queue_push(sendQueue, packet);
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wunused-result"
