@@ -53,7 +53,7 @@ void URLStreamThread::setBytesLoaded(uint32_t b)
 		{
 			timestamp_last_progress = cur;
 			loader->incRef();
-			getVm(loader->getSystemState())->addEvent(loader,_MR(Class<ProgressEvent>::getInstanceS(loader->getSystemState(),b,bytes_total)));
+			getVm(loader->getSystemState())->addEvent(loader,_MR(Class<ProgressEvent>::getInstanceS(loader->getInstanceWorker(),b,bytes_total)));
 		}
 	}
 }
@@ -74,10 +74,10 @@ void URLStreamThread::execute()
 	if(!downloader->hasFailed())
 	{
 		loader->incRef();
-		getVm(loader->getSystemState())->addEvent(loader,_MR(Class<Event>::getInstanceS(loader->getSystemState(),"open")));
+		getVm(loader->getSystemState())->addEvent(loader,_MR(Class<Event>::getInstanceS(loader->getInstanceWorker(),"open")));
 		streambuffer = cache->createReader();
 		loader->incRef();
-		getVm(loader->getSystemState())->addEvent(loader,_MR(Class<ProgressEvent>::getInstanceS(loader->getSystemState(),0,bytes_total)));
+		getVm(loader->getSystemState())->addEvent(loader,_MR(Class<ProgressEvent>::getInstanceS(loader->getInstanceWorker(),0,bytes_total)));
 		cache->waitForTermination();
 		if(!downloader->hasFailed() && !threadAborting)
 		{
@@ -100,16 +100,16 @@ void URLStreamThread::execute()
 	if(success && !threadAborting)
 	{
 		loader->incRef();
-		getVm(loader->getSystemState())->addEvent(loader,_MR(Class<ProgressEvent>::getInstanceS(loader->getSystemState(),downloader->getLength(),downloader->getLength())));
+		getVm(loader->getSystemState())->addEvent(loader,_MR(Class<ProgressEvent>::getInstanceS(loader->getInstanceWorker(),downloader->getLength(),downloader->getLength())));
 		//Send a complete event for this object
 		loader->incRef();
-		getVm(loader->getSystemState())->addEvent(loader,_MR(Class<Event>::getInstanceS(loader->getSystemState(),"complete")));
+		getVm(loader->getSystemState())->addEvent(loader,_MR(Class<Event>::getInstanceS(loader->getInstanceWorker(),"complete")));
 	}
 	else if(!success && !threadAborting)
 	{
 		//Notify an error during loading
 		loader->incRef();
-		getVm(loader->getSystemState())->addEvent(loader,_MR(Class<IOErrorEvent>::getInstanceS(loader->getSystemState())));
+		getVm(loader->getSystemState())->addEvent(loader,_MR(Class<IOErrorEvent>::getInstanceS(loader->getInstanceWorker())));
 	}
 
 	{
@@ -150,7 +150,7 @@ void URLStream::sinit(Class_base* c)
 	IDataInput::linkTraits(c);
 }
 
-ASFUNCTIONBODY_GETTER(URLStream,connected);
+ASFUNCTIONBODY_GETTER(URLStream,connected)
 
 void URLStream::buildTraits(ASObject* o)
 {
@@ -171,14 +171,14 @@ void URLStream::threadFinished(IThreadJob* finishedJob)
 
 ASFUNCTIONBODY_ATOM(URLStream,_constructor)
 {
-	EventDispatcher::_constructor(ret,sys,obj,NULL,0);
+	EventDispatcher::_constructor(ret,wrk,obj,NULL,0);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,load)
 {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	_NR<URLRequest> urlRequest;
-	ARG_UNPACK_ATOM (urlRequest);
+	ARG_CHECK(ARG_UNPACK (urlRequest));
 
 	{
 		Locker l(th->spinlock);
@@ -191,7 +191,7 @@ ASFUNCTIONBODY_ATOM(URLStream,load)
 	{
 		//Notify an error during loading
 		th->incRef();
-		sys->currentVm->addEvent(_MR(th),_MR(Class<IOErrorEvent>::getInstanceS(sys)));
+		wrk->getSystemState()->currentVm->addEvent(_MR(th),_MR(Class<IOErrorEvent>::getInstanceS(wrk)));
 		return;
 	}
 
@@ -199,6 +199,8 @@ ASFUNCTIONBODY_ATOM(URLStream,load)
 	//URLLoader ALWAYS checks for policy files, in contrast to NetStream.play().
 	SecurityManager::checkURLStaticAndThrow(th->url, ~(SecurityManager::LOCAL_WITH_FILE),
 		SecurityManager::LOCAL_WITH_FILE | SecurityManager::LOCAL_TRUSTED, true);
+	if (wrk->currentCallContext && wrk->currentCallContext->exceptionthrown)
+		return;
 
 	th->incRef();
 	URLStreamThread *job=new URLStreamThread(urlRequest, _MR(th), th->data);
@@ -222,120 +224,120 @@ void URLStream::finalize()
 	data.reset();
 }
 
-URLStream::URLStream(Class_base *c):EventDispatcher(c),data(_MNR(Class<ByteArray>::getInstanceS(c->getSystemState()))),job(NULL),connected(false) 
+URLStream::URLStream(ASWorker* wrk, Class_base *c):EventDispatcher(wrk,c),data(_MNR(Class<ByteArray>::getInstanceS(wrk))),job(NULL),connected(false) 
 {
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,bytesAvailable) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::_getBytesAvailable(ret,sys,v, args, argslen);
+	ByteArray::_getBytesAvailable(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,_getEndian) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::_getEndian(ret,sys,v, args, argslen);
+	ByteArray::_getEndian(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,_setEndian) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::_setEndian(ret,sys,v, args, argslen);
+	ByteArray::_setEndian(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,_getObjectEncoding) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::_getObjectEncoding(ret,sys,v, args, argslen);
+	ByteArray::_getObjectEncoding(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,_setObjectEncoding) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::_setObjectEncoding(ret,sys,v, args, argslen);
+	ByteArray::_setObjectEncoding(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readBoolean) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readBoolean(ret,sys,v, args, argslen);
+	ByteArray::readBoolean(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readByte) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readByte(ret,sys,v, args, argslen);
+	ByteArray::readByte(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readBytes) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readBytes(ret,sys,v, args, argslen);
+	ByteArray::readBytes(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readDouble) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readDouble(ret,sys,v, args, argslen);
+	ByteArray::readDouble(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readFloat) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readFloat(ret,sys,v, args, argslen);
+	ByteArray::readFloat(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readInt) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readInt(ret,sys,v, args, argslen);
+	ByteArray::readInt(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readMultiByte) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readMultiByte(ret,sys,v, args, argslen);
+	ByteArray::readMultiByte(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readObject) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readObject(ret,sys,v, args, argslen);
+	ByteArray::readObject(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readShort) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readShort(ret,sys,v, args, argslen);
+	ByteArray::readShort(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readUnsignedByte) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readUnsignedByte(ret,sys,v, args, argslen);
+	ByteArray::readUnsignedByte(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readUnsignedInt) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readUnsignedInt(ret,sys,v, args, argslen);
+	ByteArray::readUnsignedInt(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readUnsignedShort) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readUnsignedShort(ret,sys,v, args, argslen);
+	ByteArray::readUnsignedShort(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readUTF) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readUTF(ret,sys,v, args, argslen);
+	ByteArray::readUTF(ret,wrk,v, args, argslen);
 }
 
 ASFUNCTIONBODY_ATOM(URLStream,readUTFBytes) {
 	URLStream* th=asAtomHandler::as<URLStream>(obj);
 	asAtom v = asAtomHandler::fromObject(th->data.getPtr());
-	ByteArray::readUTFBytes(ret,sys,v, args, argslen);
+	ByteArray::readUTFBytes(ret,wrk,v, args, argslen);
 }
