@@ -71,21 +71,21 @@ void XMLSocket::finalize()
 		job->threadAborting = true;
 		job->requestClose();
 		job->threadAbort();
-		job = NULL;
+		job = nullptr;
 	}
 	timeout = 20000;
 }
 
-ASFUNCTIONBODY_GETTER_SETTER(XMLSocket, timeout);
+ASFUNCTIONBODY_GETTER_SETTER(XMLSocket, timeout)
 
 ASFUNCTIONBODY_ATOM(XMLSocket,_constructor)
 {
 	tiny_string host;
 	bool host_is_null;
 	int port;
-	ARG_UNPACK_ATOM (host, "") (port, 0);
+	ARG_CHECK(ARG_UNPACK (host, "") (port, 0));
 
-	EventDispatcher::_constructor(ret,sys,obj,NULL,0);
+	EventDispatcher::_constructor(ret,wrk,obj,NULL,0);
 
 	XMLSocket* th=asAtomHandler::as<XMLSocket>(obj);
 	host_is_null = argslen > 0 && asAtomHandler::is<Null>(args[0]);
@@ -112,17 +112,23 @@ ASFUNCTIONBODY_ATOM(XMLSocket, _close)
 void XMLSocket::connect(tiny_string host, int port)
 {
 	if (port <= 0 || port > 65535)
-		throw Class<SecurityError>::getInstanceS(getSystemState(),"Invalid port");
+	{
+		createError<SecurityError>(getInstanceWorker(),0,"Invalid port");
+		return;
+	}
 
 	if (host.empty())
 		host = getSys()->mainClip->getOrigin().getHostname();
 
 	if (isConnected())
-		throw Class<IOError>::getInstanceS(getSystemState(),"Already connected");
+	{
+		createError<IOError>(getInstanceWorker(),0,"Already connected");
+		return;
+	}
 
 	// Host shouldn't contain scheme or port
-	if (host.strchr(':') != NULL)
-		throw Class<SecurityError>::getInstanceS(getSystemState(),"Invalid hostname");
+	if (host.strchr(':') != nullptr)
+		createError<SecurityError>(getInstanceWorker(),0,"Invalid hostname");
 
 	// Check sandbox and policy file
 	size_t buflen = host.numBytes() + 22;
@@ -134,13 +140,15 @@ void XMLSocket::connect(tiny_string host, int port)
 		~(SecurityManager::LOCAL_WITH_FILE),
 		SecurityManager::LOCAL_WITH_FILE | SecurityManager::LOCAL_TRUSTED,
 		true);
+	if (getInstanceWorker()->currentCallContext && getInstanceWorker()->currentCallContext->exceptionthrown)
+		return;
 
 	SecurityManager::EVALUATIONRESULT evaluationResult;
 	evaluationResult = getSys()->securityManager->evaluateSocketConnection(url, true);
 	if(evaluationResult != SecurityManager::ALLOWED)
 	{
 		incRef();
-		getVm(getSystemState())->addEvent(_MR(this), _MR(Class<SecurityErrorEvent>::getInstanceS(getSystemState(),"No policy file allows socket connection")));
+		getVm(getSystemState())->addEvent(_MR(this), _MR(Class<SecurityErrorEvent>::getInstanceS(getInstanceWorker(),"No policy file allows socket connection")));
 		return;
 	}
 
@@ -156,7 +164,7 @@ ASFUNCTIONBODY_ATOM(XMLSocket, _connect)
 	tiny_string host;
 	bool host_is_null;
 	int port;
-	ARG_UNPACK_ATOM (host) (port);
+	ARG_CHECK(ARG_UNPACK (host) (port));
 	host_is_null = argslen > 0 && asAtomHandler::is<Null>(args[0]);
 
 	if (host_is_null)
@@ -169,7 +177,7 @@ ASFUNCTIONBODY_ATOM(XMLSocket, _send)
 {
 	XMLSocket* th=asAtomHandler::as<XMLSocket>(obj);
 	tiny_string data;
-	ARG_UNPACK_ATOM (data);
+	ARG_CHECK(ARG_UNPACK (data));
 
 	Locker l(th->joblock);
 	if (th->job)
@@ -178,7 +186,7 @@ ASFUNCTIONBODY_ATOM(XMLSocket, _send)
 	}
 	else
 	{
-		throw Class<IOError>::getInstanceS(sys,"Socket is not connected");
+		createError<IOError>(wrk,0,"Socket is not connected");
 	}
 }
 
@@ -197,7 +205,64 @@ ASFUNCTIONBODY_ATOM(XMLSocket, _connected)
 void XMLSocket::threadFinished()
 {
 	Locker l(joblock);
-	job = NULL;
+	job = nullptr;
+}
+void XMLSocket::AVM1HandleEvent(EventDispatcher *dispatcher, Event* e)
+{
+	if (dispatcher == this)
+	{
+		ASWorker* wrk = getInstanceWorker();
+		if (e->type == "connect")
+		{
+			asAtom func=asAtomHandler::invalidAtom;
+			multiname m(nullptr);
+			m.name_type=multiname::NAME_STRING;
+			m.isAttribute = false;
+			m.name_s_id=BUILTIN_STRINGS::STRING_ONCONNECT;
+			getVariableByMultiname(func,m,GET_VARIABLE_OPTION::NONE,wrk);
+			if (asAtomHandler::is<AVM1Function>(func))
+			{
+				asAtom ret=asAtomHandler::invalidAtom;
+				asAtom obj = asAtomHandler::fromObject(this);
+				asAtom arg0 = asAtomHandler::fromBool(this->isConnected());
+				asAtomHandler::as<AVM1Function>(func)->call(&ret,&obj,&arg0,1);
+				asAtomHandler::as<AVM1Function>(func)->decRef();
+			}
+		}
+		else if (e->type == "data")
+		{
+			asAtom func=asAtomHandler::invalidAtom;
+			multiname m(nullptr);
+			m.name_type=multiname::NAME_STRING;
+			m.isAttribute = false;
+			m.name_s_id=BUILTIN_STRINGS::STRING_ONDATA;
+			getVariableByMultiname(func,m,GET_VARIABLE_OPTION::NONE,wrk);
+			if (asAtomHandler::is<AVM1Function>(func))
+			{
+				asAtom ret=asAtomHandler::invalidAtom;
+				asAtom obj = asAtomHandler::fromObject(this);
+				asAtom arg0 = asAtomHandler::fromString(getSystemState(),e->as<DataEvent>()->data);
+				asAtomHandler::as<AVM1Function>(func)->call(&ret,&obj,&arg0,1);
+				asAtomHandler::as<AVM1Function>(func)->decRef();
+			}
+		}
+		else if (e->type == "close")
+		{
+			asAtom func=asAtomHandler::invalidAtom;
+			multiname m(nullptr);
+			m.name_type=multiname::NAME_STRING;
+			m.isAttribute = false;
+			m.name_s_id=BUILTIN_STRINGS::STRING_ONCLOSE;
+			getVariableByMultiname(func,m,GET_VARIABLE_OPTION::NONE,wrk);
+			if (asAtomHandler::is<AVM1Function>(func))
+			{
+				asAtom ret=asAtomHandler::invalidAtom;
+				asAtom obj = asAtomHandler::fromObject(this);
+				asAtomHandler::as<AVM1Function>(func)->call(&ret,&obj,nullptr,0);
+				asAtomHandler::as<AVM1Function>(func)->decRef();
+			}
+		}
+	}
 }
 
 XMLSocketThread::XMLSocketThread(_R<XMLSocket> _owner, const tiny_string& _hostname, int _port, int _timeout)
@@ -236,7 +301,7 @@ XMLSocketThread::~XMLSocketThread()
 		::close(signalEmitter);
 
 	void *data;
-	while ((data = g_async_queue_try_pop(sendQueue)) != NULL)
+	while ((data = g_async_queue_try_pop(sendQueue)) != nullptr)
 	{
 		tiny_string *s = (tiny_string *)data;
 		delete s;
@@ -249,17 +314,20 @@ void XMLSocketThread::execute()
 	if (!sock.connect(hostname, port))
 	{
 		owner->incRef();
-		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
+		if (owner->getSystemState()->mainClip->needsActionScript3())
+			getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getInstanceWorker())));
+		else
+			getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getInstanceWorker(),"connect")));
 		return;
 	}
 
 	owner->incRef();
-	getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getSystemState(),"connect")));
+	getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getInstanceWorker(),"connect")));
 
 	struct timeval timeout;
 	int maxfd;
 	fd_set readfds;
-	       
+	
 	while (!threadAborting)
 	{
 		FD_ZERO(&readfds);
@@ -271,11 +339,11 @@ void XMLSocketThread::execute()
 		timeout.tv_sec = 10;
 		timeout.tv_usec = 0;
 
-		int status = select(maxfd+1, &readfds, NULL, NULL, &timeout);
+		int status = select(maxfd+1, &readfds, nullptr, nullptr, &timeout);
 		if (status  < 0)
 		{
 			owner->incRef();
-			getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
+			getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getInstanceWorker())));
 			return;
 		}
 
@@ -287,7 +355,7 @@ void XMLSocketThread::execute()
 			if (nbytes < 0)
 			{
 				owner->incRef();
-				getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
+				getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getInstanceWorker())));
 				return;
 			}
 			else if (nbytes == 0)
@@ -319,20 +387,20 @@ void XMLSocketThread::readSocket(const SocketIO& sock)
 		buf[nbytes] = '\0';
 		tiny_string data(buf, true);
 		owner->incRef();
-		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<DataEvent>::getInstanceS(owner->getSystemState(),data)));
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<DataEvent>::getInstanceS(owner->getInstanceWorker(),data)));
 	}
 	else if (nbytes == 0)
 	{
 		// The server has closed the socket
 		owner->incRef();
-		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getSystemState(),"close")));
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getInstanceWorker(),"close")));
 		threadAborting = true;
 	}
 	else
 	{
 		// Error
 		owner->incRef();
-		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getInstanceWorker())));
 		threadAborting = true;
 	}
 }
@@ -344,11 +412,15 @@ void XMLSocketThread::executeCommand(char cmd, SocketIO& sock)
 		case SOCKET_COMMAND_SEND:
 		{
 			void *data;
-			while ((data = g_async_queue_try_pop(sendQueue)) != NULL)
+			while ((data = g_async_queue_try_pop(sendQueue)) != nullptr)
 			{
 				tiny_string *s = (tiny_string *)data;
 				sock.sendAll(s->raw_buf(), s->numBytes());
 				delete s;
+				// according to specs every message is terminated by a null byte
+				char buf=0;
+				sock.sendAll(&buf, 1);
+				
 			}
 			break;
 		}
