@@ -29,7 +29,6 @@
 #include "backends/decoder.h"
 #include "backends/audio.h"
 #include "NetStreamInfo.h"
-#include "scripting/flash/utils/ByteArray.h"
 
 namespace lightspark
 {
@@ -46,12 +45,13 @@ private:
 	tiny_string getContentTypeHeader() const;
 	void validateHeaderName(const tiny_string& headerName) const;
 	ASPROPERTY_GETTER_SETTER(tiny_string,contentType);
-	ASPROPERTY_GETTER_SETTER(_R<Array>,requestHeaders);
+	ASPROPERTY_GETTER_SETTER(_NR<Array>,requestHeaders);
 public:
-	URLRequest(Class_base* c, const tiny_string u="", const tiny_string m="GET", _NR<ASObject> d = NullRef);
-	void finalize();
+	URLRequest(ASWorker* wrk,Class_base* c, const tiny_string u="", const tiny_string m="GET", _NR<ASObject> d = NullRef);
+	void finalize() override;
+	bool destruct() override;
+	void prepareShutdown() override;
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o);
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(_getURL);
 	ASFUNCTION_ATOM(_setURL);
@@ -69,7 +69,7 @@ public:
 class URLRequestMethod: public ASObject
 {
 public:
-	URLRequestMethod(Class_base* c):ASObject(c){}
+	URLRequestMethod(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base*);
 };
 
@@ -79,10 +79,9 @@ private:
 	void decode(const tiny_string& s);
 	tiny_string toString_priv();
 public:
-	URLVariables(Class_base* c):ASObject(c){}
-	URLVariables(Class_base* c,const tiny_string& s);
+	URLVariables(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
+	URLVariables(ASWorker* wrk,Class_base* c,const tiny_string& s);
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o);
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(decode);
 	ASFUNCTION_ATOM(_toString);
@@ -92,7 +91,7 @@ public:
 class URLLoaderDataFormat: public ASObject
 {
 public:
-	URLLoaderDataFormat(Class_base* c):ASObject(c){}
+	URLLoaderDataFormat(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base*);
 };
 
@@ -100,7 +99,7 @@ public:
 class SharedObjectFlushStatus: public ASObject
 {
 public:
-	SharedObjectFlushStatus(Class_base* c):ASObject(c){}
+	SharedObjectFlushStatus(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base*);
 };
 
@@ -108,10 +107,11 @@ class SharedObject: public EventDispatcher
 {
 private:
 	tiny_string name;
-	bool hasData;
 public:
-	SharedObject(Class_base* c);
-	bool doFlush();
+	SharedObject(ASWorker* wrk,Class_base* c);
+	bool destruct() override;
+	void prepareShutdown() override;
+	bool doFlush(ASWorker* wrk);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(getLocal);
 	ASFUNCTION_ATOM(getRemote);
@@ -120,7 +120,7 @@ public:
 	ASFUNCTION_ATOM(close);
 	ASFUNCTION_ATOM(connect);
 	ASFUNCTION_ATOM(setProperty);
-	ASPROPERTY_GETTER_SETTER(_R<ASObject>,client);
+	ASPROPERTY_GETTER_SETTER(_NR<ASObject>,client);
 	ASPROPERTY_GETTER(_NR<ASObject>,data);
 	ASFUNCTION_ATOM(_getDefaultObjectEncoding);
 	ASFUNCTION_ATOM(_setDefaultObjectEncoding);
@@ -145,7 +145,7 @@ public:
 class DynamicPropertyOutput:public ASObject
 {
 public:
-	DynamicPropertyOutput(Class_base* c):ASObject(c){}
+	DynamicPropertyOutput(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(writeDynamicProperty);
 };
@@ -153,8 +153,7 @@ public:
 class ObjectEncoding: public ASObject
 {
 public:
-	ObjectEncoding(Class_base* c):ASObject(c){}
-	enum ENCODING { AMF0=0, AMF3=3, DEFAULT=3 };
+	ObjectEncoding(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base*);
 	ASPROPERTY_GETTER_SETTER(_NR<ASObject>, dynamicPropertyWriter);
 };
@@ -165,7 +164,7 @@ class URLLoaderThread : public DownloaderThreadBase
 {
 private:
 	_R<URLLoader> loader;
-	void execute();
+	void execute() override;
 public:
 	URLLoaderThread(_R<URLRequest> _request, _R<URLLoader> _loader);
 };
@@ -179,11 +178,11 @@ private:
 	URLLoaderThread *job;
 	uint64_t timestamp_last_progress;
 public:
-	URLLoader(Class_base* c);
-	void finalize();
+	URLLoader(ASWorker* wrk,Class_base* c);
+	void finalize() override;
+	void prepareShutdown() override;
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o);
-	void threadFinished(IThreadJob *job);
+	void threadFinished(IThreadJob *job) override;
 	void setData(_NR<ASObject> data);
 	ASObject* getData() const
 	{
@@ -191,8 +190,8 @@ public:
 	}
 	tiny_string getDataFormat();
 	void setDataFormat(const tiny_string& newFormat);
-	void setBytesTotal(uint32_t b);
-	void setBytesLoaded(uint32_t b);
+	void setBytesTotal(uint32_t b) override;
+	void setBytesLoaded(uint32_t b) override;
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(load);
 	ASFUNCTION_ATOM(close);
@@ -210,9 +209,9 @@ private:
 	asAtom result;
 	asAtom status;
 public:
-	Responder(Class_base* c):ASObject(c),result(asAtomHandler::invalidAtom),status(asAtomHandler::invalidAtom){}
+	Responder(ASWorker* wrk,Class_base* c):ASObject(wrk,c),result(asAtomHandler::invalidAtom),status(asAtomHandler::invalidAtom){}
 	static void sinit(Class_base*);
-	void finalize();
+	void finalize() override;
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(onResult);
 };
@@ -234,17 +233,16 @@ private:
 	_NR<Responder> responder;
 	uint32_t messageCount;
 	//The connection is to a flash media server
-	ObjectEncoding::ENCODING objectEncoding;
+	OBJECT_ENCODING objectEncoding;
 	PROXY_TYPE proxyType;
 	//IThreadJob interface
-	void execute();
-	void threadAbort();
-	void jobFence();
+	void execute() override;
+	void threadAbort() override;
+	void jobFence() override;
 public:
-	NetConnection(Class_base* c);
-	void finalize();
+	NetConnection(ASWorker* wrk,Class_base* c);
+	void finalize() override;
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o);
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(connect);
 	ASFUNCTION_ATOM(call);
@@ -262,13 +260,13 @@ public:
 	ASPROPERTY_GETTER_SETTER(NullableRef<ASObject>,client);
 	ASPROPERTY_GETTER_SETTER(uint32_t,maxPeerConnections);
 	ASPROPERTY_GETTER(tiny_string,nearID);
-	void afterExecution(_R<Event> e);
+	void afterExecution(_R<Event> e) override;
 };
 
 class NetStreamAppendBytesAction: public ASObject
 {
 public:
-	NetStreamAppendBytesAction(Class_base* c):ASObject(c){}
+	NetStreamAppendBytesAction(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base*);
 };
 
@@ -299,12 +297,12 @@ private:
 	Mutex mutex;
 	Mutex countermutex;
 	//IThreadJob interface for long jobs
-	void execute();
-	void threadAbort();
-	void jobFence();
+	void execute() override;
+	void threadAbort() override;
+	void jobFence() override;
 	//ITickJob interface to frame advance
-	void tick();
-	void tickFence();
+	void tick() override;
+	void tickFence() override;
 	bool isReady() const;
 	_NR<ASObject> client;
 
@@ -332,11 +330,10 @@ private:
 	_NR<ByteArray> datagenerationbuffer;
 	StreamDecoder* streamDecoder;
 public:
-	NetStream(Class_base* c);
+	NetStream(ASWorker* wrk,Class_base* c);
 	~NetStream();
-	void finalize();
+	void finalize() override;
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o);
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(play);
 	ASFUNCTION_ATOM(play2);
@@ -429,12 +426,14 @@ public:
 		@pre the object should be locked
 	*/
 	void unlock();
+	
+	void clearFrameBuffer();
 };
 
 class LocalConnection: public EventDispatcher
 {
 public:
-	LocalConnection(Class_base* c);
+	LocalConnection(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER(bool,isSupported);
@@ -450,7 +449,7 @@ public:
 class NetGroup: public EventDispatcher
 {
 public:
-	NetGroup(Class_base* c);
+	NetGroup(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 };
@@ -458,14 +457,16 @@ public:
 class FileReference: public EventDispatcher
 {
 public:
-	FileReference(Class_base* c);
+	FileReference(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
+	ASPROPERTY_GETTER(number_t,size);
+	
 };
 class FileFilter: public ASObject
 {
 public:
-	FileFilter(Class_base* c);
+	FileFilter(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
 	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(tiny_string,description);
@@ -476,15 +477,15 @@ public:
 class DRMManager: public EventDispatcher
 {
 public:
-	DRMManager(Class_base* c);
+	DRMManager(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base*);
 	ASPROPERTY_GETTER(bool,isSupported);
 };
 
-void navigateToURL(asAtom& ret,SystemState* sys,asAtom& obj,asAtom* args, const unsigned int argslen);
-void sendToURL(asAtom& ret,SystemState* sys,asAtom& obj,asAtom* args, const unsigned int argslen);
-void registerClassAlias(asAtom& ret,SystemState* sys,asAtom& obj,asAtom* args, const unsigned int argslen);
-void getClassByAlias(asAtom& ret,SystemState* sys,asAtom& obj,asAtom* args, const unsigned int argslen);
+void navigateToURL(asAtom& ret,ASWorker* wrk,asAtom& obj,asAtom* args, const unsigned int argslen);
+void sendToURL(asAtom& ret,ASWorker* wrk,asAtom& obj,asAtom* args, const unsigned int argslen);
+void registerClassAlias(asAtom& ret,ASWorker* wrk,asAtom& obj,asAtom* args, const unsigned int argslen);
+void getClassByAlias(asAtom& ret,ASWorker* wrk,asAtom& obj,asAtom* args, const unsigned int argslen);
 
 }
 
