@@ -4,8 +4,8 @@
 
 using namespace lightspark;
 
-Sample::Sample(Class_base* c):
-	ASObject(c)
+Sample::Sample(ASWorker* wrk, Class_base* c):
+	ASObject(wrk,c)
 {
 }
 
@@ -15,8 +15,8 @@ void Sample::sinit(Class_base* c)
 }
 
 
-DeleteObjectSample::DeleteObjectSample(Class_base* c):
-	Sample(c)
+DeleteObjectSample::DeleteObjectSample(ASWorker* wrk,Class_base* c):
+	Sample(wrk,c)
 {
 }
 
@@ -26,8 +26,8 @@ void DeleteObjectSample::sinit(Class_base* c)
 }
 
 
-NewObjectSample::NewObjectSample(Class_base* c):
-	Sample(c)
+NewObjectSample::NewObjectSample(ASWorker* wrk, Class_base* c):
+	Sample(wrk,c)
 {
 }
 
@@ -40,8 +40,8 @@ void NewObjectSample::sinit(Class_base* c)
 ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(NewObjectSample,object);
 ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(NewObjectSample,size);
 
-StackFrame::StackFrame(Class_base* c):
-	ASObject(c)
+StackFrame::StackFrame(ASWorker* wrk, Class_base* c):
+	ASObject(wrk,c)
 {
 }
 
@@ -65,38 +65,38 @@ ASFUNCTIONBODY_ATOM(lightspark,getGetterInvocationCount)
 {
 	_NR<ASObject> o;
 	_NR<ASQName> qname;
-	ARG_UNPACK_ATOM (o)(qname);
+	ARG_CHECK(ARG_UNPACK (o)(qname));
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.getGetterInvocationCount not implemented");
-	asAtomHandler::setInt(ret,sys,0);
+	asAtomHandler::setInt(ret,wrk,0);
 }
 ASFUNCTIONBODY_ATOM(lightspark,getInvocationCount)
 {
 	_NR<ASObject> o;
 	_NR<ASQName> qname;
-	ARG_UNPACK_ATOM (o)(qname);
+	ARG_CHECK(ARG_UNPACK (o)(qname));
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.getInvocationCount not implemented");
-	asAtomHandler::setInt(ret,sys,0);
+	asAtomHandler::setInt(ret,wrk,0);
 }
 ASFUNCTIONBODY_ATOM(lightspark,getSetterInvocationCount)
 {
 	_NR<ASObject> o;
 	_NR<ASQName> qname;
-	ARG_UNPACK_ATOM (o)(qname);
+	ARG_CHECK(ARG_UNPACK (o)(qname));
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.getSetterInvocationCount not implemented");
-	asAtomHandler::setInt(ret,sys,0);
+	asAtomHandler::setInt(ret,wrk,0);
 }
 ASFUNCTIONBODY_ATOM(lightspark,getLexicalScopes)
 {
 	_NR<IFunction> func;
-	ARG_UNPACK_ATOM (func);
+	ARG_CHECK(ARG_UNPACK (func));
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.getLexicalScopes not implemented");
-	Array* res=Class<Array>::getInstanceSNoArgs(sys);
+	Array* res=Class<Array>::getInstanceSNoArgs(wrk);
 	ret = asAtomHandler::fromObject(res);
 }
 ASFUNCTIONBODY_ATOM(lightspark,getMasterString)
 {
 	tiny_string str;
-	ARG_UNPACK_ATOM (str);
+	ARG_CHECK(ARG_UNPACK (str));
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.getMasterString not implemented");
 	asAtomHandler::setNull(ret);
 }
@@ -104,14 +104,14 @@ ASFUNCTIONBODY_ATOM(lightspark,getMemberNames)
 {
 	bool instanceNames;
 	_NR<ASObject> o;
-	ARG_UNPACK_ATOM (o)(instanceNames, false);
+	ARG_CHECK(ARG_UNPACK (o)(instanceNames, false));
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.getMemberNames not implemented");
 	asAtomHandler::setUndefined(ret);
 }
 ASFUNCTIONBODY_ATOM(lightspark,getSampleCount)
 {
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.getSampleCount not implemented");
-	asAtomHandler::setInt(ret,sys,-1);
+	asAtomHandler::setInt(ret,wrk,-1);
 }
 ASFUNCTIONBODY_ATOM(lightspark,getSamples)
 {
@@ -122,14 +122,14 @@ ASFUNCTIONBODY_ATOM(lightspark,getSamples)
 ASFUNCTIONBODY_ATOM(lightspark,getSize)
 {
 	_NR<ASObject> o;
-	ARG_UNPACK_ATOM (o);
+	ARG_CHECK(ARG_UNPACK (o));
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.getSize not implemented");
-	asAtomHandler::setInt(ret,sys,0);
+	asAtomHandler::setInt(ret,wrk,0);
 }
 ASFUNCTIONBODY_ATOM(lightspark,getSavedThis)
 {
 	_NR<ASObject> o;
-	ARG_UNPACK_ATOM (o);
+	ARG_CHECK(ARG_UNPACK (o));
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.getSavedThis not implemented");
 	asAtomHandler::setNull(ret);
 }
@@ -137,7 +137,7 @@ ASFUNCTIONBODY_ATOM(lightspark,isGetterSetter)
 {
 	_NR<ASObject> o;
 	_NR<ASQName> qname;
-	ARG_UNPACK_ATOM (o)(qname);
+	ARG_CHECK(ARG_UNPACK (o)(qname));
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.isGetterSetter not implemented");
 	asAtomHandler::setBool(ret,false);
 }
@@ -149,7 +149,7 @@ ASFUNCTIONBODY_ATOM(lightspark,pauseSampling)
 ASFUNCTIONBODY_ATOM(lightspark,sampleInternalAllocs)
 {
 	bool b;
-	ARG_UNPACK_ATOM (b);
+	ARG_CHECK(ARG_UNPACK (b));
 	LOG(LOG_NOT_IMPLEMENTED,"flash.sampler.sampleInternalAllocs not implemented");
 }
 ASFUNCTIONBODY_ATOM(lightspark,setSamplerCallback)
