@@ -26,6 +26,9 @@
 #include "scripting/flash/display/flashdisplay.h"
 #include "3rdparty/pugixml/src/pugixml.hpp"
 
+// according to TextLineMetrics specs, there are always 2 pixels added to each side of a textfield
+#define TEXTFIELD_PADDING 2
+
 namespace lightspark
 {
 class Array;
@@ -34,7 +37,7 @@ class DefineEditTextTag;
 class AntiAliasType : public ASObject
 {
 public:
-	AntiAliasType(Class_base* c):ASObject(c){}
+	AntiAliasType(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
@@ -42,7 +45,7 @@ class ASFont: public ASObject
 {
 public:
 	static std::vector<asAtom>* getFontList();
-	ASFont(Class_base* c):ASObject(c),fontType("device"){}
+	ASFont(ASWorker* wrk,Class_base* c):ASObject(wrk,c),fontType("device"){}
 	void SetFont(tiny_string& fontname,bool is_bold,bool is_italic, bool is_Embedded, bool is_EmbeddedCFF);
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(enumerateFonts);
@@ -58,8 +61,8 @@ class StyleSheet: public EventDispatcher
 private:
 	std::map<tiny_string, asAtom > styles;
 public:
-	StyleSheet(Class_base* c):EventDispatcher(c){}
-	void finalize();
+	StyleSheet(ASWorker* wrk,Class_base* c):EventDispatcher(wrk,c){}
+	void finalize() override;
 	ASFUNCTION_ATOM(getStyle);
 	ASFUNCTION_ATOM(setStyle);
 	ASFUNCTION_ATOM(_getStyleNames);
@@ -92,10 +95,10 @@ public:
 	enum GRID_FIT_TYPE { GF_NONE, GF_PIXEL, GF_SUBPIXEL };
 	enum TEXT_INTERACTION_MODE { TI_NORMAL, TI_SELECTION };
 private:
-	_NR<DisplayObject> hitTestImpl(_NR<DisplayObject> last, number_t x, number_t y, HIT_TYPE type,bool interactiveObjectsOnly) override;
-	bool renderImpl(RenderContext& ctxt) const override;
-	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const override;
-	IDrawable* invalidate(DisplayObject* target, const MATRIX& initialMatrix, bool smoothing) override;
+	_NR<DisplayObject> hitTestImpl(number_t x, number_t y, HIT_TYPE type,bool interactiveObjectsOnly) override;
+	bool renderImpl(RenderContext& ctxt) override;
+	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) override;
+	IDrawable* invalidate(DisplayObject* target, const MATRIX& initialMatrix, bool smoothing, InvalidateQueue* q, _NR<DisplayObject>* cachedBitmap) override;
 	void requestInvalidation(InvalidateQueue* q, bool forceTextureRefresh=false) override;
 	void defaultEventBehavior(_R<Event> e) override;
 	void updateText(const tiny_string& new_text);
@@ -110,7 +113,7 @@ private:
 	int32_t getMaxScrollH();
 	int32_t getMaxScrollV();
 	void textUpdated();
-	void setSizeAndPositionFromAutoSize();
+	void setSizeAndPositionFromAutoSize(bool updatewidth=true);
 	void replaceText(unsigned int begin, unsigned int end, const tiny_string& newText);
 	EDIT_TYPE type;
 	ANTI_ALIAS_TYPE antiAliasType;
@@ -119,32 +122,37 @@ private:
 	_NR<ASString> restrictChars;
 	number_t autosizeposition;
 	tiny_string tagvarname;
+	DisplayObject* tagvartarget;
 	Mutex invalidatemutex;
 	DefineEditTextTag* tag;
+	int32_t originalXPosition;
+	int32_t originalWidth;
 
 	// these are only used when drawing to DisplayObject, so they are guarranteed not to be destroyed during rendering
-	FILLSTYLE fillstyleTextColor;
+	list<FILLSTYLE> fillstyleTextColor;
 	FILLSTYLE fillstyleBackgroundColor;
 	LINESTYLE2 lineStyleBorder;
 	LINESTYLE2 lineStyleCaret;
+	Mutex* linemutex;
 	void getTextBounds(const tiny_string &txt, number_t &xmin, number_t &xmax, number_t &ymin, number_t &ymax);
 protected:
 	void afterSetLegacyMatrix() override;
 public:
-	TextField(Class_base* c, const TextData& textData=TextData(), bool _selectable=true, bool readOnly=true, const char* varname="", DefineEditTextTag* _tag=nullptr);
+	TextField(ASWorker* wrk,Class_base* c, const TextData& textData=TextData(), bool _selectable=true, bool readOnly=true, const char* varname="", DefineEditTextTag* _tag=nullptr);
+	~TextField();
 	void finalize() override;
+	void prepareShutdown() override;
 	static void sinit(Class_base* c);
-	static void buildTraits(ASObject* o);
 	void setHtmlText(const tiny_string& html);
 	void avm1SyncTagVar();
 	void UpdateVariableBinding(asAtom v) override;
 	void afterLegacyInsert() override;
-	void afterLegacyDelete(DisplayObjectContainer* par) override;
+	void afterLegacyDelete(DisplayObjectContainer* parent, bool inskipping) override;
 	void lostFocus() override;
 	void gotFocus() override;
 	void textInputChanged(const tiny_string& newtext) override;
 	void tick() override;
-	void tickFence() override {}
+	void tickFence() override;
 	uint32_t getTagID() const override;
 	
 	ASFUNCTION_ATOM(appendText);
@@ -188,6 +196,8 @@ public:
 	ASFUNCTION_ATOM(_replaceText);
 	ASFUNCTION_ATOM(_replaceSelectedText);
 	ASFUNCTION_ATOM(_getCharBoundaries);
+	ASFUNCTION_ATOM(_getdisplayAsPassword);
+	ASFUNCTION_ATOM(_setdisplayAsPassword);
 	ASPROPERTY_GETTER_SETTER(bool, alwaysShowSelection);
 	ASFUNCTION_GETTER_SETTER(background);
 	ASFUNCTION_GETTER_SETTER(backgroundColor);
@@ -195,7 +205,6 @@ public:
 	ASFUNCTION_GETTER_SETTER(borderColor);
 	ASPROPERTY_GETTER(uint32_t, caretIndex);
 	ASPROPERTY_GETTER_SETTER(bool, condenseWhite);
-	ASPROPERTY_GETTER_SETTER(bool, displayAsPassword);
 	ASPROPERTY_GETTER_SETTER(bool, embedFonts);
 	ASPROPERTY_GETTER_SETTER(int32_t, maxChars);
 	ASFUNCTION_GETTER_SETTER(multiline);
@@ -211,26 +220,29 @@ public:
 	ASPROPERTY_GETTER_SETTER(number_t, thickness);
 	ASFUNCTION_GETTER_SETTER(type);
 	ASPROPERTY_GETTER_SETTER(bool, useRichTextClipboard);
-	
-	std::string toDebugString() override;
+	ASFUNCTION_ATOM(_getTextFieldX);
+	ASFUNCTION_ATOM(_setTextFieldX);
+
+	std::string toDebugString() const override;
 };
 
 class TextFormat: public ASObject
 {
 private:
-	void onAlign(const tiny_string& old);
+	void onAlign(const asAtom& old);
 public:
-	TextFormat(Class_base* c):ASObject(c,T_OBJECT,SUBTYPE_TEXTFORMAT){}
-	bool destruct();
+	TextFormat(ASWorker* wrk,Class_base* c);
+	void finalize() override;
+	bool destruct() override;
+	void prepareShutdown() override;
 	static void sinit(Class_base* c);
-	static void buildTraits(ASObject* o);
 	ASFUNCTION_ATOM(_constructor);
-	ASPROPERTY_GETTER_SETTER(tiny_string,align);
+	ASPROPERTY_GETTER_SETTER_ATOM(align);
 	ASPROPERTY_GETTER_SETTER_ATOM(blockIndent);
 	ASPROPERTY_GETTER_SETTER_ATOM(bold);
 	ASPROPERTY_GETTER_SETTER_ATOM(bullet);
 	ASPROPERTY_GETTER_SETTER_ATOM(color);
-	ASPROPERTY_GETTER_SETTER(tiny_string,font);
+	ASPROPERTY_GETTER_SETTER_ATOM(font);
 	ASPROPERTY_GETTER_SETTER_ATOM(indent);
 	ASPROPERTY_GETTER_SETTER_ATOM(italic);
 	ASPROPERTY_GETTER_SETTER_ATOM(kerning);
@@ -240,30 +252,30 @@ public:
 	ASPROPERTY_GETTER_SETTER_ATOM(rightMargin);
 	ASPROPERTY_GETTER_SETTER_ATOM(size);
 	ASPROPERTY_GETTER_SETTER(_NR<Array>,tabStops);
-	ASPROPERTY_GETTER_SETTER(tiny_string,target);
+	ASPROPERTY_GETTER_SETTER_ATOM(target);
 	ASPROPERTY_GETTER_SETTER_ATOM(underline);
-	ASPROPERTY_GETTER_SETTER(tiny_string,url);
-	ASPROPERTY_GETTER_SETTER(tiny_string,display);
+	ASPROPERTY_GETTER_SETTER_ATOM(url);
+	ASPROPERTY_GETTER_SETTER_ATOM(display);
 };
 
 class TextFieldType: public ASObject
 {
 public:
-	TextFieldType(Class_base* c):ASObject(c){}
+	TextFieldType(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
 class TextFormatAlign: public ASObject
 {
 public:
-	TextFormatAlign(Class_base* c):ASObject(c){}
+	TextFormatAlign(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
 class TextFieldAutoSize: public ASObject
 {
 public:
-	TextFieldAutoSize(Class_base* c):ASObject(c){}
+	TextFieldAutoSize(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
@@ -274,60 +286,58 @@ private:
 	RECT bounds;
 	uint32_t tagID;
 protected:
-	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const override;
-	bool renderImpl(RenderContext& ctxt) const override
-		{ return TokenContainer::renderImpl(ctxt); }
-	_NR<DisplayObject> hitTestImpl(_NR<DisplayObject> last, number_t x, number_t y, HIT_TYPE type,bool interactiveObjectsOnly) override;
+	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) override;
+	bool renderImpl(RenderContext& ctxt) override;
+	_NR<DisplayObject> hitTestImpl(number_t x, number_t y, HIT_TYPE type,bool interactiveObjectsOnly) override;
 public:
-	StaticText(Class_base* c) : DisplayObject(c),TokenContainer(this),tagID(UINT32_MAX) {}
-	StaticText(Class_base* c, const tokensVector& tokens,const RECT& b,uint32_t _tagID):
-		DisplayObject(c),TokenContainer(this, tokens, 1.0f/1024.0f/20.0f/20.0f),bounds(b),tagID(_tagID) {}
+	StaticText(ASWorker* wrk,Class_base* c):DisplayObject(wrk,c),TokenContainer(this),tagID(UINT32_MAX) {}
+	StaticText(ASWorker* wrk,Class_base* c, const tokensVector& tokens,const RECT& b,uint32_t _tagID):
+		DisplayObject(wrk,c),TokenContainer(this, tokens, 1.0f/1024.0f/20.0f/20.0f),bounds(b),tagID(_tagID) {}
 	static void sinit(Class_base* c);
 	void requestInvalidation(InvalidateQueue* q, bool forceTextureRefresh=false) override { TokenContainer::requestInvalidation(q,forceTextureRefresh); }
-	IDrawable* invalidate(DisplayObject* target, const MATRIX& initialMatrix,bool smoothing) override
-	{ return TokenContainer::invalidate(target, initialMatrix,smoothing); }
+	IDrawable* invalidate(DisplayObject* target, const MATRIX& initialMatrix, bool smoothing, InvalidateQueue* q, _NR<DisplayObject>* cachedBitmap) override;
 	uint32_t getTagID() const override { return tagID; }
 };
 
 class FontStyle: public ASObject
 {
 public:
-	FontStyle(Class_base* c):ASObject(c){}
+	FontStyle(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
 class FontType: public ASObject
 {
 public:
-	FontType(Class_base* c):ASObject(c){}
+	FontType(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
 class TextDisplayMode: public ASObject
 {
 public:
-	TextDisplayMode(Class_base* c):ASObject(c){}
+	TextDisplayMode(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
 class TextColorType: public ASObject
 {
 public:
-	TextColorType(Class_base* c):ASObject(c){}
+	TextColorType(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
 class GridFitType: public ASObject
 {
 public:
-	GridFitType(Class_base* c):ASObject(c){}
+	GridFitType(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
 class TextInteractionMode: public ASObject
 {
 public:
-	TextInteractionMode(Class_base* c):ASObject(c){}
+	TextInteractionMode(ASWorker* wrk,Class_base* c):ASObject(wrk,c){}
 	static void sinit(Class_base* c);
 };
 
@@ -341,14 +351,14 @@ protected:
 	ASPROPERTY_GETTER_SETTER(number_t, width);
 	ASPROPERTY_GETTER_SETTER(number_t, x);
 public:
-	TextLineMetrics(Class_base* c, number_t _x=0, number_t _width=0, number_t _height=0,
+	TextLineMetrics(ASWorker* wrk,Class_base* c, number_t _x=0, number_t _width=0, number_t _height=0,
 			number_t _ascent=0, number_t _descent=0, number_t _leading=0)
-		: ASObject(c), ascent(_ascent), descent(_descent),
+		: ASObject(wrk,c,T_OBJECT,SUBTYPE_TEXTLINEMETRICS), ascent(_ascent), descent(_descent),
 		  height(_height), leading(_leading), width(_width), x(_x) {}
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_constructor);
 };
 
-};
+}
 
 #endif /* SCRIPTING_FLASH_TEXT_FLASHTEXT_H */
