@@ -34,11 +34,14 @@ void ContentElement::sinit(Class_base* c)
 	CLASS_SETUP(c, ASObject, _constructorNotInstantiatable, CLASS_SEALED);
 	REGISTER_GETTER(c,rawText);
 	REGISTER_GETTER_SETTER(c,elementFormat);
+	REGISTER_GETTER_RESULTTYPE(c,groupElement,GroupElement);
+	c->setVariableAtomByQName("GRAPHIC_ELEMENT",nsNameAndKind(),asAtomHandler::fromUInt(0xFDEF),CONSTANT_TRAIT);
 }
 ASFUNCTIONBODY_GETTER_SETTER(ContentElement,elementFormat)
 ASFUNCTIONBODY_GETTER(ContentElement,rawText)
+ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(ContentElement,groupElement)
 
-ElementFormat::ElementFormat(Class_base *c): ASObject(c,T_OBJECT,SUBTYPE_ELEMENTFORMAT),
+ElementFormat::ElementFormat(ASWorker* wrk, Class_base *c): ASObject(wrk,c,T_OBJECT,SUBTYPE_ELEMENTFORMAT),
 	alignmentBaseline("useDominantBaseline"),
 	alpha(1.0),
 	baselineShift(0.0),
@@ -108,16 +111,16 @@ ASFUNCTIONBODY_GETTER_SETTER(ElementFormat,typographicCase)
 ASFUNCTIONBODY_ATOM(ElementFormat,_constructor)
 {
 	ElementFormat* th=asAtomHandler::as<ElementFormat>(obj);
-	ARG_UNPACK_ATOM(th->fontDescription, NullRef)(th->fontSize, 12.0)(th->color, 0x000000) (th->alpha, 1.0)(th->textRotation, "auto")
+	ARG_CHECK(ARG_UNPACK(th->fontDescription, NullRef)(th->fontSize, 12.0)(th->color, 0x000000) (th->alpha, 1.0)(th->textRotation, "auto")
 			(th->dominantBaseline, "roman") (th->alignmentBaseline, "useDominantBaseline") (th->baselineShift, 0.0)(th->kerning, "on")
 			(th->trackingRight, 0.0)(th->trackingLeft, 0.0)(th->locale, "en")(th->breakOpportunity, "auto")(th->digitCase, "default")
-			(th->digitWidth, "default")(th->ligatureLevel, "common")(th->typographicCase, "default");
+			(th->digitWidth, "default")(th->ligatureLevel, "common")(th->typographicCase, "default"));
 }
 ASFUNCTIONBODY_ATOM(ElementFormat, _clone)
 {
 	ElementFormat* th=asAtomHandler::as<ElementFormat>(obj);
 
-	ElementFormat* newformat = Class<ElementFormat>::getInstanceS(sys);
+	ElementFormat* newformat = Class<ElementFormat>::getInstanceS(wrk);
 	newformat->fontDescription = th->fontDescription;
 	newformat->fontSize = th->fontSize;
 	newformat->color = th->color;
@@ -152,13 +155,27 @@ void FontDescription::sinit(Class_base* c)
 	c->isReusable = true;
 	c->setDeclaredMethodByQName("clone","",Class<IFunction>::getFunction(c->getSystemState(),_clone),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("isFontCompatible","",Class<IFunction>::getFunction(c->getSystemState(),isFontCompatible),NORMAL_METHOD,false);
-	REGISTER_GETTER_SETTER(c,cffHinting);
-	REGISTER_GETTER_SETTER(c,fontLookup);
-	REGISTER_GETTER_SETTER(c,fontName);
-	REGISTER_GETTER_SETTER(c,fontPosture);
-	REGISTER_GETTER_SETTER(c,fontWeight);
-	REGISTER_GETTER_SETTER(c,locked);
-	REGISTER_GETTER_SETTER(c,renderingMode);
+
+    c->setDeclaredMethodByQName("fontName","",Class<IFunction>::getFunction(c->getSystemState(),_getFontName),GETTER_METHOD,true);
+    c->setDeclaredMethodByQName("fontName","",Class<IFunction>::getFunction(c->getSystemState(),_setFontName),SETTER_METHOD,true);
+
+    c->setDeclaredMethodByQName("fontWeight","",Class<IFunction>::getFunction(c->getSystemState(),_getFontWeight),GETTER_METHOD,true);
+    c->setDeclaredMethodByQName("fontWeight","",Class<IFunction>::getFunction(c->getSystemState(),_setFontWeight),SETTER_METHOD,true);
+
+    c->setDeclaredMethodByQName("fontPosture","",Class<IFunction>::getFunction(c->getSystemState(),_getFontPosture),GETTER_METHOD,true);
+    c->setDeclaredMethodByQName("fontPosture","",Class<IFunction>::getFunction(c->getSystemState(),_setFontPosture),SETTER_METHOD,true);
+
+    c->setDeclaredMethodByQName("fontLookup","",Class<IFunction>::getFunction(c->getSystemState(),_getFontLookup),GETTER_METHOD,true);
+    c->setDeclaredMethodByQName("fontLookup","",Class<IFunction>::getFunction(c->getSystemState(),_setFontLookup),SETTER_METHOD,true);
+
+    c->setDeclaredMethodByQName("renderingMode","",Class<IFunction>::getFunction(c->getSystemState(),_getRenderingMode),GETTER_METHOD,true);
+    c->setDeclaredMethodByQName("renderingMode","",Class<IFunction>::getFunction(c->getSystemState(),_setRenderingMode),SETTER_METHOD,true);
+
+    c->setDeclaredMethodByQName("cffHinting","",Class<IFunction>::getFunction(c->getSystemState(),_getCffHinting),GETTER_METHOD,true);
+    c->setDeclaredMethodByQName("cffHinting","",Class<IFunction>::getFunction(c->getSystemState(),_setCffHinting),SETTER_METHOD,true);
+
+    c->setDeclaredMethodByQName("locked","",Class<IFunction>::getFunction(c->getSystemState(),_getLocked),GETTER_METHOD,true);
+    c->setDeclaredMethodByQName("locked","",Class<IFunction>::getFunction(c->getSystemState(),_setLocked),SETTER_METHOD,true);
 }
 
 bool FontDescription::destruct()
@@ -175,21 +192,15 @@ bool FontDescription::destruct()
 
 ASFUNCTIONBODY_ATOM(FontDescription,_constructor)
 {
-	LOG(LOG_NOT_IMPLEMENTED, "FontDescription class not implemented");
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    ARG_CHECK(ARG_UNPACK(th->fontName, "_serif")(th->fontWeight, "normal")(th->fontPosture, "normal")(th->fontLookup, "device")(th->renderingMode, "cff")(th->cffHinting, "horizontalStem"));
 }
-ASFUNCTIONBODY_GETTER_SETTER(FontDescription,cffHinting)
-ASFUNCTIONBODY_GETTER_SETTER(FontDescription,fontLookup)
-ASFUNCTIONBODY_GETTER_SETTER(FontDescription,fontName)
-ASFUNCTIONBODY_GETTER_SETTER(FontDescription,fontPosture)
-ASFUNCTIONBODY_GETTER_SETTER(FontDescription,fontWeight)
-ASFUNCTIONBODY_GETTER_SETTER(FontDescription,locked)
-ASFUNCTIONBODY_GETTER_SETTER(FontDescription,renderingMode)
 
 ASFUNCTIONBODY_ATOM(FontDescription, _clone)
 {
 	FontDescription* th=asAtomHandler::as<FontDescription>(obj);
 
-	FontDescription* newfontdescription = Class<FontDescription>::getInstanceS(sys);
+	FontDescription* newfontdescription = Class<FontDescription>::getInstanceS(wrk);
 	newfontdescription->cffHinting = th->cffHinting;
 	newfontdescription->fontLookup = th->fontLookup;
 	newfontdescription->fontName = th->fontName;
@@ -199,22 +210,29 @@ ASFUNCTIONBODY_ATOM(FontDescription, _clone)
 	newfontdescription->locked = false;
 	ret = asAtomHandler::fromObject(newfontdescription);
 }
+
 ASFUNCTIONBODY_ATOM(FontDescription, isFontCompatible)
 {
 	tiny_string fontName;
 	tiny_string fontWeight;
 	tiny_string fontPosture;
-	ARG_UNPACK_ATOM(fontName)(fontWeight)(fontPosture);
+	ARG_CHECK(ARG_UNPACK(fontName)(fontWeight)(fontPosture));
 	bool italic = false;
 	bool bold = false;
 	if (fontWeight == "bold")
 		bold = true;
 	else if (fontWeight != "normal")
-		throwError<ArgumentError>(kInvalidArgumentError,"fontWeight");
+	{
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"fontWeight");
+		return;
+	}
 	if (fontPosture == "italic")
 		italic = true;
 	else if (fontPosture != "normal")
-		throwError<ArgumentError>(kInvalidArgumentError,"fontPosture");
+	{
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"fontPosture");
+		return;
+	}
 	tiny_string fontStyle = bold ? (italic ? "boldItalic" : "bold") : (italic ? "italic" : "regular");
 	std::vector<asAtom>* flist = ASFont::getFontList();
 	auto it = flist->begin();
@@ -233,6 +251,219 @@ ASFUNCTIONBODY_ATOM(FontDescription, isFontCompatible)
 	asAtomHandler::setBool(ret,false);
 }
 
+ASFUNCTIONBODY_ATOM(FontDescription,isDeviceFontCompatible)
+{
+	// Similar to isFontCompatible but without embeddedCFF check.
+
+    tiny_string fontName;
+    tiny_string fontWeight;
+    tiny_string fontPosture;
+    ARG_CHECK(ARG_UNPACK(fontName)(fontWeight)(fontPosture));
+    bool italic = false;
+    bool bold = false;
+    if (fontWeight == "bold")
+        bold = true;
+    else if (fontWeight != "normal")
+	{
+        createError<ArgumentError>(wrk,kInvalidArgumentError,"fontWeight");
+		return;
+	}
+    if (fontPosture == "italic")
+        italic = true;
+    else if (fontPosture != "normal")
+	{
+        createError<ArgumentError>(wrk,kInvalidArgumentError,"fontPosture");
+		return;
+	}
+    tiny_string fontStyle = bold ? (italic ? "boldItalic" : "bold") : (italic ? "italic" : "regular");
+    std::vector<asAtom>* flist = ASFont::getFontList();
+    auto it = flist->begin();
+    while (it != flist->end())
+    {
+        ASFont* f = asAtomHandler::as<ASFont>(*it);
+        if (f->fontName == fontName &&
+            f->fontStyle == fontStyle)
+        {
+            asAtomHandler::setBool(ret,true);
+            return;
+        }
+        it++;
+    }
+    asAtomHandler::setBool(ret,false);
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_getLocked)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    ret = asAtomHandler::fromBool(th->locked);
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_setLocked)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    if (!th->locked)
+    {
+        bool value;
+        ARG_CHECK(ARG_UNPACK(value));
+        th->locked = value;
+    }
+    else
+    {
+        createError<ArgumentError>(wrk,kCantInstantiateError, "The FontDescription object is locked and cannot be modified.");
+    }
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_getFontName)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    ret = asAtomHandler::fromString(wrk->getSystemState(),th->fontName);
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_setFontName)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    if (!th->locked)
+    {
+        tiny_string value;
+        ARG_CHECK(ARG_UNPACK(value));
+        th->fontName = value;
+    }
+	else
+	{
+		createError<ArgumentError>(wrk,kCantInstantiateError, "The FontDescription object is locked and cannot be modified.");
+	}
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_getFontWeight)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    ret = asAtomHandler::fromString(wrk->getSystemState(),th->fontWeight);
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_setFontWeight)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    if (!th->locked)
+    {
+        tiny_string value;
+        ARG_CHECK(ARG_UNPACK(value));
+		if (value != "bold" && value != "normal")
+		{
+			createError<ArgumentError>(wrk,kInvalidEnumError, "fontWeight");
+			return;
+		}
+        th->fontWeight = value;
+    }
+	else
+	{
+		createError<ArgumentError>(wrk,kCantInstantiateError, "The FontDescription object is locked and cannot be modified.");
+	}
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_getFontPosture)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    ret = asAtomHandler::fromString(wrk->getSystemState(),th->fontPosture);
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_setFontPosture)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    if (!th->locked)
+    {
+        tiny_string value;
+        ARG_CHECK(ARG_UNPACK(value));
+		if (value != "italic" && value != "normal")
+		{
+			createError<ArgumentError>(wrk,kInvalidEnumError, "fontPosture");
+			return;
+		}
+        th->fontPosture = value;
+    }
+	else
+	{
+		createError<ArgumentError>(wrk,kCantInstantiateError, "The FontDescription object is locked and cannot be modified.");
+	}
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_getFontLookup)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    ret = asAtomHandler::fromString(wrk->getSystemState(),th->fontLookup);
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_setFontLookup)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    if (!th->locked)
+    {
+        tiny_string value;
+        ARG_CHECK(ARG_UNPACK(value));
+		if (value != "device" && value != "embeddedCFF")
+		{
+			createError<ArgumentError>(wrk,kInvalidEnumError, "fontLookup");
+			return;
+		}
+        th->fontLookup = value;
+    }
+	else
+	{
+		createError<ArgumentError>(wrk,kCantInstantiateError, "The FontDescription object is locked and cannot be modified.");
+	}
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_getRenderingMode)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    ret = asAtomHandler::fromString(wrk->getSystemState(),th->renderingMode);
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_setRenderingMode)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    if (!th->locked)
+    {
+        tiny_string value;
+        ARG_CHECK(ARG_UNPACK(value));
+		if (value != "cff" && value != "normal")
+		{
+			createError<ArgumentError>(wrk,kInvalidEnumError, "renderingMode");
+			return;
+		}
+        th->renderingMode = value;
+    }
+	else
+	{
+		createError<ArgumentError>(wrk,kCantInstantiateError, "The FontDescription object is locked and cannot be modified.");
+	}
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_getCffHinting)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    ret = asAtomHandler::fromString(wrk->getSystemState(),th->cffHinting);
+}
+
+ASFUNCTIONBODY_ATOM(FontDescription,_setCffHinting)
+{
+    FontDescription* th=asAtomHandler::as<FontDescription>(obj);
+    if (!th->locked)
+    {
+        tiny_string value;
+        ARG_CHECK(ARG_UNPACK(value));
+		if (value != "horizontalStem" && value != "none")
+		{
+			createError<ArgumentError>(wrk,kInvalidEnumError, "cffHinting");
+			return;
+		}
+        th->cffHinting = value;
+    }
+	else
+	{
+		createError<ArgumentError>(wrk,kCantInstantiateError, "The FontDescription object is locked and cannot be modified.");
+	}
+}
+
 void FontPosture::sinit(Class_base* c)
 {
 	CLASS_SETUP_NO_CONSTRUCTOR(c, ASObject, CLASS_FINAL | CLASS_SEALED);
@@ -290,7 +521,7 @@ void TextJustifier::sinit(Class_base* c)
 }
 ASFUNCTIONBODY_ATOM(TextJustifier,_constructor)
 {
-	throwError<ArgumentError>(kCantInstantiateError, "TextJustifier cannot be instantiated");
+	createError<ArgumentError>(wrk,kCantInstantiateError, "TextJustifier cannot be instantiated");
 }
 void SpaceJustifier::sinit(Class_base* c)
 {
@@ -312,7 +543,7 @@ ASFUNCTIONBODY_ATOM(EastAsianJustifier,_constructor)
 }
 
 
-TextBlock::TextBlock(Class_base *c): ASObject(c,T_OBJECT,SUBTYPE_TEXTBLOCK)
+TextBlock::TextBlock(ASWorker* wrk, Class_base *c): ASObject(wrk,c,T_OBJECT,SUBTYPE_TEXTBLOCK)
   ,applyNonLinearFontScaling(true),baselineFontSize(12),baselineZero("roman"),bidiLevel(0),firstLine(NullRef),lastLine(NullRef),lineRotation("rotate0")
 {
 }
@@ -339,25 +570,25 @@ void TextBlock::sinit(Class_base* c)
 	REGISTER_GETTER_SETTER(c, userData);
 }
 
-ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, applyNonLinearFontScaling);
-ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, baselineFontDescription);
-ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, baselineFontSize);
-ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, baselineZero);
-ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, bidiLevel);
-ASFUNCTIONBODY_GETTER_SETTER(TextBlock, content);
-ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextBlock, firstInvalidLine );
-ASFUNCTIONBODY_GETTER(TextBlock, firstLine);
-ASFUNCTIONBODY_GETTER(TextBlock, lastLine);
-ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, lineRotation);
-ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, textJustifier);
-ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, tabStops);
-ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextBlock, textLineCreationResult);
-ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, userData);
+ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, applyNonLinearFontScaling)
+ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, baselineFontDescription)
+ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, baselineFontSize)
+ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, baselineZero)
+ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, bidiLevel)
+ASFUNCTIONBODY_GETTER_SETTER(TextBlock, content)
+ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextBlock, firstInvalidLine )
+ASFUNCTIONBODY_GETTER(TextBlock, firstLine)
+ASFUNCTIONBODY_GETTER(TextBlock, lastLine)
+ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, lineRotation)
+ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, textJustifier)
+ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, tabStops)
+ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextBlock, textLineCreationResult)
+ASFUNCTIONBODY_GETTER_SETTER_NOT_IMPLEMENTED(TextBlock, userData)
 
 ASFUNCTIONBODY_ATOM(TextBlock,_constructor)
 {
 	TextBlock* th=asAtomHandler::as<TextBlock>(obj);
-	ARG_UNPACK_ATOM (th->content, NullRef);
+	ARG_CHECK(ARG_UNPACK (th->content, NullRef));
 	if (argslen > 1)
 		LOG(LOG_NOT_IMPLEMENTED, "TextBlock constructor ignores some parameters");
 }
@@ -369,14 +600,15 @@ ASFUNCTIONBODY_ATOM(TextBlock, createTextLine)
 	int32_t width;
 	number_t lineOffset;
 	bool fitSomething;
-	ARG_UNPACK_ATOM (previousLine, NullRef) (width, MAX_LINE_WIDTH) (lineOffset, 0.0) (fitSomething, false);
+	ARG_CHECK(ARG_UNPACK (previousLine, NullRef) (width, MAX_LINE_WIDTH) (lineOffset, 0.0) (fitSomething, false));
 
 	if (argslen > 2)
 		LOG(LOG_NOT_IMPLEMENTED, "TextBlock::createTextLine ignored some parameters");
 
-	if (!fitSomething && (width < 0 || width > MAX_LINE_WIDTH))
+	if (!fitSomething && ((width < 0) || (width > MAX_LINE_WIDTH)))
 	{
-		throwError<ArgumentError>(kOutOfRangeError,"Invalid width");
+		createError<ArgumentError>(wrk,kOutOfRangeError,"Invalid width");
+		return;
 	}
 
 	// TODO handle non TextElement Content
@@ -392,39 +624,47 @@ ASFUNCTIONBODY_ATOM(TextBlock, createTextLine)
 	LOG(LOG_NOT_IMPLEMENTED,"splitting textblock in multiple lines not implemented");
 	th->content->as<TextElement>()->text = "";
 	th->incRef();
-	_NR<TextLine> textLine = _NR<TextLine>(Class<TextLine>::getInstanceS(sys,linetext, _MNR(th)));
+	_NR<TextLine> textLine = _NR<TextLine>(Class<TextLine>::getInstanceS(wrk,linetext, _MNR(th)));
 	textLine->width = (uint32_t)width;
 	textLine->previousLine = previousLine;
+
+	// Set baseline font
+	textLine->font = th->baselineZero;
+
+	// Set font from FontDescription in TextBlock if possible
+ 	if (!th->baselineFontDescription.isNull())
+	{
+ 		textLine->font = th->baselineFontDescription->fontName;
+	}
+
+	// Set font from FontDescription in TextElement if possible
+	auto elementFormat = th->content->as<TextElement>()->elementFormat;
+	if (!elementFormat.isNull() &&
+		!elementFormat->fontDescription.isNull()   )
+	{
+		th->content->as<TextElement>()->elementFormat->fontDescription->fontName;
+	}
+
 	textLine->updateSizes();
 	if (textLine->width > textLine->textWidth)
 	{
-		textLine->decRef();
-		th->decRef();
 		asAtomHandler::setNull(ret);
 		return;
 	}
 	if (previousLine.isNull())
 	{
-		textLine->incRef();
 		th->firstLine = textLine;
 		if (th->lastLine.isNull())
-		{
-			textLine->incRef();
 			th->lastLine = textLine;
-		}
 	}
 	else
 	{
 		if (th->lastLine == previousLine)
-		{
-			th->lastLine->decRef();
-			textLine->incRef();
 			th->lastLine = textLine;
-		}
-		textLine->incRef();
 		previousLine->nextLine = textLine;
 	}
 	
+	textLine->incRef();
 	ret = asAtomHandler::fromObject(textLine.getPtr());
 }
 ASFUNCTIONBODY_ATOM(TextBlock, recreateTextLine)
@@ -435,7 +675,7 @@ ASFUNCTIONBODY_ATOM(TextBlock, recreateTextLine)
 	int32_t width;
 	number_t lineOffset;
 	bool fitSomething;
-	ARG_UNPACK_ATOM (textLine) (previousLine, NullRef) (width, MAX_LINE_WIDTH) (lineOffset, 0.0) (fitSomething, false);
+	ARG_CHECK(ARG_UNPACK (textLine) (previousLine, NullRef) (width, MAX_LINE_WIDTH) (lineOffset, 0.0) (fitSomething, false));
 
 	if (argslen > 2)
 		LOG(LOG_NOT_IMPLEMENTED, "TextBlock::recreateTextLine ignored some parameters");
@@ -448,32 +688,52 @@ ASFUNCTIONBODY_ATOM(TextBlock, recreateTextLine)
 		return;
 	}
 
-	if (!fitSomething && (width < 0 || width > MAX_LINE_WIDTH))
+	if (!fitSomething && ((width < 0) || (width > MAX_LINE_WIDTH)))
 	{
-		throwError<ArgumentError>(kOutOfRangeError,"Invalid width");
+		createError<ArgumentError>(wrk,kOutOfRangeError,"Invalid width");
+		return;
 	}
 
 	if (textLine.isNull())
 	{
-		throwError<ArgumentError>(kInvalidArgumentError,"Invalid argument: textLine");
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"textLine");
+		return;
 	}
-
-	if (!textLine->textBlock.isNull() && th != textLine->textBlock.getPtr())
+	if (textLine == previousLine)
 	{
-		throwError<ArgumentError>(kInvalidArgumentError,"Invalid argument: textLine is in different textBlock");
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"previousLine");
+		return;
 	}
-	if (fitSomething && textLine->text == "")
-		textLine->text = " ";
+	if (!previousLine.isNull() && !previousLine->textBlock.isNull() && th != previousLine->textBlock.getPtr())
+	{
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"previousLine");
+		return;
+	}
+	if (fitSomething && textLine->getText().empty())
+		textLine->setText(" ");
 	textLine->width = (uint32_t)width;
 	textLine->previousLine = previousLine;
 	textLine->updateSizes();
+	th->incRef();
+	textLine->textBlock= _MNR(th);
 	if (textLine->width > textLine->textWidth)
 	{
 		asAtomHandler::setNull(ret);
 		return;
 	}
-	if (!previousLine.isNull())
+	if (previousLine.isNull())
+	{
+		th->firstLine = textLine;
+		if (th->lastLine.isNull())
+			th->lastLine = textLine;
+	}
+	else
+	{
+		if (th->lastLine == previousLine)
+			th->lastLine = textLine;
 		previousLine->nextLine = textLine;
+	}
+
 	textLine->incRef();
 	ret = asAtomHandler::fromObject(textLine.getPtr());
 }
@@ -483,19 +743,21 @@ ASFUNCTIONBODY_ATOM(TextBlock, releaseLines)
 	TextBlock* th=asAtomHandler::as<TextBlock>(obj);
 	_NR<TextLine> firstLine;
 	_NR<TextLine> lastLine;
-	ARG_UNPACK_ATOM (firstLine) (lastLine);
+	ARG_CHECK(ARG_UNPACK (firstLine) (lastLine));
 
 	// TODO handle non TextElement Content
 	if (th->content.isNull() || !th->content->is<TextElement>())
 		return;
 
-	if (firstLine.isNull() || firstLine->textBlock != th)
+	if (firstLine.isNull() || (!firstLine->textBlock.isNull() && firstLine->textBlock != th))
 	{
-		throwError<ArgumentError>(kInvalidArgumentError,"Invalid argument: firstLine");
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"firstLine");
+		return;
 	}
-	if (lastLine.isNull() || lastLine->textBlock != th)
+	if (lastLine.isNull() || (!lastLine->textBlock.isNull() && lastLine->textBlock != th))
 	{
-		throwError<ArgumentError>(kInvalidArgumentError,"Invalid argument: lastLine");
+		createError<ArgumentError>(wrk,kInvalidArgumentError,"lastLine");
+		return;
 	}
 
 	bool afterlast = false;
@@ -515,7 +777,6 @@ ASFUNCTIONBODY_ATOM(TextBlock, releaseLines)
 			{
 				tmpLine = firstLine->previousLine;
 				firstLine->previousLine = NullRef;
-				tmpLine->decRef();
 			}
 			firstLine->textBlock = NullRef;
 			firstLine->nextLine = NullRef;
@@ -538,15 +799,12 @@ void TextElement::sinit(Class_base* c)
 	REGISTER_GETTER_SETTER(c, text);
 }
 
-ASFUNCTIONBODY_GETTER_SETTER_CB(TextElement, text,settext_cb);
+ASFUNCTIONBODY_GETTER_SETTER_CB(TextElement, text,settext_cb)
 
 ASFUNCTIONBODY_ATOM(TextElement,_constructor)
 {
 	TextElement* th=asAtomHandler::as<TextElement>(obj);
-	ARG_UNPACK_ATOM (th->text, "");
-	if (argslen > 1)
-		LOG(LOG_NOT_IMPLEMENTED, "TextElement constructor ignores some parameters");
-
+    	ARG_CHECK(ARG_UNPACK (th->text, "")(th->elementFormat, NullRef));
 }
 ASFUNCTIONBODY_ATOM(TextElement, replaceText)
 {
@@ -554,9 +812,12 @@ ASFUNCTIONBODY_ATOM(TextElement, replaceText)
 	int beginIndex;
 	int endIndex;
 	tiny_string newtext;
-	ARG_UNPACK_ATOM (beginIndex)(endIndex)(newtext);
-	if (beginIndex < 0 || endIndex < 0 || beginIndex > (int32_t)th->text.numChars())
-		throwError<RangeError>(kParamRangeError);
+	ARG_CHECK(ARG_UNPACK (beginIndex)(endIndex)(newtext));
+	if ((beginIndex < 0) || (endIndex < 0) || (beginIndex > (int32_t)th->text.numChars()))
+	{
+		createError<RangeError>(wrk,kParamRangeError);
+		return;
+	}
 	if (beginIndex > endIndex)
 	{
 		int tmp = endIndex;
@@ -583,21 +844,20 @@ ASFUNCTIONBODY_ATOM(GroupElement,_constructor)
 	LOG(LOG_NOT_IMPLEMENTED, "GroupElement constructor not implemented");
 }
 
-TextLine::TextLine(Class_base* c, tiny_string linetext, _NR<TextBlock> owner)
-  : DisplayObjectContainer(c), TextData(),nextLine(NULL),previousLine(NULL),userData(NULL)
+TextLine::TextLine(ASWorker* wrk, Class_base* c, tiny_string linetext, _NR<TextBlock> owner)
+  : DisplayObjectContainer(wrk,c), TextData(),nextLine(nullptr),previousLine(nullptr),userData(nullptr)
   ,hasGraphicElement(false),hasTabs(false),rawTextLength(0),specifiedWidth(0),textBlockBeginIndex(0)
 {
 	subtype = SUBTYPE_TEXTLINE;
 	textBlock = owner;
 
-	text = linetext;
+	setText(linetext.raw_buf());
 	updateSizes();
-	requestInvalidation(getSys());
 }
 
 void TextLine::sinit(Class_base* c)
 {
-	CLASS_SETUP(c, DisplayObjectContainer, _constructor, CLASS_FINAL | CLASS_SEALED);
+	CLASS_SETUP_NO_CONSTRUCTOR(c, DisplayObjectContainer, CLASS_FINAL | CLASS_SEALED);
 	c->setVariableAtomByQName("MAX_LINE_WIDTH",nsNameAndKind(),asAtomHandler::fromUInt((uint32_t)MAX_LINE_WIDTH),CONSTANT_TRAIT);
 	c->setDeclaredMethodByQName("getBaselinePosition","",Class<IFunction>::getFunction(c->getSystemState(),getBaselinePosition),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("descent","",Class<IFunction>::getFunction(c->getSystemState(),getDescent),GETTER_METHOD,true);
@@ -617,80 +877,90 @@ void TextLine::sinit(Class_base* c)
 	REGISTER_GETTER(c, textBlockBeginIndex);
 }
 
-
-ASFUNCTIONBODY_GETTER(TextLine, textBlock);
-ASFUNCTIONBODY_GETTER(TextLine, nextLine);
-ASFUNCTIONBODY_GETTER(TextLine, previousLine);
-ASFUNCTIONBODY_GETTER_SETTER(TextLine, validity);
-ASFUNCTIONBODY_GETTER_SETTER(TextLine, userData);
-ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextLine, hasGraphicElement);
-ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextLine, hasTabs);
-ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextLine, rawTextLength);
-ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextLine, specifiedWidth);
-ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextLine, textBlockBeginIndex);
-
-ASFUNCTIONBODY_ATOM(TextLine,_constructor)
+void TextLine::finalize()
 {
-	// Should throw ArgumentError when called from AS code
-	//throw Class<ArgumentError>::getInstanceS("Error #2012: TextLine class cannot be instantiated.");
+	textBlock.reset();
+	nextLine.reset();
+	previousLine.reset();
+	userData.reset();
+	DisplayObjectContainer::finalize();
 }
 
+ASFUNCTIONBODY_GETTER(TextLine, textBlock)
+ASFUNCTIONBODY_GETTER(TextLine, nextLine)
+ASFUNCTIONBODY_GETTER(TextLine, previousLine)
+ASFUNCTIONBODY_GETTER_SETTER(TextLine, validity)
+ASFUNCTIONBODY_GETTER_SETTER(TextLine, userData)
+ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextLine, hasGraphicElement)
+ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextLine, hasTabs)
+ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextLine, rawTextLength)
+ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextLine, specifiedWidth)
+ASFUNCTIONBODY_GETTER_NOT_IMPLEMENTED(TextLine, textBlockBeginIndex)
+
 ASFUNCTIONBODY_ATOM(TextLine, getBaselinePosition)
 {
 	LOG(LOG_NOT_IMPLEMENTED,"TextLine.getBaselinePosition");
-	asAtomHandler::setInt(ret,sys,0);
+	asAtomHandler::setInt(ret,wrk,0);
 }
 
 ASFUNCTIONBODY_ATOM(TextLine, getDescent)
 {
 	LOG(LOG_NOT_IMPLEMENTED,"TextLine.descent");
-	asAtomHandler::setInt(ret,sys,0);
+	asAtomHandler::setInt(ret,wrk,0);
 }
 
 ASFUNCTIONBODY_ATOM(TextLine, getAscent)
 {
 	TextLine* th=asAtomHandler::as<TextLine>(obj);
 	LOG(LOG_NOT_IMPLEMENTED,"TextLine.ascent");
-	asAtomHandler::setInt(ret,sys,th->textHeight);
+	asAtomHandler::setInt(ret,wrk,th->textHeight);
 }
 
 ASFUNCTIONBODY_ATOM(TextLine, getTextWidth)
 {
 	TextLine* th=asAtomHandler::as<TextLine>(obj);
-	asAtomHandler::setInt(ret,sys,th->textWidth);
+	asAtomHandler::setInt(ret,wrk,th->textWidth);
 }
 
 ASFUNCTIONBODY_ATOM(TextLine, getTextHeight)
 {
 	TextLine* th=asAtomHandler::as<TextLine>(obj);
-	asAtomHandler::setInt(ret,sys,th->textHeight);
+	asAtomHandler::setInt(ret,wrk,th->textHeight);
 }
 
 ASFUNCTIONBODY_ATOM(TextLine, getUnjustifiedTextWidth)
 {
 	TextLine* th=asAtomHandler::as<TextLine>(obj);
-	asAtomHandler::setInt(ret,sys,th->width);
+	asAtomHandler::setInt(ret,wrk,th->width);
 }
 
 void TextLine::updateSizes()
 {
-	uint32_t w,h,tw,th;
+	number_t w,h;
 	w = width;
 	h = height;
 	//Compute (text)width, (text)height
-	CairoPangoRenderer::getBounds(*this, w, h, tw, th);
-	if (w == 0)
-		w = tw;
-	if (h == 0)
-		h = th;
-	width = w; //TODO: check the case when w,h == 0
-	
+	CairoPangoRenderer::getBounds(*this,this->getText(), w, h);
 	textWidth = w;
-	height = h;
 	textHeight = h;
 }
 
-bool TextLine::boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const
+string TextLine::toDebugString() const
+{
+	string res = DisplayObjectContainer::toDebugString();
+	res += " (";
+	res += this->validity;
+	res += ")";
+	if (textBlock)
+	{
+		char buf[100];
+		sprintf(buf," ow:%p",textBlock.getPtr());
+		res += buf;
+	}
+	return res;
+}
+
+bool TextLine::boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax)
 {
 	xmin=0;
 	xmax=width;
@@ -701,15 +971,22 @@ bool TextLine::boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number
 
 void TextLine::requestInvalidation(InvalidateQueue* q, bool forceTextureRefresh)
 {
+	if (requestInvalidationForCacheAsBitmap(q))
+		return;
 	DisplayObjectContainer::requestInvalidation(q,forceTextureRefresh);
 	incRef();
 	q->addToInvalidateQueue(_MR(this));
 }
 
-IDrawable* TextLine::invalidate(DisplayObject* target, const MATRIX& initialMatrix,bool smoothing)
+IDrawable* TextLine::invalidate(DisplayObject* target, const MATRIX& initialMatrix,bool smoothing, InvalidateQueue* q, _NR<DisplayObject>* cachedBitmap)
 {
-	int32_t x,y,rx,ry;
-	uint32_t width,height,rwidth,rheight;
+	if (cachedBitmap && computeCacheAsBitmap())
+	{
+		setNeedsTextureRecalculation();
+		return getCachedBitmapDrawable(target, initialMatrix, cachedBitmap);
+	}
+	number_t x,y,rx,ry;
+	number_t width,height,rwidth,rheight;
 	number_t bxmin,bxmax,bymin,bymax;
 	if(boundsRect(bxmin,bxmax,bymin,bymax)==false)
 	{
@@ -719,14 +996,14 @@ IDrawable* TextLine::invalidate(DisplayObject* target, const MATRIX& initialMatr
 
 	//Compute the matrix and the masks that are relevant
 	bool isMask;
-	bool hasMask;
+	_NR<DisplayObject> mask;
 	MATRIX totalMatrix;
 	std::vector<IDrawable::MaskData> masks;
-	computeMasksAndMatrix(target,masks,totalMatrix,false,isMask,hasMask);
+	computeMasksAndMatrix(target,masks,totalMatrix,false,isMask,mask);
 	totalMatrix=initialMatrix.multiplyMatrix(totalMatrix);
 	computeBoundsForTransformedRect(bxmin,bxmax,bymin,bymax,x,y,width,height,totalMatrix);
 	MATRIX totalMatrix2;
-	computeMasksAndMatrix(target,masks,totalMatrix2,true,isMask,hasMask);
+	computeMasksAndMatrix(target,masks,totalMatrix2,true,isMask,mask);
 	totalMatrix2=initialMatrix.multiplyMatrix(totalMatrix2);
 	computeBoundsForTransformedRect(bxmin,bxmax,bymin,bymax,rx,ry,rwidth,rheight,totalMatrix2);
 	if(width==0 || height==0)
@@ -734,31 +1011,33 @@ IDrawable* TextLine::invalidate(DisplayObject* target, const MATRIX& initialMatr
 
 	float rotation = getConcatenatedMatrix().getRotation();
 	return new CairoPangoRenderer(*this, totalMatrix,
-				x, y, width, height,
-				rx, ry, rwidth, rheight,rotation,
+				x, y, ceil(width), ceil(height),
+				rx, ry, ceil(rwidth), ceil(rheight), rotation,
 				totalMatrix.getScaleX(),totalMatrix.getScaleY(),
-				isMask,hasMask,
+				isMask,mask,
 				1.0f,getConcatenatedAlpha(),masks,
 				1.0f,1.0f,1.0f,1.0f,
 				0.0f,0.0f,0.0f,0.0f,
-				smoothing,bxmin,bymin,0);
+				smoothing ? SMOOTH_MODE::SMOOTH_SUBPIXEL : SMOOTH_MODE::SMOOTH_NONE,0);
 }
 
-bool TextLine::renderImpl(RenderContext& ctxt) const
+bool TextLine::renderImpl(RenderContext& ctxt)
 {
 	return defaultRender(ctxt);
 }
 
-_NR<DisplayObject> TextLine::hitTestImpl(_NR<DisplayObject> last, number_t x, number_t y, DisplayObject::HIT_TYPE type,bool interactiveObjectsOnly)
+_NR<DisplayObject> TextLine::hitTestImpl(number_t x, number_t y, DisplayObject::HIT_TYPE type,bool interactiveObjectsOnly)
 {
 	number_t xmin,xmax,ymin,ymax;
 	boundsRect(xmin,xmax,ymin,ymax);
-	if( xmin <= x && x <= xmax && ymin <= y && y <= ymax && isHittable(type))
-		return last;
-	else
+	if( xmin <= x && x <= xmax && ymin <= y && y <= ymax)
 	{
 		incRef();
-		return DisplayObjectContainer::hitTestImpl(_MR(this), x, y, type, interactiveObjectsOnly);
+		return _MNR(this);
+	}
+	else
+	{
+		return DisplayObjectContainer::hitTestImpl(x, y, type, interactiveObjectsOnly);
 	}
 }
 
