@@ -18,12 +18,15 @@
 **************************************************************************/
 
 #include "scripting/abc.h"
-#include "scripting/flash/utils/flashutils.h"
+#include "scripting/flash/utils/ByteArray.h"
 #include "asobject.h"
 #include "scripting/class.h"
 #include "compat.h"
 #include "parsing/amf3_generator.h"
 #include "scripting/argconv.h"
+#include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/Integer.h"
+#include "scripting/toplevel/UInteger.h"
 #include "scripting/flash/errors/flasherrors.h"
 #include <sstream>
 #include <zlib.h>
@@ -38,7 +41,7 @@ using namespace lightspark;
 // maybe we should set this smaller
 #define BA_MAX_SIZE 0x40000000
 
-ByteArray::ByteArray(Class_base* c, uint8_t* b, uint32_t l):ASObject(c,T_OBJECT,SUBTYPE_BYTEARRAY),littleEndian(false),objectEncoding(ObjectEncoding::AMF3),currentObjectEncoding(ObjectEncoding::AMF3),
+ByteArray::ByteArray(ASWorker* wrk, Class_base* c, uint8_t* b, uint32_t l):ASObject(wrk,c,T_OBJECT,SUBTYPE_BYTEARRAY),littleEndian(false),objectEncoding(OBJECT_ENCODING::AMF3),currentObjectEncoding(OBJECT_ENCODING::AMF3),
 	position(0),bytes(b),real_len(l),len(l),shareable(false)
 {
 #ifdef MEMORY_USAGE_PROFILING
@@ -53,13 +56,46 @@ ByteArray::~ByteArray()
 #ifdef MEMORY_USAGE_PROFILING
 		getClass()->memoryAccount->removeBytes(real_len);
 #endif
-		free(bytes);
+		delete[] bytes;
+		bytes = nullptr;
+	}
+}
+
+bool ByteArray::destruct()
+{
+	if(bytes)
+	{
+#ifdef MEMORY_USAGE_PROFILING
+		getClass()->memoryAccount->removeBytes(real_len);
+#endif
+		delete[] bytes;
+		bytes = nullptr;
+	}
+	currentObjectEncoding = OBJECT_ENCODING::AMF3;
+	position = 0;
+	real_len = 0;
+	len = 0;
+	shareable = false;
+	littleEndian = false;
+	return ASObject::destruct();
+}
+
+void ByteArray::finalize()
+{
+	if(bytes)
+	{
+#ifdef MEMORY_USAGE_PROFILING
+		getClass()->memoryAccount->removeBytes(real_len);
+#endif
+		delete[] bytes;
+		bytes = nullptr;
 	}
 }
 
 void ByteArray::sinit(Class_base* c)
 {
 	CLASS_SETUP(c, ASObject, _constructor, CLASS_SEALED);
+	c->isReusable=true;
 	c->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(c->getSystemState(),_getLength,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(c->getSystemState(),_setLength),SETTER_METHOD,true);
 	c->setDeclaredMethodByQName("bytesAvailable","",Class<IFunction>::getFunction(c->getSystemState(),_getBytesAvailable,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
@@ -72,7 +108,7 @@ void ByteArray::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("defaultObjectEncoding","",Class<IFunction>::getFunction(c->getSystemState(),_getDefaultObjectEncoding,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,false);
 	c->setDeclaredMethodByQName("defaultObjectEncoding","",Class<IFunction>::getFunction(c->getSystemState(),_setDefaultObjectEncoding),SETTER_METHOD,false);
 
-	c->getSystemState()->staticByteArrayDefaultObjectEncoding = ObjectEncoding::DEFAULT;
+	c->getSystemState()->staticByteArrayDefaultObjectEncoding = OBJECT_ENCODING::DEFAULT;
 	c->setDeclaredMethodByQName("clear","",Class<IFunction>::getFunction(c->getSystemState(),clear),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("compress","",Class<IFunction>::getFunction(c->getSystemState(),_compress),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("uncompress","",Class<IFunction>::getFunction(c->getSystemState(),_uncompress),NORMAL_METHOD,true);
@@ -89,7 +125,7 @@ void ByteArray::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("readUnsignedByte","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedByte,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("readUnsignedInt","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedInt,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("readUnsignedShort","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedShort,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readObject","",Class<IFunction>::getFunction(c->getSystemState(),readObject),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readObject","",Class<IFunction>::getFunction(c->getSystemState(),readObject,0,Class<ASObject>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("readUTF","",Class<IFunction>::getFunction(c->getSystemState(),readUTF,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("readUTFBytes","",Class<IFunction>::getFunction(c->getSystemState(),readUTFBytes,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("writeBoolean","",Class<IFunction>::getFunction(c->getSystemState(),writeBoolean),NORMAL_METHOD,true);
@@ -116,14 +152,13 @@ void ByteArray::sinit(Class_base* c)
 	IDataOutput::linkTraits(c);
 }
 
-void ByteArray::buildTraits(ASObject* o)
-{
-}
-
 uint8_t* ByteArray::getBufferIntern(unsigned int size, bool enableResize)
 {
 	if (size > BA_MAX_SIZE) 
-		throwError<ASError>(kOutOfMemoryError);
+	{
+		createError<ASError>(getInstanceWorker(), kOutOfMemoryError);
+		return nullptr;
+	}
 	// The first allocation is exactly the size we need,
 	// the subsequent reallocations happen in increments of BA_CHUNK_SIZE bytes
 	uint32_t prevLen = len;
@@ -131,7 +166,7 @@ uint8_t* ByteArray::getBufferIntern(unsigned int size, bool enableResize)
 	{
 		len=size;
 		real_len=len;
-		bytes = (uint8_t*) malloc(len);
+		bytes = new uint8_t[len];
 		memset(bytes,0,len);
 #ifdef MEMORY_USAGE_PROFILING
 		getClass()->memoryAccount->addBytes(len);
@@ -149,11 +184,13 @@ uint8_t* ByteArray::getBufferIntern(unsigned int size, bool enableResize)
 		while(real_len < size)
 			real_len += BA_CHUNK_SIZE;
 		// Reallocate the buffer, in chunks of BA_CHUNK_SIZE bytes
-		uint8_t* bytes2 = (uint8_t*) realloc(bytes, real_len);
+		uint8_t* bytes2 = new uint8_t[real_len];
+		assert_and_throw(bytes2);
+		memcpy(bytes2,bytes,prevLen);
+		delete[] bytes;
 #ifdef MEMORY_USAGE_PROFILING
 		getClass()->memoryAccount->addBytes(real_len-prev_real_len);
 #endif
-		assert_and_throw(bytes2);
 		bytes = bytes2;
 		if(prevLen<size)
 			memset(bytes+prevLen,0,real_len-prevLen);
@@ -167,54 +204,6 @@ uint8_t* ByteArray::getBufferIntern(unsigned int size, bool enableResize)
 	return bytes;
 }
 
-uint16_t ByteArray::endianIn(uint16_t value)
-{
-	if(littleEndian)
-		return GUINT16_TO_LE(value);
-	else
-		return GUINT16_TO_BE(value);
-}
-
-uint32_t ByteArray::endianIn(uint32_t value)
-{
-	if(littleEndian)
-		return GUINT32_TO_LE(value);
-	else
-		return GUINT32_TO_BE(value);
-}
-
-uint64_t ByteArray::endianIn(uint64_t value)
-{
-	if(littleEndian)
-		return GUINT64_TO_LE(value);
-	else
-		return GUINT64_TO_BE(value);
-}
-
-uint16_t ByteArray::endianOut(uint16_t value)
-{
-	if(littleEndian)
-		return GUINT16_FROM_LE(value);
-	else
-		return GUINT16_FROM_BE(value);
-}
-
-uint32_t ByteArray::endianOut(uint32_t value)
-{
-	if(littleEndian)
-		return GUINT32_FROM_LE(value);
-	else
-		return GUINT32_FROM_BE(value);
-}
-
-uint64_t ByteArray::endianOut(uint64_t value)
-{
-	if(littleEndian)
-		return GUINT64_FROM_LE(value);
-	else
-		return GUINT64_FROM_BE(value);
-}
-
 ASFUNCTIONBODY_ATOM(ByteArray,_constructor)
 {
 }
@@ -222,7 +211,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,_constructor)
 ASFUNCTIONBODY_ATOM(ByteArray,_getPosition)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
-	asAtomHandler::setUInt(ret,sys,th->getPosition());
+	asAtomHandler::setUInt(ret,wrk,th->getPosition());
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,_setPosition)
@@ -236,35 +225,38 @@ ASFUNCTIONBODY_ATOM(ByteArray,_getEndian)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 	if(th->littleEndian)
-		ret = asAtomHandler::fromString(sys,Endian::littleEndian);
+		ret = asAtomHandler::fromString(wrk->getSystemState(),Endian::littleEndian);
 	else
-		ret = asAtomHandler::fromString(sys,Endian::bigEndian);
+		ret = asAtomHandler::fromString(wrk->getSystemState(),Endian::bigEndian);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,_setEndian)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
-	if(asAtomHandler::toString(args[0],sys) == Endian::littleEndian)
+	if(asAtomHandler::toString(args[0],wrk) == Endian::littleEndian)
 		th->littleEndian = true;
-	else if(asAtomHandler::toString(args[0],sys) == Endian::bigEndian)
+	else if(asAtomHandler::toString(args[0],wrk) == Endian::bigEndian)
 		th->littleEndian = false;
 	else
-		throwError<ArgumentError>(kInvalidEnumError, "endian");
+		createError<ArgumentError>(wrk,kInvalidEnumError, "endian");
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,_getObjectEncoding)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
-	asAtomHandler::setUInt(ret,sys,th->objectEncoding);
+	asAtomHandler::setUInt(ret,wrk,th->objectEncoding);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,_setObjectEncoding)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 	uint32_t value;
-	ARG_UNPACK_ATOM(value);
-	if(value!=ObjectEncoding::AMF0 && value!=ObjectEncoding::AMF3)
-		throwError<ArgumentError>(kInvalidEnumError, "objectEncoding");
+	ARG_CHECK(ARG_UNPACK(value));
+	if(value!=OBJECT_ENCODING::AMF0 && value!=OBJECT_ENCODING::AMF3)
+	{
+		createError<ArgumentError>(wrk,kInvalidEnumError, "objectEncoding");
+		return;
+	}
 
 	th->objectEncoding=value;
 	th->currentObjectEncoding=value;
@@ -272,7 +264,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,_setObjectEncoding)
 
 ASFUNCTIONBODY_ATOM(ByteArray,_getDefaultObjectEncoding)
 {
-	asAtomHandler::setUInt(ret,sys,sys->staticNetConnectionDefaultObjectEncoding);
+	asAtomHandler::setUInt(ret,wrk,wrk->getSystemState()->staticNetConnectionDefaultObjectEncoding);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,_setDefaultObjectEncoding)
@@ -280,9 +272,9 @@ ASFUNCTIONBODY_ATOM(ByteArray,_setDefaultObjectEncoding)
 	assert_and_throw(argslen == 1);
 	int32_t value = asAtomHandler::toInt(args[0]);
 	if(value == 0)
-		sys->staticByteArrayDefaultObjectEncoding = ObjectEncoding::AMF0;
+		wrk->getSystemState()->staticByteArrayDefaultObjectEncoding = OBJECT_ENCODING::AMF0;
 	else if(value == 3)
-		sys->staticByteArrayDefaultObjectEncoding = ObjectEncoding::AMF3;
+		wrk->getSystemState()->staticByteArrayDefaultObjectEncoding = OBJECT_ENCODING::AMF3;
 	else
 		throw RunTimeException("Invalid object encoding");
 }
@@ -312,9 +304,9 @@ void ByteArray::setLength(uint32_t newLen)
 #ifdef MEMORY_USAGE_PROFILING
 			getClass()->memoryAccount->removeBytes(real_len);
 #endif
-			free(bytes);
+			delete[] bytes;
 		}
-		bytes = NULL;
+		bytes = nullptr;
 		real_len = newLen;
 	}
 	len = newLen;
@@ -324,13 +316,13 @@ void ByteArray::setLength(uint32_t newLen)
 ASFUNCTIONBODY_ATOM(ByteArray,_getLength)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
-	asAtomHandler::setUInt(ret,sys,th->len);
+	asAtomHandler::setUInt(ret,wrk,th->len);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,_getBytesAvailable)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
-	asAtomHandler::setUInt(ret,sys,th->len-th->position);
+	asAtomHandler::setUInt(ret,wrk,th->len-th->position);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,readBoolean)
@@ -342,7 +334,8 @@ ASFUNCTIONBODY_ATOM(ByteArray,readBoolean)
 	if(!th->readByte(res))
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 
 	th->unlock();
@@ -355,7 +348,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,readBytes)
 	_NR<ByteArray> out;
 	uint32_t offset;
 	uint32_t length;
-	ARG_UNPACK_ATOM(out)(offset, 0)(length, 0);
+	ARG_CHECK(ARG_UNPACK(out)(offset, 0)(length, 0));
 	
 	th->lock();
 	if(length == 0)
@@ -368,12 +361,14 @@ ASFUNCTIONBODY_ATOM(ByteArray,readBytes)
 	if(th->position+length > th->len)
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 	if((uint64_t)length+offset > 0xFFFFFFFF)
 	{
 		th->unlock();
-		throw Class<RangeError>::getInstanceS(sys,"length+offset");
+		createError<RangeError>(wrk,0,"length+offset");
+		return;
 	}
 	
 	uint8_t* buf=out->getBuffer(length+offset,true);
@@ -417,10 +412,11 @@ ASFUNCTIONBODY_ATOM(ByteArray,readUTF)
 	if (!th->readUTF(res))
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 	th->unlock();
-	ret = asAtomHandler::fromObject(abstract_s(sys,res));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,res));
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,readUTFBytes)
@@ -428,16 +424,17 @@ ASFUNCTIONBODY_ATOM(ByteArray,readUTFBytes)
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 	uint32_t length;
 
-	ARG_UNPACK_ATOM (length);
+	ARG_CHECK(ARG_UNPACK (length));
 	th->lock();
 	if(th->position+length > th->len)
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 	tiny_string res;
 	th->readUTFBytes(length,res);
-	ret = asAtomHandler::fromObject(abstract_s(sys,res));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,res));
 }
 
 bool ByteArray::readUTFBytes(uint32_t length,tiny_string& ret)
@@ -454,12 +451,13 @@ bool ByteArray::readUTFBytes(uint32_t length,tiny_string& ret)
 		}
 	}
 	uint8_t *bufStart=bytes+position;
-	char* buf = g_newa(char,length+1);
+	char* buf = new char[length+1];
 	buf[length]=0;
 	strncpy(buf,(char*)bufStart,(size_t)length);
 	position+=length;
 	ret = buf;
 	ret.checkValidUTF();
+	delete[] buf;
 	return true;
 }
 bool ByteArray::readBytes(uint32_t offset, uint32_t length,uint8_t* ret)
@@ -486,12 +484,30 @@ asAtom ByteArray::readObject()
 	return ret;
 }
 
+ASObject* ByteArray::readSharedObject()
+{
+	ASObject* ret = Class<ASObject>::getInstanceS(getInstanceWorker());
+	if (getLength()<=16)
+		return ret;
+	Amf3Deserializer d(this);
+	try
+	{
+		d.readSharedObject(ret);
+	}
+	catch(LightsparkException& e)
+	{
+		LOG(LOG_ERROR,"Exception caught while parsing shared object: " << e.cause);
+	}
+	return ret;
+}
+
 void ByteArray::writeUTF(const tiny_string& str)
 {
 	getBuffer(position+str.numBytes()+2,true);
 	if(str.numBytes() > 65535)
 	{
-		throwError<RangeError>(kParamRangeError);
+		createError<RangeError>(getInstanceWorker(),kParamRangeError);
+		return;
 	}
 	uint16_t numBytes=endianIn((uint16_t)str.numBytes());
 	memcpy(bytes+position,&numBytes,2);
@@ -506,7 +522,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,writeUTF)
 	assert_and_throw(argslen==1);
 	assert_and_throw(asAtomHandler::isString(args[0]));
 	th->lock();
-	th->writeUTF(asAtomHandler::toString(args[0],sys));
+	th->writeUTF(asAtomHandler::toString(args[0],wrk));
 	th->unlock();
 }
 
@@ -516,7 +532,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,writeUTFBytes)
 	//Validate parameters
 	assert_and_throw(argslen==1);
 	assert_and_throw(asAtomHandler::isString(args[0]));
-	tiny_string str=asAtomHandler::toString(args[0],sys);
+	tiny_string str=asAtomHandler::toString(args[0],wrk);
 	th->lock();
 	th->getBuffer(th->position+str.numBytes(),true);
 	memcpy(th->bytes+th->position,str.raw_buf(),str.numBytes());
@@ -529,7 +545,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,writeMultiByte)
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 	tiny_string value;
 	tiny_string charset;
-	ARG_UNPACK_ATOM(value)(charset);
+	ARG_CHECK(ARG_UNPACK(value)(charset));
 
 	// TODO: should convert from UTF-8 to charset
 	LOG(LOG_NOT_IMPLEMENTED, "ByteArray.writeMultiByte doesn't convert charset");
@@ -541,7 +557,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,writeMultiByte)
 	th->unlock();
 }
 
-uint32_t ByteArray::writeObject(ASObject* obj)
+uint32_t ByteArray::writeObject(ASObject* obj, ASWorker* wrk)
 {
 	//Return the length of the serialized object
 
@@ -550,9 +566,52 @@ uint32_t ByteArray::writeObject(ASObject* obj)
 	map<const ASObject*, uint32_t> objMap;
 	map<const Class_base*, uint32_t> traitsMap;
 	uint32_t oldPosition=position;
-	obj->serialize(this, stringMap, objMap,traitsMap);
+	obj->serialize(this, stringMap, objMap,traitsMap,wrk);
 	return position-oldPosition;
 }
+uint32_t ByteArray::writeAtomObject(asAtom obj, ASWorker* wrk)
+{
+	//Return the length of the serialized object
+
+	//TODO: support custom serialization
+	map<tiny_string, uint32_t> stringMap;
+	map<const ASObject*, uint32_t> objMap;
+	map<const Class_base*, uint32_t> traitsMap;
+	uint32_t oldPosition=position;
+	asAtomHandler::serialize(this,stringMap,objMap,traitsMap,wrk,obj);
+	return position-oldPosition;
+}
+void ByteArray::writeSharedObject(ASObject* obj, const tiny_string& name, ASWorker* wrk)
+{
+	// write amf header
+	writeByte(0x00);
+	writeByte(0xbf);
+	uint32_t sizepos = getPosition();
+	writeUnsignedInt(0); // size of file, will be filled at end of method
+	writeByte('T');
+	writeByte('C');
+	writeByte('S');
+	writeByte('O');
+	writeByte(0x00);
+	writeByte(0x04);
+	writeByte(0x00);
+	writeByte(0x00);
+	writeByte(0x00);
+	writeByte(0x00);
+	writeUTF(name);
+	writeByte(0x00);
+	writeByte(0x00);
+	writeByte(0x00);
+	writeByte(0x03);// always store as AMF3
+
+	map<tiny_string, uint32_t> stringMap;
+	map<const ASObject*, uint32_t> objMap;
+	map<const Class_base*, uint32_t> traitsMap;
+	obj->serializeDynamicProperties(this, stringMap, objMap,traitsMap,wrk,true,true);
+	setPosition(sizepos);
+	writeUnsignedInt(GUINT32_TO_BE(getLength()-6));
+	setPosition(0);
+}
 
 ASFUNCTIONBODY_ATOM(ByteArray,writeObject)
 {
@@ -560,7 +619,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,writeObject)
 	//Validate parameters
 	assert_and_throw(argslen==1);
 	th->lock();
-	th->writeObject(asAtomHandler::toObject(args[0],sys));
+	th->writeAtomObject(args[0],wrk);
 	th->unlock();
 }
 
@@ -576,7 +635,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,writeShort)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 	int32_t value;
-	ARG_UNPACK_ATOM(value);
+	ARG_CHECK(ARG_UNPACK(value));
 
 	th->lock();
 	th->writeShort((static_cast<uint16_t>(value & 0xffff)));
@@ -631,7 +690,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,writeBoolean)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 	bool b;
-	ARG_UNPACK_ATOM (b);
+	ARG_CHECK(ARG_UNPACK (b));
 
 	th->lock();
 	if (b)
@@ -755,10 +814,11 @@ ASFUNCTIONBODY_ATOM(ByteArray, readByte)
 	if(!th->readByte(res))
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 	th->unlock();
-	asAtomHandler::setInt(ret,sys,(int8_t)res);
+	asAtomHandler::setInt(ret,wrk,(int8_t)res);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,readDouble)
@@ -767,20 +827,17 @@ ASFUNCTIONBODY_ATOM(ByteArray,readDouble)
 	assert_and_throw(argslen==0);
 
 	th->lock();
-	if(th->len < th->position+8)
+	number_t res;
+	if(!th->readDouble(res,th->position))
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
-
-	uint64_t res;
-	memcpy(&res,th->bytes+th->position,8);
 	th->position+=8;
-	res = th->endianOut(res);
 
-	double *doubleptr=reinterpret_cast<double*>(&res);
 	th->unlock();
-	asAtomHandler::setNumber(ret,sys,*doubleptr);
+	asAtomHandler::setNumber(ret,wrk,res);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,readFloat)
@@ -789,20 +846,17 @@ ASFUNCTIONBODY_ATOM(ByteArray,readFloat)
 	assert_and_throw(argslen==0);
 
 	th->lock();
-	if(th->len < th->position+4)
+	float res;
+	if(!th->readFloat(res,th->position))
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
-
-	uint32_t res;
-	memcpy(&res,th->bytes+th->position,4);
 	th->position+=4;
-	res = th->endianOut(res);
 
-	float *floatptr=reinterpret_cast<float*>(&res);
 	th->unlock();
-	asAtomHandler::setNumber(ret,sys,*floatptr);
+	asAtomHandler::setNumber(ret,wrk,res);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,readInt)
@@ -814,14 +868,15 @@ ASFUNCTIONBODY_ATOM(ByteArray,readInt)
 	if(th->len < th->position+4)
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 
 	uint32_t res;
 	memcpy(&res,th->bytes+th->position,4);
 	th->position+=4;
 	th->unlock();
-	asAtomHandler::setInt(ret,sys,(int32_t)th->endianOut(res));
+	asAtomHandler::setInt(ret,wrk,(int32_t)th->endianOut(res));
 }
 
 bool ByteArray::readShort(uint16_t& ret)
@@ -846,11 +901,12 @@ ASFUNCTIONBODY_ATOM(ByteArray,readShort)
 	if(!th->readShort(res))
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 
 	th->unlock();
-	asAtomHandler::setInt(ret,sys,(int16_t)res);
+	asAtomHandler::setInt(ret,wrk,(int16_t)res);
 }
 ASFUNCTIONBODY_ATOM(ByteArray,readUnsignedByte)
 {
@@ -861,10 +917,11 @@ ASFUNCTIONBODY_ATOM(ByteArray,readUnsignedByte)
 	if (!th->readByte(res))
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 	th->unlock();
-	asAtomHandler::setUInt(ret,sys,(uint32_t)res);
+	asAtomHandler::setUInt(ret,wrk,(uint32_t)res);
 }
 
 bool ByteArray::readUnsignedInt(uint32_t& ret)
@@ -889,10 +946,11 @@ ASFUNCTIONBODY_ATOM(ByteArray,readUnsignedInt)
 	if(!th->readUnsignedInt(res))
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 	th->unlock();
-	asAtomHandler::setUInt(ret,sys,res);
+	asAtomHandler::setUInt(ret,wrk,res);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,readUnsignedShort)
@@ -905,11 +963,12 @@ ASFUNCTIONBODY_ATOM(ByteArray,readUnsignedShort)
 	if(!th->readShort(res))
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 
 	th->unlock();
-	asAtomHandler::setUInt(ret,sys,(uint32_t)res);
+	asAtomHandler::setUInt(ret,wrk,(uint32_t)res);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,readMultiByte)
@@ -917,13 +976,14 @@ ASFUNCTIONBODY_ATOM(ByteArray,readMultiByte)
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 	uint32_t strlen;
 	tiny_string charset;
-	ARG_UNPACK_ATOM(strlen)(charset);
+	ARG_CHECK(ARG_UNPACK(strlen)(charset));
 
 	th->lock();
 	if(th->len < th->position+strlen)
 	{
 		th->unlock();
-		throwError<EOFError>(kEOFError);
+		createError<EOFError>(wrk,kEOFError);
+		return;
 	}
 
 	if (charset != "us-ascii" && charset != "utf-8")
@@ -934,7 +994,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,readMultiByte)
 	s[strlen] = 0x0;
 	tiny_string res(s,true);
 	th->unlock();
-	ret = asAtomHandler::fromObject(abstract_s(sys,res));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,res));
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,readObject)
@@ -948,7 +1008,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,readObject)
 		// it seems that contrary to the specs Adobe returns Undefined when reading from an empty ByteArray
 		asAtomHandler::setUndefined(ret);
 		return;
-		//throwError<EOFError>(kEOFError);
+		//createError<EOFError>(wrk,kEOFError);
 	}
 	ret = th->readObject();
 	th->unlock();
@@ -959,7 +1019,6 @@ ASFUNCTIONBODY_ATOM(ByteArray,readObject)
 		asAtomHandler::setUndefined(ret);
 		return;
 	}
-	ASATOM_INCREF(ret);
 }
 // replicate adobe behaviour where invalid UTF8 chars are treated as single chars
 // this is very expensive for big strings, so always do validateUtf8 first to check if parseUtf8 is neccessary
@@ -1106,30 +1165,30 @@ ASFUNCTIONBODY_ATOM(ByteArray,_toString)
 			start = 3;
 	}
 	if (validateUtf8(th->bytes+start,th->len-start))
-		ret = asAtomHandler::fromObject(abstract_s(sys,(const char*)th->bytes+start,th->len-start));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,(const char*)th->bytes+start,th->len-start));
 	else
-		ret = asAtomHandler::fromObject(abstract_s(sys,parseUtf8(th->bytes+start,th->len-start)));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,parseUtf8(th->bytes+start,th->len-start)));
 }
 
-bool ByteArray::hasPropertyByMultiname(const multiname& name, bool considerDynamic, bool considerPrototype)
+bool ByteArray::hasPropertyByMultiname(const multiname& name, bool considerDynamic, bool considerPrototype, ASWorker* wrk)
 {
 	if(considerDynamic==false)
-		return ASObject::hasPropertyByMultiname(name, considerDynamic, considerPrototype);
+		return ASObject::hasPropertyByMultiname(name, considerDynamic, considerPrototype,wrk);
 	if (!isConstructed())
 		return false;
 	unsigned int index=0;
 	if(!Array::isValidMultiname(getSystemState(),name,index))
-		return ASObject::hasPropertyByMultiname(name, considerDynamic, considerPrototype);
+		return ASObject::hasPropertyByMultiname(name, considerDynamic, considerPrototype,wrk);
 
 	return index<len;
 }
 
-GET_VARIABLE_RESULT ByteArray::getVariableByMultiname(asAtom& ret, const multiname& name, GET_VARIABLE_OPTION opt)
+GET_VARIABLE_RESULT ByteArray::getVariableByMultiname(asAtom& ret, const multiname& name, GET_VARIABLE_OPTION opt, ASWorker* wrk)
 {
 	unsigned int index=0;
 	if((opt & GET_VARIABLE_OPTION::SKIP_IMPL)!=0  || !implEnable || !Array::isValidMultiname(getSystemState(),name,index))
 	{
-		return getVariableByMultinameIntern(ret,name,this->getClass(),opt);
+		return getVariableByMultinameIntern(ret,name,this->getClass(),opt,wrk);
 	}
 
 	if(index<len)
@@ -1137,33 +1196,33 @@ GET_VARIABLE_RESULT ByteArray::getVariableByMultiname(asAtom& ret, const multina
 		lock();
 		uint8_t value = bytes[index];
 		unlock();
-		asAtomHandler::setUInt(ret,this->getSystemState(),static_cast<uint32_t>(value));
+		asAtomHandler::setUInt(ret,this->getInstanceWorker(),static_cast<uint32_t>(value));
 		return GET_VARIABLE_RESULT::GETVAR_NORMAL;
 	}
 	asAtomHandler::setUndefined(ret);
 	return GET_VARIABLE_RESULT::GETVAR_NORMAL;
 }
-GET_VARIABLE_RESULT ByteArray::getVariableByInteger(asAtom &ret, int index, GET_VARIABLE_OPTION opt)
+GET_VARIABLE_RESULT ByteArray::getVariableByInteger(asAtom &ret, int index, GET_VARIABLE_OPTION opt, ASWorker* wrk)
 {
 	if (index < 0)
-		return getVariableByIntegerIntern(ret,index,opt);
+		return getVariableByIntegerIntern(ret,index,opt,wrk);
 	if (index >=0 && uint32_t(index) < len)
 	{
 		lock();
 		uint8_t value = bytes[index];
 		unlock();
-		asAtomHandler::setUInt(ret,this->getSystemState(),static_cast<uint32_t>(value));
+		asAtomHandler::setUInt(ret,this->getInstanceWorker(),static_cast<uint32_t>(value));
 		return GET_VARIABLE_RESULT::GETVAR_NORMAL;
 	}
 	asAtomHandler::setUndefined(ret);
 	return GET_VARIABLE_RESULT::GETVAR_NORMAL;
 }
-int32_t ByteArray::getVariableByMultiname_i(const multiname& name)
+int32_t ByteArray::getVariableByMultiname_i(const multiname& name, ASWorker* wrk)
 {
 	assert_and_throw(implEnable);
 	unsigned int index=0;
 	if(!Array::isValidMultiname(getSystemState(),name,index))
-		return ASObject::getVariableByMultiname_i(name);
+		return ASObject::getVariableByMultiname_i(name,wrk);
 
 	if(index<len)
 	{
@@ -1176,14 +1235,17 @@ int32_t ByteArray::getVariableByMultiname_i(const multiname& name)
 		return _MNR(getSystemState()->getUndefinedRef());
 }
 
-multiname *ByteArray::setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool* alreadyset)
+multiname *ByteArray::setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool* alreadyset,ASWorker* wrk)
 {
 	assert_and_throw(implEnable);
 	unsigned int index=0;
 	if(!Array::isValidMultiname(getSystemState(),name,index))
-		return ASObject::setVariableByMultiname(name,o,allowConst,alreadyset);
+		return ASObject::setVariableByMultiname(name,o,allowConst,alreadyset,wrk);
 	if (index > BA_MAX_SIZE) 
-		throwError<ASError>(kOutOfMemoryError);
+	{
+		createError<ASError>(wrk,kOutOfMemoryError);
+		return nullptr;
+	}
 
 	lock();
 	if(index>=len)
@@ -1201,11 +1263,11 @@ multiname *ByteArray::setVariableByMultiname(multiname& name, asAtom& o, CONST_A
 	ASATOM_DECREF(o);
 	return nullptr;
 }
-void ByteArray::setVariableByInteger(int index, asAtom &o, ASObject::CONST_ALLOWED_FLAG allowConst)
+void ByteArray::setVariableByInteger(int index, asAtom &o, ASObject::CONST_ALLOWED_FLAG allowConst, bool* alreadyset, ASWorker* wrk)
 {
 	if (index < 0)
 	{
-		setVariableByInteger_intern(index,o,allowConst);
+		setVariableByInteger_intern(index,o,allowConst,alreadyset,wrk);
 		return;
 	}
 	lock();
@@ -1224,10 +1286,10 @@ void ByteArray::setVariableByInteger(int index, asAtom &o, ASObject::CONST_ALLOW
 
 	ASATOM_DECREF(o);
 }
-void ByteArray::setVariableByMultiname_i(multiname& name, int32_t value)
+void ByteArray::setVariableByMultiname_i(multiname& name, int32_t value, ASWorker* wrk)
 {
 	asAtom v = asAtomHandler::fromInt(value);
-	setVariableByMultiname(name, v,ASObject::CONST_NOT_ALLOWED);
+	setVariableByMultiname(name, v,ASObject::CONST_NOT_ALLOWED,nullptr,wrk);
 }
 
 void ByteArray::acquireBuffer(uint8_t* buf, int bufLen)
@@ -1237,7 +1299,7 @@ void ByteArray::acquireBuffer(uint8_t* buf, int bufLen)
 #ifdef MEMORY_USAGE_PROFILING
 		getClass()->memoryAccount->removeBytes(real_len);
 #endif
-		free(bytes);
+		delete[] bytes;
 	}
 	bytes=buf;
 	real_len=bufLen;
@@ -1284,7 +1346,10 @@ void ByteArray::writeStringVR(map<tiny_string, uint32_t>& stringMap, const tiny_
 {
 	const uint32_t len=s.numBytes();
 	if(len >= 1<<28)
-		throwError<RangeError>(kParamRangeError);
+	{
+		createError<RangeError>(getInstanceWorker(),kParamRangeError);
+		return;
+	}
 
 	//Check if the string is already in the map
 	auto it=stringMap.find(s);
@@ -1333,7 +1398,10 @@ void ByteArray::writeXMLString(std::map<const ASObject*, uint32_t>& objMap,
 			       const tiny_string& xmlstr)
 {
 	if(xmlstr.numBytes() >= 1<<28)
-		throwError<RangeError>(kParamRangeError);
+	{
+		createError<RangeError>(getInstanceWorker(),kParamRangeError);
+		return;
+	}
 
 	//Check if the XML object has been already serialized
 	auto it=objMap.find(xml);
@@ -1374,20 +1442,38 @@ void ByteArray::removeFrontBytes(int count)
 
 
 
-void ByteArray::compress_zlib()
+void ByteArray::compress_zlib(bool raw)
 {
+	z_stream strm;
+	int status;
+
 	if(len==0)
 		return;
 
+	strm.zalloc=Z_NULL;
+	strm.zfree=Z_NULL;
+	strm.opaque=Z_NULL;
+	strm.avail_in=len;
+	strm.next_in=bytes;
+	strm.avail_out=0;
+	status=deflateInit2 (&strm, Z_DEFAULT_COMPRESSION, Z_DEFLATED,
+						 raw ? -15 : 15, 
+						 8,
+						 Z_DEFAULT_STRATEGY);
+	if (status != Z_OK)
+		throw RunTimeException("zlib compress failed");
 	unsigned long buflen=compressBound(len);
-	uint8_t *compressed=(uint8_t*) malloc(buflen);
-	assert_and_throw(compressed);
-
-	if(compress(compressed, &buflen, bytes, len)!=Z_OK)
+	position=0;
+	unsigned char* compressed = new unsigned char[buflen];
+	strm.avail_out = buflen;
+	strm.next_out = compressed;
+	status = deflate (&strm, Z_FINISH);
+	if (status == Z_STREAM_ERROR || strm.avail_in != 0)
 	{
-		free(compressed);
+		delete[] compressed;
 		throw RunTimeException("zlib compress failed");
 	}
+	deflateEnd(&strm);
 
 	acquireBuffer(compressed, buflen);
 	position=buflen;
@@ -1409,7 +1495,10 @@ void ByteArray::uncompress_zlib(bool raw)
 	strm.total_out=0;
 	status=inflateInit2(&strm,raw ? -15 : 15);
 	if(status==Z_VERSION_ERROR)
-		throw Class<IOError>::getInstanceS(getSystemState(),"not valid compressed data");
+	{
+		createError<IOError>(getInstanceWorker(),0,"not valid compressed data");
+		return;
+	}
 	else if(status!=Z_OK)
 		throw RunTimeException("zlib uncompress failed");
 
@@ -1423,7 +1512,8 @@ void ByteArray::uncompress_zlib(bool raw)
 		if(status!=Z_OK && status!=Z_STREAM_END)
 		{
 			inflateEnd(&strm);
-			throw Class<IOError>::getInstanceS(getSystemState(),"not valid compressed data");
+			createError<IOError>(getInstanceWorker(),0,"not valid compressed data");
+			return;
 		}
 
 		if(strm.avail_out==0)
@@ -1437,8 +1527,9 @@ void ByteArray::uncompress_zlib(bool raw)
 	getClass()->memoryAccount->addBytes(len-real_len);
 #endif
 	real_len = len;
-	uint8_t* bytes2=(uint8_t*) realloc(bytes, len);
+	uint8_t* bytes2 = new uint8_t[len];
 	assert_and_throw(bytes2);
+	delete[] bytes;
 	bytes = bytes2;
 	memcpy(bytes, &buf[0], len);
 	position=0;
@@ -1451,7 +1542,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,_compress)
 	// and always uses the zlib algorithm
 	// but tamarin tests do not catch it, so we simply ignore any parameters provided
 	th->lock();
-	th->compress_zlib();
+	th->compress_zlib(false);
 	th->unlock();
 }
 
@@ -1470,7 +1561,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,_deflate)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 	th->lock();
-	th->compress_zlib();
+	th->compress_zlib(true);
 	th->unlock();
 }
 
@@ -1491,9 +1582,9 @@ ASFUNCTIONBODY_ATOM(ByteArray,clear)
 #ifdef MEMORY_USAGE_PROFILING
 		th->getClass()->memoryAccount->removeBytes(th->real_len);
 #endif
-		free(th->bytes);
+		delete[] th->bytes;
 	}
-	th->bytes = NULL;
+	th->bytes = nullptr;
 	th->len=0;
 	th->real_len=0;
 	th->position=0;
@@ -1512,7 +1603,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,pop)
 		th->len--;
 	}
 	th->unlock();
-	asAtomHandler::setUInt(ret,sys,(uint32_t)res);
+	asAtomHandler::setUInt(ret,wrk,(uint32_t)res);
 	
 }
 
@@ -1528,7 +1619,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,push)
 	}
 	uint32_t res = th->getLength();
 	th->unlock();
-	asAtomHandler::setUInt(ret,sys,res);
+	asAtomHandler::setUInt(ret,wrk,res);
 }
 
 // this seems to be how AS3 handles generic shift calls in Array class
@@ -1543,7 +1634,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,shift)
 		th->len--;
 	}
 	th->unlock();
-	asAtomHandler::setUInt(ret,sys,(uint32_t)res);
+	asAtomHandler::setUInt(ret,wrk,(uint32_t)res);
 }
 
 // this seems to be how AS3 handles generic unshift calls in Array class
@@ -1559,14 +1650,14 @@ ASFUNCTIONBODY_ATOM(ByteArray,unshift)
 	}
 	uint32_t res = th->getLength();
 	th->unlock();
-	asAtomHandler::setUInt(ret,sys,res);
+	asAtomHandler::setUInt(ret,wrk,res);
 }
-ASFUNCTIONBODY_GETTER(ByteArray,shareable);
+ASFUNCTIONBODY_GETTER(ByteArray,shareable)
 ASFUNCTIONBODY_ATOM(ByteArray,_setter_shareable)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 	bool value;
-	ARG_UNPACK_ATOM(value);
+	ARG_CHECK(ARG_UNPACK(value));
 	if (!th->shareable)
 		th->shareable=value;
 }
@@ -1575,17 +1666,19 @@ ASFUNCTIONBODY_ATOM(ByteArray,atomicCompareAndSwapIntAt)
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 
 	int32_t byteindex,expectedValue,newvalue;
-	ARG_UNPACK_ATOM(byteindex)(expectedValue)(newvalue);
+	ARG_CHECK(ARG_UNPACK(byteindex)(expectedValue)(newvalue));
 
-	if (byteindex < 0 || byteindex%4)
+	if ((byteindex < 0) || (byteindex%4))
 	{
-		throwError<RangeError>(kInvalidRangeError, th->getClassName());
+		createError<RangeError>(wrk,kInvalidRangeError, th->getClassName());
+		return;
 	}
 	th->lock();
 	if(byteindex >= (int32_t)th->len-4)
 	{
 		th->unlock();
-		throwError<RangeError>(kInvalidRangeError, th->getClassName());
+		createError<RangeError>(wrk,kInvalidRangeError, th->getClassName());
+		return;
 	}
 	int32_t res;
 	memcpy(&res,th->bytes+byteindex,4);
@@ -1595,13 +1688,13 @@ ASFUNCTIONBODY_ATOM(ByteArray,atomicCompareAndSwapIntAt)
 		memcpy(th->bytes+byteindex,&newvalue,4);
 	}
 	th->unlock();
-	asAtomHandler::setInt(ret,sys,res);
+	asAtomHandler::setInt(ret,wrk,res);
 }
 ASFUNCTIONBODY_ATOM(ByteArray,atomicCompareAndSwapLength)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
 	int32_t expectedLength,newLength;
-	ARG_UNPACK_ATOM(expectedLength)(newLength);
+	ARG_CHECK(ARG_UNPACK(expectedLength)(newLength));
 
 	th->lock();
 	int32_t res = th->len;
@@ -1610,19 +1703,19 @@ ASFUNCTIONBODY_ATOM(ByteArray,atomicCompareAndSwapLength)
 		th->setLength(newLength);
 	}
 	th->unlock();
-	asAtomHandler::setInt(ret,sys,res);
+	asAtomHandler::setInt(ret,wrk,res);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,_toJSON)
 {
-	ret = asAtomHandler::fromString(sys,"ByteArray");
+	ret = asAtomHandler::fromString(wrk->getSystemState(),"ByteArray");
 }
 
 void ByteArray::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
-				std::map<const Class_base*, uint32_t>& traitsMap)
+				std::map<const Class_base*, uint32_t>& traitsMap,ASWorker* wrk)
 {
-	if (out->getObjectEncoding() == ObjectEncoding::AMF0)
+	if (out->getObjectEncoding() == OBJECT_ENCODING::AMF0)
 	{
 		LOG(LOG_NOT_IMPLEMENTED,"serializing ByteArray in AMF0 not implemented");
 		return;
