@@ -22,6 +22,7 @@
 
 #include "compat.h"
 #include "swftypes.h"
+#include "threading.h"
 #include "scripting/flash/utils/flashutils.h"
 
 namespace lightspark
@@ -42,11 +43,10 @@ protected:
 	uint8_t* bytes;
 	uint32_t real_len;
 	uint32_t len;
-	void compress_zlib();
+	void compress_zlib(bool raw);
 	void uncompress_zlib(bool raw);
 	Mutex mutex;
 	uint8_t* getBufferIntern(unsigned int size, bool enableResize);
-	
 public:
 	FORCE_INLINE void lock()
 	{
@@ -56,8 +56,10 @@ public:
 	{
 		if (shareable) mutex.unlock();
 	}
-	ByteArray(Class_base* c, uint8_t* b = NULL, uint32_t l = 0);
+	ByteArray(ASWorker* wrk,Class_base* c, uint8_t* b = nullptr, uint32_t l = 0);
 	~ByteArray();
+	bool destruct() override;
+	void finalize() override;
 	//Helper interface for serialization
 	bool peekByte(uint8_t& b);
 	FORCE_INLINE bool readByte(uint8_t& b)
@@ -76,7 +78,39 @@ public:
 	bool readUTF(tiny_string& ret);
 	bool readUTFBytes(uint32_t length,tiny_string& ret);
 	bool readBytes(uint32_t offset, uint32_t length, uint8_t* ret);
+	inline bool readFloat(float& ret, uint32_t pos)
+	{
+		if(len < pos+4)
+			return false;
+		union
+		{
+			uint32_t u;
+			float f;
+		} res;
+		memcpy(&res,bytes+pos,4);
+		res.u = endianOut(res.u);
+		ret=res.f;
+		return true;
+	}
+	inline bool readDouble(number_t& ret, uint32_t pos)
+	{
+		if(len < pos+8)
+			return false;
+	
+		union
+		{
+			uint64_t u;
+			double d;
+		} res;
+		memcpy(&res,bytes+pos,8);
+		pos+=8;
+		res.u = endianOut(res.u);
+		ret=res.d;
+		return true;
+	}
+	
 	asAtom readObject();
+	ASObject* readSharedObject();
 	FORCE_INLINE void writeByte(uint8_t b)
 	{
 		getBuffer(position+1,true);
@@ -91,12 +125,13 @@ public:
 	void writeShort(uint16_t val);
 	void writeUnsignedInt(uint32_t val);
 	void writeUTF(const tiny_string& str);
-	uint32_t writeObject(ASObject* obj);
+	uint32_t writeObject(ASObject* obj,ASWorker* wrk);
+	uint32_t writeAtomObject(asAtom obj,ASWorker* wrk);
+	void writeSharedObject(ASObject* obj, const tiny_string& name, ASWorker* wrk);
 	void writeStringVR(std::map<tiny_string, uint32_t>& stringMap, const tiny_string& s);
 	void writeStringAMF0(const tiny_string& s);
 	void writeXMLString(std::map<const ASObject*, uint32_t>& objMap, ASObject *xml, const tiny_string& s);
 	void writeU29(uint32_t val);
-
 	void serializeDouble(number_t val);
 
 	void setLength(uint32_t newLen);
@@ -201,27 +236,62 @@ public:
 	}
 	uint32_t getLength() const { return len; }
 
-	uint16_t endianIn(uint16_t value);
-	uint32_t endianIn(uint32_t value);
-	uint64_t endianIn(uint64_t value);
+	FORCE_INLINE uint16_t endianIn(uint16_t value)
+	{
+		if(littleEndian)
+			return GUINT16_TO_LE(value);
+		else
+			return GUINT16_TO_BE(value);
+	}
+	FORCE_INLINE uint32_t endianIn(uint32_t value)
+	{
+		if(littleEndian)
+			return GUINT32_TO_LE(value);
+		else
+			return GUINT32_TO_BE(value);
+	}
+	FORCE_INLINE uint64_t endianIn(uint64_t value)
+	{
+		if(littleEndian)
+			return GUINT64_TO_LE(value);
+		else
+			return GUINT64_TO_BE(value);
+	}
 
-	uint16_t endianOut(uint16_t value);
-	uint32_t endianOut(uint32_t value);
-	uint64_t endianOut(uint64_t value);
+	FORCE_INLINE uint16_t endianOut(uint16_t value)
+	{
+		if(littleEndian)
+			return GUINT16_FROM_LE(value);
+		else
+			return GUINT16_FROM_BE(value);
+	}
+	FORCE_INLINE uint32_t endianOut(uint32_t value)
+	{
+		if(littleEndian)
+			return GUINT32_FROM_LE(value);
+		else
+			return GUINT32_FROM_BE(value);
+	}
+	FORCE_INLINE uint64_t endianOut(uint64_t value)
+	{
+		if(littleEndian)
+			return GUINT64_FROM_LE(value);
+		else
+			return GUINT64_FROM_BE(value);
+	}
 
 	static void sinit(Class_base* c);
-	static void buildTraits(ASObject* o);
-	GET_VARIABLE_RESULT getVariableByMultiname(asAtom &ret, const multiname& name,GET_VARIABLE_OPTION opt=NONE) override;
-	GET_VARIABLE_RESULT getVariableByInteger(asAtom& ret, int index, GET_VARIABLE_OPTION opt) override;
-	int32_t getVariableByMultiname_i(const multiname& name) override;
-	multiname* setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool *alreadyset=nullptr) override;
-	void setVariableByInteger(int index, asAtom& o, CONST_ALLOWED_FLAG allowConst) override;
-	void setVariableByMultiname_i(multiname& name, int32_t value) override;
-	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic, bool considerPrototype) override;
+	GET_VARIABLE_RESULT getVariableByMultiname(asAtom &ret, const multiname& name, GET_VARIABLE_OPTION opt, ASWorker* wrk) override;
+	GET_VARIABLE_RESULT getVariableByInteger(asAtom& ret, int index, GET_VARIABLE_OPTION opt, ASWorker* wrk) override;
+	int32_t getVariableByMultiname_i(const multiname& name, ASWorker* wrk) override;
+	multiname* setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool *alreadyset, ASWorker* wrk) override;
+	void setVariableByInteger(int index, asAtom& o, CONST_ALLOWED_FLAG allowConst,bool *alreadyset,ASWorker* wrk) override;
+	void setVariableByMultiname_i(multiname& name, int32_t value,ASWorker* wrk) override;
+	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic, bool considerPrototype, ASWorker* wrk) override;
 
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
-				std::map<const Class_base*, uint32_t>& traitsMap) override;
+				std::map<const Class_base*, uint32_t>& traitsMap, ASWorker* wrk) override;
 };
 
 }
