@@ -31,48 +31,39 @@ class Dictionary: public ASObject
 {
 friend class ABCVm;
 private:
-	typedef std::map<_R<ASObject>,asAtom,std::less<_R<ASObject>>,
-	       reporter_allocator<std::pair<const _R<ASObject>, asAtom>>> dictType;
+	typedef std::map<ASObject*, asAtom> dictType;
 	dictType data;
 	dictType::iterator findKey(ASObject *);
 	bool weakkeys;
 public:
-	Dictionary(Class_base* c);
-	bool destruct() override
-	{
-		Dictionary::dictType::iterator it = data.begin();
-		while(it!=data.end())
-		{
-			ASATOM_DECREF(it->second);
-			it++;
-		}
-		data.clear();
-		return destructIntern();
-	}
-	
+	Dictionary(ASWorker* wrk,Class_base* c);
+	void finalize() override;
+	bool destruct() override;
+	void prepareShutdown() override;
+
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o);
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(_toJSON);
 
-	GET_VARIABLE_RESULT getVariableByMultiname(asAtom& ret, const multiname& name,GET_VARIABLE_OPTION opt=NONE) override;
-	int32_t getVariableByMultiname_i(const multiname& name) override
+	GET_VARIABLE_RESULT getVariableByMultiname(asAtom& ret, const multiname& name, GET_VARIABLE_OPTION opt, ASWorker* wrk) override;
+	int32_t getVariableByMultiname_i(const multiname& name, ASWorker* wrk) override
 	{
 		assert_and_throw(implEnable);
 		throw UnsupportedException("getVariableByMultiName_i not supported for Dictionary");
 	}
-	multiname* setVariableByMultiname(multiname& name, asAtom &o, CONST_ALLOWED_FLAG allowConst, bool* alreadyset=nullptr) override;
-	void setVariableByMultiname_i(multiname& name, int32_t value) override;
-	bool deleteVariableByMultiname(const multiname& name) override;
-	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic, bool considerPrototype) override;
+	multiname* setVariableByMultiname(multiname& name, asAtom &o, CONST_ALLOWED_FLAG allowConst, bool* alreadyset,ASWorker* wrk) override;
+	void setVariableByMultiname_i(multiname& name, int32_t value, ASWorker* wrk) override;
+	bool deleteVariableByMultiname(const multiname& name, ASWorker* wrk) override;
+	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic, bool considerPrototype, ASWorker* wrk) override;
 	tiny_string toString();
 	uint32_t nextNameIndex(uint32_t cur_index) override;
 	void nextName(asAtom &ret, uint32_t index) override;
 	void nextValue(asAtom &ret, uint32_t index) override;
+	bool countCylicMemberReferences(lightspark::garbagecollectorstate& gcstate) override;
 
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
-				std::map<const Class_base*, uint32_t>& traitsMap) override;
+				std::map<const Class_base*, uint32_t>& traitsMap, ASWorker* wrk) override;
 };
 
 }
