@@ -17,9 +17,11 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#include <pcre.h>
+#include "3rdparty/avmplus/pcre/pcre.h"
 
 #include "scripting/toplevel/ASString.h"
+#include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/Integer.h"
 #include "scripting/flash/utils/ByteArray.h"
 #include "compat.h"
 #include "scripting/argconv.h"
@@ -29,29 +31,28 @@
 using namespace std;
 using namespace lightspark;
 
-ASString::ASString(Class_base* c):ASObject(c,T_STRING),currentindex(0),hasId(true),datafilled(true)
+ASString::ASString(ASWorker* wrk,Class_base* c):ASObject(wrk,c,T_STRING),hasId(true),datafilled(true)
 {
 	stringId = BUILTIN_STRINGS::EMPTY;
 }
 
-ASString::ASString(Class_base* c,const string& s) : ASObject(c,T_STRING),data(s),currentindex(0),hasId(false),datafilled(true)
+ASString::ASString(ASWorker* wrk,Class_base* c,const string& s) : ASObject(wrk,c,T_STRING),data(s),hasId(false),datafilled(true)
 {
 }
 
-ASString::ASString(Class_base* c,const tiny_string& s) : ASObject(c,T_STRING),data(s),currentindex(0),hasId(false),datafilled(true)
+ASString::ASString(ASWorker* wrk,Class_base* c,const tiny_string& s) : ASObject(wrk,c,T_STRING),data(s),hasId(false),datafilled(true)
 {
 }
 
-ASString::ASString(Class_base* c,const char* s) : ASObject(c,T_STRING),data(s, /*copy:*/true),currentindex(0),hasId(false),datafilled(true)
+ASString::ASString(ASWorker* wrk,Class_base* c,const char* s) : ASObject(wrk,c,T_STRING),data(s, /*copy:*/true),hasId(false),datafilled(true)
 {
 }
 
-ASString::ASString(Class_base* c,const char* s, uint32_t len) : ASObject(c,T_STRING)
+ASString::ASString(ASWorker* wrk,Class_base* c,const char* s, uint32_t len) : ASObject(wrk,c,T_STRING)
 {
 	data = std::string(s,len);
 	hasId = false;
 	datafilled=true;
-	currentindex=0;
 }
 
 ASFUNCTIONBODY_ATOM(ASString,_constructor)
@@ -59,7 +60,7 @@ ASFUNCTIONBODY_ATOM(ASString,_constructor)
 	ASString* th=asAtomHandler::as<ASString>(obj);
 	if(args && argslen==1)
 	{
-		th->data=asAtomHandler::toString(args[0],sys);
+		th->data=asAtomHandler::toString(args[0],wrk);
 		th->hasId = false;
 		th->stringId = UINT32_MAX;
 		th->datafilled = true;
@@ -71,16 +72,16 @@ ASFUNCTIONBODY_ATOM(ASString,_getLength)
 	// fast path if obj is ASString
 	if (asAtomHandler::isStringID(obj))
 	{
-		asAtomHandler::setInt(ret,sys,int32_t(sys->getStringFromUniqueId(asAtomHandler::toStringId(obj,sys)).numChars()));
+		asAtomHandler::setInt(ret,wrk,int32_t(wrk->getSystemState()->getStringFromUniqueId(asAtomHandler::toStringId(obj,wrk)).numChars()));
 	}
 	else if (asAtomHandler::isString(obj))
 	{
 		ASString* th = asAtomHandler::getObjectNoCheck(obj)->as<ASString>();
-		asAtomHandler::setInt(ret,sys,int32_t(th->getData().numChars()));
+		asAtomHandler::setInt(ret,wrk,int32_t(th->getData().numChars()));
 	}
 	else
 	{
-		asAtomHandler::setInt(ret,sys,int32_t(tiny_string(asAtomHandler::toString(obj,sys)).numChars()));
+		asAtomHandler::setInt(ret,wrk,int32_t(tiny_string(asAtomHandler::toString(obj,wrk)).numChars()));
 	}
 }
 
@@ -135,21 +136,17 @@ void ASString::sinit(Class_base* c)
 	c->prototype->setVariableByQName("localeCompare","",Class<IFunction>::getFunction(c->getSystemState(),localeCompare_prototype,1,Class<Integer>::getRef(c->getSystemState()).getPtr()),CONSTANT_TRAIT);
 }
 
-void ASString::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(ASString,search)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
+	tiny_string data = asAtomHandler::toString(obj,wrk);
 	int res = -1;
 	if(argslen == 0 || asAtomHandler::isUndefined(args[0]))
 	{
-		asAtomHandler::setInt(ret,sys,res);
+		asAtomHandler::setInt(ret,wrk,res);
 		return;
 	}
 
-	int options=PCRE_UTF8|PCRE_NEWLINE_ANY;//|PCRE_JAVASCRIPT_COMPAT;
+	int options=PCRE_UTF8|PCRE_NEWLINE_ANY|PCRE_NO_UTF8_CHECK;//|PCRE_JAVASCRIPT_COMPAT;
 	tiny_string restr;
 	if(asAtomHandler::is<RegExp>(args[0]))
 	{
@@ -166,15 +163,16 @@ ASFUNCTIONBODY_ATOM(ASString,search)
 	}
 	else
 	{
-		restr = asAtomHandler::toString(args[0],sys);
+		restr = asAtomHandler::toString(args[0],wrk);
 	}
 
 	const char* error;
 	int errorOffset;
-	pcre* pcreRE=pcre_compile(restr.raw_buf(), options, &error, &errorOffset,NULL);
+	pcre* pcreRE=pcre_compile(restr.raw_buf(), options, &error, &errorOffset,nullptr);
 	if(error)
 	{
-		asAtomHandler::setInt(ret,sys,res);
+		pcre_free(pcreRE);
+		asAtomHandler::setInt(ret,wrk,res);
 		return;
 	}
 	int capturingGroups;
@@ -182,33 +180,34 @@ ASFUNCTIONBODY_ATOM(ASString,search)
 	if(infoOk!=0)
 	{
 		pcre_free(pcreRE);
-		asAtomHandler::setInt(ret,sys,res);
+		asAtomHandler::setInt(ret,wrk,res);
 		return;
 	}
 	pcre_extra extra;
-	extra.match_limit_recursion=200;
+	extra.match_limit_recursion=500;
 	extra.flags = PCRE_EXTRA_MATCH_LIMIT_RECURSION;
 	int ovector[(capturingGroups+1)*3];
 	int offset=0;
 	//Global is not used in search
-	int rc=pcre_exec(pcreRE, &extra, data.raw_buf(), data.numBytes(), offset, 0, ovector, (capturingGroups+1)*3);
+	int rc=pcre_exec(pcreRE, &extra, data.raw_buf(), data.numBytes(), offset, PCRE_NO_UTF8_CHECK, ovector, (capturingGroups+1)*3);
 	if(rc<0)
 	{
 		//No matches or error
 		pcre_free(pcreRE);
-		asAtomHandler::setInt(ret,sys,res);
+		asAtomHandler::setInt(ret,wrk,res);
 		return;
 	}
+	pcre_free(pcreRE);
 	res=ovector[0];
 	// pcre_exec returns byte position, so we have to convert it to character position 
 	tiny_string tmp = data.substr_bytes(0, res);
 	res = tmp.numChars();
-	asAtomHandler::setInt(ret,sys,res);
+	asAtomHandler::setInt(ret,wrk,res);
 }
 
 ASFUNCTIONBODY_ATOM(ASString,match)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
+	tiny_string data = asAtomHandler::toString(obj,wrk);
 	if(argslen == 0 || asAtomHandler::isNull(args[0]) || asAtomHandler::isUndefined(args[0]))
 	{
 		asAtomHandler::setNull(ret);
@@ -224,12 +223,12 @@ ASFUNCTIONBODY_ATOM(ASString,match)
 	}
 	else
 	{
-		re = Class<RegExp>::getInstanceS(sys,asAtomHandler::toString(args[0],sys));
+		re = Class<RegExp>::getInstanceS(wrk,asAtomHandler::toString(args[0],wrk));
 	}
 
 	if (re->global)
 	{
-		Array *resarr = Class<Array>::getInstanceSNoArgs(sys);
+		Array *resarr = Class<Array>::getInstanceSNoArgs(wrk);
 		int prevLastIndex = 0;
 		re->lastIndex = 0;
 
@@ -251,7 +250,9 @@ ASFUNCTIONBODY_ATOM(ASString,match)
 			prevLastIndex = re->lastIndex;
 
 			assert(match->is<Array>());
-			resarr->push(match->as<Array>()->at(0));
+			asAtom a = match->as<Array>()->at(0);
+			ASATOM_INCREF(a);
+			resarr->push(a);
 		}
 
 		// According to ECMA we should return Null if resarr
@@ -271,7 +272,7 @@ ASFUNCTIONBODY_ATOM(ASString,match)
 
 ASFUNCTIONBODY_ATOM(ASString,_toString)
 {
-	if(Class<ASString>::getClass(sys)->prototype->getObj() == asAtomHandler::getObject(obj))
+	if(Class<ASString>::getClass(wrk->getSystemState())->prototype->getObj() == asAtomHandler::getObject(obj))
 	{
 		ret = asAtomHandler::fromStringID(BUILTIN_STRINGS::EMPTY);
 		return;
@@ -279,7 +280,8 @@ ASFUNCTIONBODY_ATOM(ASString,_toString)
 	if(!asAtomHandler::is<ASString>(obj))
 	{
 		LOG(LOG_ERROR,"String.toString is not generic:"<<asAtomHandler::toDebugString(obj));
-		throw Class<TypeError>::getInstanceS(sys,"String.toString is not generic");
+		createError<TypeError>(wrk,0,"String.toString is not generic");
+		return;
 	}
 	assert_and_throw(argslen==0);
 
@@ -290,12 +292,13 @@ ASFUNCTIONBODY_ATOM(ASString,_toString)
 
 ASFUNCTIONBODY_ATOM(ASString,split)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
-	Array* res=Class<Array>::getInstanceSNoArgs(sys);
+	tiny_string data;
+	asAtomHandler::getStringView(data,obj,wrk);
+	Array* res=Class<Array>::getInstanceSNoArgs(wrk);
 	uint32_t limit = 0x7fffffff;
-	if(argslen == 0 )
+	if(argslen == 0)
 	{
-		res->push(asAtomHandler::fromObject(abstract_s(sys,data)));
+		res->push(asAtomHandler::fromObject(abstract_s(wrk,data)));
 		ret = asAtomHandler::fromObject(res);
 		return;
 	}
@@ -317,7 +320,7 @@ ASFUNCTIONBODY_ATOM(ASString,split)
 			{
 				if (res->size() >= limit)
 					break;
-				res->push(asAtomHandler::fromObject(abstract_s(sys, tiny_string::fromChar(*i) ) ));
+				res->push(asAtomHandler::fromObject(abstract_s(wrk, tiny_string::fromChar(*i) ) ));
 			}
 			ret = asAtomHandler::fromObject(res);
 			return;
@@ -347,7 +350,7 @@ ASFUNCTIONBODY_ATOM(ASString,split)
 		do
 		{
 			//offset is a byte offset that must point to the beginning of an utf8 character
-			int rc=pcre_exec(pcreRE, &extra, data.raw_buf(), data.numBytes(), offset, 0, ovector, (capturingGroups+1)*3);
+			int rc=pcre_exec(pcreRE, &extra, data.raw_buf(), data.numBytes(), offset, PCRE_NO_UTF8_CHECK, ovector, (capturingGroups+1)*3);
 			end=ovector[0];
 			if(rc<0)
 				break;
@@ -359,7 +362,7 @@ ASFUNCTIONBODY_ATOM(ASString,split)
 			if (res->size() >= limit)
 				break;
 			//Extract string from last match until the beginning of the current match
-			ASObject* s=abstract_s(sys,data.substr_bytes(lastMatch,end-lastMatch));
+			ASObject* s=abstract_s(wrk,data.substr_bytes(lastMatch,end-lastMatch));
 			res->push(asAtomHandler::fromObject(s));
 			lastMatch=offset=ovector[1];
 
@@ -369,21 +372,22 @@ ASFUNCTIONBODY_ATOM(ASString,split)
 				if (res->size() >= limit)
 					break;
 				//use string interface through raw(), because we index on bytes, not on UTF-8 characters
-				ASObject* s=abstract_s(sys,data.substr_bytes(ovector[i*2],ovector[i*2+1]-ovector[i*2]));
+				ASObject* s=abstract_s(wrk,data.substr_bytes(ovector[i*2],ovector[i*2+1]-ovector[i*2]));
 				res->push(asAtomHandler::fromObject(s));
 			}
 		}
 		while(end<data.numBytes() && res->size() < limit);
 		if(res->size() < limit && lastMatch != data.numBytes()+1)
 		{
-			ASObject* s=abstract_s(sys,data.substr_bytes(lastMatch,data.numBytes()-lastMatch));
+			ASObject* s=abstract_s(wrk,data.substr_bytes(lastMatch,data.numBytes()-lastMatch));
 			res->push(asAtomHandler::fromObject(s));
 		}
 		pcre_free(pcreRE);
 	}
 	else
 	{
-		const tiny_string& del=asAtomHandler::toString(args[0],sys);
+		tiny_string del;
+		asAtomHandler::getStringView(del,args[0],wrk);
 		if(del.empty())
 		{
 			//the string is empty, so split every character
@@ -398,7 +402,7 @@ ASFUNCTIONBODY_ATOM(ASString,split)
 				if (j >= limit)
 					break;
 				j++;
-				res->push(asAtomHandler::fromObject(abstract_s(sys, tiny_string::fromChar(*i) ) ));
+				res->push(asAtomHandler::fromObject(abstract_s(wrk, tiny_string::fromChar(*i) ) ));
 			}
 			ret = asAtomHandler::fromObject(res);
 			return;
@@ -412,9 +416,13 @@ ASFUNCTIONBODY_ATOM(ASString,split)
 				match++;
 			if(match==-1)
 				match= len;
-			ASObject* s=abstract_s(sys,data.substr(start,(match-start)));
 			if (res->size() >= limit)
 				break;
+			ASObject* s;
+			if (data.isSinglebyte()) // fast path for ascii strings to avoid unneccessary buffer copying
+				s=abstract_s(wrk,data.raw_buf()+start,match-start,match-start,data.isSinglebyte(),data.hasNullEntries());
+			else
+				s=abstract_s(wrk,data.substr(start,(match-start)));
 			res->push(asAtomHandler::fromObject(s));
 			start=match+del.numChars();
 			if (start == len)
@@ -422,13 +430,11 @@ ASFUNCTIONBODY_ATOM(ASString,split)
 		}
 		while(start<len && res->size() < limit);
 	}
-
 	ret = asAtomHandler::fromObject(res);
 }
 
 ASFUNCTIONBODY_ATOM(ASString,substr)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
 	int start=0;
 	if(argslen>=1)
 	{
@@ -440,14 +446,20 @@ ASFUNCTIONBODY_ATOM(ASString,substr)
 			return;
 		}
 	}
+	int numchars=0;
+	if (asAtomHandler::isStringID(obj))
+		numchars = wrk->getSystemState()->getStringFromUniqueId(asAtomHandler::getStringId(obj)).numChars();
+	else if (asAtomHandler::is<ASString>(obj))
+		numchars = asAtomHandler::as<ASString>(obj)->getData().numChars();
+	else
+		numchars = asAtomHandler::toString(obj,wrk).numChars();
 	if(start<0) {
-		start=data.numChars()+start;
+		start=numchars+start;
 		if(start<0)
 			start=0;
 	}
-	if(start>(int)data.numChars())
-		start=data.numChars();
-
+	if(start>numchars)
+		start=numchars;
 	int len=0x7fffffff;
 	if (argslen==2 && !asAtomHandler::is<Undefined>(args[1]))
 	{
@@ -458,16 +470,31 @@ ASFUNCTIONBODY_ATOM(ASString,substr)
 		}
 		else
 			len=asAtomHandler::toInt(args[1]);
+		if (len<0)
+		{
+			len+=numchars;
+			if(len<0)
+				len=0;
+		}
 	}
-	ret = asAtomHandler::fromObject(abstract_s(sys,data.substr(start,len)));
+
+	if (asAtomHandler::isStringID(obj))
+		ret = asAtomHandler::fromObject(abstract_s(wrk,wrk->getSystemState()->getStringFromUniqueId(asAtomHandler::getStringId(obj)).substr(start,len)));
+	else if (asAtomHandler::is<ASString>(obj))
+	{
+		ASString* th = asAtomHandler::as<ASString>(obj);
+		ret = asAtomHandler::fromObject(abstract_s(wrk,th->data.substr_bytes(th->getBytePosition(start),start+len >= numchars ? UINT32_MAX  : (th->getBytePosition(start+len)-th->getBytePosition(start)))));
+	}
+	else
+		ret = asAtomHandler::fromObject(abstract_s(wrk,asAtomHandler::toString(obj,wrk).substr(start,len)));
 }
 
 ASFUNCTIONBODY_ATOM(ASString,substring)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
+	tiny_string data = asAtomHandler::toString(obj,wrk);
 
 	number_t start, end;
-	ARG_UNPACK_ATOM (start,0) (end,0x7fffffff);
+	ARG_CHECK(ARG_UNPACK (start,0) (end,0x7fffffff));
 	if(start<0 || std::isnan(start))
 		start=0;
 	if(start>(int)data.numChars() || std::isinf(start))
@@ -484,7 +511,7 @@ ASFUNCTIONBODY_ATOM(ASString,substring)
 		end=tmp;
 	}
 
-	ret = asAtomHandler::fromObject(abstract_s(sys,data.substr(start,end-start)));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,data.substr(start,end-start)));
 }
 
 number_t ASString::toNumber()
@@ -619,14 +646,22 @@ TRISTATE ASString::isLess(ASObject* r)
 	//ECMA-262 11.8.5 algorithm
 	assert_and_throw(implEnable);
 	asAtom rprim=asAtomHandler::invalidAtom;
-	r->toPrimitive(rprim);
+	number_t a=0;
+	number_t b=0;
+	bool isrefcounted;
+	r->toPrimitive(rprim,isrefcounted);
 	if(getObjectType()==T_STRING && asAtomHandler::isString(rprim))
 	{
-		ASString* rstr=static_cast<ASString*>(asAtomHandler::toObject(rprim,getSystemState()));
-		return (getData()<rstr->getData())?TTRUE:TFALSE;
+		ASString* rstr=static_cast<ASString*>(asAtomHandler::toObject(rprim,getInstanceWorker()));
+		TRISTATE res = (getData()<rstr->getData())?TTRUE:TFALSE;
+		if (isrefcounted)
+			ASATOM_DECREF(rprim);
+		return res;
 	}
-	number_t a=toNumber();
-	number_t b=asAtomHandler::toNumber(rprim);
+	a=toNumber();
+	b=asAtomHandler::toNumber(rprim);
+	if (isrefcounted)
+		ASATOM_DECREF(rprim);
 	if(std::isnan(a) || std::isnan(b))
 		return TUNDEFINED;
 	return (a<b)?TTRUE:TFALSE;
@@ -636,16 +671,19 @@ TRISTATE ASString::isLessAtom(asAtom& r)
 {
 	//ECMA-262 11.8.5 algorithm
 	assert_and_throw(implEnable);
+	bool isrefcounted=false;
 	asAtom rprim = r;
 	if (asAtomHandler::getObject(r))
-		asAtomHandler::getObject(r)->toPrimitive(rprim);
+		asAtomHandler::getObject(r)->toPrimitive(rprim,isrefcounted);
 	if(getObjectType()==T_STRING && asAtomHandler::isString(rprim))
 	{
-		ASString* rstr=static_cast<ASString*>(asAtomHandler::toObject(rprim,getSystemState()));
+		ASString* rstr=static_cast<ASString*>(asAtomHandler::toObject(rprim,getInstanceWorker()));
 		return (getData()<rstr->getData())?TTRUE:TFALSE;
 	}
 	number_t a=toNumber();
 	number_t b=asAtomHandler::toNumber(rprim);
+	if (isrefcounted)
+		ASATOM_DECREF(rprim);
 	if(std::isnan(a) || std::isnan(b))
 		return TUNDEFINED;
 	return (a<b)?TTRUE:TFALSE;
@@ -653,9 +691,9 @@ TRISTATE ASString::isLessAtom(asAtom& r)
 
 void ASString::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
-				std::map<const Class_base*, uint32_t>& traitsMap)
+				std::map<const Class_base*, uint32_t>& traitsMap,ASWorker* wrk)
 {
-	if (out->getObjectEncoding() == ObjectEncoding::AMF0)
+	if (out->getObjectEncoding() == OBJECT_ENCODING::AMF0)
 	{
 		out->writeByte(amf0_string_marker);
 		out->writeStringAMF0(getData());
@@ -667,9 +705,25 @@ void ASString::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& string
 	}
 }
 
+string ASString::toDebugString() const
+{
+	tiny_string ret;
+	if (!datafilled && hasId)
+		ret = std::string("\"") + std::string(getSystemState()->getStringFromUniqueId(stringId)) + "\"_id";
+	else
+		ret = std::string("\"") + std::string(data) + "\"";
+#ifndef _NDEBUG
+	char buf[300];
+	sprintf(buf,"(%p/%d/%d/%d%s) ",this,this->getRefCount(),this->storedmembercount,this->getConstant(),this->isConstructed()?"":" not constructed");
+	ret += buf;
+#endif
+	return ret;
+}
+
 ASFUNCTIONBODY_ATOM(ASString,slice)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
+	tiny_string data;
+	asAtomHandler::getStringView(data,obj,wrk);
 	int startIndex=0;
 	if(argslen>=1)
 		startIndex=asAtomHandler::toInt(args[0]);
@@ -679,7 +733,7 @@ ASFUNCTIONBODY_ATOM(ASString,slice)
 			startIndex=0;
 	}
 	if(startIndex>(int)data.numChars())
-		startIndex=data.numChars();
+		startIndex=(int)data.numChars();
 
 	int endIndex=0x7fffffff;
 	if(argslen>=2)
@@ -690,20 +744,25 @@ ASFUNCTIONBODY_ATOM(ASString,slice)
 			endIndex=0;
 	}
 	if(endIndex>(int)data.numChars())
-		endIndex=data.numChars();
+		endIndex=(int)data.numChars();
 	if(endIndex<=startIndex)
 		ret = asAtomHandler::fromStringID(BUILTIN_STRINGS::EMPTY);
 	else
-		ret = asAtomHandler::fromObject(abstract_s(sys,data.substr(startIndex,endIndex-startIndex)));
+	{
+		if (data.isSinglebyte()) // fast path for ascii strings to avoid unneccessary buffer copying
+			ret = asAtomHandler::fromObject(abstract_s(wrk,data.raw_buf()+startIndex,endIndex-startIndex,endIndex-startIndex,data.isSinglebyte(),data.hasNullEntries()));
+		else
+			ret = asAtomHandler::fromObject(abstract_s(wrk,data.substr(startIndex,endIndex-startIndex)));
+	}
 }
 ASFUNCTIONBODY_ATOM(ASString,charAt)
 {
 	number_t index;
-	ARG_UNPACK_ATOM (index, 0);
+	ARG_CHECK(ARG_UNPACK (index, 0));
 	// fast path if obj is ASString
 	if (asAtomHandler::isStringID(obj))
 	{
-		const tiny_string& s = sys->getStringFromUniqueId(asAtomHandler::toStringId(obj,sys));
+		const tiny_string& s = wrk->getSystemState()->getStringFromUniqueId(asAtomHandler::toStringId(obj,wrk));
 		int maxIndex=s.numChars();
 		if(index<0 || index>=maxIndex || std::isinf(index))
 		{
@@ -711,7 +770,7 @@ ASFUNCTIONBODY_ATOM(ASString,charAt)
 			return;
 		}
 		uint32_t c = s.charAt(index);
-		ret = c < BUILTIN_STRINGS_CHAR_MAX ? asAtomHandler::fromStringID(c) : asAtomHandler::fromObject(abstract_s(sys, tiny_string::fromChar(c) ));
+		ret = c < BUILTIN_STRINGS_CHAR_MAX ? asAtomHandler::fromStringID(c) : asAtomHandler::fromObject(abstract_s(wrk, tiny_string::fromChar(c) ));
 		return;
 	}
 	else if (asAtomHandler::isString(obj))
@@ -724,36 +783,18 @@ ASFUNCTIONBODY_ATOM(ASString,charAt)
 			ret = asAtomHandler::fromStringID(BUILTIN_STRINGS::EMPTY);
 			return;
 		}
-		uint32_t c;
-		if (th->getData().isSinglebyte())
-			c = th->getData().charAt(index);
-		else
-		{
-			if (index == 0)
-			{
-				th->currentpos = th->getData().begin();
-				th->currentindex = 0;
-				c = *(th->currentpos);
-			}
-			else if (th->currentpos.isValid() && th->currentindex == index-1)
-			{
-				th->currentindex++;
-				c = *(++th->currentpos);
-			}
-			else
-				c = th->getData().charAt(index);
-		}
-		ret = c < BUILTIN_STRINGS_CHAR_MAX ? asAtomHandler::fromStringID(c) : asAtomHandler::fromObject(abstract_s(sys, tiny_string::fromChar(c) ));
+		uint32_t c = g_utf8_get_char(th->getData().raw_buf()+th->getBytePosition(index));
+		ret = c < BUILTIN_STRINGS_CHAR_MAX ? asAtomHandler::fromStringID(c) : asAtomHandler::fromObject(abstract_s(wrk, tiny_string::fromChar(c) ));
 		return;
 	}
-	tiny_string data = asAtomHandler::toString(obj,sys);
+	tiny_string data = asAtomHandler::toString(obj,wrk);
 	int maxIndex=data.numChars();
 	if(index<0 || index>=maxIndex || std::isinf(index))
 		ret = asAtomHandler::fromStringID(BUILTIN_STRINGS::EMPTY);
 	else
 	{
 		uint32_t c = data.charAt(index);
-		ret = c < BUILTIN_STRINGS_CHAR_MAX ? asAtomHandler::fromStringID(c) : asAtomHandler::fromObject(abstract_s(sys, tiny_string::fromChar(c) ));
+		ret = c < BUILTIN_STRINGS_CHAR_MAX ? asAtomHandler::fromStringID(c) : asAtomHandler::fromObject(abstract_s(wrk, tiny_string::fromChar(c) ));
 	}
 }
 
@@ -761,55 +802,37 @@ ASFUNCTIONBODY_ATOM(ASString,charCodeAt)
 {
 	int64_t index;
 	
-	ARG_UNPACK_ATOM (index, 0);
+	ARG_CHECK(ARG_UNPACK (index, 0));
 
 	// fast path if obj is ASString
 	if (asAtomHandler::isStringID(obj))
 	{
-		tiny_string s = sys->getStringFromUniqueId(asAtomHandler::getStringId(obj));
+		tiny_string s = wrk->getSystemState()->getStringFromUniqueId(asAtomHandler::getStringId(obj));
 		if(index<0 || index>=(int64_t)s.numChars())
-			asAtomHandler::setNumber(ret,sys,Number::NaN);
+			asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		else
-			asAtomHandler::setInt(ret,sys,(int32_t)s.charAt(index));
+			asAtomHandler::setInt(ret,wrk,(int32_t)s.charAt(index));
 	}
 	else if (asAtomHandler::isString(obj) && asAtomHandler::getObject(obj))
 	{
 		ASString* th = asAtomHandler::as<ASString>(obj);
 		if(index<0 || index>=(int64_t)th->getData().numChars())
-			asAtomHandler::setNumber(ret,sys,Number::NaN);
+			asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		else
 		{
-			uint32_t c;
-			if (th->getData().isSinglebyte())
-				c = th->getData().charAt(index);
-			else
-			{
-				if (index == 0)
-				{
-					th->currentpos = th->getData().begin();
-					th->currentindex = 0;
-					c = *(th->currentpos);
-				}
-				else if (th->currentpos.isValid() && th->currentindex == index-1)
-				{
-					th->currentindex++;
-					c = *(++th->currentpos);
-				}
-				else
-					c = th->getData().charAt(index);
-			}
-			asAtomHandler::setInt(ret,sys,(int32_t)c);
+			uint32_t c = g_utf8_get_char(th->getData().raw_buf()+th->getBytePosition(index));
+			asAtomHandler::setInt(ret,wrk,(int32_t)c);
 		}
 	}
 	else
 	{
-		tiny_string data = asAtomHandler::toString(obj,sys);
+		tiny_string data = asAtomHandler::toString(obj,wrk);
 		if(index<0 || index>=(int64_t)data.numChars())
-			asAtomHandler::setNumber(ret,sys,Number::NaN);
+			asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		else
 		{
 			//Character codes are expected to be positive
-			asAtomHandler::setInt(ret,sys,(int32_t)data.charAt(index));
+			asAtomHandler::setInt(ret,wrk,(int32_t)data.charAt(index));
 		}
 	}
 }
@@ -818,35 +841,38 @@ ASFUNCTIONBODY_ATOM(ASString,indexOf)
 {
 	if (argslen == 0)
 	{
-		asAtomHandler::setInt(ret,sys,-1);
+		asAtomHandler::setInt(ret,wrk,-1);
 		return;
 	}
-	tiny_string data = asAtomHandler::toString(obj,sys);
-	tiny_string arg0=asAtomHandler::toString(args[0],sys);
+	tiny_string data;
+	asAtomHandler::getStringView(data,obj,wrk);
+	tiny_string arg0;
+	asAtomHandler::getStringView(arg0,args[0],wrk);
 	int startIndex=0;
 	if(argslen>1)
 		startIndex=asAtomHandler::toInt(args[1]);
 	startIndex = imin(imax(startIndex, 0), data.numChars());
 
-	size_t pos = data.find(arg0.raw_buf(), startIndex);
+	size_t pos = data.find(arg0, startIndex);
 	if(pos == data.npos)
-		asAtomHandler::setInt(ret,sys,-1);
+		asAtomHandler::setInt(ret,wrk,-1);
 	else
-		asAtomHandler::setInt(ret,sys,(int32_t)pos);
+		asAtomHandler::setInt(ret,wrk,(int32_t)pos);
 }
 
 ASFUNCTIONBODY_ATOM(ASString,lastIndexOf)
 {
 	assert_and_throw(argslen==1 || argslen==2);
-	tiny_string data = asAtomHandler::toString(obj,sys);
-	tiny_string val=asAtomHandler::toString(args[0],sys);
+	tiny_string data;
+	asAtomHandler::getStringView(data,obj,wrk);
+	tiny_string val=asAtomHandler::toString(args[0],wrk);
 	size_t startIndex=data.npos;
 	if(argslen > 1 && !asAtomHandler::isUndefined(args[1]) && !std::isnan(asAtomHandler::toNumber(args[1])) && !(asAtomHandler::toNumber(args[1]) > 0 && std::isinf(asAtomHandler::toNumber(args[1]))))
 	{
 		int32_t i = asAtomHandler::toInt(args[1]);
 		if(i<0)
 		{
-			asAtomHandler::setInt(ret,sys,-1);
+			asAtomHandler::setInt(ret,wrk,-1);
 			return;
 		}
 		startIndex = i;
@@ -856,64 +882,67 @@ ASFUNCTIONBODY_ATOM(ASString,lastIndexOf)
 
 	size_t pos=data.rfind(val.raw_buf(), startIndex);
 	if(pos==data.npos)
-		asAtomHandler::setInt(ret,sys,-1);
+		asAtomHandler::setInt(ret,wrk,-1);
 	else
-		asAtomHandler::setInt(ret,sys,(int32_t)pos);
+		asAtomHandler::setInt(ret,wrk,(int32_t)pos);
 }
 
 ASFUNCTIONBODY_ATOM(ASString,toLowerCase)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
-	if (asAtomHandler::is<ASString>(obj) && asAtomHandler::toStringId(obj,sys) != UINT32_MAX)
-		ret = asAtomHandler::fromStringID(sys->getUniqueStringId(data.lowercase()));
+	tiny_string data = asAtomHandler::toString(obj,wrk);
+	if (asAtomHandler::is<ASString>(obj) && asAtomHandler::toStringId(obj,wrk) != UINT32_MAX)
+		ret = asAtomHandler::fromStringID(wrk->getSystemState()->getUniqueStringId(data.lowercase()));
 	else
-		ret = asAtomHandler::fromObject(abstract_s(sys,data.lowercase()));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,data.lowercase()));
 }
 
 ASFUNCTIONBODY_ATOM(ASString,toUpperCase)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
-	if (asAtomHandler::is<ASString>(obj) && asAtomHandler::toStringId(obj,sys) != UINT32_MAX)
-		ret = asAtomHandler::fromStringID(sys->getUniqueStringId(data.uppercase()));
+	tiny_string data = asAtomHandler::toString(obj,wrk);
+	if (asAtomHandler::is<ASString>(obj) && asAtomHandler::toStringId(obj,wrk) != UINT32_MAX)
+		ret = asAtomHandler::fromStringID(wrk->getSystemState()->getUniqueStringId(data.uppercase()));
 	else
-		ret = asAtomHandler::fromObject(abstract_s(sys,data.uppercase()));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,data.uppercase()));
 }
 ASFUNCTIONBODY_ATOM(ASString,localeCompare)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
+	tiny_string data = asAtomHandler::toString(obj,wrk);
 	tiny_string other;
-	ARG_UNPACK_ATOM_MORE_ALLOWED(other);
+	ARG_CHECK(ARG_UNPACK_MORE_ALLOWED(other));
 	if (argslen > 1)
 		LOG(LOG_NOT_IMPLEMENTED,"localeCompare with more than one parameter not implemented");
-	if (sys->getSwfVersion() < 11)
+	if (wrk->getSystemState()->getSwfVersion() < 11)
 	{
 		if (asAtomHandler::is<Null>(args[0]) || asAtomHandler::is<Undefined>(args[0]))
 		{
-			asAtomHandler::setInt(ret,sys,data == "" ? 1 : 0);
+			asAtomHandler::setInt(ret,wrk,data == "" ? 1 : 0);
 			return;
 		}
 	}
 	int res = data.compare(other);
-	asAtomHandler::setInt(ret,sys,res);
+	asAtomHandler::setInt(ret,wrk,res);
 }
 ASFUNCTIONBODY_ATOM(ASString,localeCompare_prototype)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
+	tiny_string data = asAtomHandler::toString(obj,wrk);
 	tiny_string other;
-	ARG_UNPACK_ATOM_MORE_ALLOWED(other);
+	ARG_CHECK(ARG_UNPACK_MORE_ALLOWED(other));
 	if (argslen > 1)
-		throwError<ArgumentError>(kWrongArgumentCountError, "localeCompare", "1",Integer::toString(argslen));
+	{
+		createError<ArgumentError>(wrk,kWrongArgumentCountError, "localeCompare", "1",Integer::toString(argslen));
+		return;
+	}
 
-	if (sys->getSwfVersion() < 11)
+	if (wrk->getSystemState()->getSwfVersion() < 11)
 	{
 		if (asAtomHandler::is<Null>(args[0]) || asAtomHandler::is<Undefined>(args[0]))
 		{
-			asAtomHandler::setInt(ret,sys,data == "" ? 1 : 0);
+			asAtomHandler::setInt(ret,wrk,data == "" ? 1 : 0);
 			return;
 		}
 	}
 	int res = data.compare(other);
-	asAtomHandler::setInt(ret,sys,res);
+	asAtomHandler::setInt(ret,wrk,res);
 }
 
 ASFUNCTIONBODY_ATOM(ASString,fromCharCode)
@@ -923,7 +952,7 @@ ASFUNCTIONBODY_ATOM(ASString,fromCharCode)
 		ret = asAtomHandler::fromStringID(asAtomHandler::toUInt(args[0])& 0xFFFF);
 		return;
 	}
-	ASString* res=abstract_s(sys)->as<ASString>();
+	ASString* res=abstract_s(wrk)->as<ASString>();
 	for(uint32_t i=0;i<argslen;i++)
 	{
 		res->stringId = UINT32_MAX;
@@ -935,25 +964,24 @@ ASFUNCTIONBODY_ATOM(ASString,fromCharCode)
 
 ASFUNCTIONBODY_ATOM(ASString,replace)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
+	tiny_string data;
+	asAtomHandler::getStringView(data,obj,wrk);
 	enum REPLACE_TYPE { STRING=0, FUNC };
 	REPLACE_TYPE type;
-	ASString* res=abstract_s(sys,data)->as<ASString>();
-
+	ASString* res=abstract_s(wrk,data.raw_buf(),data.numBytes(), data.numChars(), data.isSinglebyte(), data.hasNullEntries())->as<ASString>();
 	tiny_string replaceWith;
 	if(argslen < 2)
 	{
 		type = STRING;
-		replaceWith="";
 	}
 	else if(!asAtomHandler::isFunction(args[1]))
 	{
 		type = STRING;
-		replaceWith=asAtomHandler::toString(args[1],sys);
+		asAtomHandler::getStringView(replaceWith,args[1],wrk);
 	}
 	else
 	{
-		replaceWith=asAtomHandler::toString(args[1],sys);
+		asAtomHandler::getStringView(replaceWith,args[1],wrk);
 		type = FUNC;
 	}
 	if(asAtomHandler::is<RegExp>(args[0]))
@@ -968,7 +996,7 @@ ASFUNCTIONBODY_ATOM(ASString,replace)
 		}
 
 		int capturingGroups;
-		int infoOk=pcre_fullinfo(pcreRE, NULL, PCRE_INFO_CAPTURECOUNT, &capturingGroups);
+		int infoOk=pcre_fullinfo(pcreRE, nullptr, PCRE_INFO_CAPTURECOUNT, &capturingGroups);
 		if(infoOk!=0)
 		{
 			pcre_free(pcreRE);
@@ -985,7 +1013,7 @@ ASFUNCTIONBODY_ATOM(ASString,replace)
 		do
 		{
 			tiny_string replaceWithTmp = replaceWith;
-			int rc=pcre_exec(pcreRE, &extra, res->getData().raw_buf(), res->getData().numBytes(), offset, 0, ovector, (capturingGroups+1)*3);
+			int rc=pcre_exec(pcreRE, &extra, res->getData().raw_buf(), res->getData().numBytes(), offset, PCRE_NO_UTF8_CHECK, ovector, (capturingGroups+1)*3);
 			if(rc<0)
 			{
 				//No matches or error
@@ -997,62 +1025,68 @@ ASFUNCTIONBODY_ATOM(ASString,replace)
 			if(type==FUNC)
 			{
 				//Get the replace for this match
-				asAtom* subargs = g_newa(asAtom, 3+capturingGroups);
+				vector<asAtom> subargs;
+				subargs.reserve(3+capturingGroups);
+
 				//we index on bytes, not on UTF-8 characters
-				subargs[0]=asAtomHandler::fromObject(abstract_s(sys,res->data.substr_bytes(ovector[0],ovector[1]-ovector[0])));
+				subargs.push_back(asAtomHandler::fromObject(abstract_s(wrk,res->data.substr_bytes(ovector[0],ovector[1]-ovector[0]))));
 				for(int i=0;i<capturingGroups;i++)
-					subargs[i+1]=asAtomHandler::fromObject(abstract_s(sys,res->data.substr_bytes(ovector[i*2+2],ovector[i*2+3]-ovector[i*2+2])));
-				subargs[capturingGroups+1]=asAtomHandler::fromInt((int32_t)(ovector[0]-retDiff));
-				
-				subargs[capturingGroups+2]=asAtomHandler::fromObject(abstract_s(sys,data));
+				{
+					if (ovector[i*2+2] >= 0)
+						subargs.push_back(asAtomHandler::fromObject(abstract_s(wrk,res->data.substr_bytes(ovector[i*2+2],ovector[i*2+3]-ovector[i*2+2]))));
+					else
+						subargs.push_back(asAtomHandler::undefinedAtom);
+				}
+				subargs.push_back(asAtomHandler::fromInt((int32_t)(ovector[0]-retDiff)));
+				subargs.push_back(asAtomHandler::fromObject(abstract_s(wrk,data)));
 				asAtom ret=asAtomHandler::invalidAtom;
 				asAtom obj = asAtomHandler::nullAtom;
 				if (asAtomHandler::as<IFunction>(args[1])->closure_this) // use closure as "this" in function call
-					obj = asAtomHandler::fromObject(asAtomHandler::as<IFunction>(args[1])->closure_this.getPtr());
-				asAtomHandler::callFunction(args[1],ret,obj, subargs, 3+capturingGroups,true);
-				replaceWithTmp=asAtomHandler::toString(ret,sys).raw_buf();
+					obj = asAtomHandler::fromObject(asAtomHandler::as<IFunction>(args[1])->closure_this);
+				asAtomHandler::callFunction(args[1],wrk,ret,obj, subargs.data(), subargs.size(),true);
+				replaceWithTmp=asAtomHandler::toString(ret,wrk).raw_buf();
 				ASATOM_DECREF(ret);
 			} else {
-					size_t pos, ipos = 0;
-					tiny_string group;
-					int i, j;
-					while((pos = replaceWithTmp.find("$", ipos)) != tiny_string::npos) {
-						i = 0;
-						ipos = pos;
-						/* docu is not clear what to do if the $nn value is higher 
+				size_t pos, ipos = 0;
+				tiny_string group;
+				int i, j;
+				while((pos = replaceWithTmp.find("$", ipos)) != tiny_string::npos) {
+					i = 0;
+					ipos = pos;
+					/* docu is not clear what to do if the $nn value is higher 
 						 * than the number of matching groups, 
 						 * but the ecma3/String/eregress_104375 test indicates
 						 * that we should take it as an $n value
 						 */
-						   
-						while (++ipos < replaceWithTmp.numChars() && i<10) {
+					
+					while (++ipos < replaceWithTmp.numChars() && i<10) {
 						j = replaceWithTmp.charAt(ipos)-'0';
-							if (j <0 || j> 9 || rc < 10*i + j)
-								break;
-							i = 10*i + j;
-						}
-						if (i == 0)
-						{
-							switch (replaceWithTmp.charAt(ipos))
-							{
-								case '$':
-									replaceWithTmp.replace(ipos,1,"");
-									break;
-								case '&':
-									replaceWithTmp.replace(ipos-1,2,res->getData().substr_bytes(ovector[0], ovector[1]-ovector[0]));
-									break;
-								case '`':
-									replaceWithTmp.replace(ipos-1,2,prevsubstring);
-									break;
-								case '\'':
-									replaceWithTmp.replace(ipos-1,2,res->getData().substr_bytes(ovector[1],res->getData().numBytes()-ovector[1]));
-									break;
-							}
-							continue;
-						}
-						group = (i >= rc) ? "" : res->getData().substr_bytes(ovector[i*2], ovector[i*2+1]-ovector[i*2]);
-						replaceWithTmp.replace(pos, ipos-pos, group);
+						if ((j <0) || (j> 9) || (rc < 10*i + j))
+							break;
+						i = 10*i + j;
 					}
+					if (i == 0)
+					{
+						switch (replaceWithTmp.charAt(ipos))
+						{
+							case '$':
+								replaceWithTmp.replace(ipos,1,"");
+								break;
+							case '&':
+								replaceWithTmp.replace(ipos-1,2,res->getData().substr_bytes(ovector[0], ovector[1]-ovector[0]));
+								break;
+							case '`':
+								replaceWithTmp.replace(ipos-1,2,prevsubstring);
+								break;
+							case '\'':
+								replaceWithTmp.replace(ipos-1,2,res->getData().substr_bytes(ovector[1],res->getData().numBytes()-ovector[1]));
+								break;
+						}
+						continue;
+					}
+					group = (i >= rc) ? "" : res->getData().substr_bytes(ovector[i*2], ovector[i*2+1]-ovector[i*2]);
+					replaceWithTmp.replace(pos, ipos-pos, group);
+				}
 			}
 			prevsubstring += res->getData().substr_bytes(ovector[0],ovector[1]-ovector[0]);
 			res->hasId = false;
@@ -1064,12 +1098,12 @@ ASFUNCTIONBODY_ATOM(ASString,replace)
 			retDiff+=replaceWithTmp.numBytes()-(ovector[1]-ovector[0]);
 		}
 		while(re->global);
-
 		pcre_free(pcreRE);
 	}
 	else
 	{
-		const tiny_string& s=asAtomHandler::toString(args[0],sys);
+		tiny_string s;
+		asAtomHandler::getStringView(s,args[0],wrk);
 		int index=res->getData().find(s,0);
 		if(index==-1) //No result
 		{
@@ -1079,18 +1113,17 @@ ASFUNCTIONBODY_ATOM(ASString,replace)
 		res->hasId = false;
 		res->getData().replace(index,s.numChars(),replaceWith);
 	}
-
 	ret = asAtomHandler::fromObject(res);
 }
 
 ASFUNCTIONBODY_ATOM(ASString,concat)
 {
-	tiny_string data = asAtomHandler::toString(obj,sys);
-	ASString* res=abstract_s(sys,data)->as<ASString>();
+	tiny_string data = asAtomHandler::toString(obj,wrk);
+	ASString* res=abstract_s(wrk,data)->as<ASString>();
 	for(unsigned int i=0;i<argslen;i++)
 	{
 		res->hasId = false;
-		res->getData()+=asAtomHandler::toString(args[i],sys).raw_buf();
+		res->getData()+=asAtomHandler::toString(args[i],wrk).raw_buf();
 	}
 
 	ret = asAtomHandler::fromObject(res);
@@ -1102,7 +1135,7 @@ ASFUNCTIONBODY_ATOM(ASString,generator)
 	if (argslen == 0)
 		ret = asAtomHandler::fromStringID(BUILTIN_STRINGS::EMPTY);
 	else
-		ret = asAtomHandler::fromObject(abstract_s(sys,asAtomHandler::toString(args[0],sys)));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,asAtomHandler::toString(args[0],wrk)));
 }
 
 bool ASString::isEcmaSpace(uint32_t c)
