@@ -19,6 +19,7 @@
 
 #include "parsing/amf3_generator.h"
 #include "scripting/toplevel/Date.h"
+#include "scripting/toplevel/Number.h"
 #include "scripting/class.h"
 #include "scripting/argconv.h"
 #include "scripting/flash/utils/ByteArray.h"
@@ -27,7 +28,7 @@
 using namespace std;
 using namespace lightspark;
 
-Date::Date(Class_base* c):ASObject(c,T_OBJECT,SUBTYPE_DATE),extrayears(0), nan(false), datetime(NULL),datetimeUTC(NULL)
+Date::Date(ASWorker* wrk, Class_base* c):ASObject(wrk,c,T_OBJECT,SUBTYPE_DATE),extrayears(0), nan(false), datetime(nullptr),datetimeUTC(nullptr)
 {
 }
 
@@ -186,7 +187,7 @@ ASFUNCTIONBODY_ATOM(Date,_constructor)
 	{
 		number_t nm = Number::NaN;
 		if (asAtomHandler::isString(args[0]))
-			nm = parse(asAtomHandler::toString(args[0],sys));
+			nm = parse(asAtomHandler::toString(args[0],wrk));
 		else
 			nm = asAtomHandler::toNumber(args[0]);
 		if (std::isnan(nm) || std::isinf(nm))
@@ -196,7 +197,7 @@ ASFUNCTIONBODY_ATOM(Date,_constructor)
 	} else
 	{
 		number_t year, month, day, hour, minute, second, millisecond;
-		ARG_UNPACK_ATOM (year, 1970) (month, 0) (day, 1) (hour, 0) (minute, 0) (second, 0) (millisecond, 0);
+		ARG_CHECK(ARG_UNPACK (year, 1970) (month, 0) (day, 1) (hour, 0) (minute, 0) (second, 0) (millisecond, 0));
 		if (fabs(year) < 100 ) year = 1900 + year;
 		th->MakeDate(year, month+1, day, hour, minute,second, millisecond,argslen > 3);
 	}
@@ -269,28 +270,28 @@ void Date::MakeDate(int64_t year, int64_t month, int64_t day, int64_t hour, int6
 
 ASFUNCTIONBODY_ATOM(Date,generator)
 {
-	Date* th=Class<Date>::getInstanceS(getSys());
+	Date* th=Class<Date>::getInstanceS(wrk);
 	GDateTime* tmp = g_date_time_new_now_utc();
 	th->MakeDateFromMilliseconds(g_date_time_to_unix(tmp)*1000 + g_date_time_get_microsecond (tmp)/1000);
 	g_date_time_unref(tmp);
 
-	ret = asAtomHandler::fromObject(abstract_s(th->getSystemState(),th->toString()));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,th->toString()));
 }
 
 ASFUNCTIONBODY_ATOM(Date,UTC)
 {
 	for (uint32_t i = 0; i < argslen; i++) {
 		if(asAtomHandler::isNumeric(args[i]) && std::isnan(asAtomHandler::toNumber(args[i]))) {
-			asAtomHandler::setNumber(ret,sys,Number::NaN);
+			asAtomHandler::setNumber(ret,wrk,Number::NaN);
 			return;
 		}
 	}
 	number_t year, month, day, hour, minute, second, millisecond;
-	ARG_UNPACK_ATOM (year) (month) (day, 1) (hour, 0) (minute, 0) (second, 0) (millisecond, 0);
-	_R<Date> dt=_MR(Class<Date>::getInstanceS(sys));
+	ARG_CHECK(ARG_UNPACK (year) (month) (day, 1) (hour, 0) (minute, 0) (second, 0) (millisecond, 0));
+	_R<Date> dt=_MR(Class<Date>::getInstanceS(wrk));
 	dt->MakeDate(year, month+1, day, hour, minute,second, millisecond,false);
 	if(dt->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	ret = dt->msSinceEpoch();
@@ -300,182 +301,182 @@ ASFUNCTIONBODY_ATOM(Date,getTimezoneOffset)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	GTimeSpan diff = g_date_time_get_utc_offset(th->datetime);
-	asAtomHandler::setInt(ret,sys,(int32_t)(-diff/G_TIME_SPAN_MINUTE));
+	asAtomHandler::setInt(ret,wrk,(int32_t)(-diff/G_TIME_SPAN_MINUTE));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getUTCFullYear)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,th->extrayears + g_date_time_get_year(th->datetimeUTC));
+	asAtomHandler::setInt(ret,wrk,th->extrayears + g_date_time_get_year(th->datetimeUTC));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getUTCMonth)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_month(th->datetimeUTC)-1);
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_month(th->datetimeUTC)-1);
 }
 
 ASFUNCTIONBODY_ATOM(Date,getUTCDate)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_day_of_month(th->datetimeUTC));
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_day_of_month(th->datetimeUTC));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getUTCDay)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_day_of_week(th->datetimeUTC)%7);
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_day_of_week(th->datetimeUTC)%7);
 }
 
 ASFUNCTIONBODY_ATOM(Date,getUTCHours)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_hour(th->datetimeUTC));
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_hour(th->datetimeUTC));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getUTCMinutes)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_minute(th->datetimeUTC));
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_minute(th->datetimeUTC));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getUTCSeconds)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_second(th->datetimeUTC));
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_second(th->datetimeUTC));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getUTCMilliseconds)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,(int32_t)(th->milliseconds % 1000));
+	asAtomHandler::setInt(ret,wrk,(int32_t)(th->milliseconds % 1000));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getFullYear)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,th->extrayears + g_date_time_get_year(th->datetime));
+	asAtomHandler::setInt(ret,wrk,th->extrayears + g_date_time_get_year(th->datetime));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getMonth)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_month(th->datetime)-1);
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_month(th->datetime)-1);
 }
 
 ASFUNCTIONBODY_ATOM(Date,getDate)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_day_of_month(th->datetime));
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_day_of_month(th->datetime));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getDay)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_day_of_week(th->datetime)%7);
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_day_of_week(th->datetime)%7);
 }
 
 ASFUNCTIONBODY_ATOM(Date,getHours)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_hour(th->datetime));
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_hour(th->datetime));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getMinutes)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_minute(th->datetime));
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_minute(th->datetime));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getSeconds)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,g_date_time_get_second(th->datetime));
+	asAtomHandler::setInt(ret,wrk,g_date_time_get_second(th->datetime));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getMilliseconds)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
-	asAtomHandler::setInt(ret,sys,(int32_t)(th->milliseconds % 1000));
+	asAtomHandler::setInt(ret,wrk,(int32_t)(th->milliseconds % 1000));
 }
 
 ASFUNCTIONBODY_ATOM(Date,getTime)
 {
 	if (!asAtomHandler::is<Date>(obj)) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	ret = th->msSinceEpoch();
@@ -487,11 +488,11 @@ ASFUNCTIONBODY_ATOM(Date,setFullYear)
 	if (argslen == 0)
 	{
 		th->nan = true;
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	number_t y, m, d;
-	ARG_UNPACK_ATOM (y) (m, 0) (d, 0);
+	ARG_CHECK(ARG_UNPACK (y) (m, 0) (d, 0));
 
 	if (argslen > 1)
 		m++;
@@ -506,7 +507,7 @@ ASFUNCTIONBODY_ATOM(Date,setFullYear)
 ASFUNCTIONBODY_ATOM(Date,fullYearSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setFullYear(o,sys,obj, args, min(argslen, (unsigned)1));
+	Date::setFullYear(o,wrk,obj, args, min(argslen, (unsigned)1));
 	ASATOM_DECREF(o);
 }
 
@@ -514,10 +515,10 @@ ASFUNCTIONBODY_ATOM(Date,setMonth)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t m, d;
-	ARG_UNPACK_ATOM (m) (d, 0);
+	ARG_CHECK(ARG_UNPACK (m) (d, 0));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	if (argslen < 2)
@@ -529,7 +530,7 @@ ASFUNCTIONBODY_ATOM(Date,setMonth)
 ASFUNCTIONBODY_ATOM(Date,monthSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setMonth(o,sys,obj, args, min(argslen, (unsigned)1));
+	Date::setMonth(o,wrk,obj, args, min(argslen, (unsigned)1));
 	ASATOM_DECREF(o);
 }
 
@@ -537,10 +538,10 @@ ASFUNCTIONBODY_ATOM(Date,setDate)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t d;
-	ARG_UNPACK_ATOM (d);
+	ARG_CHECK(ARG_UNPACK (d));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	th->MakeDate(g_date_time_get_year(th->datetime)+th->extrayears, g_date_time_get_month(th->datetime), d, g_date_time_get_hour(th->datetime),g_date_time_get_minute(th->datetime),g_date_time_get_second(th->datetime),th->milliseconds % 1000,true);
@@ -550,7 +551,7 @@ ASFUNCTIONBODY_ATOM(Date,setDate)
 ASFUNCTIONBODY_ATOM(Date,dateSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setDate(o,sys,obj, args, argslen);
+	Date::setDate(o,wrk,obj, args, argslen);
 	ASATOM_DECREF(o);
 }
 
@@ -558,10 +559,10 @@ ASFUNCTIONBODY_ATOM(Date,setHours)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t h, min, sec, ms;
-	ARG_UNPACK_ATOM (h) (min, 0) (sec, 0) (ms, 0);
+	ARG_CHECK(ARG_UNPACK (h) (min, 0) (sec, 0) (ms, 0));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	if (!min) min = g_date_time_get_minute(th->datetime);
@@ -574,7 +575,7 @@ ASFUNCTIONBODY_ATOM(Date,setHours)
 ASFUNCTIONBODY_ATOM(Date,hoursSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setHours(o,sys,obj, args, min(argslen, (unsigned)1));
+	Date::setHours(o,wrk,obj, args, min(argslen, (unsigned)1));
 	ASATOM_DECREF(o);
 }
 
@@ -582,10 +583,10 @@ ASFUNCTIONBODY_ATOM(Date,setMinutes)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t min, sec, ms;
-	ARG_UNPACK_ATOM (min) (sec, 0) (ms, 0);
+	ARG_CHECK(ARG_UNPACK (min) (sec, 0) (ms, 0));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	if (!sec) sec = g_date_time_get_second(th->datetime);
@@ -597,7 +598,7 @@ ASFUNCTIONBODY_ATOM(Date,setMinutes)
 ASFUNCTIONBODY_ATOM(Date,minutesSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setMinutes(o,sys,obj, args, argslen);
+	Date::setMinutes(o,wrk,obj, args, argslen);
 	ASATOM_DECREF(o);
 }
 
@@ -606,10 +607,10 @@ ASFUNCTIONBODY_ATOM(Date,setSeconds)
 	Date* th=asAtomHandler::as<Date>(obj);
 
 	number_t sec, ms;
-	ARG_UNPACK_ATOM (sec) (ms, 0);
+	ARG_CHECK(ARG_UNPACK (sec) (ms, 0));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	if (!ms) ms = th->milliseconds % 1000;
@@ -620,7 +621,7 @@ ASFUNCTIONBODY_ATOM(Date,setSeconds)
 ASFUNCTIONBODY_ATOM(Date,secondsSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setSeconds(o,sys,obj, args, min(argslen, (unsigned)1));
+	Date::setSeconds(o,wrk,obj, args, min(argslen, (unsigned)1));
 	ASATOM_DECREF(o);
 }
 
@@ -628,10 +629,10 @@ ASFUNCTIONBODY_ATOM(Date,setMilliseconds)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t ms;
-	ARG_UNPACK_ATOM (ms);
+	ARG_CHECK(ARG_UNPACK (ms));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	th->MakeDate(g_date_time_get_year(th->datetime)+th->extrayears, g_date_time_get_month(th->datetime), g_date_time_get_day_of_month(th->datetime),  g_date_time_get_hour(th->datetime), g_date_time_get_minute(th->datetime), g_date_time_get_second(th->datetime), ms,true);
@@ -641,7 +642,7 @@ ASFUNCTIONBODY_ATOM(Date,setMilliseconds)
 ASFUNCTIONBODY_ATOM(Date,millisecondsSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setMilliseconds(o,sys,obj, args, argslen);
+	Date::setMilliseconds(o,wrk,obj, args, argslen);
 	ASATOM_DECREF(o);
 }
 
@@ -649,7 +650,7 @@ ASFUNCTIONBODY_ATOM(Date,setUTCFullYear)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t y, m, d;
-	ARG_UNPACK_ATOM (y) (m, 0) (d, 0);
+	ARG_CHECK(ARG_UNPACK (y) (m, 0) (d, 0));
 
 	if (argslen > 1)
 		m++;
@@ -664,7 +665,7 @@ ASFUNCTIONBODY_ATOM(Date,setUTCFullYear)
 ASFUNCTIONBODY_ATOM(Date,UTCFullYearSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setUTCFullYear(o,sys,obj, args, min(argslen, (unsigned)1));
+	Date::setUTCFullYear(o,wrk,obj, args, min(argslen, (unsigned)1));
 	ASATOM_DECREF(o);
 }
 
@@ -672,10 +673,10 @@ ASFUNCTIONBODY_ATOM(Date,setUTCMonth)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t m, d;
-	ARG_UNPACK_ATOM (m) (d, 0);
+	ARG_CHECK(ARG_UNPACK (m) (d, 0));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	if (argslen < 2)
@@ -687,7 +688,7 @@ ASFUNCTIONBODY_ATOM(Date,setUTCMonth)
 ASFUNCTIONBODY_ATOM(Date,UTCMonthSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setUTCMonth(o,sys,obj, args, min(argslen, (unsigned)1));
+	Date::setUTCMonth(o,wrk,obj, args, min(argslen, (unsigned)1));
 	ASATOM_DECREF(o);
 }
 
@@ -695,10 +696,10 @@ ASFUNCTIONBODY_ATOM(Date,setUTCDate)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t d;
-	ARG_UNPACK_ATOM (d);
+	ARG_CHECK(ARG_UNPACK (d));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	th->MakeDate(g_date_time_get_year(th->datetimeUTC)+th->extrayears, g_date_time_get_month(th->datetimeUTC), d, g_date_time_get_hour(th->datetimeUTC),g_date_time_get_minute(th->datetimeUTC),g_date_time_get_second(th->datetimeUTC),th->milliseconds % 1000,false);
@@ -708,7 +709,7 @@ ASFUNCTIONBODY_ATOM(Date,setUTCDate)
 ASFUNCTIONBODY_ATOM(Date,UTCDateSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setUTCDate(o,sys,obj, args, argslen);
+	Date::setUTCDate(o,wrk,obj, args, argslen);
 	ASATOM_DECREF(o);
 }
 
@@ -716,10 +717,10 @@ ASFUNCTIONBODY_ATOM(Date,setUTCHours)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t h, min, sec, ms;
-	ARG_UNPACK_ATOM (h) (min, 0) (sec, 0) (ms, 0);
+	ARG_CHECK(ARG_UNPACK (h) (min, 0) (sec, 0) (ms, 0));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	if (!min) min = g_date_time_get_minute(th->datetimeUTC);
@@ -732,7 +733,7 @@ ASFUNCTIONBODY_ATOM(Date,setUTCHours)
 ASFUNCTIONBODY_ATOM(Date,UTCHoursSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setUTCHours(o,sys,obj, args, min(argslen, (unsigned)1));
+	Date::setUTCHours(o,wrk,obj, args, min(argslen, (unsigned)1));
 	ASATOM_DECREF(o);
 }
 
@@ -740,10 +741,10 @@ ASFUNCTIONBODY_ATOM(Date,setUTCMinutes)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t min, sec, ms;
-	ARG_UNPACK_ATOM (min) (sec, 0) (ms, 0);
+	ARG_CHECK(ARG_UNPACK (min) (sec, 0) (ms, 0));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	if (!sec) sec = g_date_time_get_second(th->datetimeUTC);
@@ -755,7 +756,7 @@ ASFUNCTIONBODY_ATOM(Date,setUTCMinutes)
 ASFUNCTIONBODY_ATOM(Date,UTCMinutesSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setUTCMinutes(o,sys,obj, args, argslen==0?0:1);
+	Date::setUTCMinutes(o,wrk,obj, args, argslen==0?0:1);
 	ASATOM_DECREF(o);
 }
 
@@ -764,10 +765,10 @@ ASFUNCTIONBODY_ATOM(Date,setUTCSeconds)
 	Date* th=asAtomHandler::as<Date>(obj);
 
 	number_t sec, ms;
-	ARG_UNPACK_ATOM (sec) (ms, 0);
+	ARG_CHECK(ARG_UNPACK (sec) (ms, 0));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	if (!ms) ms = th->milliseconds % 1000;
@@ -778,7 +779,7 @@ ASFUNCTIONBODY_ATOM(Date,setUTCSeconds)
 ASFUNCTIONBODY_ATOM(Date,UTCSecondsSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setUTCSeconds(o,sys,obj, args, min(argslen, (unsigned)1));
+	Date::setUTCSeconds(o,wrk,obj, args, min(argslen, (unsigned)1));
 	ASATOM_DECREF(o);
 }
 
@@ -786,10 +787,10 @@ ASFUNCTIONBODY_ATOM(Date,setUTCMilliseconds)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
 	number_t ms;
-	ARG_UNPACK_ATOM (ms);
+	ARG_CHECK(ARG_UNPACK (ms));
 
 	if (th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	th->MakeDate(g_date_time_get_year(th->datetimeUTC)+th->extrayears, g_date_time_get_month(th->datetimeUTC), g_date_time_get_day_of_month(th->datetimeUTC),  g_date_time_get_hour(th->datetimeUTC), g_date_time_get_minute(th->datetimeUTC), g_date_time_get_second(th->datetimeUTC), ms,false);
@@ -799,25 +800,25 @@ ASFUNCTIONBODY_ATOM(Date,setUTCMilliseconds)
 ASFUNCTIONBODY_ATOM(Date,UTCMillisecondsSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setUTCMilliseconds(o,sys,obj, args, argslen);
+	Date::setUTCMilliseconds(o,wrk,obj, args, argslen);
 	ASATOM_DECREF(o);
 }
 
 ASFUNCTIONBODY_ATOM(Date,setTime)
 {
 	number_t ms;
-	ARG_UNPACK_ATOM (ms, Number::NaN);
+	ARG_CHECK(ARG_UNPACK (ms, Number::NaN));
 	if (!asAtomHandler::is<Date>(obj))
 	{
-		multiname name(NULL);
+		multiname name(nullptr);
 		name.name_type=multiname::NAME_STRING;
-		name.name_s_id=sys->getUniqueStringId("value");
-		name.ns.emplace_back(sys,BUILTIN_STRINGS::EMPTY,NAMESPACE);
-		name.ns.emplace_back(sys,BUILTIN_STRINGS::STRING_AS3NS,NAMESPACE);
+		name.name_s_id=wrk->getSystemState()->getUniqueStringId("value");
+		name.ns.emplace_back(wrk->getSystemState(),BUILTIN_STRINGS::EMPTY,NAMESPACE);
+		name.ns.emplace_back(wrk->getSystemState(),BUILTIN_STRINGS::STRING_AS3NS,NAMESPACE);
 		name.isAttribute = true;
-		asAtom v = asAtomHandler::fromNumber(sys,ms,false);
-		asAtomHandler::toObject(obj,sys)->setVariableByMultiname(name,v,CONST_NOT_ALLOWED);
-		asAtomHandler::setNumber(ret,sys,ms);
+		asAtom v = asAtomHandler::fromNumber(wrk,ms,false);
+		asAtomHandler::toObject(obj,wrk)->setVariableByMultiname(name,v,CONST_NOT_ALLOWED,nullptr,wrk);
+		asAtomHandler::setNumber(ret,wrk,ms);
 		return;
 	}
 	assert_and_throw(asAtomHandler::is<Date>(obj));
@@ -826,7 +827,7 @@ ASFUNCTIONBODY_ATOM(Date,setTime)
 	if (std::isnan(ms))
 	{
 		th->nan = true;
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	th->MakeDateFromMilliseconds(int64_t(ms));
@@ -837,7 +838,7 @@ ASFUNCTIONBODY_ATOM(Date,setTime)
 ASFUNCTIONBODY_ATOM(Date,timeSetter)
 {
 	asAtom o=asAtomHandler::invalidAtom;
-	Date::setTime(o,sys,obj, args, argslen);
+	Date::setTime(o,wrk,obj, args, argslen);
 	ASATOM_DECREF(o);
 }
 
@@ -845,12 +846,12 @@ ASFUNCTIONBODY_ATOM(Date,valueOf)
 {
 	if (!asAtomHandler::is<Date>(obj))
 	{
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	Date* th=asAtomHandler::as<Date>(obj);
 	if(th->nan) {
-		asAtomHandler::setNumber(ret,sys,Number::NaN);
+		asAtomHandler::setNumber(ret,wrk,Number::NaN);
 		return;
 	}
 	ret = th->msSinceEpoch();
@@ -858,7 +859,7 @@ ASFUNCTIONBODY_ATOM(Date,valueOf)
 
 asAtom Date::msSinceEpoch()
 {
-	return asAtomHandler::fromNumber(getSystemState(),(number_t)getMsSinceEpoch(),false);
+	return asAtomHandler::fromNumber(getInstanceWorker(),(number_t)getMsSinceEpoch(),false);
 }
 int64_t Date::getMsSinceEpoch()
 {
@@ -869,7 +870,7 @@ int64_t Date::getMsSinceEpoch()
 tiny_string Date::toString()
 {
 	assert_and_throw(implEnable);
-	return toString_priv(false, "%a %b %e %H:%M:%S GMT%z");
+	return toString_priv(false, "%a %b %_e %H:%M:%S GMT%z");
 }
 
 int Date::getYear()
@@ -905,29 +906,29 @@ ASFUNCTIONBODY_ATOM(Date,_toString)
 {
 	if (!asAtomHandler::is<Date>(obj))
 	{
-		ret = asAtomHandler::fromString(sys,"Invalid Date");
+		ret = asAtomHandler::fromString(wrk->getSystemState(),"Invalid Date");
 		return;
 	}
 	Date* th=asAtomHandler::as<Date>(obj);
-	ret = asAtomHandler::fromObject(abstract_s(sys,th->toString()));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,th->toString()));
 }
 
 ASFUNCTIONBODY_ATOM(Date,toUTCString)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
-	ret = asAtomHandler::fromObject(abstract_s(sys,th->toString_priv(true,"%a %b %e %H:%M:%S UTC")));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,th->toString_priv(true,"%a %b %_e %H:%M:%S UTC")));
 }
 
 ASFUNCTIONBODY_ATOM(Date,toDateString)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
-	ret = asAtomHandler::fromObject(abstract_s(sys,th->toString_priv(false,"%a %b %e")));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,th->toString_priv(false,"%a %b %_e")));
 }
 
 ASFUNCTIONBODY_ATOM(Date,toTimeString)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
-	ret = asAtomHandler::fromObject(abstract_s(sys,g_date_time_format(th->datetime, "%H:%M:%S GMT%z")));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,g_date_time_format(th->datetime, "%H:%M:%S GMT%z")));
 }
 
 
@@ -939,7 +940,7 @@ ASFUNCTIONBODY_ATOM(Date,toLocaleString)
 		ret = asAtomHandler::fromStringID(BUILTIN_STRINGS::EMPTY);
 		return;
 	}
-	tiny_string res = th->toString_priv(false,"%a %b %e");
+	tiny_string res = th->toString_priv(false,"%a %b %_e");
 	gchar* fs = g_date_time_format(th->datetime, " %I:%M:%S");
 	res += fs;
 	if (g_date_time_get_hour(th->datetime) > 12)
@@ -947,28 +948,27 @@ ASFUNCTIONBODY_ATOM(Date,toLocaleString)
 	else
 		res += " AM";
 	g_free(fs);
-	ret = asAtomHandler::fromObject(abstract_s(sys,res));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,res));
 }
 ASFUNCTIONBODY_ATOM(Date,toLocaleDateString)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
-	ret = asAtomHandler::fromObject(abstract_s(sys,th->toString_priv(false,"%a %b %e")));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,th->toString_priv(false,"%a %b %_e")));
 }
 ASFUNCTIONBODY_ATOM(Date,toLocaleTimeString)
 {
 	Date* th=asAtomHandler::as<Date>(obj);
-	ret = asAtomHandler::fromObject(abstract_s(sys,g_date_time_format(th->datetime, "%H:%M:%S %Z%z")));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,g_date_time_format(th->datetime, "%H:%M:%S %Z%z")));
 }
 
 ASFUNCTIONBODY_ATOM(Date,_parse)
 {
-	asAtomHandler::setNumber(ret,sys,parse(asAtomHandler::toString(args[0],sys)));
+	asAtomHandler::setNumber(ret,wrk,parse(asAtomHandler::toString(args[0],wrk)));
 }
 
 static const char* months[] = { "Jan", "Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
 number_t Date::parse(tiny_string str)
 {
-
 	char mo[4],tzd[20];
 	int c,d=0,h=0,m=0,s=0,y=0,mon=-1;
 	bool bvalid = false;
@@ -1155,7 +1155,7 @@ number_t Date::parse(tiny_string str)
 				y += 1900;
 			if (mon >= 1 && mon <= 12 && d >= 1 && d <= 31 && h >= 0 && h <= 23 && m >= 0 && m <= 59 && s >= 0 && s <= 59)
 			{
-				_R<Date> dt=_MR(Class<Date>::getInstanceS(getSys()));
+				_R<Date> dt=_MR(Class<Date>::getInstanceS(getWorker()));
 				if (tz == 0)
 					dt->MakeDate(y, mon, d, h, m, s, 0,bIsLocalTime);
 				else
@@ -1164,7 +1164,6 @@ number_t Date::parse(tiny_string str)
 			}
 		}
 	}
-	
 	return res;
 }
 bool Date::isEqual(ASObject* r)
@@ -1193,7 +1192,7 @@ TRISTATE Date::isLessAtom(asAtom& r)
 
 tiny_string Date::format(const char *fmt, bool utc)
 {
-	gchar* fs = g_date_time_format(utc? this->datetimeUTC : this->datetime, "%a %b %e %H:%M:%S GMT%z");
+	gchar* fs = g_date_time_format(utc? this->datetimeUTC : this->datetime, "%a %b %_e %H:%M:%S GMT%z");
 	tiny_string res(fs);
 	g_free(fs);
 	return res;
@@ -1201,9 +1200,9 @@ tiny_string Date::format(const char *fmt, bool utc)
 
 void Date::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
-				std::map<const Class_base*, uint32_t>& traitsMap)
+				std::map<const Class_base*, uint32_t>& traitsMap,ASWorker* wrk)
 {
-	if (out->getObjectEncoding() == ObjectEncoding::AMF0)
+	if (out->getObjectEncoding() == OBJECT_ENCODING::AMF0)
 	{
 		LOG(LOG_NOT_IMPLEMENTED,"serializing Date in AMF0 not implemented");
 		return;
