@@ -21,6 +21,8 @@
 #include "parsing/amf3_generator.h"
 #include "scripting/argconv.h"
 #include "scripting/toplevel/Integer.h"
+#include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/UInteger.h"
 #include "scripting/flash/utils/ByteArray.h"
 
 using namespace std;
@@ -28,9 +30,9 @@ using namespace lightspark;
 
 ASFUNCTIONBODY_ATOM(Integer,_toString)
 {
-	if(Class<Integer>::getClass(sys)->prototype->getObj() == asAtomHandler::getObject(obj))
+	if(Class<Integer>::getClass(wrk->getSystemState())->prototype->getObj() == asAtomHandler::getObject(obj))
 	{
-		ret = asAtomHandler::fromString(sys,"0");
+		ret = asAtomHandler::fromString(wrk->getSystemState(),"0");
 		return;
 	}
 
@@ -42,25 +44,28 @@ ASFUNCTIONBODY_ATOM(Integer,_toString)
 	{
 		char buf[20];
 		snprintf(buf,20,"%i",asAtomHandler::toInt(obj));
-		ret = asAtomHandler::fromObject(abstract_s(sys,buf));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,buf));
 	}
 	else
 	{
 		tiny_string s=Number::toStringRadix(asAtomHandler::toNumber(obj), radix);
-		ret = asAtomHandler::fromObject(abstract_s(sys,s));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,s));
 	}
 }
 
 ASFUNCTIONBODY_ATOM(Integer,_valueOf)
 {
-	if(Class<Integer>::getClass(sys)->prototype->getObj() == asAtomHandler::getObject(obj))
+	if(Class<Integer>::getClass(wrk->getSystemState())->prototype->getObj() == asAtomHandler::getObject(obj))
 	{
-		asAtomHandler::setInt(ret,sys,0);
+		asAtomHandler::setInt(ret,wrk,0);
 		return;
 	}
 
 	if(!asAtomHandler::isInteger(obj))
-			throw Class<TypeError>::getInstanceS(sys,"");
+	{
+		createError<TypeError>(wrk,0,"");
+		return;
+	}
 
 	ASATOM_INCREF(obj);
 	ret = obj;
@@ -80,9 +85,9 @@ ASFUNCTIONBODY_ATOM(Integer,_constructor)
 ASFUNCTIONBODY_ATOM(Integer,generator)
 {
 	if (argslen == 0)
-		asAtomHandler::setInt(ret,sys,(int32_t)0);
+		asAtomHandler::setInt(ret,wrk,(int32_t)0);
 	else
-		asAtomHandler::setInt(ret,sys,asAtomHandler::toInt(args[0]));
+		asAtomHandler::setInt(ret,wrk,asAtomHandler::toInt(args[0]));
 }
 
 TRISTATE Integer::isLess(ASObject* o)
@@ -143,8 +148,11 @@ TRISTATE Integer::isLess(ASObject* o)
 	}
 	
 	asAtom val2p=asAtomHandler::invalidAtom;
-	o->toPrimitive(val2p);
+	bool isrefcounted;
+	o->toPrimitive(val2p,isrefcounted);
 	double val2=asAtomHandler::toNumber(val2p);
+	if (isrefcounted)
+		ASATOM_DECREF(val2p);
 	if(std::isnan(val2)) return TUNDEFINED;
 	return (val<val2)?TTRUE:TFALSE;
 }
@@ -176,8 +184,11 @@ TRISTATE Integer::isLessAtom(asAtom& r)
 	}
 	
 	asAtom val2p=asAtomHandler::invalidAtom;
-	asAtomHandler::getObject(r)->toPrimitive(val2p);
+	bool isrefcounted;
+	asAtomHandler::getObject(r)->toPrimitive(val2p,isrefcounted);
 	double val2=asAtomHandler::toNumber(val2p);
+	if (isrefcounted)
+		ASATOM_DECREF(val2p);
 	if(std::isnan(val2)) return TUNDEFINED;
 	return (val<val2)?TTRUE:TFALSE;
 }
@@ -204,7 +215,7 @@ bool Integer::isEqual(ASObject* o)
 	}
 }
 
-tiny_string Integer::toString()
+tiny_string Integer::toString() const
 {
 	return Integer::toString(val);
 }
@@ -239,23 +250,28 @@ void Integer::sinit(Class_base* c)
 	c->isReusable = true;
 	c->setVariableAtomByQName("MAX_VALUE",nsNameAndKind(),asAtomHandler::fromInt(numeric_limits<int32_t>::max()),CONSTANT_TRAIT);
 	c->setVariableAtomByQName("MIN_VALUE",nsNameAndKind(),asAtomHandler::fromInt(numeric_limits<int32_t>::min()),CONSTANT_TRAIT);
-	c->setDeclaredMethodByQName("toString",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toString),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("toFixed",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toFixed,1),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("toExponential",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toExponential,1),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("toPrecision",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toPrecision,1),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("valueOf",AS3,Class<IFunction>::getFunction(c->getSystemState(),_valueOf),NORMAL_METHOD,true);
-	c->prototype->setVariableByQName("toExponential","",Class<IFunction>::getFunction(c->getSystemState(),Integer::_toExponential, 1),CONSTANT_TRAIT);
-	c->prototype->setVariableByQName("toFixed","",Class<IFunction>::getFunction(c->getSystemState(),Integer::_toFixed, 1),CONSTANT_TRAIT);
-	c->prototype->setVariableByQName("toPrecision","",Class<IFunction>::getFunction(c->getSystemState(),Integer::_toPrecision, 1),CONSTANT_TRAIT);
-	c->prototype->setVariableByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),Integer::_toString),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("valueOf","",Class<IFunction>::getFunction(c->getSystemState(),_valueOf),DYNAMIC_TRAIT);
+	c->setDeclaredMethodByQName("toString",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toString,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("toFixed",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toFixed,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("toExponential",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toExponential,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("toPrecision",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toPrecision,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("valueOf",AS3,Class<IFunction>::getFunction(c->getSystemState(),_valueOf,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->prototype->setVariableByQName("toExponential","",Class<IFunction>::getFunction(c->getSystemState(),Integer::_toExponential,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),CONSTANT_TRAIT);
+	c->prototype->setVariableByQName("toFixed","",Class<IFunction>::getFunction(c->getSystemState(),Integer::_toFixed,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),CONSTANT_TRAIT);
+	c->prototype->setVariableByQName("toPrecision","",Class<IFunction>::getFunction(c->getSystemState(),Integer::_toPrecision,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),CONSTANT_TRAIT);
+	c->prototype->setVariableByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),Integer::_toString,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("valueOf","",Class<IFunction>::getFunction(c->getSystemState(),_valueOf,1,Class<Integer>::getRef(c->getSystemState()).getPtr()),DYNAMIC_TRAIT);
 }
 
 void Integer::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
-				std::map<const Class_base*, uint32_t>& traitsMap)
+				std::map<const Class_base*, uint32_t>& traitsMap,ASWorker* wrk)
 {
-	if (out->getObjectEncoding() == ObjectEncoding::AMF0)
+	serializeValue(out,val);
+}
+
+void Integer::serializeValue(ByteArray* out, int32_t val)
+{
+	if (out->getObjectEncoding() == OBJECT_ENCODING::AMF0)
 	{
 		// write as double
 		out->writeByte(amf0_number_marker);
@@ -510,7 +526,7 @@ ASFUNCTIONBODY_ATOM(Integer,_toExponential)
 {
 	double v = asAtomHandler::toNumber(obj);
 	int32_t fractionDigits;
-	ARG_UNPACK_ATOM(fractionDigits, 0);
+	ARG_CHECK(ARG_UNPACK(fractionDigits, 0));
 	if (argslen == 0 || asAtomHandler::is<Undefined>(args[0]))
 	{
 		if (v == 0)
@@ -518,24 +534,24 @@ ASFUNCTIONBODY_ATOM(Integer,_toExponential)
 		else
 			fractionDigits = imin(imax((int32_t)ceil(::log10(::fabs(v))), 1), 20);
 	}
-	ret = asAtomHandler::fromObject(abstract_s(sys,Number::toExponentialString(v, fractionDigits)));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,Number::toExponentialString(v, fractionDigits)));
 }
 
 ASFUNCTIONBODY_ATOM(Integer,_toFixed)
 {
 	int fractiondigits;
-	ARG_UNPACK_ATOM (fractiondigits, 0);
-	ret = asAtomHandler::fromObject(abstract_s(sys,Number::toFixedString(asAtomHandler::toNumber(obj), fractiondigits)));
+	ARG_CHECK(ARG_UNPACK (fractiondigits, 0));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,Number::toFixedString(asAtomHandler::toNumber(obj), fractiondigits)));
 }
 
 ASFUNCTIONBODY_ATOM(Integer,_toPrecision)
 {
 	if (argslen == 0 || asAtomHandler::is<Undefined>(args[0]))
 	{
-		ret = asAtomHandler::fromObject(abstract_s(sys,asAtomHandler::toString(obj,sys)));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,asAtomHandler::toString(obj,wrk)));
 		return;
 	}
 	int precision;
-	ARG_UNPACK_ATOM (precision);
-	ret = asAtomHandler::fromObject(abstract_s(sys,Number::toPrecisionString(asAtomHandler::toNumber(obj), precision)));
+	ARG_CHECK(ARG_UNPACK (precision));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,Number::toPrecisionString(asAtomHandler::toNumber(obj), precision)));
 }
