@@ -20,6 +20,8 @@
 #include "parsing/amf3_generator.h"
 #include "scripting/argconv.h"
 #include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/Integer.h"
+#include "scripting/toplevel/UInteger.h"
 #include "scripting/toplevel/Math.h"
 #include "scripting/flash/utils/ByteArray.h"
 #include "3rdparty/avmplus/core/d2a.h"
@@ -51,8 +53,12 @@ number_t ASObject::toNumber()
 		{
 			//everything else is an Object regarding to the spec
 			asAtom val2p=asAtomHandler::invalidAtom;
-			toPrimitive(val2p,NUMBER_HINT);
-			return asAtomHandler::toNumber(val2p);
+			bool isrefcounted;
+			toPrimitive(val2p,isrefcounted,NUMBER_HINT);
+			number_t res = asAtomHandler::toNumber(val2p);
+			if (isrefcounted)
+				ASATOM_DECREF(val2p);
+			return res;
 		}
 	}
 }
@@ -120,104 +126,54 @@ TRISTATE Number::isLess(ASObject* o)
 		default:
 		{
 			asAtom val2p=asAtomHandler::invalidAtom;
-			o->toPrimitive(val2p,NUMBER_HINT);
+			bool isrefcounted;
+			o->toPrimitive(val2p,isrefcounted,NUMBER_HINT);
 			double val2=asAtomHandler::toNumber(val2p);
+			if (isrefcounted)
+				ASATOM_DECREF(val2p);
 			if(std::isnan(val2)) return TUNDEFINED;
 			return (toNumber()<val2)?TTRUE:TFALSE;
 		}
 	}
 }
 
-/*
- * This purges trailing zeros from the right, i.e.
- * "144124.45600" -> "144124.456"
- * "144124.000" -> "144124"
- * "144124.45600e+12" -> "144124.456e+12"
- * "144124.45600e+07" -> 144124.456e+7"
- * and it transforms the ',' into a '.' if found.
- */
-void Number::purgeTrailingZeroes(char* buf)
-{
-	int i=strlen(buf)-1;
-	uint32_t Epos = 0;
-	if(i>4 && buf[i-3] == 'e')
-	{
-		Epos = i-3;
-		i=i-4;
-	}
-	for(;i>0;i--)
-	{
-		if(buf[i]!='0')
-			break;
-	}
-	bool commaFound=false;
-	if(buf[i]==',' || buf[i]=='.')
-	{
-		i--;
-		commaFound=true;
-	}
-	if(Epos)
-	{
-		//copy e+12 to the current place
-		strncpy(buf+i+1,buf+Epos,5);
-		if(buf[i+3] == '0')
-		{
-			//this looks like e+07, so turn it into e+7
-			buf[i+3] = buf[i+4];
-			buf[i+4] = '\0';
-		}
-	}
-	else
-		buf[i+1]='\0';
-
-	if(commaFound)
-		return;
-
-	//Also change the comma to the point if needed
-	for(;i>0;i--)
-	{
-		if(buf[i]==',')
-		{
-			buf[i]='.';
-			break;
-		}
-	}
-}
-
 ASFUNCTIONBODY_ATOM(Number,_toString)
 {
-	if(Class<Number>::getClass(sys)->prototype->getObj() == asAtomHandler::getObject(obj))
+	if(Class<Number>::getClass(wrk->getSystemState())->prototype->getObj() == asAtomHandler::getObject(obj))
 	{
-		ret = asAtomHandler::fromString(sys,"0");
+		ret = asAtomHandler::fromString(wrk->getSystemState(),"0");
 		return;
 	}
 	if(!asAtomHandler::isNumeric(obj))
-		throwError<TypeError>(kInvokeOnIncompatibleObjectError, "Number.toString");
+	{
+		createError<TypeError>(wrk,kInvokeOnIncompatibleObjectError, "Number.toString");
+		return;
+	}
 	int radix=10;
-	ARG_UNPACK_ATOM (radix,10);
+	ARG_CHECK(ARG_UNPACK (radix,10));
 
 	if((radix==10) || (asAtomHandler::is<Number>(obj) &&
 					   (std::isnan(asAtomHandler::toNumber(obj)) ||
 					   (std::isinf(asAtomHandler::toNumber(obj))))))
 	{
 		//see e 15.7.4.2
-		ret = asAtomHandler::fromObject(abstract_s(sys,asAtomHandler::toString(obj,sys)));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,asAtomHandler::toString(obj,wrk)));
 	}
 	else
 	{
-		ret = asAtomHandler::fromObject(abstract_s(sys,Number::toStringRadix(asAtomHandler::toNumber(obj), radix)));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,Number::toStringRadix(asAtomHandler::toNumber(obj), radix)));
 	}
 }
 ASFUNCTIONBODY_ATOM(Number,_toLocaleString)
 {
-	if(Class<Number>::getClass(sys)->prototype->getObj() == asAtomHandler::getObject(obj))
+	if(Class<Number>::getClass(wrk->getSystemState())->prototype->getObj() == asAtomHandler::getObject(obj))
 	{
-		ret = asAtomHandler::fromString(sys,"0");
+		ret = asAtomHandler::fromString(wrk->getSystemState(),"0");
 		return;
 	}
 	if(!asAtomHandler::isNumeric(obj))
 	{
-		ret = asAtomHandler::fromString(sys,"0");
+		ret = asAtomHandler::fromString(wrk->getSystemState(),"0");
 		return;
 	}
 	int radix=10;
@@ -227,23 +183,23 @@ ASFUNCTIONBODY_ATOM(Number,_toLocaleString)
 					   (std::isinf(asAtomHandler::toNumber(obj))))))
 	{
 		//see e 15.7.4.2
-		ret = asAtomHandler::fromObject(abstract_s(sys,asAtomHandler::toString(obj,sys)));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,asAtomHandler::toString(obj,wrk)));
 	}
 	else
 	{
-		ret = asAtomHandler::fromObject(abstract_s(sys,Number::toStringRadix(asAtomHandler::toNumber(obj), radix)));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,Number::toStringRadix(asAtomHandler::toNumber(obj), radix)));
 	}
 }
 
 ASFUNCTIONBODY_ATOM(Number,generator)
 {
 	if(argslen==0)
-		asAtomHandler::setNumber(ret,sys,0.);
+		asAtomHandler::setNumber(ret,wrk,0.);
 	else
-		asAtomHandler::setNumber(ret,sys,asAtomHandler::toNumber(args[0]));
+		asAtomHandler::setNumber(ret,wrk,asAtomHandler::toNumber(args[0]));
 }
 
-tiny_string Number::toString()
+tiny_string Number::toString() const
 {
 	return Number::toString(isfloat ? dval : ival);
 }
@@ -567,8 +523,11 @@ tiny_string Number::toString(number_t value, DTOSTRMODE mode, int32_t precision)
 
 tiny_string Number::toStringRadix(number_t val, int radix)
 {
-	if(radix < 2 || radix > 36)
-		throwError<RangeError>(kInvalidRadixError, Integer::toString(radix));
+	if((radix < 2) || (radix > 36))
+	{
+		createError<RangeError>(getWorker(),kInvalidRadixError, Integer::toString(radix));
+		return "";
+	}
 
 	if(std::isnan(val) || std::isinf(val))
 		return Number::toString(val);
@@ -595,62 +554,62 @@ void Number::sinit(Class_base* c)
 {
 	CLASS_SETUP(c, ASObject, _constructor, CLASS_SEALED | CLASS_FINAL);
 	c->isReusable = true;
-	c->setVariableAtomByQName("NEGATIVE_INFINITY",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),-numeric_limits<double>::infinity(),true),CONSTANT_TRAIT);
-	c->setVariableAtomByQName("POSITIVE_INFINITY",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),numeric_limits<double>::infinity(),true),CONSTANT_TRAIT);
-	c->setVariableAtomByQName("MAX_VALUE",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),numeric_limits<double>::max(),true),CONSTANT_TRAIT);
-	c->setVariableAtomByQName("MIN_VALUE",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),numeric_limits<double>::min(),true),CONSTANT_TRAIT);
-	c->setVariableAtomByQName("NaN",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),numeric_limits<double>::quiet_NaN(),true),CONSTANT_TRAIT);
-	c->setDeclaredMethodByQName("toString",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toString),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("toFixed",AS3,Class<IFunction>::getFunction(c->getSystemState(),toFixed,1),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("toExponential",AS3,Class<IFunction>::getFunction(c->getSystemState(),toExponential,1),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("toPrecision",AS3,Class<IFunction>::getFunction(c->getSystemState(),toPrecision,1),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("valueOf",AS3,Class<IFunction>::getFunction(c->getSystemState(),_valueOf),NORMAL_METHOD,true);
-	c->prototype->setVariableByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),Number::_toString),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("toLocaleString","",Class<IFunction>::getFunction(c->getSystemState(),Number::_toLocaleString),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("toFixed","",Class<IFunction>::getFunction(c->getSystemState(),Number::toFixed, 1),CONSTANT_TRAIT);
-	c->prototype->setVariableByQName("toExponential","",Class<IFunction>::getFunction(c->getSystemState(),Number::toExponential, 1),CONSTANT_TRAIT);
-	c->prototype->setVariableByQName("toPrecision","",Class<IFunction>::getFunction(c->getSystemState(),Number::toPrecision, 1),CONSTANT_TRAIT);
-	c->prototype->setVariableByQName("valueOf","",Class<IFunction>::getFunction(c->getSystemState(),_valueOf),DYNAMIC_TRAIT);
+	c->setVariableAtomByQName("NEGATIVE_INFINITY",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),-numeric_limits<double>::infinity(),true),CONSTANT_TRAIT);
+	c->setVariableAtomByQName("POSITIVE_INFINITY",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),numeric_limits<double>::infinity(),true),CONSTANT_TRAIT);
+	c->setVariableAtomByQName("MAX_VALUE",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),numeric_limits<double>::max(),true),CONSTANT_TRAIT);
+	c->setVariableAtomByQName("MIN_VALUE",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),numeric_limits<double>::min(),true),CONSTANT_TRAIT);
+	c->setVariableAtomByQName("NaN",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),numeric_limits<double>::quiet_NaN(),true),CONSTANT_TRAIT);
+	c->setDeclaredMethodByQName("toString",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toString,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("toFixed",AS3,Class<IFunction>::getFunction(c->getSystemState(),toFixed,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("toExponential",AS3,Class<IFunction>::getFunction(c->getSystemState(),toExponential,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("toPrecision",AS3,Class<IFunction>::getFunction(c->getSystemState(),toPrecision,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("valueOf",AS3,Class<IFunction>::getFunction(c->getSystemState(),_valueOf,0,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->prototype->setVariableByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),Number::_toString,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("toLocaleString","",Class<IFunction>::getFunction(c->getSystemState(),Number::_toLocaleString,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("toFixed","",Class<IFunction>::getFunction(c->getSystemState(),Number::toFixed,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),CONSTANT_TRAIT);
+	c->prototype->setVariableByQName("toExponential","",Class<IFunction>::getFunction(c->getSystemState(),Number::toExponential,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),CONSTANT_TRAIT);
+	c->prototype->setVariableByQName("toPrecision","",Class<IFunction>::getFunction(c->getSystemState(),Number::toPrecision,1,Class<ASString>::getRef(c->getSystemState()).getPtr()),CONSTANT_TRAIT);
+	c->prototype->setVariableByQName("valueOf","",Class<IFunction>::getFunction(c->getSystemState(),_valueOf,0,Class<Number>::getRef(c->getSystemState()).getPtr()),DYNAMIC_TRAIT);
 
 	// if needed add AVMPLUS definitions
 	if(c->getSystemState()->flashMode==SystemState::AVMPLUS)
 	{
-		c->setVariableAtomByQName("E",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),2.71828182845905,true),CONSTANT_TRAIT,false);
-		c->setVariableAtomByQName("LN10",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),2.302585092994046,true),CONSTANT_TRAIT,false);
-		c->setVariableAtomByQName("LN2",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),0.6931471805599453,true),CONSTANT_TRAIT,false);
-		c->setVariableAtomByQName("LOG10E",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),0.4342944819032518,true),CONSTANT_TRAIT,false);
-		c->setVariableAtomByQName("LOG2E",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),1.442695040888963387,true),CONSTANT_TRAIT,false);
-		c->setVariableAtomByQName("PI",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),3.141592653589793,true),CONSTANT_TRAIT,false);
-		c->setVariableAtomByQName("SQRT1_2",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),0.7071067811865476,true),CONSTANT_TRAIT,false);
-		c->setVariableAtomByQName("SQRT2",nsNameAndKind(),asAtomHandler::fromNumber(c->getSystemState(),1.4142135623730951,true),CONSTANT_TRAIT,false);
+		c->setVariableAtomByQName("E",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),2.71828182845905,true),CONSTANT_TRAIT,false);
+		c->setVariableAtomByQName("LN10",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),2.302585092994046,true),CONSTANT_TRAIT,false);
+		c->setVariableAtomByQName("LN2",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),0.6931471805599453,true),CONSTANT_TRAIT,false);
+		c->setVariableAtomByQName("LOG10E",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),0.4342944819032518,true),CONSTANT_TRAIT,false);
+		c->setVariableAtomByQName("LOG2E",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),1.442695040888963387,true),CONSTANT_TRAIT,false);
+		c->setVariableAtomByQName("PI",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),3.141592653589793,true),CONSTANT_TRAIT,false);
+		c->setVariableAtomByQName("SQRT1_2",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),0.7071067811865476,true),CONSTANT_TRAIT,false);
+		c->setVariableAtomByQName("SQRT2",nsNameAndKind(),asAtomHandler::fromNumber(c->getInstanceWorker(),1.4142135623730951,true),CONSTANT_TRAIT,false);
 		
-		c->setDeclaredMethodByQName("abs","",Class<IFunction>::getFunction(c->getSystemState(),Math::abs,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("acos","",Class<IFunction>::getFunction(c->getSystemState(),Math::acos,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("asin","",Class<IFunction>::getFunction(c->getSystemState(),Math::asin,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("atan","",Class<IFunction>::getFunction(c->getSystemState(),Math::atan,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("atan2","",Class<IFunction>::getFunction(c->getSystemState(),Math::atan2,2),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("ceil","",Class<IFunction>::getFunction(c->getSystemState(),Math::ceil,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("cos","",Class<IFunction>::getFunction(c->getSystemState(),Math::cos,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("exp","",Class<IFunction>::getFunction(c->getSystemState(),Math::exp,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("floor","",Class<IFunction>::getFunction(c->getSystemState(),Math::floor,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("log","",Class<IFunction>::getFunction(c->getSystemState(),Math::log,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("max","",Class<IFunction>::getFunction(c->getSystemState(),Math::_max,2),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("min","",Class<IFunction>::getFunction(c->getSystemState(),Math::_min,2),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("pow","",Class<IFunction>::getFunction(c->getSystemState(),Math::pow,2),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("random","",Class<IFunction>::getFunction(c->getSystemState(),Math::random),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("round","",Class<IFunction>::getFunction(c->getSystemState(),Math::round,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("sin","",Class<IFunction>::getFunction(c->getSystemState(),Math::sin,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("sqrt","",Class<IFunction>::getFunction(c->getSystemState(),Math::sqrt,1),NORMAL_METHOD,false,false);
-		c->setDeclaredMethodByQName("tan","",Class<IFunction>::getFunction(c->getSystemState(),Math::tan,1),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("abs","",Class<IFunction>::getFunction(c->getSystemState(),Math::abs,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("acos","",Class<IFunction>::getFunction(c->getSystemState(),Math::acos,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("asin","",Class<IFunction>::getFunction(c->getSystemState(),Math::asin,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("atan","",Class<IFunction>::getFunction(c->getSystemState(),Math::atan,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("atan2","",Class<IFunction>::getFunction(c->getSystemState(),Math::atan2,2,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("ceil","",Class<IFunction>::getFunction(c->getSystemState(),Math::ceil,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("cos","",Class<IFunction>::getFunction(c->getSystemState(),Math::cos,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("exp","",Class<IFunction>::getFunction(c->getSystemState(),Math::exp,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("floor","",Class<IFunction>::getFunction(c->getSystemState(),Math::floor,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("log","",Class<IFunction>::getFunction(c->getSystemState(),Math::log,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("max","",Class<IFunction>::getFunction(c->getSystemState(),Math::_max,2,Class<Number>::getRef(c->getSystemState()).getPtr(),Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("min","",Class<IFunction>::getFunction(c->getSystemState(),Math::_min,2,Class<Number>::getRef(c->getSystemState()).getPtr(),Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("pow","",Class<IFunction>::getFunction(c->getSystemState(),Math::pow,2,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("random","",Class<IFunction>::getFunction(c->getSystemState(),Math::random,0,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("round","",Class<IFunction>::getFunction(c->getSystemState(),Math::round,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("sin","",Class<IFunction>::getFunction(c->getSystemState(),Math::sin,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("sqrt","",Class<IFunction>::getFunction(c->getSystemState(),Math::sqrt,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
+		c->setDeclaredMethodByQName("tan","",Class<IFunction>::getFunction(c->getSystemState(),Math::tan,1,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,false,false);
 	}
 }
 
-string Number::toDebugString()
+string Number::toDebugString() const
 {
 	string ret = toString()+(isfloat ? "d" : "di");
 #ifndef NDEBUG
 	char buf[300];
-	sprintf(buf,"(%p / %d/%d)",this,this->getRefCount(),this->getConstant());
+	sprintf(buf,"(%p/%d/%d/%d)",this,this->getRefCount(),this->storedmembercount,this->getConstant());
 	ret += buf;
 #endif
 	return ret;
@@ -688,14 +647,17 @@ ASFUNCTIONBODY_ATOM(Number,toFixed)
 {
 	number_t val = asAtomHandler::toNumber(obj);
 	int fractiondigits;
-	ARG_UNPACK_ATOM (fractiondigits,0);
-	ret = asAtomHandler::fromObject(abstract_s(sys,toFixedString(val, fractiondigits)));
+	ARG_CHECK(ARG_UNPACK (fractiondigits,0));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,toFixedString(val, fractiondigits)));
 }
 
 tiny_string Number::toFixedString(double v, int32_t fractionDigits)
 {
-	if (fractionDigits < 0 || fractionDigits > 20)
-		throwError<RangeError>(kInvalidPrecisionError);
+	if ((fractionDigits < 0) || (fractionDigits > 20))
+	{
+		createError<RangeError>(getWorker(),kInvalidPrecisionError);
+		return "";
+	}
 	return toString(v,DTOSTR_FIXED,fractionDigits);
 }
 
@@ -703,16 +665,19 @@ ASFUNCTIONBODY_ATOM(Number,toExponential)
 {
 	double v = asAtomHandler::toNumber(obj);
 	int32_t fractionDigits;
-	ARG_UNPACK_ATOM(fractionDigits, 0);
+	ARG_CHECK(ARG_UNPACK(fractionDigits, 0));
 	if (argslen == 0 || asAtomHandler::is<Undefined>(args[0]))
 		fractionDigits = imin(imax(Number::countSignificantDigits(v)-1, 1), 20);
-	ret =asAtomHandler::fromObject(abstract_s(sys,toExponentialString(v, fractionDigits)));
+	ret =asAtomHandler::fromObject(abstract_s(wrk,toExponentialString(v, fractionDigits)));
 }
 
 tiny_string Number::toExponentialString(double v, int32_t fractionDigits)
 {
-	if (fractionDigits < 0 || fractionDigits > 20)
-		throwError<RangeError>(kInvalidPrecisionError);
+	if ((fractionDigits < 0) || (fractionDigits > 20))
+	{
+		createError<RangeError>(getWorker(),kInvalidPrecisionError);
+		return "";
+	}
 	return toString(v,DTOSTR_EXPONENTIAL,fractionDigits);
 }
 
@@ -795,42 +760,47 @@ ASFUNCTIONBODY_ATOM(Number,toPrecision)
 	double v = asAtomHandler::toNumber(obj);
 	if (argslen == 0 || asAtomHandler::is<Undefined>(args[0]))
 	{
-		ret = asAtomHandler::fromObject(abstract_s(sys,toString(v)));
+		ret = asAtomHandler::fromObject(abstract_s(wrk,toString(v)));
 		return;
 	}
 
 	int32_t precision;
-	ARG_UNPACK_ATOM(precision);
-	ret = asAtomHandler::fromObject(abstract_s(sys,toPrecisionString(v, precision)));
+	ARG_CHECK(ARG_UNPACK(precision));
+	ret = asAtomHandler::fromObject(abstract_s(wrk,toPrecisionString(v, precision)));
 }
 
 tiny_string Number::toPrecisionString(double v, int32_t precision)
 {
-	if (precision < 1 || precision > 21)
-		throwError<RangeError>(kInvalidPrecisionError);
+	if ((precision < 1) || (precision > 21))
+	{
+		createError<RangeError>(getWorker(),kInvalidPrecisionError);
+		return "";
+	}
 	return toString(v,DTOSTR_PRECISION,precision);
 }
 
 ASFUNCTIONBODY_ATOM(Number,_valueOf)
 {
-	if(Class<Number>::getClass(sys)->prototype->getObj() == asAtomHandler::getObject(obj))
+	if(Class<Number>::getClass(wrk->getSystemState())->prototype->getObj() == asAtomHandler::getObject(obj))
 	{
-		asAtomHandler::setInt(ret,sys,0);
+		asAtomHandler::setInt(ret,wrk,0);
 		return;
 	}
 
 	if(!asAtomHandler::isNumeric(obj))
-		throwError<TypeError>(kInvokeOnIncompatibleObjectError);
-
+	{
+		createError<TypeError>(wrk,kInvokeOnIncompatibleObjectError);
+		return;
+	}
 	ASATOM_INCREF(obj);
 	ret = obj;
 }
 
 void Number::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
-				std::map<const Class_base*, uint32_t>& traitsMap)
+				std::map<const Class_base*, uint32_t>& traitsMap,ASWorker* wrk)
 {
-	if (out->getObjectEncoding() == ObjectEncoding::AMF0)
+	if (out->getObjectEncoding() == OBJECT_ENCODING::AMF0)
 	{
 		out->writeByte(amf0_number_marker);
 		out->serializeDouble(toNumber());
