@@ -30,7 +30,7 @@ class XML: public ASObject, public XMLBase
 {
 friend class XMLList;
 public:
-	typedef std::vector<_R<XML>> XMLVector;
+	typedef std::vector<_NR<XML>> XMLVector;
 	typedef std::vector<_R<Namespace>> NSVector;
 private:
 	_NR<XMLList> childrenlist;
@@ -70,19 +70,21 @@ private:
 	void handleNotification(const tiny_string& command, asAtom value, asAtom detail);
 	// Append node or attribute to this. Concatenates adjacent
 	// text nodes.
-	void appendChild(_R<XML> child);
-	void prependChild(_R<XML> child);
+	void appendChild(_NR<XML> child);
+	void prependChild(_NR<XML> child);
 	static void normalizeRecursive(XML *node);
 	void addTextContent(const tiny_string& str);
 	void RemoveNamespace(Namespace *ns);
 	void getComments(XMLVector& ret);
 	void getprocessingInstructions(XMLVector& ret, tiny_string name);
-	void CheckCyclicReference(XML* node);
+	bool CheckCyclicReference(XML* node);
 public:
-	XML(Class_base* c);
-	XML(Class_base* c,const std::string& str);
-	XML(Class_base* c,const pugi::xml_node& _n, XML* parent=nullptr, bool fromXMLList=false);
+	XML(ASWorker* wrk,Class_base* c);
+	XML(ASWorker* wrk,Class_base* c,const std::string& str);
+	XML(ASWorker* wrk,Class_base* c,const pugi::xml_node& _n, XML* parent=nullptr, bool fromXMLList=false);
+	void finalize() override;
 	bool destruct() override;
+	void prepareShutdown() override;
 	
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(_toString);
@@ -143,29 +145,28 @@ public:
 	ASFUNCTION_ATOM(setNotification);
 	ASFUNCTION_ATOM(notification);
 
-	static void buildTraits(ASObject* o){}
 	static void sinit(Class_base* c);
 	
 	static bool getPrettyPrinting();
 	static unsigned int getParseMode();
-	static XML* createFromString(SystemState *sys, const tiny_string& s, bool usefirstchild=false);
-	static XML* createFromNode(const pugi::xml_node& _n, XML* parent=NULL, bool fromXMLList=false);
+	static XML* createFromString(ASWorker* wrk, const tiny_string& s, bool usefirstchild=false);
+	static XML* createFromNode(ASWorker* wrk,const pugi::xml_node& _n, XML* parent=nullptr, bool fromXMLList=false);
 
 	const tiny_string getName() const { return nodename;}
 	uint32_t getNamespaceURI() const { return nodenamespace_uri;}
-	XMLList* getChildrenlist() { return childrenlist ? childrenlist.getPtr() : NULL; }
+	XMLList* getChildrenlist() { return childrenlist ? childrenlist.getPtr() : nullptr; }
 	
 	
-	void getDescendantsByQName(const tiny_string& name, uint32_t ns, bool bIsAttribute, XMLVector& ret) const;
+	void getDescendantsByQName(const multiname& name, XMLVector& ret) const;
 	void getElementNodes(const tiny_string& name, XMLVector& foundElements);
-	GET_VARIABLE_RESULT getVariableByMultiname(asAtom& ret, const multiname& name, GET_VARIABLE_OPTION opt=NONE) override;
-	GET_VARIABLE_RESULT getVariableByInteger(asAtom &ret, int index, GET_VARIABLE_OPTION opt) override;
-	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic, bool considerPrototype) override;
-	bool hasProperty(const multiname& name,bool checkXMLPropsOnly, bool considerDynamic, bool considerPrototype);
-	multiname* setVariableByMultiname(multiname& name, asAtom &o, CONST_ALLOWED_FLAG allowConst, bool *alreadyset=nullptr) override;
-	void setVariableByInteger(int index, asAtom &o, ASObject::CONST_ALLOWED_FLAG allowConst) override;
-	multiname *setVariableByMultinameIntern(multiname& name, asAtom &o, CONST_ALLOWED_FLAG allowConst, bool replacetext);
-	bool deleteVariableByMultiname(const multiname& name) override;
+	GET_VARIABLE_RESULT getVariableByMultiname(asAtom& ret, const multiname& name, GET_VARIABLE_OPTION opt, ASWorker* wrk) override;
+	GET_VARIABLE_RESULT getVariableByInteger(asAtom &ret, int index, GET_VARIABLE_OPTION opt, ASWorker* wrk) override;
+	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic, bool considerPrototype, ASWorker* wrk) override;
+	bool hasProperty(const multiname& name, bool checkXMLPropsOnly, bool considerDynamic, bool considerPrototype, ASWorker* wrk);
+	multiname* setVariableByMultiname(multiname& name, asAtom &o, CONST_ALLOWED_FLAG allowConst, bool *alreadyset, lightspark::ASWorker* wrk) override;
+	void setVariableByInteger(int index, asAtom &o, ASObject::CONST_ALLOWED_FLAG allowConst, bool* alreadyset,ASWorker* wrk) override;
+	multiname *setVariableByMultinameIntern(multiname& name, asAtom &o, CONST_ALLOWED_FLAG allowConst, bool replacetext, bool* alreadyset, ASWorker* wrk);
+	bool deleteVariableByMultiname(const multiname& name, ASWorker* wrk) override;
 	static bool isValidMultiname(SystemState *sys, const multiname& name, uint32_t& index);
 
 	void setTextContent(const tiny_string& content);
@@ -187,7 +188,7 @@ public:
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
-				std::map<const Class_base*, uint32_t>& traitsMap) override;
+				std::map<const Class_base*, uint32_t>& traitsMap, ASWorker* wrk) override;
 	void dumpTreeObjects(int indent=0);
 };
 }
