@@ -29,11 +29,8 @@
 #include <string>
 #include "swftypes.h"
 #include "scripting/flash/display/flashdisplay.h"
-#include "scripting/flash/net/flashnet.h"
-#include "scripting/flash/utils/IntervalManager.h"
 #include "timer.h"
 #include "memory_support.h"
-#include "platforms/engineutils.h"
 
 class uncompressing_filter;
 
@@ -47,8 +44,10 @@ class ControlTag;
 class DownloadManager;
 class DisplayListTag;
 class DictionaryTag;
+class DefineScalingGridTag;
 class ExtScriptObject;
 class InputThread;
+class IntervalManager;
 class ParseThread;
 class PluginManager;
 class RenderThread;
@@ -63,6 +62,8 @@ class SecurityDomain;
 class Class_inherit;
 class FontTag;
 class SoundTransform;
+class ASFile;
+class EngineData;
 
 class RootMovieClip: public MovieClip
 {
@@ -75,6 +76,8 @@ private:
 	RGB Background;
 	Mutex dictSpinlock;
 	std::unordered_map < uint32_t, DictionaryTag* > dictionary;
+	Mutex scalinggridsmutex;
+	std::unordered_map < uint32_t, RECT > scalinggrids;
 	std::map < QName, DictionaryTag* > classesToBeBound;
 	std::map < tiny_string,FontTag* > embeddedfonts;
 	std::map < uint32_t,FontTag* > embeddedfontsByID;
@@ -87,23 +90,25 @@ private:
 	 * but mainClip->getStage()->* instead.
 	 */
 	void initFrame() override;
-	void advanceFrame() override;
+	void advanceFrame(bool implicit) override;
 	void executeFrameScript() override;
 	ACQUIRE_RELEASE_FLAG(finishedLoading);
 	
 	unordered_map<uint32_t,_NR<IFunction>> avm1ClassConstructors;
 	unordered_map<uint32_t,AVM1InitActionTag*> avm1InitActionTags;
 public:
-	RootMovieClip(_NR<LoaderInfo> li, _NR<ApplicationDomain> appDomain, _NR<SecurityDomain> secDomain, Class_base* c);
+	RootMovieClip(ASWorker* wrk,_NR<LoaderInfo> li, _NR<ApplicationDomain> appDomain, _NR<SecurityDomain> secDomain, Class_base* c);
 	~RootMovieClip();
 	void destroyTags();
 	bool destruct() override;
 	void finalize() override;
+	void prepareShutdown() override;
 	bool hasFinishedLoading() override { return ACQUIRE_READ(finishedLoading); }
 	bool isWaitingForParser() { return waitingforparser; }
 	void constructionComplete() override;
 	void afterConstruction() override;
 	bool needsActionScript3() const override { return this->usesActionScript3;}
+	ParseThread* parsethread;
 	uint32_t version;
 	uint32_t fileLength;
 	bool hasSymbolClass;
@@ -118,21 +123,19 @@ public:
 	void addToDictionary(DictionaryTag* r);
 	DictionaryTag* dictionaryLookup(int id);
 	DictionaryTag* dictionaryLookupByName(uint32_t nameID);
+	void addToScalingGrids(const DefineScalingGridTag* r);
+	RECT* ScalingGridsLookup(int id);
 	void resizeCompleted();
 	void labelCurrentFrame(const STRING& name);
 	void commitFrame(bool another);
 	void revertFrame();
 	void parsingFailed();
-	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const override;
+	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) override;
 	void DLL_PUBLIC setOrigin(const tiny_string& u, const tiny_string& filename="");
 	URLInfo& getOrigin() { return origin; }
 	void DLL_PUBLIC setBaseURL(const tiny_string& url);
 	const URLInfo& getBaseURL();
-/*	ASObject* getVariableByQName(const tiny_string& name, const tiny_string& ns);
-	void setVariableByQName(const tiny_string& name, const tiny_string& ns, ASObject* o);
-	void setVariableByMultiname(multiname& name, asAtom o);
-	void setVariableByString(const std::string& s, ASObject* o);*/
-	static RootMovieClip* getInstance(_NR<LoaderInfo> li, _R<ApplicationDomain> appDomain, _R<SecurityDomain> secDomain);
+	static RootMovieClip* getInstance(ASWorker* wrk, _NR<LoaderInfo> li, _R<ApplicationDomain> appDomain, _R<SecurityDomain> secDomain);
 	/*
 	 * The application domain for this clip
 	 */
@@ -141,6 +144,15 @@ public:
 	 * The security domain for this clip
 	 */
 	_NR<SecurityDomain> securityDomain;
+	//map of all classed defined in the swf. They own one reference to each class/template
+	//key is the stringID of the class name (without namespace)
+	std::multimap<uint32_t, Class_base*> customClasses;
+	/*
+	 * Support for class aliases in AMF3 serialization
+	 */
+	std::map<tiny_string, _R<Class_base> > aliasMap;
+	std::map<QName,std::unordered_set<uint32_t>*> customclassoverriddenmethods;
+	std::map<QName, Template_base*> templates;
 	//DisplayObject interface
 	_NR<RootMovieClip> getRoot() override;
 	_NR<Stage> getStage() override;
@@ -206,6 +218,7 @@ private:
 	bool shutdown;
 	bool firsttick;
 	bool localstorageallowed;
+	bool influshing;
 	RenderThread* renderThread;
 	InputThread* inputThread;
 	EngineData* engineData;
@@ -263,7 +276,7 @@ private:
 	Mutex profileDataSpinlock;
 
 	Mutex mutexFrameListeners;
-	std::set<_R<DisplayObject>> frameListeners;
+	std::set<DisplayObject*> frameListeners;
 	/*
 	   The head of the invalidate queue
 	*/
@@ -306,8 +319,7 @@ private:
 	Cond mainsignalCond;
 	void systemFinalize();
 	std::map<tiny_string, Class_base *> classnamemap;
-	set<ASObject*> constantrefs;
-	list<_R<DisplayObject>> listResetParent;
+	unordered_set<DisplayObject*> listResetParent;
 public:
 	void setURL(const tiny_string& url) DLL_PUBLIC;
 	tiny_string getDumpedSWFPath() const { return dumpedSWFPath;}
@@ -338,13 +350,10 @@ public:
 	bool isShuttingDown() const DLL_PUBLIC;
 	bool isOnError() const DLL_PUBLIC;
 	void setShutdownFlag() DLL_PUBLIC;
+	void signalTerminated();
 	std::map<tiny_string, _R<SharedObject> > sharedobjectmap;
 	bool localStorageAllowed() const { return localstorageallowed; }
 	void setLocalStorageAllowed(bool allowed);
-	void registerConstantRef(ASObject* obj)
-	{
-		constantrefs.insert(obj);
-	}
 	void tick() override;
 	void tickFence() override;
 	RenderThread* getRenderThread() const { return renderThread; }
@@ -407,15 +416,9 @@ public:
 	//Application starting time in milliseconds
 	uint64_t startTime;
 
-	//map of all classed defined in the swf. They own one reference to each class/template
-	//key is the stringID of the class name (without namespace)
-	std::multimap<uint32_t, Class_base*> customClasses;
 	//This is an array of fixed size, we can avoid using std::vector
 	Class_base** builtinClasses;
-	std::map<QName, Template_base*> templates;
 
-	std::map<QName,std::unordered_set<uint32_t>*> customclassoverriddenmethods;
-	
 	//Flags for command line options
 	bool useInterpreter;
 	bool useFastInterpreter;
@@ -458,6 +461,7 @@ public:
 	Mutex workerMutex;
 	void addWorker(ASWorker* w);
 	void removeWorker(ASWorker* w);
+	void addEventToBackgroundWorkers(_NR<EventDispatcher> obj, _R<Event> ev);
 
 	//Stuff to be done once for process and not for plugin instance
 	static void staticInit() DLL_PUBLIC;
@@ -467,7 +471,7 @@ public:
 	IntervalManager* intervalManager;
 	SecurityManager* securityManager;
 	LocaleManager* localeManager;
-    CurrencyManager* currencyManager;
+	CurrencyManager* currencyManager;
 	ExtScriptObject* extScriptObject;
 
 	enum SCALE_MODE { EXACT_FIT=0, NO_BORDER=1, NO_SCALE=2, SHOW_ALL=3 };
@@ -477,14 +481,15 @@ public:
 	//TODO: Those should be different for each security domain
 	//NAMING: static$CLASSNAME$$PROPERTYNAME$
 	//	NetConnection
-	ObjectEncoding::ENCODING staticNetConnectionDefaultObjectEncoding;
-	ObjectEncoding::ENCODING staticByteArrayDefaultObjectEncoding;
-	ObjectEncoding::ENCODING staticSharedObjectDefaultObjectEncoding;
+	OBJECT_ENCODING staticNetConnectionDefaultObjectEncoding;
+	OBJECT_ENCODING staticByteArrayDefaultObjectEncoding;
+	OBJECT_ENCODING staticSharedObjectDefaultObjectEncoding;
 	bool staticSharedObjectPreventBackup;
 	
-	//enterFrame event management
-	void registerFrameListener(_R<DisplayObject> clip);
-	void unregisterFrameListener(_R<DisplayObject> clip);
+	//broadcast event management
+	void registerFrameListener(DisplayObject* clip);
+	void unregisterFrameListener(DisplayObject* clip);
+	void addBroadcastEvent(const tiny_string& event);
 
 	//Invalidation queue management
 	void addToInvalidateQueue(_R<DisplayObject> d) override;
@@ -495,10 +500,6 @@ public:
 	//Resize support
 	void resizeCompleted();
 
-	/*
-	 * Support for class aliases in AMF3 serialization
-	 */
-	std::map<tiny_string, _R<Class_base> > aliasMap;
 #ifdef PROFILING_SUPPORT
 	void setProfilingOutput(const tiny_string& t) DLL_PUBLIC;
 	const tiny_string& getProfilingOutput() const;
@@ -540,6 +541,7 @@ public:
 	void openPageInBrowser(const tiny_string& url, const tiny_string& window);
 
 	void showMouseCursor(bool visible);
+	void setMouseHandCursor(bool sethand);
 	void waitRendering() DLL_PUBLIC;
 	EngineData* getEngineData() { return engineData;}
 	uint32_t getSwfVersion();
@@ -551,23 +553,26 @@ public:
 	void waitMainSignal() DLL_PUBLIC;
 	void sendMainSignal() DLL_PUBLIC;
 
-	void dumpStacktrace();
-	
 	// static class properties are named static_<classname>_<propertyname>
 	_NR<SoundTransform> static_SoundMixer_soundTransform;
 	int static_SoundMixer_bufferTime;
 	_NR<ASObject> static_ObjectEncoding_dynamicPropertyWriter;
+	tiny_string static_Multitouch_inputMode;
+	_NR<ASFile> static_ASFile_applicationDirectory;
+	_NR<ASFile> static_ASFile_applicationStorageDirectory;
 
 	ACQUIRE_RELEASE_FLAG(isinitialized);
 	Mutex initializedMutex;
 	Cond initializedCond;
 	void waitInitialized();
-	void getClassInstanceByName(asAtom &ret, const tiny_string& clsname);
+	void getClassInstanceByName(ASWorker* wrk, asAtom &ret, const tiny_string& clsname);
 	Mutex resetParentMutex;
-	void addDisplayObjectToResetParentList(_R<DisplayObject> child)
+	void addDisplayObjectToResetParentList(DisplayObject* child)
 	{
 		Locker l(resetParentMutex);
-		listResetParent.push_back(child);
+		child->incRef();
+		child->addStoredMember();
+		listResetParent.insert(child);
 	}
 	void resetParentList()
 	{
@@ -576,6 +581,7 @@ public:
 		while (it != listResetParent.end())
 		{
 			(*it)->setParent(nullptr);
+			(*it)->removeStoredMember();
 			it = listResetParent.erase(it);
 		}
 	}
@@ -585,7 +591,7 @@ public:
 		auto it = listResetParent.begin();
 		while (it != listResetParent.end())
 		{
-			if ((*it).getPtr()==d)
+			if ((*it)==d)
 				return true;
 			it++;
 		}
@@ -597,8 +603,9 @@ public:
 		auto it = listResetParent.begin();
 		while (it != listResetParent.end())
 		{
-			if ((*it).getPtr()==d)
+			if ((*it)==d)
 			{
+				d->removeStoredMember();
 				listResetParent.erase(it);
 				break;
 			}
@@ -620,27 +627,32 @@ public:
 	ParseThread(std::istream& in, RootMovieClip *root) DLL_PUBLIC;
 	~ParseThread();
 	FILE_TYPE getFileType() const { return fileType; }
-        _NR<DisplayObject> getParsedObject();
+	_NR<DisplayObject> getParsedObject();
 	RootMovieClip* getRootMovie() const;
 	static FILE_TYPE recognizeFile(uint8_t c1, uint8_t c2, uint8_t c3, uint8_t c4);
-	void execute();
+	void execute() override;
 	_NR<ApplicationDomain> applicationDomain;
 	_NR<SecurityDomain> securityDomain;
+	void getSWFByteArray(ByteArray* ba);
+	void addExtensions(std::vector<tiny_string>& ext) { extensions = ext; }
 private:
+	std::vector<tiny_string> extensions;
 	std::istream& f;
 	uncompressing_filter* uncompressingFilter;
 	std::streambuf* backend;
+	std::streambuf* bytearraybuf;
 	Loader *loader;
 	_NR<DisplayObject> parsedObject;
 	Mutex objectSpinlock;
 	tiny_string url;
 	FILE_TYPE fileType;
-	void threadAbort();
-	void jobFence() {}
+	void threadAbort() override;
+	void jobFence() override {}
 	void parseSWFHeader(RootMovieClip *root, UI8 ver);
 	void parseSWF(UI8 ver);
 	void parseBitmap();
 	void setRootMovie(RootMovieClip *root);
+	void parseExtensions(RootMovieClip* root);
 };
 
 /* Returns the thread-specific SystemState */
