@@ -36,6 +36,11 @@
 #include "scripting/flash/display/BitmapData.h"
 #include "scripting/flash/geom/flashgeom.h"
 #include "scripting/toplevel/toplevel.h"
+#include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/Boolean.h"
+#include "scripting/toplevel/Integer.h"
+#include "scripting/toplevel/UInteger.h"
+#include "parsing/tags.h"
 
 
 using namespace std;
@@ -130,11 +135,11 @@ const tiny_string multiname::normalizedNameUnresolved(SystemState* sys) const
 	}
 }
 
-void multiname::setName(asAtom& n,SystemState* sys)
+void multiname::setName(asAtom& n,ASWorker* w)
 {
-	if (name_type==NAME_OBJECT && name_o!=NULL) {
+	if (name_type==NAME_OBJECT && name_o!=nullptr) {
 		name_o->decRef();
-		name_o = NULL;
+		name_o = nullptr;
 	}
 
 	switch(asAtomHandler::getObjectType(n))
@@ -168,24 +173,24 @@ void multiname::setName(asAtom& n,SystemState* sys)
 			ASQName* qname=asAtomHandler::as<ASQName>(n);
 			name_s_id=qname->local_name;
 			name_type = NAME_STRING;
-			isInteger=Array::isIntegerWithoutLeadingZeros(sys->getStringFromUniqueId(name_s_id));
+			isInteger=Array::isIntegerWithoutLeadingZeros(w->getSystemState()->getStringFromUniqueId(name_s_id));
 		}
 		break;
 	case T_STRING:
 		{
-			name_s_id=asAtomHandler::toStringId(n,sys);
+			name_s_id=asAtomHandler::toStringId(n,w);
 			name_type = NAME_STRING;
-			isInteger=Array::isIntegerWithoutLeadingZeros(sys->getStringFromUniqueId(name_s_id));
+			isInteger=Array::isIntegerWithoutLeadingZeros(w->getSystemState()->getStringFromUniqueId(name_s_id));
 		}
 		break;
 	case T_NULL:
-		name_o=sys->getNullRef();
+		name_o=w->getSystemState()->getNullRef();
 		name_type = NAME_OBJECT;
 		name_s_id = UINT32_MAX;
 		isInteger=false;
 		break;
 	case T_UNDEFINED:
-		name_o=sys->getUndefinedRef();
+		name_o=w->getSystemState()->getUndefinedRef();
 		name_type = NAME_OBJECT;
 		name_s_id = UINT32_MAX;
 		isInteger=false;
@@ -205,7 +210,7 @@ void multiname::resetNameIfObject()
 	if(name_type==NAME_OBJECT && name_o)
 	{
 		name_o->decRef();
-		name_o=NULL;
+		name_o=nullptr;
 	}
 }
 
@@ -481,7 +486,7 @@ std::ostream& lightspark::operator<<(std::ostream& s, const MATRIX& r)
 {
 	s << "| " << r.xx << ' ' << r.yx << " |" << std::endl;
 	s << "| " << r.xy << ' ' << r.yy << " |" << std::endl;
-	s << "| " << (int)r.x0 << ' ' << (int)r.y0 << " |" << std::endl;
+	s << "| " << r.x0 << ' ' << r.y0 << " |" << std::endl;
 	return s;
 }
 
@@ -539,7 +544,7 @@ std::istream& lightspark::operator>>(std::istream& s, LINESTYLEARRAY& v)
 	UI8 LineStyleCount;
 	s >> LineStyleCount;
 	if(LineStyleCount==0xff)
-		LOG(LOG_ERROR,_("Line array extended not supported"));
+		LOG(LOG_ERROR,"Line array extended not supported");
 	if(v.version<4)
 	{
 		for(int i=0;i<LineStyleCount;i++)
@@ -570,7 +575,7 @@ std::istream& lightspark::operator>>(std::istream& s, MORPHLINESTYLEARRAY& v)
 	UI8 LineStyleCount;
 	s >> LineStyleCount;
 	if(LineStyleCount==0xff)
-		LOG(LOG_ERROR,_("Line array extended not supported"));
+		LOG(LOG_ERROR,"Line array extended not supported");
 	assert(v.version==1 || v.version==2);
 	if(v.version==1)
 	{
@@ -632,7 +637,7 @@ std::istream& lightspark::operator>>(std::istream& s, MORPHFILLSTYLEARRAY& v)
 	UI8 FillStyleCount;
 	s >> FillStyleCount;
 	if(FillStyleCount==0xff)
-		LOG(LOG_ERROR,_("Fill array extended not supported"));
+		LOG(LOG_ERROR,"Fill array extended not supported");
 	for(int i=0;i<FillStyleCount;i++)
 	{
 		MORPHFILLSTYLE t;
@@ -749,7 +754,7 @@ std::istream& lightspark::operator>>(std::istream& in, TEXTRECORD& v)
 	v.TextRecordType=UB(1,bs);
 	v.StyleFlagsReserved=UB(3,bs);
 	if(v.StyleFlagsReserved)
-		LOG(LOG_ERROR,_("Reserved bits not so reserved"));
+		LOG(LOG_ERROR,"Reserved bits not so reserved");
 	v.StyleFlagsHasFont=UB(1,bs);
 	v.StyleFlagsHasColor=UB(1,bs);
 	v.StyleFlagsHasYOffset=UB(1,bs);
@@ -910,7 +915,7 @@ std::istream& lightspark::operator>>(std::istream& s, FILLSTYLE& v)
 	}
 	else
 	{
-		LOG(LOG_ERROR,_("Not supported fill style ") << (int)v.FillStyleType);
+		LOG(LOG_ERROR,"Not supported fill style " << (int)v.FillStyleType);
 	throw ParseException("Not supported fill style");
 	}
 	return s;
@@ -957,7 +962,7 @@ std::istream& lightspark::operator>>(std::istream& s, MORPHFILLSTYLE& v)
 	}
 	else
 	{
-		LOG(LOG_ERROR,_("Not supported fill style 0x") << hex << (int)v.FillStyleType << " at "<< s.tellg()<< dec <<  _("... Aborting"));
+		LOG(LOG_ERROR,"Not supported fill style 0x" << hex << (int)v.FillStyleType << " at "<< s.tellg()<< dec <<  "... Aborting");
 	}
 	return s;
 }
@@ -1026,10 +1031,11 @@ SHAPERECORD::SHAPERECORD(SHAPE* p,BitStream& bs):parent(p),MoveBits(0),MoveDelta
 		{
 			LineStyle=UB(parent->NumLineBits,bs);
 		}
-		if(StateNewStyles && parent->version >= 2)
+		// contrary to spec StateNewStyles is also allowed in DefineShapeTag
+		if(StateNewStyles)// && parent->version >= 2)
 		{
 			SHAPEWITHSTYLE* ps=dynamic_cast<SHAPEWITHSTYLE*>(parent);
-			if(ps==NULL)
+			if(ps==nullptr)
 				throw ParseException("Malformed SWF file");
 			bs.pos=0;
 			FILLSTYLEARRAY a(ps->FillStyles.version);
@@ -1162,8 +1168,8 @@ std::istream& lightspark::operator>>(std::istream& stream, MATRIX& v)
 		v.xy = 0;
 	}
 	int NTranslateBits=UB(5,bs);
-	v.x0=SB(NTranslateBits,bs)/20;
-	v.y0=SB(NTranslateBits,bs)/20;
+	v.x0=SB(NTranslateBits,bs)/20.0;
+	v.y0=SB(NTranslateBits,bs)/20.0;
 	return stream;
 }
 
@@ -1240,7 +1246,7 @@ std::istream& lightspark::operator>>(std::istream& stream, FILTER& v)
 			stream >> v.GradientBevelFilter;
 			break;
 		default:
-			LOG(LOG_ERROR,_("Unsupported Filter Id ") << (int)v.FilterID);
+			LOG(LOG_ERROR,"Unsupported Filter Id " << (int)v.FilterID);
 			throw ParseException("Unsupported Filter Id");
 	}
 	return stream;
@@ -1256,7 +1262,7 @@ std::istream& lightspark::operator>>(std::istream& stream, GLOWFILTER& v)
 	v.InnerGlow = UB(1,bs);
 	v.Knockout = UB(1,bs);
 	v.CompositeSource = UB(1,bs);
-	bs.discard(5);
+	v.Passes = UB(5,bs);
 
 	return stream;
 }
@@ -1273,7 +1279,7 @@ std::istream& lightspark::operator>>(std::istream& stream, DROPSHADOWFILTER& v)
 	v.InnerShadow = UB(1,bs);
 	v.Knockout = UB(1,bs);
 	v.CompositeSource = UB(1,bs);
-	bs.discard(5);
+	v.Passes = UB(5,bs);
 
 	return stream;
 }
@@ -1303,7 +1309,7 @@ std::istream& lightspark::operator>>(std::istream& stream, BEVELFILTER& v)
 	v.Knockout = UB(1,bs);
 	v.CompositeSource = UB(1,bs);
 	v.OnTop = UB(1,bs);
-	bs.discard(4);
+	v.Passes = UB(4,bs);
 
 	return stream;
 }
@@ -1333,7 +1339,8 @@ std::istream& lightspark::operator>>(std::istream& stream, GRADIENTGLOWFILTER& v
 	v.InnerGlow = UB(1,bs);
 	v.Knockout = UB(1,bs);
 	v.CompositeSource = UB(1,bs);
-	bs.discard(5);
+	v.OnTop = UB(1,bs);
+	v.Passes = UB(4,bs);
 
 	return stream;
 }
@@ -1393,7 +1400,7 @@ std::istream& lightspark::operator>>(std::istream& stream, GRADIENTBEVELFILTER&
 	v.Knockout = UB(1,bs);
 	v.CompositeSource = UB(1,bs);
 	v.OnTop = UB(1,bs);
-	bs.discard(4);
+	v.Passes = UB(4,bs);
 
 	return stream;
 }
@@ -1514,81 +1521,90 @@ std::istream& lightspark::operator>>(std::istream& s, CLIPACTIONS& v)
 	}
 	return s;
 }
-ASObject* lightspark::abstract_s(SystemState *sys)
+ASObject* lightspark::abstract_s(ASWorker* wrk)
 {
-	ASString* ret= Class<ASString>::getInstanceSNoArgs(sys);
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(wrk);
 	ret->stringId = BUILTIN_STRINGS::EMPTY;
 	ret->hasId = true;
 	ret->datafilled=true;
 	return ret;
 }
-ASObject* lightspark::abstract_s(SystemState *sys, const char* s, uint32_t len)
+ASObject* lightspark::abstract_s(ASWorker* wrk, const char* s, uint32_t len)
 {
-	ASString* ret= Class<ASString>::getInstanceSNoArgs(sys);
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(wrk);
 	ret->data = std::string(s,len);
 	ret->stringId = UINT32_MAX;
 	ret->hasId = false;
 	ret->datafilled=true;
 	return ret;
 }
-ASObject* lightspark::abstract_s(SystemState *sys, const char* s)
+ASObject* lightspark::abstract_s(ASWorker* wrk, const char* s, int numbytes, int numchars, bool issinglebyte, bool hasNull)
 {
-	ASString* ret= Class<ASString>::getInstanceSNoArgs(sys);
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(wrk);
+	ret->data.setValue(s,numbytes,numchars,issinglebyte,hasNull,true);
+	ret->stringId = UINT32_MAX;
+	ret->hasId = false;
+	ret->datafilled=true;
+	return ret;
+}
+ASObject* lightspark::abstract_s(ASWorker* wrk, const char* s)
+{
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(wrk);
 	ret->data = s;
 	ret->stringId = UINT32_MAX;
 	ret->hasId = false;
 	ret->datafilled=true;
 	return ret;
 }
-ASObject* lightspark::abstract_s(SystemState *sys, const tiny_string& s)
+ASObject* lightspark::abstract_s(ASWorker* wrk, const tiny_string& s)
 {
-	ASString* ret= Class<ASString>::getInstanceSNoArgs(sys);
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(wrk);
 	ret->data = s;
 	ret->stringId = UINT32_MAX;
 	ret->hasId = false;
 	ret->datafilled=true;
 	return ret;
 }
-ASObject* lightspark::abstract_s(SystemState *sys, uint32_t stringId)
+ASObject* lightspark::abstract_s(ASWorker* wrk, uint32_t stringId)
 {
-	ASString* ret= Class<ASString>::getInstanceSNoArgs(sys);
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(wrk);
 	ret->stringId = stringId;
 	ret->hasId = true;
 	ret->datafilled=false;
 	return ret;
 }
-ASObject* lightspark::abstract_d(SystemState* sys,number_t i)
+ASObject* lightspark::abstract_d(ASWorker* wrk, number_t i)
 {
-	Number* ret=Class<Number>::getInstanceSNoArgs(sys);
+	Number* ret=Class<Number>::getInstanceSNoArgs(wrk);
 	ret->dval = i;
 	ret->isfloat = true;
 	return ret;
 }
-ASObject* lightspark::abstract_d_constant(SystemState* sys,number_t i)
+ASObject* lightspark::abstract_d_constant(ASWorker* wrk, number_t i)
 {
-	Number* ret=new (sys->unaccountedMemory) Number(Class<Number>::getRef(sys).getPtr());
+	Number* ret=new (wrk->getSystemState()->unaccountedMemory) Number(wrk,Class<Number>::getRef(wrk->getSystemState()).getPtr());
 	ret->dval = i;
 	ret->isfloat = true;
 	ret->setRefConstant();
 	return ret;
 }
-ASObject* lightspark::abstract_di(SystemState* sys,int64_t i)
+ASObject* lightspark::abstract_di(ASWorker* wrk, int64_t i)
 {
-	Number* ret=Class<Number>::getInstanceSNoArgs(sys);
+	Number* ret=Class<Number>::getInstanceSNoArgs(wrk);
 	ret->ival = i;
 	ret->isfloat = false;
 	return ret;
 }
-ASObject* lightspark::abstract_i(SystemState *sys, int32_t i)
+ASObject* lightspark::abstract_i(ASWorker* wrk, int32_t i)
 {
-	Integer* ret=Class<Integer>::getInstanceSNoArgs(sys);
+	Integer* ret=Class<Integer>::getInstanceSNoArgs(wrk);
 	ret->val = i;
 	return ret;
 }
 
-ASObject* lightspark::abstract_ui(SystemState *sys, uint32_t i)
+ASObject* lightspark::abstract_ui(ASWorker* wrk, uint32_t i)
 {
-	UInteger* ret=Class<UInteger>::getInstanceSNoArgs(sys);
+	UInteger* ret=Class<UInteger>::getInstanceSNoArgs(wrk);
 	ret->val = i;
 	return ret;
 }
@@ -1641,7 +1657,7 @@ void lightspark::stringToQName(const tiny_string& tmp, tiny_string& name, tiny_s
 	ns="";
 }
 
-RunState::RunState():last_FP(-1),FP(0),next_FP(0),stop_FP(false),explicit_FP(false),creatingframe(false),frameadvanced(false),avm1ScriptExecuted(false)
+RunState::RunState():last_FP(-1),FP(0),next_FP(0),stop_FP(false),explicit_FP(false),creatingframe(false),frameadvanced(false)
 {
 }
 
@@ -1700,6 +1716,35 @@ FILLSTYLE& FILLSTYLE::operator=(FILLSTYLE r)
 	return *this;
 }
 
+LINESTYLE2::LINESTYLE2(const LINESTYLE2& r):StartCapStyle(r.StartCapStyle),JointStyle(r.JointStyle),HasFillFlag(r.HasFillFlag),
+	NoHScaleFlag(r.NoHScaleFlag),NoVScaleFlag(r.NoVScaleFlag),PixelHintingFlag(r.PixelHintingFlag),
+	Width(r.Width),MiterLimitFactor(r.MiterLimitFactor),Color(r.Color),
+	FillType(r.FillType),version(r.version)
+{
+}
+
+LINESTYLE2::~LINESTYLE2()
+{
+}
+
+LINESTYLE2& LINESTYLE2::operator=(LINESTYLE2 r)
+{
+	StartCapStyle=r.StartCapStyle;
+	JointStyle=r.JointStyle;
+	HasFillFlag=r.HasFillFlag;
+	NoHScaleFlag=r.NoHScaleFlag;
+	NoVScaleFlag=r.NoVScaleFlag;
+	PixelHintingFlag=r.PixelHintingFlag;
+	NoClose=r.NoClose;
+	EndCapStyle=r.EndCapStyle;
+	Width=r.Width;
+	MiterLimitFactor=r.MiterLimitFactor;
+	Color=r.Color;
+	FillType=r.FillType;
+	version=r.version;
+	return *this;
+}
+
 nsNameAndKind::nsNameAndKind(SystemState* sys,const tiny_string& _name, NS_KIND _kind)
 {
 	nsNameId = sys->getUniqueStringId(_name);
@@ -1747,7 +1792,7 @@ nsNameAndKind::nsNameAndKind(ABCContext* c, uint32_t nsContextIndex)
 {
 	const namespace_info& ns=c->constant_pool.namespaces[nsContextIndex];
 	nsNameId=c->getString(ns.name);
-	nsNameAndKindImpl tmp(nsNameId, (NS_KIND)(int)ns.kind,((NS_KIND)(int)ns.kind)==PROTECTED_NAMESPACE ? c->root.getPtr() : nullptr);
+	nsNameAndKindImpl tmp(nsNameId, (NS_KIND)(int)ns.kind,((NS_KIND)(int)ns.kind)==PROTECTED_NAMESPACE ? c->root : nullptr);
 	//Give an id hint, in case the namespace is created in the map
 	c->root->getSystemState()->getUniqueNamespaceId(tmp, c->namespaceBaseId+nsContextIndex, nsRealId, nsId);
 	//Special handling for private namespaces, they are always compared by id
@@ -1784,7 +1829,7 @@ RGB::RGB(const tiny_string& colorstr):Red(0),Green(0),Blue(0)
 	if (s[0] == '#')
 		s++;
 
-	gint64 color = g_ascii_strtoll(s, NULL, 16);
+	gint64 color = g_ascii_strtoll(s, nullptr, 16);
 	Red = (color >> 16) & 0xFF;
 	Green = (color >> 8) & 0xFF;
 	Blue = color & 0xFF;
@@ -1820,9 +1865,6 @@ std::istream& lightspark::operator>>(std::istream& stream, SOUNDINFO& v)
 	if (v.HasEnvelope)
 	{
 		stream >> v.EnvPoints;
-		if (v.EnvPoints)
-			LOG(LOG_NOT_IMPLEMENTED,"SOUNDENVELOPE settings are read but not used");
-			
 		for (unsigned int i = 0; i < v.EnvPoints;i++)
 		{
 			SOUNDENVELOPE env;
