@@ -47,6 +47,8 @@ enum BUILTIN_STRINGS { EMPTY=0, STRING_WILDCARD='*', ANY=BUILTIN_STRINGS_CHAR_MA
 					   ,STRING_ONENTERFRAME,STRING_ONMOUSEMOVE,STRING_ONMOUSEDOWN,STRING_ONMOUSEUP,STRING_ONPRESS,STRING_ONRELEASE,STRING_ONRELEASEOUTSIDE,STRING_ONMOUSEWHEEL, STRING_ONLOAD
 					   ,STRING_OBJECT,STRING_UNDEFINED,STRING_BOOLEAN,STRING_NUMBER,STRING_STRING,STRING_FUNCTION_LOWERCASE,STRING_ONROLLOVER,STRING_ONROLLOUT
 					   ,STRING_PROTO,STRING_TARGET,STRING_FLASH_EVENTS_IEVENTDISPATCHER,STRING_ADDEVENTLISTENER,STRING_REMOVEEVENTLISTENER,STRING_DISPATCHEVENT,STRING_HASEVENTLISTENER
+					   ,STRING_ONCONNECT,STRING_ONDATA,STRING_ONCLOSE,STRING_ONSELECT
+					   ,STRING_ADD,STRING_ALPHA,STRING_DARKEN,STRING_DIFFERENCE,STRING_ERASE,STRING_HARDLIGHT,STRING_INVERT,STRING_LAYER,STRING_LIGHTEN,STRING_MULTIPLY,STRING_OVERLAY,STRING_SCREEN,STRING_SUBTRACT
 					   ,LAST_BUILTIN_STRING };
 enum BUILTIN_NAMESPACES { EMPTY_NS=0, AS3_NS };
 
@@ -64,7 +66,9 @@ enum CLASS_SUBTYPE { SUBTYPE_NOT_SET, SUBTYPE_PROXY, SUBTYPE_REGEXP, SUBTYPE_XML
 					 ,SUBTYPE_CONTEXT3D,SUBTYPE_TEXTUREBASE,SUBTYPE_TEXTURE,SUBTYPE_CUBETEXTURE,SUBTYPE_RECTANGLETEXTURE,SUBTYPE_VIDEOTEXTURE,SUBTYPE_VECTOR3D,SUBTYPE_NETSTREAM
 					 ,SUBTYPE_WORKER,SUBTYPE_WORKERDOMAIN,SUBTYPE_MUTEX,SUBTYPE_AVM1FUNCTION,SUBTYPE_SAMPLEDATA_EVENT
 					 ,SUBTYPE_BITMAPFILTER,SUBTYPE_GLOWFILTER,SUBTYPE_DROPSHADOWFILTER,SUBTYPE_GRADIENTGLOWFILTER,SUBTYPE_BEVELFILTER,SUBTYPE_COLORMATRIXFILTER,SUBTYPE_BLURFILTER,SUBTYPE_CONVOLUTIONFILTER,SUBTYPE_DISPLACEMENTFILTER,SUBTYPE_GRADIENTBEVELFILTER,SUBTYPE_SHADERFILTER
-					 ,SUBTYPE_THROTTLE_EVENT,SUBTYPE_CONTEXTMENUEVENT,SUBTYPE_GAMEINPUTEVENT, SUBTYPE_GAMEINPUTDEVICE, SUBTYPE_VIDEO
+					 ,SUBTYPE_THROTTLE_EVENT,SUBTYPE_CONTEXTMENUEVENT,SUBTYPE_GAMEINPUTEVENT, SUBTYPE_GAMEINPUTDEVICE, SUBTYPE_VIDEO, SUBTYPE_MESSAGECHANNEL, SUBTYPE_CONDITION
+					 ,SUBTYPE_FILE, SUBTYPE_FILEMODE, SUBTYPE_FILESTREAM, SUBTYPE_FILEREFERENCE, SUBTYPE_DATAGRAMSOCKET, SUBTYPE_NATIVEWINDOW,SUBTYPE_EXTENSIONCONTEXT,SUBTYPE_SIMPLEBUTTON,SUBTYPE_SHAPE,SUBTYPE_MORPHSHAPE
+					 ,SUBTYPE_URLLOADER,SUBTYPE_URLREQUEST,SUBTYPE_DICTIONARY,SUBTYPE_TEXTLINEMETRICS,SUBTYPE_XMLNODE,SUBTYPE_XMLDOCUMENT,SUBTYPE_LOADER
 				   };
  
 enum STACK_TYPE{STACK_NONE=0,STACK_OBJECT,STACK_INT,STACK_UINT,STACK_NUMBER,STACK_BOOLEAN};
@@ -73,6 +77,15 @@ enum AS_BLENDMODE { BLENDMODE_NORMAL=0, BLENDMODE_LAYER=2, BLENDMODE_MULTIPLY=3,
 					BLENDMODE_ADD=8, BLENDMODE_SUBTRACT=9,BLENDMODE_INVERT=10,BLENDMODE_ALPHA=11,BLENDMODE_ERASE=12,BLENDMODE_OVERLAY=13,BLENDMODE_HARDLIGHT=14
 				  };
 
+enum LS_VIDEO_CODEC { H264=0, H263, VP6, VP6A, GIF };
+// "Audio coding formats" from Chapter 11 in SWF documentation (except for LINEAR_PCM_FLOAT_PLATFORM_ENDIAN)
+enum LS_AUDIO_CODEC { CODEC_NONE=-1, LINEAR_PCM_PLATFORM_ENDIAN=0, ADPCM=1, MP3=2, LINEAR_PCM_LE=3, AAC=10, LINEAR_PCM_FLOAT_PLATFORM_ENDIAN = 100 };
+
+enum OBJECT_ENCODING { AMF0=0, AMF3=3, DEFAULT=3 };
+
+// Enum used during early binding in abc_optimizer.cpp
+enum EARLY_BIND_STATUS { NOT_BINDED=0, CANNOT_BIND=1, BINDED };
+
 inline std::ostream& operator<<(std::ostream& s, const STACK_TYPE& st)
 {
 	switch(st)
@@ -118,6 +131,7 @@ class MovieClip;
 class Frame;
 class AVM1Function;
 class AVM1context;
+class ASWorker;
 struct namespace_info;
 
 struct multiname;
@@ -443,7 +457,7 @@ struct multiname: public memory_reporter
 
 	const tiny_string qualifiedString(SystemState *sys, bool forDescribeType=false) const;
 	/* sets name_type, name_s/name_d based on the object n */
-	void setName(union asAtom &n, SystemState *sys);
+	void setName(union asAtom &n, ASWorker* w);
 	void resetNameIfObject();
 	inline bool isQName() const { return ns.size() == 1; }
 	bool toUInt(SystemState *sys, uint32_t& out, bool acceptStringFractions=false, bool* isNumber=nullptr) const;
@@ -505,11 +519,27 @@ public:
 	}
 };
 
-//TODO: Really implement or suppress
-typedef UI32_SWF FIXED;
+class FIXED
+{
+friend std::istream& operator>>(std::istream& s, FIXED& v);
+protected:
+	int32_t val;
+public:
+	FIXED():val(0){}
+	FIXED(int32_t v):val(v){}
+	operator number_t() const{ return number_t(val)/65536.0; }
+};
 
-//TODO: Really implement or suppress
-typedef UI16_SWF FIXED8;
+class FIXED8
+{
+friend std::istream& operator>>(std::istream& s, FIXED8& v);
+protected:
+	int16_t val;
+public:
+	FIXED8():val(0){}
+	FIXED8(int16_t v):val(v){}
+	operator number_t() const{ return number_t(val)/256.0; }
+};
 
 class RECORDHEADER
 {
@@ -741,6 +771,19 @@ inline std::istream& operator>>(std::istream& s, DOUBLE& v)
 	return s;
 }
 
+inline std::istream& operator>>(std::istream& s, FIXED& v)
+{
+	s.read((char*)&v.val,4);
+	v.val=GINT32_FROM_LE(v.val);
+	return s;
+}
+inline std::istream& operator>>(std::istream& s, FIXED8& v)
+{
+	s.read((char*)&v.val,2);
+	v.val=GINT16_FROM_LE(v.val);
+	return s;
+}
+
 inline std::istream& operator>>(std::istream& s, RECORDHEADER& v)
 {
 	s >> v.CodeAndLen;
@@ -789,7 +832,7 @@ public:
 	FB(int s,BitStream& stream)
 	{
 		if(s>32)
-			LOG(LOG_ERROR,_("Fixed point bit field wider than 32 bit not supported"));
+			LOG(LOG_ERROR,"Fixed point bit field wider than 32 bit not supported");
 		buf=stream.readBits(s);
 		if(buf>>(s-1)&1)
 		{
@@ -832,7 +875,7 @@ public:
 			i++;
 		}*/
 		if(s>32)
-			LOG(LOG_ERROR,_("Unsigned bit field wider than 32 bit not supported"));
+			LOG(LOG_ERROR,"Unsigned bit field wider than 32 bit not supported");
 		buf=stream.readBits(s);
 	}
 	operator int() const
@@ -849,7 +892,7 @@ public:
 	SB(int s,BitStream& stream)
 	{
 		if(s>32)
-			LOG(LOG_ERROR,_("Signed bit field wider than 32 bit not supported"));
+			LOG(LOG_ERROR,"Signed bit field wider than 32 bit not supported");
 		buf=stream.readBits(s);
 		if(buf>>(s-1)&1)
 		{
@@ -1028,6 +1071,7 @@ public:
 	int InterpolationMode;
 	FIXED8 StartFocalPoint;
 	FIXED8 EndFocalPoint;
+	std::map<uint16_t,FILLSTYLE> fillstylecache;
 
 	~MORPHFILLSTYLE(){}
 };
@@ -1045,6 +1089,9 @@ class LINESTYLE2
 {
 public:
 	LINESTYLE2(uint8_t v):StartCapStyle(0),JointStyle(0),HasFillFlag(false),NoHScaleFlag(false),NoVScaleFlag(false),PixelHintingFlag(0),FillType(v),version(v){}
+	LINESTYLE2(const LINESTYLE2& r);
+	LINESTYLE2& operator=(LINESTYLE2 r);
+	virtual ~LINESTYLE2();
 	int StartCapStyle;
 	int JointStyle;
 	bool HasFillFlag;
@@ -1082,6 +1129,7 @@ public:
 	UB NoClose;
 	UB EndCapStyle;
 	UI16_SWF MiterLimitFactor;
+	std::map<uint16_t,LINESTYLE2> linestylecache;
 };
 
 class LINESTYLEARRAY
@@ -1256,7 +1304,7 @@ public:
     FIXED BlurY;
     FIXED Angle;
     FIXED Distance;
-    UB Passes;
+    int Passes;
     FIXED8 Strength;
     bool InnerShadow;
     bool Knockout;
@@ -1268,7 +1316,7 @@ class BLURFILTER
 public:
 	FIXED BlurX;
 	FIXED BlurY;
-	UB Passes;
+	int Passes;
 };
 
 class GLOWFILTER
@@ -1277,7 +1325,7 @@ public:
     RGBA GlowColor;
     FIXED BlurX;
     FIXED BlurY;
-    UB Passes;
+    int Passes;
     FIXED8 Strength;
     bool InnerGlow;
     bool Knockout;
@@ -1293,7 +1341,7 @@ public:
     FIXED BlurY;
     FIXED Angle;
     FIXED Distance;
-    UB Passes;
+    int Passes;
     FIXED8 Strength;
     bool InnerShadow;
     bool Knockout;
@@ -1310,11 +1358,12 @@ public:
     FIXED BlurY;
     FIXED Angle;
     FIXED Distance;
-    UB Passes;
+    int Passes;
     FIXED8 Strength;
     bool InnerGlow;
     bool Knockout;
     bool CompositeSource;
+	bool OnTop;
 };
 
 class CONVOLUTIONFILTER
@@ -1345,7 +1394,7 @@ public:
     FIXED BlurY;
     FIXED Angle;
     FIXED Distance;
-    UB Passes;
+    int Passes;
     FIXED8 Strength;
     bool InnerShadow;
     bool Knockout;
@@ -1486,7 +1535,6 @@ public:
 	bool explicit_FP;
 	bool creatingframe;
 	bool frameadvanced;
-	bool avm1ScriptExecuted;
 	RunState();
 	inline void reset()
 	{
@@ -1497,7 +1545,6 @@ public:
 		explicit_FP = false;
 		creatingframe = false;
 		frameadvanced = false;
-		avm1ScriptExecuted=false;
 	}
 };
 class Activation_object;
@@ -1507,7 +1554,7 @@ public:
 	static void PushStack(std::stack<asAtom>& stack,const asAtom& a);
 	static asAtom PopStack(std::stack<asAtom>& stack);
 	static asAtom PeekStack(std::stack<asAtom>& stack);
-	static void executeActions(DisplayObject* clip, AVM1context* context, const std::vector<uint8_t> &actionlist, uint32_t startactionpos, std::map<uint32_t, union asAtom> &scopevariables, asAtom *result = nullptr, asAtom* obj = nullptr, asAtom *args = nullptr, uint32_t num_args=0, const std::vector<uint32_t>& paramnames=std::vector<uint32_t>(), const std::vector<uint8_t>& paramregisternumbers=std::vector<uint8_t>(),
+	static void executeActions(DisplayObject* clip, AVM1context* context, const std::vector<uint8_t> &actionlist, uint32_t startactionpos, std::map<uint32_t, asAtom> &scopevariables, bool fromInitAction = false, asAtom *result = nullptr, asAtom* obj = nullptr, asAtom *args = nullptr, uint32_t num_args=0, const std::vector<uint32_t>& paramnames=std::vector<uint32_t>(), const std::vector<uint8_t>& paramregisternumbers=std::vector<uint8_t>(),
 			bool preloadParent=false, bool preloadRoot=false, bool suppressSuper=true, bool preloadSuper=false, bool suppressArguments=false, bool preloadArguments=false, bool suppressThis=true, bool preloadThis=false, bool preloadGlobal=false, AVM1Function *caller = nullptr, AVM1Function *callee = nullptr, Activation_object *actobj=nullptr, asAtom* superobj=nullptr);
 };
 class BUTTONCONDACTION
@@ -1532,17 +1579,18 @@ public:
 	uint32_t startactionpos;
 	std::vector<uint8_t> actions;
 };
-
-ASObject* abstract_i(SystemState *sys, int32_t i);
-ASObject* abstract_ui(SystemState *sys, uint32_t i);
-ASObject* abstract_d(SystemState *sys, number_t i);
-ASObject* abstract_d_constant(SystemState *sys, number_t i);
-ASObject* abstract_di(SystemState *sys, int64_t i);
-ASObject* abstract_s(SystemState *sys);
-ASObject* abstract_s(SystemState *sys, const char* s, uint32_t len);
-ASObject* abstract_s(SystemState *sys, const char* s);
-ASObject* abstract_s(SystemState *sys, const tiny_string& s);
-ASObject* abstract_s(SystemState *sys, uint32_t stringId);
+class ASWorker;
+ASObject* abstract_i(ASWorker* wrk, int32_t i);
+ASObject* abstract_ui(ASWorker* wrk, uint32_t i);
+ASObject* abstract_d(ASWorker* wrk, number_t i);
+ASObject* abstract_d_constant(ASWorker* wrk, number_t i);
+ASObject* abstract_di(ASWorker* wrk, int64_t i);
+ASObject* abstract_s(ASWorker* wrk);
+ASObject* abstract_s(ASWorker* wrk, const char* s, uint32_t len);
+ASObject* abstract_s(ASWorker* wrk, const char* s, int numbytes, int numchars, bool issinglebyte, bool hasNull);
+ASObject* abstract_s(ASWorker* wrk, const char* s);
+ASObject* abstract_s(ASWorker* wrk, const tiny_string& s);
+ASObject* abstract_s(ASWorker* wrk, uint32_t stringId);
 ASObject* abstract_null(SystemState *sys);
 ASObject* abstract_undefined(SystemState *sys);
 ASObject* abstract_b(SystemState *sys, bool b);
