@@ -23,10 +23,11 @@
 #include "compat.h"
 #include "logger.h"
 #include "swf.h"
+#include "scripting/flash/system/flashsystem.h"
 
 using namespace lightspark;
 
-ThreadPool::ThreadPool(SystemState* s):num_jobs(0),stopFlag(false)
+ThreadPool::ThreadPool(SystemState* s):num_jobs(0),stopFlag(false),runcount(0)
 {
 	m_sys=s;
 	for(uint32_t i=0;i<NUM_THREADS;i++)
@@ -97,8 +98,10 @@ int ThreadPool::job_worker(void *d)
 		IThreadJob* myJob=data->pool->jobs.front();
 		data->pool->jobs.pop_front();
 		data->pool->curJobs[data->index]=myJob;
+		data->pool->runcount++;
 		l.release();
 
+		setTLSWorker(myJob->fromWorker);
 		chronometer.checkpoint();
 		try
 		{
@@ -113,7 +116,7 @@ int ThreadPool::job_worker(void *d)
 		}
 		catch(LightsparkException& e)
 		{
-			LOG(LOG_ERROR,_("Exception in ThreadPool ") << e.what());
+			LOG(LOG_ERROR,"Exception in ThreadPool " << e.what());
 			data->pool->m_sys->setError(e.cause);
 		}
 		catch(std::exception& e)
@@ -126,6 +129,7 @@ int ThreadPool::job_worker(void *d)
 
 		l.acquire();
 		data->pool->curJobs[data->index]=nullptr;
+		data->pool->runcount--;
 		l.release();
 
 		//jobFencing is allowed to happen outside the mutex
@@ -137,13 +141,54 @@ int ThreadPool::job_worker(void *d)
 void ThreadPool::addJob(IThreadJob* j)
 {
 	Locker l(mutex);
+	j->setWorker(getWorker());
 	if(stopFlag)
 	{
 		j->jobFence();
 		return;
 	}
 	assert(j);
-	jobs.push_back(j);
-	num_jobs.signal();
+	if (runcount == NUM_THREADS)
+	{
+		// no slot available, we create an additional thread
+		runAdditionalThread(j);
+	}
+	else
+	{
+		jobs.push_back(j);
+		num_jobs.signal();
+	}
+}
+void ThreadPool::runAdditionalThread(IThreadJob* j)
+{
+	SDL_Thread* t = SDL_CreateThread(additional_job_worker,"additionalThread",j);
+	SDL_DetachThread(t);
 }
 
+int ThreadPool::additional_job_worker(void* d)
+{
+	IThreadJob* myJob=(IThreadJob*)d;
+
+	setTLSSys(myJob->fromWorker->getSystemState());
+	setTLSWorker(myJob->fromWorker);
+	try
+	{
+		myJob->execute();
+	}
+	catch(JobTerminationException& ex)
+	{
+		LOG(LOG_NOT_IMPLEMENTED,"Job terminated");
+	}
+	catch(LightsparkException& e)
+	{
+		LOG(LOG_ERROR,"Exception in AdditionalThread " << e.what());
+		myJob->fromWorker->getSystemState()->setError(e.cause);
+	}
+	catch(std::exception& e)
+	{
+		LOG(LOG_ERROR,"std Exception in AdditionalThread:"<<myJob<<" "<<e.what());
+		myJob->fromWorker->getSystemState()->setError(e.what());
+	}
+	myJob->jobFence();
+	return 0;
+}
