@@ -143,10 +143,13 @@ public:
 		lighted=true;
 	}
 };
+class ASWorker;
 
 class IThreadJob
 {
 friend class ThreadPool;
+private:
+	ASWorker* fromWorker;
 public:
 	/*
 	 * Set to true by the ThreadPool just before threadAbort()
@@ -177,30 +180,35 @@ public:
 	 * 'delete this'.
 	 */
 	virtual void jobFence()=0;
-	IThreadJob() : threadAborting(false) {}
+	IThreadJob() : fromWorker(nullptr),threadAborting(false) {}
 	virtual ~IThreadJob() {}
+	void setWorker(ASWorker* w) { fromWorker = w;}
 };
 
-template<class T, uint32_t size>
+template<class T>
 class BlockingCircularQueue
 {
 private:
-	T queue[size];
+	T* queue;
 	//Counting semaphores for the queue
 	Semaphore freeBuffers;
 	Semaphore usedBuffers;
 	uint32_t bufferHead;
 	uint32_t bufferTail;
-	bool empty;
+	uint32_t size;
+	ACQUIRE_RELEASE_FLAG(empty);
 public:
-	BlockingCircularQueue():freeBuffers(size),usedBuffers(0),bufferHead(0),bufferTail(0),empty(true)
+	BlockingCircularQueue(uint32_t _size):freeBuffers(_size),usedBuffers(0),bufferHead(0),bufferTail(0),size(_size),empty(true)
 	{
+		aligned_malloc((void**)&queue,16,size*sizeof(T));
+		for(uint32_t i=0;i<size;i++)
+			queue[i].init();
 	}
-	template<class GENERATOR>
-	BlockingCircularQueue(const GENERATOR& g):freeBuffers(size),usedBuffers(0),bufferHead(0),bufferTail(0),empty(true)
+	~BlockingCircularQueue()
 	{
 		for(uint32_t i=0;i<size;i++)
-			g.init(queue[i]);
+			queue[i].cleanup();
+		aligned_free(queue);
 	}
 	bool isEmpty() const { return empty; }
 	T& front()
