@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2013  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -89,7 +89,7 @@ int main(int argc, char* argv[])
 	//One of useInterpreter or useJit must be enabled
 	if(!(useInterpreter || useJit))
 	{
-		LOG(LOG_ERROR,_("No execution model enabled"));
+		LOG(LOG_ERROR,"No execution model enabled");
 		exit(-1);
 	}
 	sys->useInterpreter=useInterpreter;
@@ -114,15 +114,14 @@ int main(int argc, char* argv[])
 		ifstream f(fileNames[i]);
 		if(f.is_open())
 		{
-			sys->mainClip->incRef();
-			ABCContext* context=new ABCContext(_MR(sys->mainClip), f, vm);
+			ABCContext* context=new ABCContext(sys->mainClip, f, vm);
 			contexts.push_back(context);
 			f.close();
 			vm->addEvent(NullRef,_MR(new (sys->unaccountedMemory) ABCContextInitEvent(context,false)));
 		}
 		else
 		{
-			LOG(LOG_ERROR, fileNames[i] << _(" could not be opened for execution"));
+			LOG(LOG_ERROR, fileNames[i] << " could not be opened for execution");
 		}
 	}
 	vm->start();
