@@ -290,13 +290,18 @@ bool tiny_string::endsWith(const char* o) const
  * returns index of character */
 uint32_t tiny_string::find(const tiny_string& needle, uint32_t start) const
 {
-	//TODO: omit copy into std::string
-	size_t bytestart = g_utf8_offset_to_pointer(buf,start) - buf;
-	size_t bytepos = std::string(*this).find(needle.raw_buf(),bytestart,needle.numBytes());
-	if(bytepos == std::string::npos)
+	if (isASCII && !hasNull)
+	{
+		// fast path for ascii strings
+		const char* p = strstr(buf+start,needle.raw_buf());
+		return (p ? p-buf : npos);
+	}
+	gchar* gp = g_utf8_offset_to_pointer(buf,start);
+	gchar* found =g_strstr_len(gp,-1,needle.raw_buf());
+	if(found == nullptr)
 		return npos;
 	else
-		return g_utf8_pointer_to_offset(buf,buf+bytepos);
+		return start + g_utf8_pointer_to_offset(gp,found);
 }
 
 uint32_t tiny_string::rfind(const tiny_string& needle, uint32_t start) const
@@ -419,15 +424,36 @@ tiny_string& tiny_string::replace(uint32_t pos1, uint32_t n1, const tiny_string&
 
 tiny_string& tiny_string::replace_bytes(uint32_t bytestart, uint32_t bytenum, const tiny_string& o)
 {
-	//TODO avoid copy into std::string
-	*this = std::string(*this).replace(bytestart,bytenum,std::string(o));
-	this->init();
+	uint32_t newlen = this->stringSize+o.numBytes()-bytenum;
+	assert(bytestart+bytenum<stringSize);
+	char* newbuf = new char[newlen];
+	memcpy(newbuf,this->raw_buf(),bytestart);
+	memcpy(newbuf+bytestart,o.raw_buf(),o.numBytes());
+	memcpy(newbuf+bytestart+o.numBytes(),this->raw_buf()+bytestart+bytenum,this->stringSize-(bytestart+bytenum));
+	newbuf[newlen-1] = '\0';
+	if(type==DYNAMIC)
+	{
+		reportMemoryChange(-stringSize);
+		delete[] buf;
+	}
+	this->type=DYNAMIC;
+	this->buf=newbuf;
+	this->stringSize=newlen;
+	if (this->isASCII && o.isASCII)
+	{
+		this->numchars = newlen-1;
+		this->hasNull |= o.hasNull;
+	}
+	else
+		this->init();
 	return *this;
 }
 
-tiny_string tiny_string::substr_bytes(uint32_t start, uint32_t len) const
+tiny_string tiny_string::substr_bytes(uint32_t start, uint32_t len, bool resultisascii) const
 {
 	tiny_string ret;
+	if (start >= stringSize)
+		return ret;
 	if ((len == UINT32_MAX) || (start+len >= stringSize))
 		len =stringSize-(start+1);
 	assert(start+len < stringSize);
@@ -438,6 +464,11 @@ tiny_string tiny_string::substr_bytes(uint32_t start, uint32_t len) const
 	ret.stringSize = len+1;
 	if (this->isASCII && !this->hasNull)
 		ret.numchars = len;
+	else if (resultisascii)
+	{
+		ret.numchars = len;
+		ret.hasNull=false;
+	}
 	else
 		ret.init();
 	return ret;
@@ -446,13 +477,13 @@ tiny_string tiny_string::substr_bytes(uint32_t start, uint32_t len) const
 tiny_string tiny_string::substr(uint32_t start, uint32_t len) const
 {
 	assert_and_throw(start <= numChars());
-	if(start+len > numChars())
+	if(len > numChars()-start)
 		len = numChars()-start;
 	if (isASCII)
 		return substr_bytes(start, len);
 	uint32_t bytestart = g_utf8_offset_to_pointer(buf,start) - buf;
-	uint32_t byteend = g_utf8_offset_to_pointer(buf,start+len) - buf;
-	return substr_bytes(bytestart, byteend-bytestart);
+	uint32_t byteend = g_utf8_offset_to_pointer(buf+bytestart,len) - (buf+bytestart);
+	return substr_bytes(bytestart, byteend,byteend-bytestart == len && !this->hasNull);
 }
 
 tiny_string tiny_string::substr(uint32_t start, const CharIterator& end) const
@@ -1178,6 +1209,50 @@ int tiny_string::compare(const tiny_string& r) const
 	return res;
 }
 
+tiny_string tiny_string::toQuotedString() const
+{
+	tiny_string res("\"");
+	for (CharIterator it=this->begin(); it!=this->end(); it++)
+	{
+		switch (*it)
+		{
+			case '\b':
+				res += "\\b";
+				break;
+			case '\f':
+				res += "\\f";
+				break;
+			case '\n':
+				res += "\\n";
+				break;
+			case '\r':
+				res += "\\r";
+				break;
+			case '\t':
+				res += "\\t";
+				break;
+			case '\"':
+				res += "\\\"";
+				break;
+			case '\\':
+				res += "\\\\";
+				break;
+			default:
+				if ((*it < 0x20) || (*it > 0xff))
+				{
+					char hexstr[7];
+					sprintf(hexstr,"\\u%04x",*it);
+					res += hexstr;
+				}
+				else
+					res += *it;
+				break;
+		}
+	}
+	res += "\"";
+	return res;
+}
+
 #ifdef MEMORY_USAGE_PROFILING
 void tiny_string::reportMemoryChange(int32_t change) const
 {
